-- Config.lua
-- Sets up the Setting for the Addon

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
-- Library Creation
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion, true) == nil) then
	Addon = LibStub("AceAddon-3.0"):NewAddon(AddonName.." "..AddonVersion, "AceEvent-3.0", "AceHook-3.0");
else
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
end

----------------------------------------------------------------------------------------------------
-- Libraries
----------------------------------------------------------------------------------------------------
--local LibObjectClass = LibStub:GetLibrary("FoxObjectClass");

-- Config Defaults
local configDefaults = {
	profile = {
		-- Main
		ShowStatbars 			= true,
		Debug					= false,
		RearrangeFormBar		= false,
		ShowBlizzOptions		= true,
		PositionFoxGoldTracker 	= true,
		ShowBagBar 				= true,
		PositionCarbonite		= true,
		Enabled					= true,
		HideXPBar				= false,
		ShowResourceBar 		= true,
		ShowBagBar 				= true,
		useLocalTime			= "Local",
		Layout 					= "Original",
		Style 					= "Original",
		Color 					= {r = 0.1, g = 0.1, b = 0.1, a = 0.6},
		ColorPre 				= "Grey",
	},
	char = {
		LoadedBefore = false,
		buttonData = {
			bars = {
				['**'] = {
					['**'] = {
						kind = "", value = "", hotkey = ""
					},
					[ 0] = {},
					[ 1] = {},
					[ 2] = {},
					[ 3] = {},
					[ 4] = {},
					[ 5] = {},
					[ 6] = {},
					[ 7] = {},
					[ 8] = {},
					[ 9] = {},
					[10] = {},
					[11] = {}
				},
				["FoxActionBarMain"] = {},
				["FoxActionBar1"] = {},
				["FoxActionBar2"] = {},
				["FoxActionBar3"] = {},
				["FoxActionBar4"] = {},
				["FoxActionBarStance"] = {},
				["FoxActionBarPet"] = {},
			}
		},
		UseCustomConfig = false,
	}
}


-- Config GUI
local options = {
    name = AddonName,
    handler = Addon,
    type = 'group',
    args = {
		enableToggle = {
			type = 'toggle',
			order = 0;
			name = 'Enable',
			desc = 'Addon is Enabled?',
			set = 'SetEnable',
			get = 'GetEnable',
			disabled  = function() return InCombatLockdown(); end,
		},
		debugOn = {
			type = 'toggle',
			name = 'Debug',
			desc = 'Turn On/Off Debugging?',
			set = 'SetDebug',
			get = 'GetDebug',
		},
		--[[
		useCustomConfig = {
			type = 'toggle',
			name = 'Custom Config',
			desc = 'Turn On/Off Custom Config?',
			set = 'SetCustomConfig',
			get = 'GetCustomConfig',
		},
		]]--
		resetToDefaults = {
			type = 'execute',
			name = 'Reset',
			desc = 'Reset Addon to Defaults?',
			func = 'ResetToDefaults',
		},
		mainOptions={
			type = 'group',
			name = 'Main Options',
			args={
				showBlizzOptions = {
					order = 1,
					type = 'toggle',
					name = 'Show Blizzard Options',
					desc = 'Show the Default Blizzard Options Bar?',
					set  = 'SetShowBlizzOptions',
					get  = 'GetShowBlizzOptions',
				},
				rearrangeFormBar = {
					order = 2,
					type = 'toggle',
					name = 'Rearrange Formbar',
					desc = 'Rearrange the Formbar?',
					set  = 'SetRearrangeFormBar',
					get  = 'GetRearrangeFormBar',
				},
				hideXPBar = {
					order = 3,
					type = 'toggle',
					name = 'Hide XP Bar',
					desc = 'Hide the Exp Bar?',
					set  = 'SetHideXPBar',
					get  = 'GetHideXPBar',
				},
				ShowResourceBar = {
					order = 4,
					type = 'toggle',
					name = 'Show Resource Bar',
					desc = 'Hide the Resource Bar?',
					set  = 'SetShowResourceBar',
					get  = 'GetShowResourceBar',
				},
				ShowBagBar = {
					order = 5,
					type = 'toggle',
					name = 'Show Bag Bar',
					desc = 'Hide the Bag Bar?',
					set  = 'SetShowBagBar',
					get  = 'GetShowBagBar',
				},
				UseLocalTime = {
					order = 6,
					type = 'select',
					name = 'What Time Source',
					desc = 'Use Local Time or Realm Time?',
					set  = 'SetUseLocalTime',
					get  = 'GetUseLocalTime',
					values = { ['Local'] = "Local", ['Realm'] = "Realm", ['Both'] = "Both", },
				},
				
				layoutChoice = {
					order = 30,
					type = 'select',
					name = 'Layout Used',
					desc = 'The Layout to be used.',
					get  = 'GetLayoutUsed',
					set  = 'SetLayoutUsed',
					values = { ['Original'] = "Original", ['Layout2'] = "New Layout", },
				},
				styleChoice = {
					order = 31,
					type = 'select',
					name = 'Style Used',
					desc = 'The Style to be used.',
					get  = 'GetStyleUsed',
					set  = 'SetStyleUsed',
					values = { ['Original'] = "Original", ['Square'] = "New Square Style", },
					style = 'dropdown',
				},
				colorPreChoice = {
					order = 32,
					type = 'select',
					name = 'PreBuilt Color Used',
					desc = 'The Color to be used.',
					get  = 'GetColorPreUsed',
					set  = 'SetColorPreUsed',
					values = { ['Black'] = "Black", ['Grey'] = "Grey", ['Red'] = "Red", ['Blue'] = "Blue", ['Green'] = "Green", ['Teal'] = "Teal", ['Light'] = "Light", ['Custom'] = "Custom", },
					style = 'dropdown',
				},
				colorChoice = {
					order = 33,
					type = 'color',
					name = 'Custom Color Used',
					desc = 'The Color to be used.',
					get  = 'GetColorUsed',
					set  = 'SetColorUsed',
					hasAlpha = true,
				},
				--[[
				positionFoxGoldTracker = {
					type = 'toggle',
					name = 'Position Gold Tracker',
					desc = 'Position the Fox Gold Tracker?',
					set  = 'SetPositionFoxGoldTracker',
					get  = 'GetPositionFoxGoldTracker',
				},
				--]]
				barHotkeyDefaults={
					type = 'group',
					name = 'Bar Hotkey Defaults',
					args={
						
					},
				},
			},
		},
		showConfig = {
			type = "execute",
			name = "Show Config",
			guiHidden = true, 
			dialogHidden = true, 
			dropdownHidden = true, 
			cmdHidden = false,
			func = "ToggleConfig",
		},
		
	},
}

function Addon.InitializeConfig()
	-- Setup Database for Config
	Addon.db = LibStub("AceDB-3.0"):New("FoxSmoothUIDB", configDefaults, true);
	
	-- Setup Config Panel
	-- Config, Setup Profile Switching
	options.args.mainOptions.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(Addon.db);
	
	-- Config, Setup Main
	LibStub("AceConfig-3.0"):RegisterOptionsTable(AddonName.."-"..AddonVersion, options, {"foxsmoothui", "fsui"});
	
	-- Config, Setup GUI
	Addon.ConfigFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(AddonName.."-"..AddonVersion, AddonName, parent);
	
	Addon.DebugPrint("Config Setup");
	
end
-- Config Variables Loaded
function Addon.ConfigVariablesLoaded()
	
	Addon.DebugPrint("Variables Loaded");
end
	
	
----------------------------------------------------------------------------------------------------
-- Config
----------------------------------------------------------------------------------------------------
-- Main
function Addon:ResetToDefaults()

	if (InCombatLockdown()) then
		return;
	end
	
	Addon.DebugPrint("ResetToDefaults");
	
	-- Main
	Addon.db.char.LoadedBefore 				= false;
	Addon.db.profile.ShowStatbars 			= true;
	Addon.db.profile.RearrangeFormBar		= false;

	--HideUIPanel(InterfaceOptionsFrame);
	--HideUIPanel(GameMenuFrame);
	
	-- Set Main Bar Hotkey
	Addon.ClearBarsHotkeys()
	Addon.SetupKeybindingDataBar(Addon.GetMainActionBar(), 0);
	
	-- Show First Time Prompt
	Addon.FirstLoad.Prompt();
end

function Addon:GetDebug()
	return Addon.db.profile.Debug;
end
function Addon:SetDebug(info, value)
	if (value == false) then
		Addon.DebugPrint("Turning Debug: Off");
	end
	-- Set Value
	Addon.db.profile.Debug = value;
	
	if (value) then
		Addon.DebugPrint("Turning Debug: On");
	end
	
end

function Addon:GetCustomConfig()
	return Addon.db.char.UseCustomConfig;
end
function Addon:SetCustomConfig(info, value)
	Addon.DebugPrint("Turning Debug: Off");
	-- Set Value
	Addon.db.char.UseCustomConfig = value;
	
	Addon.Config.Load();

	Addon.DebugPrint("UseCustomConfig: "..tostring(value));
end

function Addon:GetShowStatbars(info)
	return Addon.db.profile.ShowStatbars;
end
function Addon:SetShowStatbars(info, value)
	Addon.DebugPrint("SetShowStatbars: "..tostring(value));
	
	-- Set Value
	Addon.db.profile.ShowStatbars = value;
	
	-- Update Bars
	Addon.Update_BottomController(true);
end

function Addon:GetRearrangeFormBar(info)
	return Addon.db.profile.RearrangeFormBar;
end
function Addon:SetRearrangeFormBar(info, value)
	Addon.DebugPrint("SetShowMainMenuBar: "..tostring(value));
	
	-- Set Value
	Addon.db.profile.RearrangeFormBar = value;
	
	-- Update UI
	--Addon.Update_BottomController(true);
	Addon.Bottom.ControllerArr.Update()
end

function Addon:GetPositionFoxGoldTracker(info)
	return Addon.db.profile.PositionFoxGoldTracker;
end
function Addon:SetPositionFoxGoldTracker(info, value)
	Addon.DebugPrint("SetPositionFoxGoldTracker: "..tostring(value));
	
	-- Set Value
	Addon.db.profile.PositionFoxGoldTracker = value;
	
	-- Update UI
	Addon.Update_BottomController(true);
end

function Addon:GetShowBlizzOptions(info)
	return Addon.db.profile.ShowBlizzOptions;
end
function Addon:SetShowBlizzOptions(info, value)
	Addon.DebugPrint("SetShowBlizzOptions: "..tostring(value));
	
	-- Set Value
	Addon.db.profile.ShowBlizzOptions = value;
	
	-- Update UI
	Addon.Bottom.UpdateOptions();
end


function Addon:GetHideXPBar(info)
	return Addon.db.profile.HideXPBar;
end
function Addon:SetHideXPBar(info, value)
	Addon.DebugPrint("SetHideXPBar: "..tostring(value));
	
	-- Set Value
	Addon.db.profile.HideXPBar = value;
	
	-- Update UI
	Addon.Bottom.UpdateOptions();
end

function Addon:GetShowResourceBar(info)
	return Addon.db.profile.ShowResourceBar;
end
function Addon:SetShowResourceBar(info, value)
	Addon.DebugPrint("SetShowResourceBar: "..tostring(value));
	
	-- Set Value
	Addon.db.profile.ShowResourceBar = value;
	
	-- Update UI
	Addon.Bottom.UpdateOptions();
end

function Addon:GetShowBagBar(info)
	return Addon.db.profile.ShowBagBar;
end
function Addon:SetShowBagBar(info, value)
	Addon.DebugPrint("SetShowBagBar: "..tostring(value));
	
	-- Set Value
	Addon.db.profile.ShowBagBar = value;
	
	-- Update UI
	Addon.Bottom.UpdateOptions();
end

function Addon:GetUseLocalTime(info)
	return Addon.db.profile.useLocalTime;
end
function Addon:SetUseLocalTime(info, value)
	Addon.DebugPrint("SetUseLocalTime: "..tostring(value));
	
	-- Set Value
	Addon.db.profile.useLocalTime = value;
	
	-- Update UI
	Addon.Bottom.UpdateOptions();
end

function Addon:GetLayoutUsed(info)
	return Addon.db.profile.Layout or "Original";
end
function Addon:SetLayoutUsed(info, value)
	Addon.DebugPrint("SetLayoutUsed: "..tostring(value));
	
	-- Set Value
	Addon.db.profile.Layout = value
	
	-- Update UI
	Addon.Layout.SetLayout(value);
end


function Addon:GetStyleUsed(info)
	return Addon.db.profile.Style or "Original";
end
function Addon:SetStyleUsed(info, value)
	Addon.DebugPrint("SetStyleUsed: "..tostring(value));
	
	-- Set Value
	Addon.db.profile.Style = value;
	
	-- Update UI
	Addon.Styler.SetStyle(value);
end

function Addon:GetColorUsed(info)
	local c = Addon.db.profile.Color;
	if (c) then
		return c.r, c.g, c.b, c.a;
	end
	return 0.1, 0.1, 0.1, 0.6;
end
function Addon:SetColorUsed(info, r, g, b, a)
	Addon.DebugPrint("SetColorUsed: "..tostring(r)..", "..tostring(g)..", ", tostring(b)..", "..tostring(a));
	
	-- Set Value
	Addon.db.profile.Color = Addon.db.profile.Color or {};
	Addon.db.profile.Color.r = r;
	Addon.db.profile.Color.g = g;
	Addon.db.profile.Color.b = b;
	Addon.db.profile.Color.a = a;
	
	-- Update UI
	if (Addon.db.profile.ColorPre == "Custom") then
		Addon.Styler.Style_SetColor(Addon.db.profile.Color);
	else
		Addon.Styler.Style_SetColor(Addon.Styler.Colors[Addon.db.profile.ColorPre]);
	end
end

function Addon:GetColorPreUsed(info)
	return Addon.db.profile.ColorPre;
end
function Addon:SetColorPreUsed(info, value)
	Addon.DebugPrint("SetColorUsed: "..tostring(value));
	
	-- Set Value
	Addon.db.profile.ColorPre = value;
	
	-- Update UI
	if (Addon.db.profile.ColorPre == "Custom") then
		Addon.Styler.Style_SetColor(Addon.db.profile.Color);
	else
		Addon.Styler.Style_SetColor(Addon.Styler.Colors[Addon.db.profile.ColorPre]);
	end
end



function Addon:resetBarMainHotkeys()
	Addon.DebugPrint("resetBarMainHotkeys");
	
	-- Main
	Addon.SetupKeybindingDataBar(Addon.GetMainActionBar(), 	0);
end
function Addon:resetBar1Hotkeys()
	Addon.DebugPrint("resetBar1Hotkeys");
	
	-- 2nd Bar
	Addon.SetupKeybindingDataBar(Addon.ActionBar1, 		1);
end
function Addon:resetBar2Hotkeys()
	Addon.DebugPrint("resetBar2Hotkeys");
	
	-- 3rd Bar
	Addon.SetupKeybindingDataBar(Addon.ActionBar2, 		2);
end

function Addon:emptyFunction()
end

function Addon.ClearKeybindingDataBar(bar)
	if (bar == nil) then
		return;
	end
	local button;
	local key = "";
	for buttonId = 0, 11, 1 do
		
		button 	= bar:GetButton(buttonId);
		if (button ~= nil) then
			button:SetKeybinding(key);
		else
			Addon.DebugPrint("id["..buttonId.."] == nil");
		end
	end
end
function Addon:clearBarMainHotkeys()
	Addon.ClearKeybindingDataBar(Addon.ActionBarMain)
end
function Addon.ClearBarsHotkeys()
	Addon.BarTable = Addon.BarTable or {};
	if (#Addon.BarTable > 0) then
		-- Loop through Functions in table
		for i = 1, #Addon.BarTable, 1 do
			--------------------------------------------
			-- Save
			--------------------------------------------
			if (Addon.BarTable[i][1] ~= nil) then
				Addon.ClearKeybindingDataBar(Addon.BarTable[i][1])
			end
		end
	end
end


function Addon:ToggleConfig()
	Addon.DebugPrint("ToggleConfig()")
	if (Addon.ConfigFrame:IsShown() and InterfaceOptionsFrame:IsShown()) then
		InterfaceOptionsFrame:Hide();
	else
		InterfaceOptionsFrame_OpenToCategory(AddonName);
	end
end


function Addon:GetEnable()
	return Addon:IsEnabled();
end
function Addon:SetEnable(info, value)
	if (Addon:IsEnabled()) then
		Addon.DebugPrint("ToggleEnable(); Addon is now Disabled!");
		Addon:Disable();
	else
		Addon.DebugPrint("ToggleEnable(); Addon is now Enabled!");
		Addon:Enable();
	end
	
	-- Update UI
	--Addon.Update_BottomController(true);
end
----------------------------------------------------------------------------------------------------

function Addon.GetOptions()
	return options;
end
function Addon.UpdateOptions()
	-- Config, Setup Main
	LibStub("AceConfig-3.0"):RegisterOptionsTable(AddonName.."-"..AddonVersion, options, {"foxsmoothui", "fsui"});
end

Addon.ConfigOptionsFuncs = Addon.ConfigOptionsFuncs or {};
function Addon.AddConfigOptions()
	-------------------------------------------------------------
	-- Added Code for a Mimimalistic Appoach to Adding Modules
	-------------------------------------------------------------
	-- Safety Check
	Addon.ConfigOptionsFuncs = Addon.ConfigOptionsFuncs or {};
	
	actionBarOptionsOffset = 0;
	actionBarOptionsCount = 0;
	options.args.mainOptions.args.barHotkeyDefaults.args = {};
	
	-- Safety Check
	if (#Addon.ConfigOptionsFuncs > 0) then
		-- Loop through Functions in table
		for i = 1, #Addon.ConfigOptionsFuncs, 1 do
			-- Call Function i
			local status = pcall(Addon.ConfigOptionsFuncs[i]);
		end
	end
end

Addon.Config = Addon.Config or {};
local actionBarOptionsOffset = actionBarOptionsOffset or 0;
function Addon.Config.GetActionBarOptionsOffset()
	return actionBarOptionsOffset;
end
function Addon.Config.SetActionBarOptionsOffset(i)
	actionBarOptionsOffset = i;
end

local actionBarOptionsCount = actionBarOptionsCount or 0;
function Addon.Config.GetActionBarOptionsCount()
	return actionBarOptionsCount;
end
function Addon.Config.SetActionBarOptionsCount(i)
	actionBarOptionsCount = i;
end

Addon.EnableFunctions = Addon.EnableFunctions or {};
table.insert(Addon.EnableFunctions, Addon.AddConfigOptions);