-- DataManager.lua
-- Handles Saving, Loading, & Defaults

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--
Addon.BarTable = Addon.BarTable or {};

function Addon.LoadButtons()
	-- Safety Check
	Addon.BarTable = Addon.BarTable or {};
	
	-- profile[barName][buttonId] == { kind, value, hotkey };
	--Addon.LoadBar(Addon.ActionBarMain, true);
	--Addon.LoadBar(Addon.ActionBar1);
	--Addon.LoadBar(Addon.ActionBar2);
	--Addon.LoadBar(Addon.ActionBar3);
	--Addon.LoadBar(Addon.ActionBar4);
	
	--Addon.LoadBar(Addon.PetBar, true);
	--Addon.LoadBar(Addon.StanceBar, true);
	
	-- Save Bars
	if (#Addon.BarTable > 0) then
		-- Loop through Functions in table
		for i = 1, #Addon.BarTable, 1 do
			--------------------------------------------
			-- Save
			--------------------------------------------
			if (Addon.BarTable[i][1] ~= nil) then
				Addon.LoadBar(Addon.BarTable[i][1], Addon.BarTable[i][2]);
			end
		end
	end
end

function Addon.LoadBar(bar, hotkeyOnly)
	local barName = bar:GetName();
	local kind;
	local value;
	local hotkey;
	--local foxkind;
	--local foxvalue;
	
	local spec1_kind;
	local spec1_value;
	local spec2_kind;
	local spec2_value;
	for buttonId = 0, (bar.buttonCount - 1), 1 do
		local button = bar:GetButton(buttonId);
		if (button ~= nil) then
			if (hotkeyOnly) then
				hotkey 	= Addon.db.char.buttonData.bars[barName][buttonId].hotkey;
				button:SetKeybinding(hotkey);
			else
				kind 	= Addon.db.char.buttonData.bars[barName][buttonId].kind;
				value 	= Addon.db.char.buttonData.bars[barName][buttonId].value;
				hotkey 	= Addon.db.char.buttonData.bars[barName][buttonId].hotkey;
				
				--foxkind 	= Addon.db.char.buttonData.bars[barName][buttonId].foxkind;
				--foxvalue 	= Addon.db.char.buttonData.bars[barName][buttonId].foxvalue;
				
				spec1_kind 	= Addon.db.char.buttonData.bars[barName][buttonId].spec1_kind;
				spec1_value = Addon.db.char.buttonData.bars[barName][buttonId].spec1_value;
				spec2_kind 	= Addon.db.char.buttonData.bars[barName][buttonId].spec2_kind;
				spec2_value = Addon.db.char.buttonData.bars[barName][buttonId].spec2_value;
				
				local customType 			= Addon.db.char.buttonData.bars[barName][buttonId].customType;
				local customUsingTexture 	= Addon.db.char.buttonData.bars[barName][buttonId].customUsingTexture;
				local customTexture 		= Addon.db.char.buttonData.bars[barName][buttonId].customTexture;
				local macroText 			= Addon.db.char.buttonData.bars[barName][buttonId].macroText;
				--if (kind ~= "battlepet") then
					--button:Load(kind, value, hotkey, foxkind, foxvalue);
					button:Load(kind, value, hotkey, customType, customUsingTexture, customTexture, macroText);
					
				
				--end
				
			end
			button.updateAllHotkeysFunc = Addon.UpdateAllHotkeysFunc;
		end
	end
end

function Addon.SaveButtons()
	-- Safety Check
	Addon.BarTable = Addon.BarTable or {};
	
	-- profile[barName][buttonId] == { kind, value, hotkey };
	--Addon.SaveBar(Addon.ActionBarMain, true);
	--Addon.SaveBar(Addon.ActionBar1);
	--Addon.SaveBar(Addon.ActionBar2);
	--Addon.SaveBar(Addon.ActionBar3);
	--Addon.SaveBar(Addon.ActionBar4);
	
	--Addon.SaveBar(Addon.PetBar);
	--Addon.SaveBar(Addon.StanceBar);
	
	-- Save Bars
	if (#Addon.BarTable > 0) then
		-- Loop through Functions in table
		for i = 1, #Addon.BarTable, 1 do
			--------------------------------------------
			-- Save
			--------------------------------------------
			if (Addon.BarTable[i][1] ~= nil) then
				Addon.SaveBar(Addon.BarTable[i][1], Addon.BarTable[i][2]);
			end
		end
	end
end

function Addon.SaveBar(bar, hotkeyOnly)
	local barName = bar:GetName();
	local kind;
	local value;
	local hotkey;
	--local foxkind;
	--local foxvalue;
	
	local spec1_kind;
	local spec1_value;
	local spec2_kind;
	local spec2_value;
	
	local customType, customUsingTexture, customTexture
	if (bar ~= nil) then
		for buttonId = 0, (bar.buttonCount - 1), 1 do
			local button = bar:GetButton(buttonId);
			if (button ~= nil) then
				-- Get Data
				--kind, value, hotkey, foxkind, foxvalue = button:Save();
				kind, value, hotkey, customType, customUsingTexture, customTexture, macroText = button:Save();
				
				if (hotkeyOnly) then
					-- Save Data
					Addon.db.char.buttonData.bars[barName][buttonId].hotkey 	= hotkey;
				else
					Addon.db.char.buttonData.bars[barName][buttonId].kind 		= kind;
					Addon.db.char.buttonData.bars[barName][buttonId].value 		= value;
					Addon.db.char.buttonData.bars[barName][buttonId].hotkey 	= hotkey;
					--Addon.db.char.buttonData.bars[barName][buttonId].foxkind 	= foxkind;
					--Addon.db.char.buttonData.bars[barName][buttonId].foxvalue 	= foxvalue;
					
					Addon.db.char.buttonData.bars[barName][buttonId].customType 			= customType;
					Addon.db.char.buttonData.bars[barName][buttonId].customUsingTexture 	= customUsingTexture;
					Addon.db.char.buttonData.bars[barName][buttonId].customTexture 			= customTexture;
					Addon.db.char.buttonData.bars[barName][buttonId].macroText 				= macroText;
					
					
					Addon.db.char.buttonData.bars[barName][buttonId].spec1_kind 	= spec1_kind;
					Addon.db.char.buttonData.bars[barName][buttonId].spec1_value 	= spec1_value;
					Addon.db.char.buttonData.bars[barName][buttonId].spec2_kind 	= spec2_kind;
					Addon.db.char.buttonData.bars[barName][buttonId].spec2_value 	= spec2_value;
				end
			end
		end
	end
end

function Addon.UpdateAllHotkeysFunc(frameName, hotkey)
	-- Safety Check
	Addon.BarTable = Addon.BarTable or {};
	
	--Addon.UpdateBarHotkeys(Addon.ActionBarMain, frameName, hotkey);
	--Addon.UpdateBarHotkeys(Addon.ActionBar1, 	frameName, hotkey);
	--Addon.UpdateBarHotkeys(Addon.ActionBar2, 	frameName, hotkey);
	--Addon.UpdateBarHotkeys(Addon.ActionBar3, 	frameName, hotkey);
	--Addon.UpdateBarHotkeys(Addon.ActionBar4, 	frameName, hotkey);
	
	-- Save Bars
	if (#Addon.BarTable > 0) then
		-- Loop through Functions in table
		for i = 1, #Addon.BarTable, 1 do
			--------------------------------------------
			-- Save
			--------------------------------------------
			if (Addon.BarTable[i][1] ~= nil) then
				Addon.UpdateBarHotkeys(Addon.BarTable[i][1], frameName, hotkey);
			end
		end
	end
end

function Addon.UpdateBarHotkeys(bar, frameName, hotkey)
	if (bar == nil) then
		return;
	end
	
	for buttonId = 0, (bar.buttonCount - 1), 1 do
		local button = bar:GetButton(buttonId);
		if (button ~= nil) then
			if (button:GetName() ~= frameName) then
				if (button:GetAttribute("hotkey") == hotkey) then
					button:SetKeybinding(nil);
				end
			end
		end
	end
end
