-- FirstTimePrompt.lua
-- Main functions of the Addon

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

-- Initial Setup
local KeybindingKeys 	= {[0] = '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '='};
			
----------------------------------------------------------------------------------------------------
-- First Time Setup
----------------------------------------------------------------------------------------------------
function Addon.FirstLoad.Prompt()
	if (true) then
		return;
	end
	-- Setup Prompt Functions
	local function OnAccept1()
		-- Hide Current Prompt
		StaticPopup_Hide(Addon:GetName().."InitialSetup1");
		
		-- Prompt for Action Data Setup
		StaticPopup_Show(Addon:GetName().."InitialSetup2");
	end
	local function OnCancel1()
		-- Set Setup Flag
		Addon.db.char.LoadedBefore = true;
	end
	local function OnAlt1()
		-- Don't Set Setup Flag
		Addon.db.char.LoadedBefore = false;
	end
	
	local function OnAccept2()
		-- Hide Current Prompt
		StaticPopup_Hide(Addon:GetName().."InitialSetup2");
		
		-- Setup Keybinding Data
		Addon.FirstLoad.SetupKeybindingData();
		
		-- Prompt for Keybinding Data Setup
		StaticPopup_Show(Addon:GetName().."InitialSetup3");
	end
	local function OnCancel2()
		StaticPopup_Hide(Addon:GetName().."InitialSetup2");
		
		-- Prompt for Keybinding Data Setup
		StaticPopup_Show(Addon:GetName().."InitialSetup3");
	end
	
	local function OnAccept3()
		-- Hide Current Prompt
		StaticPopup_Hide(Addon:GetName().."InitialSetup3");
		
		-- Pull Action Data
		Addon.FirstLoad.PullButtonData();
		-- Save Button Data
		Addon.SaveButtons()
		
		-- Set Setup Flag
		Addon.db.char.LoadedBefore = true;
	end
	local function OnCancel3()
		StaticPopup_Hide(Addon:GetName().."InitialSetup3");

		-- Set Setup Flag
		Addon.db.char.LoadedBefore = true;
	end
	
	-- Setup Prompts
	StaticPopupDialogs[Addon:GetName().."InitialSetup1"] = {
		text = "This is the first time you've used "..Addon:GetName().." would you like to run the Initial Setup?",
		button1 = "Yes",
		button2 = "No",
		button3 = "Later",
		OnAccept = OnAccept1,
		OnCancel = OnCancel1,
		OnAlt = OnAlt1,
		timeout = 0,
		whileDead = true,
		preferredIndex = 3,
	}
	StaticPopupDialogs[Addon:GetName().."InitialSetup2"] = {
		text = "Would you like to use the default keybindings for the Main 3 Actionbars?",
		button1 = "Yes",
		button2 = "No",
		OnAccept = OnAccept2,
		OnCancel = OnCancel2,
		timeout = 0,
		whileDead = true,
		preferredIndex = 3,
	}
	StaticPopupDialogs[Addon:GetName().."InitialSetup3"] = {
		text = "Would you like to pull all of your Actionbutton Actions?",
		button1 = "Yes",
		button2 = "No",
		OnAccept = OnAccept3,
		OnCancel = OnCancel3,
		timeout = 0,
		whileDead = true,
		preferredIndex = 3,
	}
	
	-- Show Initial Prompt
	StaticPopup_Show(Addon:GetName().."InitialSetup1");
end

--[[ Actionbar Info
-- OldBarIds	Buttons		NewBarId	Bar
--	0			 1-12		0			Actionbar 1
--	1			13-24					Actionbar 2
--	2			25-36		3			MultibarRight
--	3			37-48		4			MultibarLeft
--	4			49-60		2			MultibarBottomRight
--	5			61-72		1			MultibarBottomLeft
--							6			Stance Bar
]]--
function Addon.FirstLoad.PullButtonData()
	-- Don't Pull Data from first bar as it uses the default action button slot data.
	--Addon.FirstLoad.GetActionsFromBar(Addon.ActionBarMain, 	0);
	Addon.FirstLoad.GetActionsFromBar(Addon.ActionBar1, 	5);
	Addon.FirstLoad.GetActionsFromBar(Addon.ActionBar2, 	4);
	Addon.FirstLoad.GetActionsFromBar(Addon.ActionBar3, 	2);
	Addon.FirstLoad.GetActionsFromBar(Addon.ActionBar4, 	3);
	
end

function Addon.FirstLoad.GetActionsFromBar(bar, oldBarId)
	local button;
	for i=0, 11, 1 do
		local kind, value, subType = GetActionInfo(i + (oldBarId * 12) + 1);
		if (kind == "item") then
			if (value ~= nil) then
				value 	= ("item:%d"):format(value);
			else
				kind 	= "";
				value 	= "";
			end
		elseif (kind == "companion") then
			--kind = subType;
			--if ((subType == "MOUNT") or (subType == "MOUNT")) then
				--value = GetCompanionIndexBySpellId(value, subType);
			--end
		end
		button = bar:GetButton(i);
		if (button ~= nil) then
			if (value ~= "nil") then
				button:SetAction(kind, value);
			end
		end
	end
end

function Addon.FirstLoad.SetupKeybindingData()
	-- Setup Actionbar Hotkeys
	--Addon.SetupKeybindingDataBar(Addon.ActionBarMain, 	0);
	--Addon.SetupKeybindingDataBar(Addon.ActionBar1, 		1);
	--Addon.SetupKeybindingDataBar(Addon.ActionBar2, 		2);
end
function Addon.SetupKeybindingDataBar(bar, id)
	if (bar == nil) then
		return;
	end
	local button;
	local key;
	for buttonId = 0, 11, 1 do
		button 	= bar:GetButton(buttonId);
		if (button ~= nil) then
			key 	= Addon.FirstLoad.GetKey(id, buttonId);
			button:SetKeybinding(key);
		end
	end
end

function Addon.FirstLoad.GetKey(id, buttonId)
	local key = KeybindingKeys[buttonId] or "";
	if (key == "") then
		return key;
	elseif (id == 1) then
		key = "SHIFT-"..key;
	elseif (id == 2) then
		key = "CTRL-"..key;
	elseif (id == -1) then
		key = "";
	end
	return key;
end
----------------------------------------------------------------------------------------------------

local function GetMainActionBar()
	return nil;
end
Addon.GetMainActionBar = Addon.GetMainActionBar or GetMainActionBar;