-- Bottom\InfoBar.lua
-- Creates an InfoBar that can be placed on the Top or Bottom of the Screen.

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator	= LibStub:GetLibrary("FoxFrameCreator");
----------------------------------------------------------------------------------------------------


Addon.Frames 			= Addon.Frames or {};
Addon.Frames.InfoBar 	= Addon.Frames.InfoBar or {};

local undefinedModule 	= "undef";
local infoBarName 		= "InfoBar";
local OnStyle;
local OnStyle_child;

function Addon.Frames.InfoBar.New(self, anchor, moduleName, id)
	
	local name = "FSUI_"..(moduleName or undefinedModule).."_"..infoBarName..(id or "");
	--local self = self or CreateFrame("Frame", name, UIParent);
	local self;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name);
	else
		self = libFrameCreator.Setup(_G[name]);
	end
	
	self:ClearAllPoints();
	self:SetPoint("Left", anchor, "Left");
	self:SetPoint("Right", anchor, "Right");
	
	self:SetHeight(24);
	
	self.OnStyle = OnStyle;
	
	self.framesLeft = self.framesLeft or {};
	self.framesRight = self.framesRight or {};
	
	self.AddFrameLeft = Addon.Frames.InfoBar.AddFrameLeft;
	self.AddFrameRight = Addon.Frames.InfoBar.AddFrameRight;
	
	
	self.RemoveFrameLeft 	= Addon.Frames.InfoBar.RemoveFrameLeft;
	self.RemoveFrameRight 	= Addon.Frames.InfoBar.RemoveFrameRight;
	self.RemoveFrame 		= Addon.Frames.InfoBar.RemoveFrame
	
	return self;
end

function OnStyle(self)
	if (#self.framesLeft > 0) then
		-- for
		for i = 1, #self.framesLeft, 1 do
			local frame = self.framesLeft[i];
			-- Remove Old Border
			frame:SetBackdrop(nil);
			frame.OnStyle = OnStyle_child;
		end
	end
	
	if (#self.framesRight > 0) then
		-- for
		for i = 1, #self.framesRight, 1 do
			local frame = self.framesRight[i];
			-- Remove Old Border
			frame:SetBackdrop(nil);
			frame.OnStyle = OnStyle_child;
		end
	end
	
end
function OnStyle_child(self)
	self:SetBackdrop(nil);
end

local GetInfoBar;
function GetInfoBar(self)
	return self.infoBarFrame;
end

local tableContains;
local tableGetKey;
-- Add Frame
function Addon.Frames.InfoBar.AddFrameLeft(self, frame)
	if (tableContains(self.framesLeft, frame)) then
		return;
	end
	if (frame.inInfoBar) then
		return;
	end
	
	local anchor = self;
	local anchorPoint = "Left";
	
	if (#self.framesLeft > 0) then
		anchor = self.framesLeft[#self.framesLeft];
		anchorPoint = "Right";
	end
	frame:ClearAllPoints();
	frame:SetPoint("Left", anchor, anchorPoint, 0, 0);
	
	frame:SetParent(self);
	
	-- Show Frame
	frame:Show();
	
	-- Add Reference to InfoBar
	frame.infoBarFrame = self;
	frame.GetInfoBar = GetInfoBar;
	
	-- Remove Old Border
	frame:SetBackdrop(nil);
	
	-- Add Flag
	frame.inInfoBar = true;
	
	-- Add frame to table
	table.insert(self.framesLeft, frame);
end

function Addon.Frames.InfoBar.AddFrameRight(self, frame)
	if (tableContains(self.framesRight, frame)) then
		return;
	end
	if (frame.inInfoBar) then
		return;
	end
	
	local anchor = self;
	local anchorPoint = "Right";
	
	if (#self.framesRight > 0) then
		anchor = self.framesRight[#self.framesRight];
		anchorPoint = "Left";
	end
	frame:ClearAllPoints();
	frame:SetPoint("Right", anchor, anchorPoint, -6, 0);
	
	frame:SetParent(self);
	
	-- Show Frame
	frame:Show();
	
	-- Add Reference to InfoBar
	frame.infoBarFrame = self;
	frame.GetInfoBar = GetInfoBar;
	
	-- Remove Old Border
	frame:SetBackdrop(nil);
	
	-- Add Flag
	frame.inInfoBar = true;
	
	-- Add frame to table
	table.insert(self.framesRight, frame);
end

function tableContains(table, element)
  for _, value in pairs(table) do
    if (value == element) then
      return true
    end
  end
  return false
end

function tableGetKey(table, element)
  for key, value in pairs(table) do
    if (value == element) then
      return key
    end
  end
  return false
end

function Addon.Frames.InfoBar.RemoveFrame(self, frame)
	if (tableContains(self.framesLeft, frame)) then
		Addon.Frames.InfoBar.RemoveFrameLeft(self, frame);
	elseif (tableContains(self.framesRight, frame)) then
		Addon.Frames.InfoBar.RemoveFrameRight(self, frame)
	end
end

function Addon.Frames.InfoBar.RemoveFrameLeft(self, frame)
	if (not tableContains(self.framesLeft, frame)) then
		return;
	end
	local key = tableGetKey(self.framesLeft, frame);
	table.remove(self.framesLeft, key);
	
	-- Remove Flag
	frame.inInfoBar = nil;
	
	-- Remove Position Info
	frame:ClearAllPoints();
	
	-- Hide Frame
	frame:Hide();
	
	-- Remove Reference to InfoBar
	self.infoBarFrame = nil;
	
	-- Remove OnStyle Event
	frame.OnStyle = nil;
	
end

function Addon.Frames.InfoBar.RemoveFrameRight(self, frame)
	if (not tableContains(self.framesRight, frame)) then
		return;
	end
	local key = tableGetKey(self.framesRight, frame);
	table.remove(self.framesRight, key);
	
	-- Remove Flag
	frame.inInfoBar = nil;
	
	-- Remove Position Info
	frame:ClearAllPoints();
	
	-- Hide Frame
	frame:Hide();
	
	-- Remove Reference to InfoBar
	self.infoBarFrame = nil;
	
	-- Remove OnStyle Event
	frame.OnStyle = nil;
end


