-- StatBars.lua
-- Handles the StatBar Creation

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFoxStatusBar = LibStub:GetLibrary("FoxStatusbarLib-1.0");
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
Addon.Frames 	= Addon.Frames or {};
local module 	= Addon.Frames;

-- Safety Check
module.HealthBar 	= module.HealthBar or {};
module.ResourceBar 	= module.ResourceBar or {};
module.XPBar 		= module.XPBar or {};
module.RepBar 		= module.RepBar or {};

local statBarVertWidth 	= 48;
local statBarVertHeight	= 160;
local statBarHorzWidth	= 160;
local statBarHorzHeight	= 24;

-- Stat Bar Names
local prefix 					= "FSUI_"
local healthBarName 			= "_HealthBar";
local manaBarName				= "_ResourceBar";
local expBarName				= "_ExpBar";
local repBarName				= "_RepBar";

-- Function Definitions
local GetName;
----------------------------------------------------------------------------------------------------

function module.HealthBar.Create(self, parent, moduleName)
	local name = GetName("health", moduleName);
	self = libFoxStatusBar.Create(self, "health", nil, nil, nil, statBarVertWidth, statBarVertHeight, true, nil, name, "Interface\\Addons\\"..addonName.."\\Libraries");
	self:SetParent(parent);
	
	return self;
end
function module.ResourceBar.Create(self, parent, moduleName)
	local name = GetName("mana", moduleName);
	self = libFoxStatusBar.Create(self, "mana", nil, nil, nil, statBarVertWidth, statBarVertHeight, true, nil, name, "Interface\\Addons\\"..addonName.."\\Libraries");
	self:SetParent(parent);
	
	return self;
end
function module.XPBar.Create(self, parent, moduleName)
	local name = GetName("exp", moduleName);
	self = libFoxStatusBar.Create(self, "exp", nil, nil, nil, statBarHorzWidth, statBarHorzHeight, false, nil, name, "Interface\\Addons\\"..addonName.."\\Libraries");
	self:SetParent(parent);
	--Addon.SetupXPBarTooltip(self);
	
	return self;
end
function module.RepBar.Create(self, parent, moduleName)
	local name = GetName("rep", moduleName);
	self = libFoxStatusBar.Create(self, "rep", nil, nil, nil, statBarHorzWidth, statBarHorzHeight, false, nil, name,	"Interface\\Addons\\"..addonName.."\\Libraries");
	self:SetParent(parent);
	
	return self;
end

function module.HealthBar.Setup(self, actionBarFrame)
	-- Set Health Bar's Position Data
	self:ClearAllPoints();
	self:SetPoint("Top", 		actionBarFrame, 	"Top", 		0, 0);
	self:SetPoint("Bottom", 	UIParent, 			"Bottom", 	0, 0);
	self:SetPoint("Right", 		actionBarFrame, 	"Left", 	0, 0);
	
	-- Show
	self:Show();
end
function module.ResourceBar.Setup(self, actionBarFrame)
	-- Set Mana Bar's Position Data
	self:ClearAllPoints();
	self:SetPoint("Top", 	actionBarFrame, 	"Top", 		0, 0);
	self:SetPoint("Bottom", UIParent, 			"Bottom", 	0, 0);
	self:SetPoint("Left", 	actionBarFrame, 	"Right", 	0, 0);
	
	-- Show
	self:Show();
end
function module.XPBar.Setup(self, actionBarFrame)
	-- Set Experience Bar's Position Data
	self:ClearAllPoints();
	self:SetWidth(actionBarFrame:GetWidth());
	
	-- Hide
	self:Hide();
	
	-- Add Reference for Show/Hide Function
	local ActionBarFrameRef = "ActionBarFrameRef";
	
	self:SetFrameRef(ActionBarFrameRef, 	actionBarFrame);
end
function module.RepBar.Setup(self, actionBarFrame)
	-- Set Reputation Bar's Position Data
	self:ClearAllPoints();
	self:SetWidth(actionBarFrame:GetWidth());
	
	-- Hide
	self:Hide();
	
	-- Add Reference for Show/Hide Function
	local ActionBarFrameRef = "ActionBarFrameRef";
	
	self:SetFrameRef(ActionBarFrameRef, 	actionBarFrame);
end

function GetName(modType, moduleName)
	if (modType == "health") then
		return prefix..moduleName..healthBarName;
	elseif (modType == "mana") then
		return prefix..moduleName..manaBarName;
	elseif (modType == "exp") then
		return prefix..moduleName..expBarName;
	elseif (modType == "rep") then
		return prefix..moduleName..repBarName;
	end
end
