-- StatBars.lua
-- Handles the StatBar Creation

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFoxStatusBar = LibStub:GetLibrary("FoxStatusbarLib-1.0");
local libFrameCreator	= LibStub:GetLibrary("FoxFrameCreator");
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
Addon.Frames 	= Addon.Frames or {};
local module 	= Addon.Frames;

-- Safety Check
local statBarVertWidth 	= 48;
local statBarVertHeight	= 160;
local statBarHorzWidth	= 160;
local statBarHorzHeight	= 24;

-- Stat Bar Names
local prefix 					= "FSUI_"
local healthBarName 			= "_HealthBar";

-- Function Definitions
local GetName;
----------------------------------------------------------------------------------------------------
local module   = Addon.Frames or {};

module.UnitInfo = module.UnitInfo or {};

local moduleName = "UnitInfo"
local unitInfoName = "UnitInfoFrame";


local UpdateRune;
local CreateRunes;
local CreateRune;
local OnEvent;

-- Rune Control
local HideRunes;
local ShowRunes;

function module.UnitInfo.New()
	local name = unitInfoName;

	local self;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name);
	else
		self = libFrameCreator.Setup(_G[name]);
	end

	self:SetPoint("Center");
	libFrameCreator.MakeFrameDraggable(self);

	self:SetWidth(164);
	self:SetHeight(24);
	local text = "Resources";
	self.title =  libFrameCreator.CreateLabel(self.title, self, text, "TopLeft", self, "TopLeft", 0, 0, 80, 20, "Center", "Middle")
	self.title:SetWidth(164);

	self.resourceBar = self.resourceBar or Addon.Frames.ResourceBar.Create(self.resourceBar, self, moduleName);
	local bar = self.resourceBar;

	bar:ClearAllPoints();
	bar:SetPoint("Top", self, "Bottom", 0, -30);
	bar:SetPoint("Left", self, "Left", 0, 0);
	bar:SetPoint("Right", self, "Right", 0, 0);
	bar:SetHeight(26);
	bar:MakeHorizontal();
	bar:Show();

	self.hpBar = self.hpBar or Addon.Frames.HealthBar.Create(self.hpBar, self, moduleName)
	local bar = self.hpBar;

	bar:ClearAllPoints();
	bar:SetPoint("Top", self.resourceBar, "Bottom", 0, 0);
	bar:SetPoint("Left", self, "Left", 0, 0);
	bar:SetPoint("Right", self, "Right", 0, 0);
	bar:SetHeight(26);
	bar:MakeHorizontal();
	bar:Show();

	self.runes = CreateRunes(self);

	self:SetScript("OnEvent", OnEvent);
	self:RegisterEvent("RUNE_POWER_UPDATE");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("RUNE_TYPE_UPDATE");
	
	
	self.HideRunes = HideRunes;
	self.ShowRunes = ShowRunes;
	
	
	return self;
end

function CreateRunes(self)
	
	self.runes = self.runes or {};
	runes = self.runes;
	runes[1] = CreateRune(runes[1], self, 1, "BottomLeft", self.resourceBar, 		"TopLeft", 8, 6)
	runes[2] = CreateRune(runes[2], self, 2, "Left",   runes[1], 	"Right", 8, 0);
	runes[3] = CreateRune(runes[3], self, 5, "Left",   runes[2], 	"Right", 8, 0);
	runes[4] = CreateRune(runes[4], self, 6, "Left",   runes[3], 	"Right", 8, 0);
	runes[5] = CreateRune(runes[5], self, 3, "Left",   runes[4], 	"Right", 8, 0);
	runes[6] = CreateRune(runes[6], self, 4, "Left",   runes[5], 	"Right", 8, 0);
	
	RuneButton_Update(runes[1], 1);
	RuneButton_Update(runes[2], 2);
	RuneButton_Update(runes[5], 3);
	RuneButton_Update(runes[6], 4);
	RuneButton_Update(runes[3], 5);
	RuneButton_Update(runes[4], 6);

	--UpdateRune(1);
	--UpdateRune(2);
	--UpdateRune(3);
	--UpdateRune(4);
	--UpdateRune(5);
	--UpdateRune(6);

	--"UI-PlayerFrame-Deathknight-Ring"
	
	return runes;
end

function CreateRune(self, parent, runeIndex, point, anchor, anchorPoint, x, y)
	local self = self or CreateFrame("Button", parent:GetName().."Rune"..runeIndex, parent, "RuneButtonIndividualTemplate");

	local size = 18;
	local size_Cooldown = 15;

	self:SetWidth(size);
	self:SetHeight(size);
	self.cooldown = _G[self:GetName().."Cooldown"];

	self.cooldown:SetHeight(size_Cooldown);

	self:ClearAllPoints();
	self:SetPoint(point, anchor, anchorPoint, x, y);

	return self;
end

function UpdateRune(runeIndex)
	local runeButton = _G["UnitInfoFrameRune"..runeIndex];
	local cooldown = runeButton.cooldown

	local start, duration, runeReady = GetRuneCooldown(runeIndex);

	if not runeReady  then
		if start then
			CooldownFrame_SetTimer(cooldown, start, duration, 1);
		end
		runeButton.energize:Stop();
	else
		cooldown:Hide();
		runeButton.shine:SetVertexColor(1, 1, 1);
		RuneButton_ShineFadeIn(runeButton.shine)
	end
	--runeButton:Show();
end

local MAX_RUNES = 6;
function OnEvent(self, event, ...)
	if (event == "PLAYER_ENTERING_WORLD") then
		for i = 1, MAX_RUNES do
            local runeButton = _G["UnitInfoFrameRune"..i];
            if runeButton then
                RuneButton_Update(runeButton, i, true);
            end
        end
		
	elseif ( event == "RUNE_POWER_UPDATE" ) then
		local runeIndex, isEnergize = ...;
		if runeIndex and runeIndex >= 1 and runeIndex <= MAX_RUNES  then
            local runeButton = _G["UnitInfoFrameRune"..runeIndex];
            local cooldown = runeButton.cooldown;
             
            local start, duration, runeReady = GetRuneCooldown(runeIndex);
             
            if not runeReady  then
                if start then
                    CooldownFrame_SetTimer(cooldown, start, duration, 1);
                end
                runeButton.energize:Stop();
            else
                cooldown:Hide();
                runeButton.shine:SetVertexColor(1, 1, 1);
                RuneButton_ShineFadeIn(runeButton.shine)
            end
             
            if isEnergize  then
                runeButton.energize:Play();
            end
        else
            assert(false, "Bad rune index")
        end
		
	elseif ( event == "RUNE_TYPE_UPDATE" ) then
        local runeIndex = ...;
        if ( runeIndex and runeIndex >= 1 and runeIndex <= MAX_RUNES ) then
            RuneButton_Update(_G["UnitInfoFrameRune"..runeIndex], runeIndex);
        end
		
	end
end

function HideRunes(self)
	self.runes[1]:Hide()
	self.runes[2]:Hide()
	self.runes[3]:Hide()
	self.runes[4]:Hide()
	self.runes[5]:Hide()
	self.runes[6]:Hide()
	
	self.resourceBar:SetPoint("Top", self, "Bottom", 0, 0);
end

function ShowRunes(self)
	self.runes[1]:Show()
	self.runes[2]:Show()
	self.runes[3]:Show()
	self.runes[4]:Show()
	self.runes[5]:Show()
	self.runes[6]:Show()
	
	self.resourceBar:SetPoint("Top", self, "Bottom", 0, -30);
end

--
--module.UnitInfoFrame = module.New();

