-- Title:		FoxStatusbarTemplate
-- Author: 		Fox536
-- Version: 	1.0
-- Description:	
-- Features:			
----------------------------------------------------------------------------------------------------
-- Notes
----------------------------------------------------------------------------------------------------
--
----------------------------------------------------------------------------------------------------
--  Local Tables
----------------------------------------------------------------------------------------------------
local lib 		= LibStub:NewLibrary("FoxStatusbarLib-1.0", 2);

----------------------------------------------------------------------------------------------------
--  Library Setup
----------------------------------------------------------------------------------------------------
if not lib then
	return;
end
lib = lib or {};

----------------------------------------------------------------------------------------------------
-- Local Variables
----------------------------------------------------------------------------------------------------
local Statusbar = lib;

Statusbar.libPath = "";

----------------------------------------------------------------------------------------------------
-- Main Setup Functions
----------------------------------------------------------------------------------------------------
local function GetBackgroundPath()
	if (Statusbar.libPath ~= nil) then
		return Statusbar.libPath.."\\FoxStatusbarLib\\Graphics\\BarBack1.tga";
	end
	return "Interface\\FoxGraphics\\FoxStatusbarLib\\BarBack1.tga";
end
local function GetFillPath()
	if (Statusbar.libPath ~= nil) then
		return Statusbar.libPath.."\\FoxStatusbarLib\\Graphics\\BarFill1.tga";
	end
	return "Interface\\FoxGraphics\\FoxStatusbarLib\\BarFill1.tga";
end
-- GetBackdrop, Backdrop
	-- Creates a Backdrop for a Statusbar.
local function GetBackdrop()
	local backdrop 			= {};
	backdrop.bgFile			= "";
	backdrop.edgeFile		= "Interface\\Tooltips\\UI-Tooltip-Border";
	backdrop.tile			= false;
	backdrop.tileSize		= 16;
	backdrop.edgeSize		= 12;
	backdrop.insets			= {};
	backdrop.insets.left	= 3;
	backdrop.insets.right	= 3;
	backdrop.insets.top		= 3;
	backdrop.insets.bottom	= 3;
	return backdrop;
end

local function SetupFrame(self, width, height)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	width 	= width or 160;
	height 	= height or 24;
	------------------------------------------------------------------------------------------------
	
	------------------------------------------------------------------------------------------------
	-- Setup
	------------------------------------------------------------------------------------------------
	-- Set Backdrop
	self:SetBackdrop(GetBackdrop());
	
	-- Set Size
	self:SetSize(width, height);
	------------------------------------------------------------------------------------------------
end

local function SetupBackground(self)
	if (self.background == nil) then
		self.background = self:CreateTexture(nil, "BACKGROUND");
	end
	
	self.background:SetPoint("TOPLEFT", 	 3, -3);
	self.background:SetPoint("BOTTOMRIGHT", -3,  3);
	
	self.background:SetTexture(GetBackgroundPath());
end

local function SetupText(self, anchorPoint, anchorX, anchorY, justifyH, justifyV)
	-- Set Points
	self:ClearAllPoints();
	self:SetPoint(anchorPoint, anchorX, anchorY);
	-- Set Font
	self:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE");
	-- Set Justification
	self:SetJustifyH(justifyH);
	self:SetJustifyV(justifyV);
	-- Set Shadow
	self:SetShadowOffset(1, -1);
	-- Set Color
	self:SetTextColor(1.00, 1.00, 1.00);
end
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Events
----------------------------------------------------------------------------------------------------
local function OnEvent(self, event, arg1, ...)
	if ((self.barType == 0) or (self.barType == 1)) then
		self:Update();
		if (self.Updated ~= nil) then
			self.Updated(self);
		end
	elseif ((self.barType == 2) or (self.barType == 3)) then
		if ((self.unitId ~= nil) and (self.unitId ~= "")) then
			if (self.unitId == arg1) then
				self:Update();
				if (self.Updated ~= nil) then
					self.Updated(self);
				end
			end
		end
	end
end
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Update Value Functions
----------------------------------------------------------------------------------------------------
local function HideRested(self)
	if (InCombatLockdown() == nil) then
		self.restedBar:Hide();
	end
end
local function ShowRested(self)
	if (InCombatLockdown() == nil) then
		self.restedBar:Show();
	end
end
local function UpdateRestedValue(self, maxValue, value)
	-- Default Color (Yellow)
	local r = 0.90;
	local g = 0.70;
	local b = 0.00;
	
	local restedXPBonus	= GetXPExhaustion() or 0;
	local minValue 		= 0;
	local expMax 		= maxValue or UnitXPMax("player");
	local expCurrent 	= value or UnitXP("player");
	local expNeeded 	= 0;
	if (expMax ~= 0) then
		expNeeded 		= expMax - expCurrent;
	end
	-- Check if player has more then a level of rested exp
	if (restedXPBonus > expNeeded) then
		-- Change Color (Green)
		r = 0.00;
		g = 0.60;
		b = 0.10;
	end
	restedXPBonus = restedXPBonus + expCurrent;
	-- Insure Bar doesn't over take it's parent frame
	if (restedXPBonus > expMax) then
		restedXPBonus = expMax;
	end
	
	-- Update Min & Max
	self.restedBar:SetMinMaxValues(0, expMax);
	-- Update Value
	self.restedBar:SetValue(restedXPBonus);
	-- Update Color
	self.restedBar:SetStatusBarColor(r, g, b, 0.80);
	
	if (restedXPBonus > 0) then
		-- Show Rested Bar
		ShowRested(self);
	
	else
		-- Hide Rested Bar
		HideRested(self);

	end
end
local function UpdateExpBar(self)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	minValue 	= 0;
	maxValue 	= UnitXPMax("player");
	value 		= UnitXP("player");
	------------------------------------------------------------------------------------------------
	
	-- Update RestedBar
	UpdateRestedValue(self, maxValue, value);
	
	-- Update Values
	self:SetMinMaxValues(minValue, maxValue);
	self:SetValue(value);
end
local function UpdateRepBar(self)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	local minValue	=   0;
	local maxValue 	= 100;
	local value 	=   0;
	local name, standingID, barMin, barMax, barValue = GetWatchedFactionInfo();
	------------------------------------------------------------------------------------------------
	
	if ((name ~= nil) and (name ~= "")) then
		-- Normalize Values
		barMax 		= barMax - barMin;
		barValue 	= barValue - barMin;
		barMin 		= 0;
		
		-- Set Values
		minValue 	= barMin;
		maxValue 	= barMax;
		value 		= barValue;
		
		-- Show
		if (InCombatLockdown() == nil) then
			self:Show();
		end
		
	else
		-- Hide
		if (InCombatLockdown() == nil) then
			self:Hide();
		end
	end
	
	-- Hide RestedBar
	HideRested(self);
	
	-- Update Values
	self:SetMinMaxValues(minValue, maxValue);
	self:SetValue(value);
end
local function UpdateHealthBar(self)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	local minValue	=   0;
	local maxValue 	= 100;
	local value 	=   0;
	------------------------------------------------------------------------------------------------
	
	-- Set Values
	maxValue 	= UnitHealthMax("player");
	value 		= UnitHealth("player");
	
	-- Hide RestedBar
	HideRested(self);
	
	-- Update Values
	self:SetMinMaxValues(minValue, maxValue);
	self:SetValue(value);
end
local function UpdateManaBar(self)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	local minValue	=   0;
	local maxValue 	= 100;
	local value 	=   0;
	------------------------------------------------------------------------------------------------
	
	-- Set Values
	maxValue 	= UnitPowerMax("player");
	value 		= UnitPower("player");
	
	-- Hide RestedBar
	HideRested(self);
	
	-- Update Values
	self:SetMinMaxValues(minValue, maxValue);
	self:SetValue(value);
end
local function UpdateValue(self)
	-- Exp Bar
	if (self.barType == 0) then
		UpdateExpBar(self);
	-- Rep Bar
	elseif (self.barType == 1) then
		UpdateRepBar(self);
	-- Health Bar
	elseif (self.barType == 2) then
		UpdateHealthBar(self);
	-- Mana Bar
	elseif (self.barType == 3) then
		UpdateManaBar(self);
	end
end
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Update Color Functions
----------------------------------------------------------------------------------------------------
local function UpdateColorExpBar(self)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	local r = 0.58;
	local g = 0.00;
	local b = 0.55;
	local exhaustionStateID = 0;
	------------------------------------------------------------------------------------------------
	
	-- Rested Check
	exhaustionStateID = GetRestState();
	if (exhaustionStateID == 1) then
		-- Use Rested Color
		r = 0.00;
		g = 0.39;
		b = 0.88;
	end
	
	-- Set Color
	self.bar:SetStatusBarColor(r, g, b);
end
local function UpdateColorRepBar(self)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	local r = FACTION_BAR_COLORS[4].r;
	local g = FACTION_BAR_COLORS[4].g;
	local b = FACTION_BAR_COLORS[4].b;
	local name, standingID, barMin, barMax, barValue = GetWatchedFactionInfo();
	------------------------------------------------------------------------------------------------
	
	-- Standing Check
	if (name ~= nil) then
		-- Use Standing Colors
		r = FACTION_BAR_COLORS[standingID].r or FACTION_BAR_COLORS[4].r;
		g = FACTION_BAR_COLORS[standingID].g or FACTION_BAR_COLORS[4].g;
		b = FACTION_BAR_COLORS[standingID].b or FACTION_BAR_COLORS[4].b;
	end
	
	-- Set Color
	self.bar:SetStatusBarColor(r, g, b);
end
local function UpdateColorHealthBar(self)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	local r = 0.00;
	local g = 1.00;
	local b = 0.00;
	------------------------------------------------------------------------------------------------
	
	local minValue, maxValue 	= self:GetMinMaxValues();
	local value 				= self:GetValue();
	
	if ( (value < minValue) or (value > maxValue) ) then
		return {r, g, b};
	end
	if ( (maxValue - minValue) > 0 ) then
		value = (value - minValue) / (maxValue - minValue);
	else
		value = 0;
	end
	if(value > 0.5) then
		r = (1.0 - value) * 2;
		g = 1.0;
	else
		r = 1.0;
		g = value * 2;
	end
	b = 0.0;
	
	-- Set Color
	self.bar:SetStatusBarColor(r, g, b);
end
local function UpdateColorManaBar(self)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	local r = PowerBarColor["MANA"].r;
	local g = PowerBarColor["MANA"].g;
	local b = PowerBarColor["MANA"].b;
	local powerType, powerToken = UnitPowerType("player");
	local colors;
	------------------------------------------------------------------------------------------------
	
	-- Mana Color Check
	if (PowerBarColor[powerToken]) then
		-- Power Token Found, use it
		colors = PowerBarColor[powerToken];
	elseif (PowerBarColor[powerType]) then
		-- Power Type Found, use it
		colors = PowerBarColor[powerType];
	else
		-- Use Default Mana Color
		colors = PowerBarColor["MANA"];
	end
	
	-- Set Color
	self.bar:SetStatusBarColor(colors.r, colors.g, colors.b);
end
local function UpdateColor(self)
	-- if Bar is using custom Color
	if ((self.color.r ~= nil) and (self.color.g ~= nil) and (self.color.b ~= nil)) then
		-- Set Color
		self.bar:SetStatusBarColor(self.color.r, self.color.g, self.color.b);
		
	-- if Bar is Exp Bar
	elseif (self.barType == 0) then
		UpdateColorExpBar(self);
		
	-- if Bar is Rep Bar
	elseif (self.barType == 1) then
		UpdateColorRepBar(self);
		
	-- if Bar is Health Bar
	elseif (self.barType == 2) then
		UpdateColorHealthBar(self);
		
	-- if Bar is Mana Bar
	elseif (self.barType == 3) then
		UpdateColorManaBar(self)
		
	end
end
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Update Text Functions
----------------------------------------------------------------------------------------------------
local function ClearText(self)
	self.bar.TextFirst:SetText("");
	self.bar.TextMiddle:SetText("");
	self.bar.TextLast:SetText("");
end
local function UpdateText(self)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	local minValue, maxValue 	= self:GetMinMaxValues();
	local value 				= self:GetValue();
	local percent 				= 0;
	local needed 				= 0;
	------------------------------------------------------------------------------------------------
	
	-- Set Values
	if (maxValue ~= 0) then
		percent = (value / maxValue) * 100;
		if (self.barType < 2) then
			needed 	= maxValue - value;
		end
	end

	-- Clear Old Text
	ClearText(self);

	-- Set Text
	if (self.barType < 2) then
		self:SetFirstText(("Needed: %d"):format(needed));
		self:SetMiddleText(value.." / "..maxValue);
		self:SetLastText(("%.2f%%"):format(percent));
	else
		self:SetFirstText(("%.0f%%"):format(percent));
		self:SetMiddleText(value);
		self:SetLastText(maxValue);
	end
end
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Api Functions, Get/Set
----------------------------------------------------------------------------------------------------
function Statusbar.GetOrientation(self)
	return self.orientation;
end
function Statusbar.SetOrientation(self, orientation)
	if (orientation == "VERTICAL") then
		self:MakeVertical();
	elseif (orientation == "HORIZONTAL") then
		self:MakeHorizontal();
	else
		-- Default
		self:MakeHorizontal();
	end
end

function Statusbar.GetBackgroundTexture(self)
	return self.background;
end
function Statusbar.SetBackgroundTexture(self, texture)
	if (self ~= nil) then
		if (self:GetBackgroundTexture() ~= nil) then
			self:GetBackgroundTexture():SetTexture(texture);
		end
	end
end

function Statusbar.RotateBackground(self, radians)
	if (self ~= nil) then
		if (self:GetBackgroundTexture() ~= nil) then
			self:GetBackgroundTexture():SetRotation(radians);
		end
	end
end

function Statusbar.GetBarTexture(self)
	if (self.bar ~= nil) then
		return self.bar:GetStatusBarTexture();
	end
	return "";
end
function Statusbar.SetBarTexture(self, texture)
	if (self.bar ~= nil) then
		self.bar:SetStatusBarTexture(texture);
		self.bar:GetStatusBarTexture():SetHorizTile(false);
		self.bar:GetStatusBarTexture():SetVertTile(false);
	end
	if (self.restedBar ~= nil) then
		self.restedBar:SetStatusBarTexture(texture);
		self.restedBar:GetStatusBarTexture():SetHorizTile(false);
		self.restedBar:GetStatusBarTexture():SetVertTile(false);
	end
end

function Statusbar.GetValue(self)
	if (self.bar ~= nil) then
		return self.bar:GetValue();
	end
	return 0;
end
function Statusbar.SetValue(self, value)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	value 		= value 	or  50;
	------------------------------------------------------------------------------------------------
	
	if (self.bar ~= nil) then
		self.bar:SetValue(value);
	end
end

function Statusbar.GetMinValue(self)
	if (self.bar ~= nil) then
		local minValue, maxValue = self.bar:GetMinMaxValues();
		return minValue;
	end
	return 0;
end
function Statusbar.SetMinValue(self, minValue)
	local maxValue = self:GetMaxValue();
	self:SetMinMaxValues(minValue, maxValue);
end

function Statusbar.GetMaxValue(self)
	if (self.bar ~= nil) then
		local minValue, maxValue = self.bar:GetMinMaxValues();
		return maxValue;
	end
	return 0;
end
function Statusbar.SetMaxValue(self, maxValue)
	local minValue = self:GetMinValue();
	self:SetMinMaxValues(minValue, maxValue);
end

function Statusbar.GetMinMaxValues(self)
	if (self.bar ~= nil) then
		return self.bar:GetMinMaxValues();
	end
	return 0, 0;
end
function Statusbar.SetMinMaxValues(self, minValue, maxValue)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	minValue 	= minValue  or   0;
	maxValue 	= maxValue  or 100;
	------------------------------------------------------------------------------------------------
	
	if (self.bar ~= nil) then
		self.bar:SetMinMaxValues(minValue, maxValue);
	end
end

function Statusbar.GetBarColor(self)
	if (self.color == nil) then
		self.color = { r = nil, g = nil, b = nil };
	end
	return self.color.r, self.color.g, self.color.b;
end
function Statusbar.SetBarColor(self, r, g, b, clear)
	self.color = { r, g, b };
	if (clear) then
		self.color = { r = nil, g = nil, b = nil };
	end
	UpdateColor(self);
end
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Api Functions, Methods
----------------------------------------------------------------------------------------------------
-- MakeVertical, void
	-- Changes the bar to display Vertically
function Statusbar.MakeVertical(self)
	-- Set Points
	self.bar:ClearAllPoints();
	self.bar:SetPoint("TOPLEFT", 3, -3);
	self.bar:SetPoint("BOTTOMRIGHT", -3, 3);
	
	self.restedBar:ClearAllPoints();
	self.restedBar:SetPoint("TOPLEFT", 3, -3);
	self.restedBar:SetPoint("BOTTOMRIGHT", -3, 3);
	
	-- Setup Bar to be Vertical
	self.bar:SetRotatesTexture(true);
	self.bar:SetOrientation("VERTICAL");
	self.restedBar:SetRotatesTexture(true);
	self.restedBar:SetOrientation("VERTICAL");
	
	-- Set Text Layout
	SetupText(self.bar.TextFirst,  "TOP",     1, -12, "CENTER", "TOP");
	SetupText(self.bar.TextMiddle, "CENTER",  1,   0, "CENTER", "MIDDLE");
	SetupText(self.bar.TextLast,   "BOTTOM",  1,   4, "CENTER", "BOTTOM");
	
	-- Turn Background Texture
	self:RotateBackground(rad(90));
	
	-- Set Statusbar's Variables
	self.orientation = "Vertical";
end

-- MakeHorizontal, void
	-- Changes the bar to display Horizontally
function Statusbar.MakeHorizontal(self)
	-- Set Points
	self.bar:ClearAllPoints();
	self.bar:SetPoint("TOPLEFT", 3, -3);
	self.bar:SetPoint("BOTTOMRIGHT", -3, 3);
	
	self.restedBar:ClearAllPoints();
	self.restedBar:SetPoint("TOPLEFT", 3, -3);
	self.restedBar:SetPoint("BOTTOMRIGHT", -3, 3);
	
	-- Setup Bar to be Horizontal
	self.bar:SetRotatesTexture(false);
	self.bar:SetOrientation("HORIZONTAL");
	self.restedBar:SetRotatesTexture(false);
	self.restedBar:SetOrientation("HORIZONTAL");
	
	SetupText(self.bar.TextFirst,  "LEFT",     4, 0, "LEFT",	"MIDDLE");
	SetupText(self.bar.TextMiddle, "CENTER",   0, 0, "CENTER",	"MIDDLE");
	SetupText(self.bar.TextLast,   "RIGHT",  -12, 0, "RIGHT", 	"MIDDLE");
	
	-- Normalize Background Texture
	self:RotateBackground(rad(0));
	
	-- Set Orientation
	self.orientation = "Horizontal";
end

-- Update, void
	-- Updates the Statusbar
function Statusbar.Update(self)
	if (self == nil) then
		return;
	end
	-- Update the Statusbar's Value
	UpdateValue(self);
	-- Update the Statusbar's Color
	UpdateColor(self);
	-- Update the Statusbar's Text
	UpdateText(self);
end

-- SetFirstText, void
	-- Set's the Left Text of the Statusbar
function Statusbar.SetFirstText(self, text)
	if (text == nil) then
		text = "";
	end
	self.bar.TextFirst:SetText(text);
end

-- SetMiddleText, void
	-- Set's the Center Text of the Statusbar
function Statusbar.SetMiddleText(self, text)
	if (text == nil) then
		text = "";
	end
	self.bar.TextMiddle:SetText(text);
end

-- SetLastText, void
	-- Set's the Right Text of the Statusbar
function Statusbar.SetLastText(self, text)
	if (text == nil) then
		text = "";
	end
	self.bar.TextLast:SetText(text);
end


-- Graphics Path
function Statusbar.SetLibPath(libPath)
	if (libPath ~= nil) then
		Statusbar.libPath = libPath;
	end
end
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Setup
----------------------------------------------------------------------------------------------------
-- SetupFunctions, void
	-- Set's the Statusbar's Methods
local function SetupFunctions(self)
	self.GetOrientation 		= Statusbar.GetOrientation;
	self.SetOrientation 		= Statusbar.SetOrientation;
	
	self.GetBackgroundTexture 	= Statusbar.GetBackgroundTexture;
	self.SetBackgroundTexture 	= Statusbar.SetBackgroundTexture;
	
	self.GetBarTexture 			= Statusbar.GetBarTexture;
	self.SetBarTexture 			= Statusbar.SetBarTexture;

	self.GetValue 				= Statusbar.GetValue;
	self.SetValue 				= Statusbar.SetValue;

	self.GetMinValue 			= Statusbar.GetMinValue;
	self.SetMinValue 			= Statusbar.SetMinValue;

	self.GetMaxValue 			= Statusbar.GetMaxValue;
	self.SetMaxValue 			= Statusbar.SetMaxValue;

	self.GetMinMaxValues 		= Statusbar.GetMinMaxValues;
	self.SetMinMaxValues 		= Statusbar.SetMinMaxValues;

	self.GetBarColor 			= Statusbar.GetBarColor;
	self.SetBarColor 			= Statusbar.SetBarColor;

	self.Update 				= Statusbar.Update;
	self.MakeVertical 			= Statusbar.MakeVertical;
	self.MakeHorizontal 		= Statusbar.MakeHorizontal;
	self.SetFirstText 			= Statusbar.SetFirstText;
	self.SetMiddleText 			= Statusbar.SetMiddleText;
	self.SetLastText 			= Statusbar.SetLastText;
	self.RotateBackground 		= Statusbar.RotateBackground;
end

local function SetupEventsExp(self)
	self:SetScript("OnEvent", OnEvent);
	self:RegisterEvent("PLAYER_XP_UPDATE");
	self:RegisterEvent("GUILD_XP_UPDATE");
	self:RegisterEvent("UPDATE_FACTION");
end
local function SetupEventsHealth(self)
	self:SetScript("OnEvent", OnEvent);
	self:RegisterEvent("UNIT_HEALTH");
end
local function SetupEventsMana(self)
	self:SetScript("OnEvent", OnEvent);
	self:RegisterEvent("UNIT_POWER");
	self:RegisterEvent("UNIT_DISPLAYPOWER");
end
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Create
----------------------------------------------------------------------------------------------------
-- Create, Frame
	-- Creates a Statusbar of given type and returns it's Frame.
		-- self: 		A Frame for the Statusbar, if nil is used it will be created.
		-- barType:		The Statusbar's Type 
							--	0:	Exp Bar
							--	1:	Rep Bar
							--	2:	Health Bar
							--	3:	Mana Bar
							--	20:	Other Bar (Default Green Color);
		-- minValue:	The Minimum Value for the Custom Bar type
		-- maxValue:	The Maximum Value for the Custom Bar type
		-- value:		The Initial Value for the Custom Bar type
		-- width:		The Width of the Bar
		-- height:		The Height of the Bar
		-- isVertical:	Detirmines whether the fills Left to Right(Horizontally, false), or Bottom to Top(Vertically, true)
		-- unitId:		The Unit to watch for Stat Changes for Heatlh/Mana Bars, if nil defaults to player
function Statusbar.Create(self, barType, minValue, maxValue, value, width, height, isVertical, unitId, name)
	------------------------------------------------------------------------------------------------
	-- Setup Frame
	------------------------------------------------------------------------------------------------
	if (self == nil) then
		self = CreateFrame("Frame", name, UIParent, "SecureHandlerStateTemplate, SecureHandlerShowHideTemplate");
	end
	-- Setup Frame
	SetupFrame(self, width, height);
	-- Setup Frame's new Functions
	SetupFunctions(self);
	------------------------------------------------------------------------------------------------
	
	
	------------------------------------------------------------------------------------------------
	-- Setup Background
	------------------------------------------------------------------------------------------------
	SetupBackground(self);
	------------------------------------------------------------------------------------------------
	
	
	------------------------------------------------------------------------------------------------
	-- Setup Statusbars
	------------------------------------------------------------------------------------------------
	-- Create Statusbar's Bar
	if (self.bar == nil) then
		self.bar = CreateFrame("StatusBar", self:GetName().."_Bar", self);
	end
	
	-- Create Statusbar's RestedBar
	if (self.restedBar == nil) then
		self.restedBar = CreateFrame("StatusBar", self:GetName().."_RestedBar", self);
	end
	
	-- Set Statusbars Z-Level
	self.bar:SetFrameLevel(3);
	self.restedBar:SetFrameLevel(2);
	
	-- Set Bar Texture
	self:SetBarTexture(GetFillPath());
	
	self.bar:SetRotatesTexture(true);
	self.restedBar:SetRotatesTexture(true);
	
	-- Set Min, Max, & Value
	self:SetMinMaxValues(minValue, maxValue);
	self:SetValue(value);
	------------------------------------------------------------------------------------------------
	
	
	------------------------------------------------------------------------------------------------
	-- Setup Text
	------------------------------------------------------------------------------------------------
	-- Create Statusbar's Left Text
	if (self.bar.TextFirst == nil) then
		self.bar.TextFirst = self.bar:CreateFontString(nil, "OVERLAY");
	end

	-- Create Statusbar's Center Text
	if (self.bar.TextMiddle == nil) then
		self.bar.TextMiddle = self.bar:CreateFontString(nil, "OVERLAY");
	end
	SetupText(self.bar.TextMiddle, "CENTER", 0, 0, "CENTER", "MIDDLE");

	-- Create Statusbar's Right Text
	if (self.bar.TextLast == nil) then
		self.bar.TextLast = self.bar:CreateFontString(nil, "OVERLAY");
	end
		
	-- Set Orientation
	if (isVertical) then
		self.isVertical = true;
		self:MakeVertical();
	else
		self.isVertical = false;
		self:MakeHorizontal();
	end
	------------------------------------------------------------------------------------------------
	
	
	------------------------------------------------------------------------------------------------
	-- Setup Variables
	------------------------------------------------------------------------------------------------
	-- Setup Statusbar's Type
	self.barType 	= barType or 20;
	if (self.barType > 1) then
		self.unitId = unitId or "player";
	elseif (self.barType == 20) then
		self.unitId = unitId;
	end
	-- Setup Statusbar's Color
	self.color 		= {};
	self.color.r	= nil;
	self.color.g	= nil;
	self.color.b	= nil;
	------------------------------------------------------------------------------------------------
	
	------------------------------------------------------------------------------------------------
	-- Setup Event
	------------------------------------------------------------------------------------------------
	-- Exp Bar
	if (self.barType == 0) then
		SetupEventsExp(self);
	-- Rep Bar
	elseif (self.barType == 1) then
		SetupEventsExp(self);
	-- Health Bar
	elseif (self.barType == 2) then
		self.unitId = unitId or "player";
		SetupEventsHealth(self);
	-- Mana Bar
	elseif (self.barType == 3) then
		self.unitId = unitId or "player";
		SetupEventsMana(self);
	end
	------------------------------------------------------------------------------------------------
	
	-- Hide until ready to Use
	self:Hide();
	
	-- Update
	self:Update();
	
	-- Return Statusbar
	return self;
end
----------------------------------------------------------------------------------------------------