-- Title:		FoxStatusbarTemplate
-- Author: 		Fox536
-- Version: 	2.0a
-- Description:	Allows for Easy Creation of a Statusbar
-- Features:	Pre-Made Health, Mana, Exp, Rep Bar Templates
----------------------------------------------------------------------------------------------------
-- Notes
----------------------------------------------------------------------------------------------------
--
----------------------------------------------------------------------------------------------------
--  Local Tables
----------------------------------------------------------------------------------------------------
local lib 		= LibStub:NewLibrary("FoxStatusbarLib-1.0", 2);

----------------------------------------------------------------------------------------------------
--  Library Setup
----------------------------------------------------------------------------------------------------
if not lib then
	return;
end
lib = lib or {};

----------------------------------------------------------------------------------------------------
-- Local Variables
----------------------------------------------------------------------------------------------------
lib.libPath = "";

-- Function Declarations
----------------------------------------------------------------------------------------------------
-- Setup Frame 
local SetupFrame;
local GetBackdrop;
local SetupMethods;

local SetupBackground;
local GetBackgroundPath;
local GetFillPath;
local GetFillHPath;

-- Event
local SetupEvents;

-- Text
local SetupText;

-- Event Handlers
local OnEvent;
local OnUpdate;

-- Update Functions
local UpdateValue;
local UpdateSecondaryBar;

local UpdateColor
local GetHealthColor;

local ClearText;
local UpdateText;

-- Logic Functions
local CheckValue;
local TableContains;
local numberSub;

-- Api
-- lib.Update(self) 			-- Update the Bar

-- lib.MakeVertical(self)		-- Make the bar Fill and Display Vertically

-- lib.MakeHorizontal(self)		-- Make the bar Fill and Display Horizontally

-- Set Graphics Path (Local, Global)
-- lib.SetLocalLibPath(self, libPath)	
-- lib.SetGlobalLibPath(libPath)		

-- Label Text (First, Middle, Last)
-- lib.SetFirstText(self, text)
-- lib.SetMiddleText(self, text)
-- lib.SetLastText(self, text)
-- lib.SetText(self, label, text)

-- Description Missing
-- lib.RotateBackground(self, radians)

-- Get/Set 
-- lib.GetOrientation(self)
-- lib.SetOrientation(self, orientation)

-- lib.GetBackgroundTexture(self)
-- lib.SetBackgroundTexture(self, texture)

-- lib.GetBarTexture(self)
-- lib.SetBarTexture(self, texture)

-- lib.GetValue(self)
-- lib.SetValue(self, value)

-- lib.GetMinValue(self)
-- lib.SetMinValue(self, minValue)

-- lib.GetMaxValue(self)
-- lib.SetMaxValue(self, maxValue)

-- lib.GetMinMaxValues(self)
-- lib.SetMinMaxValues(self, minValue, maxValue)

-- lib.GetBarColor(self)
-- lib.SetBarColor(self, r, g, b, clear)

-- lib.GetValue(self)
-- lib.SetValue(self, value)

-- lib.GetMinValue(self)
-- lib.SetMinValue(self, minValue)

-- lib.GetMaxValue(self)
-- lib.SetMaxValue(self, maxValue)

-- lib.GetMinMaxValues(self)
-- lib.SetMinMaxValues(self, minValue, maxValue)

-- lib.GetBarColor(self)
-- lib.SetBarColor(self, r, g, b, clear)
----------------------------------------------------------------------------------------------------

-- Variables
----------------------------------------------------------------------------------------------------
-- Check Tables
local barNeedsUnitId 	= {"health", "mana"};
local barHasEvents 		= {"health", "mana", "exp", "rep"};

-- Events
BarEvents 			= {}
BarEvents["health"] = {"UNIT_HEALTH", 		"UNIT_HEALTH_FREQUENT", "UNIT_HEALTH_PREDICTION"};
BarEvents["mana"] 	= {"UNIT_POWER", 		"UNIT_DISPLAYPOWER"};
BarEvents["exp"] 	= {"PLAYER_XP_UPDATE", 	"GUILD_XP_UPDATE", 		"UPDATE_FACTION"};
BarEvents["rep"] 	= {"PLAYER_XP_UPDATE", 	"GUILD_XP_UPDATE", 		"UPDATE_FACTION"};

-- Colors
local colorExp = {};
colorExp.r = 0.58;
colorExp.g = 0.00;
colorExp.b = 0.55;

local colorExp_Rested = {};
colorExp_Rested.r = 0.00;
colorExp_Rested.g = 0.39;
colorExp_Rested.b = 0.88;

local colorsExpRested_Normal = {};
colorsExpRested_Normal.r = 0.90;
colorsExpRested_Normal.g = 0.70;
colorsExpRested_Normal.b = 0.00;

local colorsExpRested_Over = {};
colorsExpRested_Over.r = 0.00;
colorsExpRested_Over.g = 0.60;
colorsExpRested_Over.b = 0.10;
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Create
----------------------------------------------------------------------------------------------------
-- Creates a Statusbar of given type and returns it's Frame.
	-- self: 		A Frame for the Statusbar, if nil is used it will be created.
	-- barType:		The Statusbar's Type 
		--	"exp"			=	Exp Bar
		--	"rep			=	Rep Bar
		--	"health"		=	Health Bar
		--	"mana"			=	Mana Bar
		--	"nil", nil 		=	Other Bar (Default Green Color);
	-- minValue:	The Minimum Value for the Custom Bar type
	-- maxValue:	The Maximum Value for the Custom Bar type
	-- value:		The Initial Value for the Custom Bar type
	-- width:		The Width of the Bar
	-- height:		The Height of the Bar
	-- isVertical:	Detirmines whether the fills Left to Right(Horizontally, false), or Bottom to Top(Vertically, true)
	-- unitId:		The Unit to watch for Stat Changes for Heatlh/Mana Bars, if nil defaults to player
function lib.Create(self, barType, minValue, maxValue, value, width, height, isVertical, unitId, name, libPath)
	-- Params
	------------------------------------------------------------------------------------------------
	-- Size Params
	width 		= CheckValue(width, 	1, 	nil);
	height 		= CheckValue(height, 	1, 	nil);
	
	-- Value Params
	value 		= CheckValue(value, 	minValue, 	maxValue);
	minValue 	= CheckValue(minValue, 	nil, 		maxValue);
	maxValue 	= CheckValue(maxValue, 	minValue, 	nil);
	------------------------------------------------------------------------------------------------
	

	-- Setup Frame
	------------------------------------------------------------------------------------------------
	if (self == nil) then
		self = CreateFrame("Frame", name, UIParent, "SecureHandlerStateTemplate, SecureHandlerShowHideTemplate");
	end
	
	-- Setup Frame
	SetupFrame(self, width, height);
	
	-- Setup Frame's new Functions
	SetupMethods(self);
	
	-- Setup LibPath
	self:SetLocalLibPath(libPath);
	
	-- Setup Background
	SetupBackground(self);
	self.SetupBackground = SetupBackground;
	------------------------------------------------------------------------------------------------
	
	-- Setup Bars
	------------------------------------------------------------------------------------------------
	-- Create Statusbar's Bar
	if (self.bar == nil) then
		local barName = nil;
		if (name ~= nil) then
			barName = name.."_Bar";
		end
		self.bar = CreateFrame("StatusBar", barName, self);
	end
	
	-- Create secondaryBar
	if (self.secondaryBar == nil) then
		local barName = nil;
		if (name ~= nil) then
			barName = name.."_secondaryBar";
		end
		self.secondaryBar = CreateFrame("StatusBar", barName, self);
	end
	
	-- Set Statusbars Z-Level
	self.bar:SetFrameLevel(3);
	self.secondaryBar:SetFrameLevel(2);
	
	-- Set Bar Texture
	self:SetBarTexture(GetFillPath(self));
	
	self.bar:SetRotatesTexture(true);
	self.secondaryBar:SetRotatesTexture(true);
	
	-- Set Min, Max, & Value
	self:SetMinMaxValues(minValue, maxValue);
	self:SetValue(value);
	------------------------------------------------------------------------------------------------
	
	
	-- Setup Text
	------------------------------------------------------------------------------------------------
	-- Create First Label
	if (self.bar.labelFirst == nil) then
		self.bar.labelFirst = self.bar:CreateFontString(nil, "OVERLAY");
	end

	-- Create Statusbar's Middle Label
	if (self.bar.labelMiddle == nil) then
		self.bar.labelMiddle = self.bar:CreateFontString(nil, "OVERLAY");
	end
	SetupText(self.bar.labelMiddle, "CENTER", 0, 0, "CENTER", "MIDDLE");

	-- Create Statusbar's Last Label
	if (self.bar.labelLast == nil) then
		self.bar.labelLast = self.bar:CreateFontString(nil, "OVERLAY");
	end
		
	-- Set Orientation
	if (isVertical) then
		self.isVertical = true;
		self:MakeVertical();
	else
		self.isVertical = false;
		self:MakeHorizontal();
	end
	------------------------------------------------------------------------------------------------
	
	
	-- Setup Variables
	------------------------------------------------------------------------------------------------
	-- Setup Statusbar's Type
	self.barType 	= barType or "nil";
	if (TableContains(barNeedsUnitId, barType)) then
		self.unitId = unitId or "player";
	else
		self.unitId = unitId;
	end
	
	-- Setup Statusbar's Color
	self.color 		= {};
	self.color.r	= nil;
	self.color.g	= nil;
	self.color.b	= nil;
	------------------------------------------------------------------------------------------------
	
	
	-- Setup Event
	------------------------------------------------------------------------------------------------
	if (TableContains(barHasEvents, barType)) then
		SetupEvents(self, barType);
	end
	
	if ((barType == "health") or (barType == "mana")) then
		self:SetScript("OnUpdate", OnUpdate);
	else
		self:SetScript("OnUpdate", nil);
	end
	
	self:SetScript("OnSizeChanged", UpdateValue)
	------------------------------------------------------------------------------------------------
	
	-- Hide until ready to use
	self:Hide();
	
	-- Update
	self:Update();
	
	-- Return Bar
	return self;
end

----------------------------------------------------------------------------------------------------
-- Setup
----------------------------------------------------------------------------------------------------
-- Setup Frame
function SetupFrame(self, width, height)
	-- Local Variables
	width 	= width or 160;
	height 	= height or 24;
	------------------------------------------------------------------------------------------------
	
	-- Set Backdrop
	self:SetBackdrop(GetBackdrop());
	
	-- Set Size
	self:SetSize(width, height);
end

-- Setup Backdrop for Frame
function GetBackdrop()
	local backdrop 			= {};
	backdrop.bgFile			= "";
	backdrop.edgeFile		= "Interface\\Tooltips\\UI-Tooltip-Border";
	backdrop.tile			= false;
	backdrop.tileSize		= 16;
	backdrop.edgeSize		= 12;
	backdrop.insets			= {};
	backdrop.insets.left	= 3;
	backdrop.insets.right	= 3;
	backdrop.insets.top		= 3;
	backdrop.insets.bottom	= 3;
	return backdrop;
end

-- Setup Methods
function SetupMethods(self)
	self.GetOrientation 		= lib.GetOrientation;
	self.SetOrientation 		= lib.SetOrientation;
	
	self.GetBackgroundTexture 	= lib.GetBackgroundTexture;
	self.SetBackgroundTexture 	= lib.SetBackgroundTexture;
	
	self.GetBarTexture 			= lib.GetBarTexture;
	self.SetBarTexture 			= lib.SetBarTexture;

	self.GetValue 				= lib.GetValue;
	self.SetValue 				= lib.SetValue;

	self.GetMinValue 			= lib.GetMinValue;
	self.SetMinValue 			= lib.SetMinValue;

	self.GetMaxValue 			= lib.GetMaxValue;
	self.SetMaxValue 			= lib.SetMaxValue;

	self.GetMinMaxValues 		= lib.GetMinMaxValues;
	self.SetMinMaxValues 		= lib.SetMinMaxValues;

	self.GetBarColor 			= lib.GetBarColor;
	self.SetBarColor 			= lib.SetBarColor;

	self.Update 				= lib.Update;
	self.MakeVertical 			= lib.MakeVertical;
	self.MakeHorizontal 		= lib.MakeHorizontal;
	self.SetFirstText 			= lib.SetFirstText;
	self.SetMiddleText 			= lib.SetMiddleText;
	self.SetLastText 			= lib.SetLastText;
	self.SetText				= lib.SetText;
	self.RotateBackground 		= lib.RotateBackground;
	
	self.SetLocalLibPath		= lib.SetLocalLibPath;
end

-- Setup Background Texture
function SetupBackground(self)
	if (self.background == nil) then
		self.background = self:CreateTexture(nil, "BACKGROUND");
	end
	
	self.background:SetPoint("TOPLEFT", 	 3, -3);
	self.background:SetPoint("BOTTOMRIGHT", -3,  3);
	
	self.background:SetTexture(GetBackgroundPath(self));
end

function GetBackgroundPath(self)
	if (self.libPath ~= nil) then
		return self.libPath.."\\FoxStatusbarLib\\Graphics\\BarBack1.tga";
	elseif (lib.libPath ~= nil) then
		return lib.libPath.."\\FoxStatusbarLib\\Graphics\\BarBack1.tga";
	end
	return "Interface\\FoxGraphics\\FoxStatusbarLib\\BarBack1.tga";
end
function GetFillPath(self)
	if (self.libPath ~= nil) then
		return self.libPath.."\\FoxStatusbarLib\\Graphics\\BarFill1.tga";
	elseif (lib.libPath ~= nil) then
		return lib.libPath.."\\FoxStatusbarLib\\Graphics\\BarFill1.tga";
	end
	return "Interface\\FoxGraphics\\FoxStatusbarLib\\BarFill1.tga";
end

function GetFillHPath(self)
	if (self.libPath ~= nil) then
		return self.libPath.."\\FoxStatusbarLib\\Graphics\\BarFillH.tga";
	elseif (lib.libPath ~= nil) then
		return lib.libPath.."\\FoxStatusbarLib\\Graphics\\BarFillH.tga";
	end
	return "Interface\\FoxGraphics\\FoxStatusbarLib\\BarFillH.tga";
end

lib.GetFillPath = GetFillPath;
lib.GetFillHPath = GetFillHPath;

-- Setup Events
function SetupEvents(self, barType)
	if (self == nil) then
		return;
	end
	
	local events = BarEvents[barType];
	local event;
	for key, value in pairs(events) do
		if (value ~= nil) then
			self:RegisterEvent(value);
		end
    end
	
	-- Set the OnScript Fucntion
	self:SetScript("OnEvent", OnEvent);
end

-- Setup Text
function SetupText(self, anchorPoint, anchorX, anchorY, justifyH, justifyV)
	-- Set Points
	self:ClearAllPoints();
	self:SetPoint(anchorPoint, anchorX, anchorY);
	-- Set Font
	self:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE");
	-- Set Justification
	self:SetJustifyH(justifyH);
	self:SetJustifyV(justifyV);
	-- Set Shadow
	self:SetShadowOffset(1, -1);
	-- Set Color
	self:SetTextColor(1.00, 1.00, 1.00);
end

----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Update Functions
----------------------------------------------------------------------------------------------------
function UpdateValue(self)
	local unitId = "player";
	if (self.unitId ~= nil) then
		unitId = self.unitId;
		if (unitId == "") then
			unitId = "player";
		end
	end
	
	-- Health
	if (self.barType == "health") then
		-- Get Variables
		local minValue	= 0;
		local maxValue 	= UnitHealthMax(unitId) or 100;
		local value 	= UnitHealth(unitId) 	or   0;
		
		-- Update Values
		self:SetMinMaxValues(minValue, maxValue);
		self:SetValue(value);
		
	-- Mana
	elseif (self.barType == "mana") then
		-- Get Variables
		local minValue	= 0;
		local maxValue 	= UnitPowerMax(unitId)	or 100;
		local value 	= UnitPower(unitId) 	or   0;
		
		-- Update Values
		self:SetMinMaxValues(minValue, maxValue);
		self:SetValue(value);
		
	-- Exp
	elseif (self.barType == "exp") then
		-- Get Variables
		local minValue	= 0;
		local maxValue 	= UnitXPMax(unitId)	or 100;
		local value 	= UnitXP(unitId) 	or   0;
		
		-- Update Values
		self:SetMinMaxValues(minValue, maxValue);
		self:SetValue(value);
		
	-- Rep
	elseif (self.barType == "rep") then
		local minValue	=   0;
		local maxValue 	= 100;
		local value 	=   0;
		local name, standingID, barMin, barMax, barValue = GetWatchedFactionInfo();
		------------------------------------------------------------------------------------------------
		
		if ((name ~= nil) and (name ~= "")) then
			-- Normalize Values
			barMax 		= barMax - barMin;
			barValue 	= barValue - barMin;
			barMin 		= 0;
			
			-- Set Values
			minValue 	= barMin;
			maxValue 	= barMax;
			value 		= barValue;
			
			-- Show
			if (not InCombatLockdown()) then
				self:Show();
			end
			
		else
			-- Hide
			if (not InCombatLockdown()) then
				self:Hide();
			end
		end
		
		-- Update Values
		self:SetMinMaxValues(minValue, maxValue);
		self:SetValue(value);
	end
	
	-- Update secondaryBar
	UpdateSecondaryBar(self, self.barType);
end
function UpdateSecondaryBar(self, barType)
	if (barType == "health") then
		local unitId = "player";
		if (self.unitId ~= nil) then
			unitId = self.unitId;
			if (unitId == "") then
				unitId = "player";
			end
		end
	
		local minValue	= 0;
		local maxValue 	= UnitHealthMax(unitId) or 100;
		local value 	= UnitGetIncomingHeals(unitId) or 0;
		
		-- Update Min & Max
		self.secondaryBar:SetMinMaxValues(minValue, maxValue);
		
		-- Update Value
		self.secondaryBar:SetValue(value);
		
		self.secondaryBar:SetStatusBarColor(GetHealthColor(value, minValue, maxValue), 0.8);
		
		if (value > 0) then
			self.secondaryBar:Show();
		else
			self.secondaryBar:Hide();
		end
		
	elseif (barType == "Exp") then
		local c = colorsExpRested_Normal;
		
		local restedXPBonus	= GetXPExhaustion() 	or   0;
		local minValue 		= 0;
		local expMax 		= UnitXPMax("player") 	or 100;
		local expCurrent 	= UnitXP("player")		or   0;
		local expNeeded 	= 0;
		if (expMax ~= 0) then
			expNeeded 		= expMax - expCurrent;
		end
		
		-- Check if player has more then a level of rested exp
		if (restedXPBonus > expNeeded) then
			c = colorsExpRested_Over;
		end
		
		restedXPBonus = restedXPBonus + expCurrent;
		-- Insure Bar doesn't over take it's parent frame
		if (restedXPBonus > expMax) then
			restedXPBonus = expMax;
		end
		
		-- Update Min & Max
		self.secondaryBar:SetMinMaxValues(0, expMax);
		-- Update Value
		self.secondaryBar:SetValue(restedXPBonus);
		
		-- Update Color
		self.secondaryBar:SetStatusBarColor(c.r, c.g, c.b, 0.80);
		
		-- Show/Hide Bar as needed
		if (restedXPBonus > 0) then
			-- Show Rested Bar
			self.secondaryBar:Show();
		
		else
			-- Hide Rested Bar
			self.secondaryBar:Hide();

		end
		
	else
		-- Hide as it's unnecessary
		self.secondaryBar:Hide();
		
	end
end

function UpdateColor(self)
	-- if Bar is using custom Color
	if ((self.color.r ~= nil) and (self.color.g ~= nil) and (self.color.b ~= nil)) then
		-- Set Color
		self.bar:SetStatusBarColor(self.color.r, self.color.g, self.color.b);
		self.secondaryBar:SetStatusBarColor(self.color.r, self.color.g, self.color.b);
		
	-- if Bar is Health Bar
	elseif (self.barType == "health") then
		-- Set Color
		self.bar:SetStatusBarColor(GetHealthColor(self:GetValue(), self:GetMinMaxValues()));
		
	-- if Bar is Mana Bar
	elseif (self.barType == "mana") then
		local unitId = "player";
		if (self.unitId ~= nil) then
			unitId = self.unitId;
			if (unitId == "") then
				unitId = "player";
			end
		end
		local powerType, powerToken = UnitPowerType(unitId);
		local c;
		------------------------------------------------------------------------------------------------
		
		-- Mana Color Check
		if (PowerBarColor[powerToken]) then
			-- Power Token Found, use it
			c = PowerBarColor[powerToken];
			
		elseif (PowerBarColor[powerType]) then
			-- Power Type Found, use it
			c = PowerBarColor[powerType];
			
		else
			-- Use Default Mana Color
			c = PowerBarColor["MANA"];
		end
		
		-- Set Color
		self.bar:SetStatusBarColor(c.r, c.g, c.b);
	
	-- if Bar is Exp Bar
	elseif (self.barType == "exp") then
		-- Color Table
		local c = colorExp;
		
		-- Rested Check
		if (GetRestState() == 1) then
			-- Use Rested Color
			c = colorExp_Rested;
		end
		
		-- Set Color
		self.bar:SetStatusBarColor(c.r, c.g, c.b);
		
	-- if Bar is Rep Bar
	elseif (self.barType == "rep") then
		local r = FACTION_BAR_COLORS[4].r;
		local g = FACTION_BAR_COLORS[4].g;
		local b = FACTION_BAR_COLORS[4].b;
		local name, standingID, barMin, barMax, barValue = GetWatchedFactionInfo();
		------------------------------------------------------------------------------------------------

		-- Standing Check
		if (name ~= nil) then
			-- Use Standing Colors
			r = FACTION_BAR_COLORS[standingID].r or FACTION_BAR_COLORS[4].r;
			g = FACTION_BAR_COLORS[standingID].g or FACTION_BAR_COLORS[4].g;
			b = FACTION_BAR_COLORS[standingID].b or FACTION_BAR_COLORS[4].b;
		end

		-- Set Color
		self.bar:SetStatusBarColor(r, g, b);
		
	end
end
function GetHealthColor(value, minValue, maxValue)
	local r = 0.00;
	local g = 1.00;
	local b = 0.00;
	
	if ( (value < minValue) or (value > maxValue) ) then
		return r, g, b;
	elseif ( (maxValue - minValue) > 0 ) then
		value = (value - minValue) / (maxValue - minValue);
	else
		value = 0;
	end
	
	if(value > 0.5) then
		r = (1.0 - value) * 2;
		g = 1.0;
	else
		r = 1.0;
		g = value * 2;
	end
	return r, g, b;
end

function ClearText(self)
	self.bar.labelFirst:SetText("");
	self.bar.labelMiddle:SetText("");
	self.bar.labelLast:SetText("");
end
function UpdateText(self)
	local minValue, maxValue 	= self:GetMinMaxValues();
	local value 				= self:GetValue();
	local percent 				= 0;
	local needed 				= 0;
	------------------------------------------------------------------------------------------------
	
	-- Set Values
	if (maxValue ~= 0) then
		percent = (value / maxValue) * 100;
		if ((self.barType == "exp") or (self.barType == "rep")) then
			needed 	= maxValue - value;
		end
	end

	-- Clear Old Text
	ClearText(self);

	-- Set Text
	if ((self.barType == "exp") or (self.barType == "rep")) then
		self:SetFirstText(("Needed: %d"):format(needed));
		self:SetMiddleText(value.." / "..maxValue);
		self:SetLastText(("%.2f%%"):format(percent));
	else
		self:SetFirstText(("%.0f%%"):format(percent));
		self:SetMiddleText(numberSub(value, 100000, "k"));
		self:SetLastText(numberSub(maxValue, 100000, "k"));
	end
end
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Events
----------------------------------------------------------------------------------------------------
function OnEvent(self, event, arg1, ...)
	if ((self.barType == "health") or (self.barType == "mana")) then
		if ((self.unitId ~= nil) and (self.unitId ~= "")) then
			if (self.unitId == arg1) then
				self:Update();
				if (self.Updated ~= nil) then
					self.Updated(self);
				end
			end
		end
	elseif ((self.barType == "exp") or (self.barType == "rep")) then
		self:Update();
		if (self.Updated ~= nil) then
			self.Updated(self);
		end
	end
end
function OnUpdate(self, elapsed)
	self.elapsed = self.elapsed or 0;
	self.elapsed = self.elapsed + elapsed;
	if (self.elapsed >= 0.5) then
		-- Reset Elapsed
		self.elapsed = 0.0;
		
		-- Run Update
		UpdateValue(self);
	end
end
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Logic Functions
----------------------------------------------------------------------------------------------------
-- Check Table for Value
function TableContains(tbl, item)
    for key, value in pairs(tbl) do
        if value == item then return key end
    end
    return false
end

-- Check Value for Errors
function CheckValue(value, minValue, maxValue)
	if (type(value) == "number") then
		if ((type(minValue) == "number") and (type(maxValue) == "number")) then
			if (minValue >= maxValue) then
				error("Min Value can't be the Same as Max Value");
			end
		end
		
		if (minValue ~= nil) then
			if (value < minValue) then
				value = minValue;
			end
		end
		
		if (maxValue ~= nil) then
			if (value > maxValue) then
				value = maxValue;
			end
		end
		
		return value;
	end
	return minValue;
end

-- Check Value for Thousands, Millions, ect.
function numberSub(value, maxValue, subString)
	if (value >= maxValue) then
		maxValue = maxValue / 1000;
		
		value = value /  maxValue;
		value = math.floor(value);
		value = value / 10;
		value = tostring(value)..subString;
		return value;
	end
	return value;
end
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Api
----------------------------------------------------------------------------------------------------
-- Update the Bar
function lib.Update(self)
	if (self == nil) then
		return;
	end
	-- Update the Statusbar's Value
	UpdateValue(self);
	-- Update the Statusbar's Color
	UpdateColor(self);
	-- Update the Statusbar's Text
	UpdateText(self);
end

-- Make the bar Fill and Display Vertically
function lib.MakeVertical(self)
	-- Set Points
	self.bar:ClearAllPoints();
	self.bar:SetPoint("TOPLEFT", 3, -3);
	self.bar:SetPoint("BOTTOMRIGHT", -3, 3);
	
	self.secondaryBar:ClearAllPoints();
	self.secondaryBar:SetPoint("TOPLEFT", 3, -3);
	self.secondaryBar:SetPoint("BOTTOMRIGHT", -3, 3);
	
	-- Setup Bar to be Vertical
	self.bar:SetRotatesTexture(true);
	self.bar:SetOrientation("VERTICAL");
	self.secondaryBar:SetRotatesTexture(true);
	self.secondaryBar:SetOrientation("VERTICAL");
	
	-- Set Text Layout
	SetupText(self.bar.labelFirst,  "TOP",     1, -12, "CENTER", "TOP");
	SetupText(self.bar.labelMiddle, "CENTER",  1,   0, "CENTER", "MIDDLE");
	SetupText(self.bar.labelLast,   "BOTTOM",  1,   4, "CENTER", "BOTTOM");
	
	self:SetBarTexture(GetFillPath(self));
	-- Turn Background Texture
	self:RotateBackground(rad(90));
	
	-- Set Statusbar's Variables
	self.orientation = "Vertical";
end

-- Make the bar Fill and Display Horizontally
function lib.MakeHorizontal(self)
	-- Set Points
	self.bar:ClearAllPoints();
	self.bar:SetPoint("TOPLEFT", 3, -3);
	self.bar:SetPoint("BOTTOMRIGHT", -3, 3);
	
	self.secondaryBar:ClearAllPoints();
	self.secondaryBar:SetPoint("TOPLEFT", 3, -3);
	self.secondaryBar:SetPoint("BOTTOMRIGHT", -3, 3);
	
	-- Setup Bar to be Horizontal
	self.bar:SetRotatesTexture(false);
	self.bar:SetOrientation("HORIZONTAL");
	self.secondaryBar:SetRotatesTexture(false);
	self.secondaryBar:SetOrientation("HORIZONTAL");
	
	SetupText(self.bar.labelFirst,  "BottomLeft",     4, 2, "LEFT",	"MIDDLE");
	SetupText(self.bar.labelMiddle, "Bottom",   0, 2, "CENTER",	"MIDDLE");
	SetupText(self.bar.labelLast,   "BottomRight",  -12, 2, "RIGHT", 	"MIDDLE");
	
	self:SetBarTexture(GetFillHPath(self));
	-- Normalize Background Texture
	self:RotateBackground(rad(0));
	
	-- Set Orientation
	self.orientation = "Horizontal";
end

-- Set Graphics Path (Local, Global)
function lib.SetLocalLibPath(self, libPath)
	if (self ~= nil) then
		self.libPath = libPath;
	end
end
function lib.SetLibPath(libPath)
	lib.libPath = libPath;
end

-- Label Text (First, Middle, Last)
function lib.SetFirstText(self, text)
	if (text == nil) then
		text = "";
	end
	self.bar.labelFirst:SetText(text);
end
function lib.SetMiddleText(self, text)
	if (text == nil) then
		text = "";
	end
	self.bar.labelMiddle:SetText(text);
end
function lib.SetLastText(self, text)
	if (text == nil) then
		text = "";
	end
	self.bar.labelLast:SetText(text);
end
function lib.SetText(self, label, text)
	if (self ~= nil) then
		if (text == nil) then
			text = "";
		end
		if ((label == 1) or (label == "first")) then
			self.bar.labelFirst:SetText(text);
		elseif ((label == 2) or (label == "middle")) then
			self.bar.labelMiddle:SetText(text);
		elseif ((label == 3) or (label == "last")) then
			self.bar.labelLast:SetText(text);
		end
	end
end

function lib.GetOrientation(self)
	return self.orientation;
end
function lib.SetOrientation(self, orientation)
	if (orientation == "VERTICAL") then
		self:MakeVertical();
	elseif (orientation == "HORIZONTAL") then
		self:MakeHorizontal();
	else
		-- Default
		self:MakeHorizontal();
	end
end

function lib.GetBackgroundTexture(self)
	return self.background;
end
function lib.SetBackgroundTexture(self, texture)
	if (self ~= nil) then
		if (self:GetBackgroundTexture() ~= nil) then
			self:GetBackgroundTexture():SetTexture(texture);
		end
	end
end

function lib.RotateBackground(self, radians)
	if (self ~= nil) then
		if (self:GetBackgroundTexture() ~= nil) then
			self:GetBackgroundTexture():SetRotation(radians);
		end
	end
end

function lib.GetBarTexture(self)
	if (self.bar ~= nil) then
		return self.bar:GetStatusBarTexture();
	end
	return "";
end
function lib.SetBarTexture(self, texture)
	if (self.bar ~= nil) then
		self.bar:SetStatusBarTexture(texture);
		self.bar:GetStatusBarTexture():SetHorizTile(false);
		self.bar:GetStatusBarTexture():SetVertTile(false);
	end
	if (self.secondaryBar ~= nil) then
		self.secondaryBar:SetStatusBarTexture(texture);
		self.secondaryBar:GetStatusBarTexture():SetHorizTile(false);
		self.secondaryBar:GetStatusBarTexture():SetVertTile(false);
	end
end

function lib.GetValue(self)
	if (self.bar ~= nil) then
		return self.bar:GetValue();
	end
	return 0;
end
function lib.SetValue(self, value)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	value 		= value 	or  50;
	------------------------------------------------------------------------------------------------
	
	if (self.bar ~= nil) then
		self.bar:SetValue(value);
	end
end

function lib.GetMinValue(self)
	if (self.bar ~= nil) then
		local minValue, maxValue = self.bar:GetMinMaxValues();
		return minValue;
	end
	return 0;
end
function lib.SetMinValue(self, minValue)
	local maxValue = self:GetMaxValue();
	self:SetMinMaxValues(minValue, maxValue);
end

function lib.GetMaxValue(self)
	if (self.bar ~= nil) then
		local minValue, maxValue = self.bar:GetMinMaxValues();
		return maxValue;
	end
	return 0;
end
function lib.SetMaxValue(self, maxValue)
	local minValue = self:GetMinValue();
	self:SetMinMaxValues(minValue, maxValue);
end

function lib.GetMinMaxValues(self)
	if (self.bar ~= nil) then
		return self.bar:GetMinMaxValues();
	end
	return 0, 0;
end
function lib.SetMinMaxValues(self, minValue, maxValue)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	minValue 	= minValue  or   0;
	maxValue 	= maxValue  or 100;
	------------------------------------------------------------------------------------------------
	
	if (self.bar ~= nil) then
		self.bar:SetMinMaxValues(minValue, maxValue);
	end
end

function lib.GetBarColor(self)
	if (self.color == nil) then
		self.color = { r = nil, g = nil, b = nil };
	end
	return self.color.r, self.color.g, self.color.b;
end
function lib.SetBarColor(self, r, g, b, clear)
	self.color = { r, g, b };
	if (clear) then
		self.color = { r = nil, g = nil, b = nil };
	end
	UpdateColor(self);
end

-- Value Methods
----------------------------------------------------------------------------------------------------
function lib.GetValue(self)
	if (self.bar ~= nil) then
		return self.bar:GetValue();
	end
	return 0;
end
function lib.SetValue(self, value)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	value 		= value 	or  50;
	------------------------------------------------------------------------------------------------
	
	if (self.bar ~= nil) then
		self.bar:SetValue(value);
	end
end

function lib.GetMinValue(self)
	if (self.bar ~= nil) then
		local minValue, maxValue = self.bar:GetMinMaxValues();
		return minValue;
	end
	return 0;
end
function lib.SetMinValue(self, minValue)
	local maxValue = self:GetMaxValue();
	self:SetMinMaxValues(minValue, maxValue);
end

function lib.GetMaxValue(self)
	if (self.bar ~= nil) then
		local minValue, maxValue = self.bar:GetMinMaxValues();
		return maxValue;
	end
	return 0;
end
function lib.SetMaxValue(self, maxValue)
	local minValue = self:GetMinValue();
	self:SetMinMaxValues(minValue, maxValue);
end

function lib.GetMinMaxValues(self)
	if (self.bar ~= nil) then
		return self.bar:GetMinMaxValues();
	end
	return 0, 0;
end
function lib.SetMinMaxValues(self, minValue, maxValue)
	------------------------------------------------------------------------------------------------
	-- Local Variables
	------------------------------------------------------------------------------------------------
	minValue 	= minValue  or   0;
	maxValue 	= maxValue  or 100;
	------------------------------------------------------------------------------------------------
	
	if (self.bar ~= nil) then
		self.bar:SetMinMaxValues(minValue, maxValue);
	end
end

function lib.GetBarColor(self)
	if (self.color == nil) then
		self.color = { r = nil, g = nil, b = nil };
	end
	return self.color.r, self.color.g, self.color.b;
end
function lib.SetBarColor(self, r, g, b, clear)
	self.color = { r, g, b };
	if (clear) then
		self.color = { r = nil, g = nil, b = nil };
	end
	UpdateColor(self);
end
----------------------------------------------------------------------------------------------------

