-- libFoxActionBarBase.lua
-- Contains the base of all Actionbars in Addon.

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local LibraryName 		= "libFoxActionBarBase";
local LibraryVersion	= "1.0";
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName, true) == nil) then
	lib = LibStub:NewLibrary(LibraryName, LibraryVersion);
else
	lib = LibStub:GetLibrary(LibraryName);
end

----------------------------------------------------------------------------------------------------
--  Get Console
----------------------------------------------------------------------------------------------------
Console 			= LibStub:GetLibrary("FoxConsole");

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

-- Libraries
local libFrameCreator 	= LibStub:GetLibrary("FoxFrameCreator");
local libActionButton	= LibStub:GetLibrary("FoxActionButton 2.0");
local libFoxObjectClass	= LibStub:GetLibrary("FoxObjectClass");

-- Variables
lib.ButtonPad 			= 2;
lib.UpdateTime			= 15.0

-- Creation Functions
function lib.New(name, orientation, count, static, addonName, addonGroup)
	local self;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name, true);
	else
		self = libFrameCreator.New(_G[name], true);
	end
	self.ClassData = libFoxObjectClass.Class.New(name);
	-- Setup
	lib.Setup(self, orientation, count, static, addonName, addonGroup)
	
	-- Return
	return self;
end

-- Setup
function lib.Setup(self, orientation, count, static, addonName, addonGroup)
	-- Add Class Data
	lib.AddClassData(self);
	
	-- Position
	self:ClearAllPoints();
	self:SetPoint("CENTER", UIParent, 0, 0);
	
	-- Set Appearance
	self.Orientation = orientation or "Horizontal";
	
	-- Add Event Setup
	lib.AddEventMethods(self);
	
	-- Create Buttons
	lib.CreateActionButtons(self, count, addonName, addonGroup);
	
	-- Return
	return self;
end
function lib.AddClassData(self)
	-- Object Methods
	self.UpdateSize 				= lib.UpdateSize;
	self.UpdateButtonPosition 		= lib.UpdateButtonPosition;
	
	self.OnInitialize 				= lib.OnInitialize;
	self.OnEnable 					= lib.OnEnable;
	self.OnDisable 					= lib.OnDisable;
	
	self.Update 					= lib.Update;
	
	self.RePosition					= lib.RePosition;
	
	-- Class Methods
	self.ClassData.OnInitialize 	= lib.OnInitialize;
	self.ClassData.OnEnable 		= lib.OnEnable;
	self.ClassData.OnDisable 		= lib.OnDisable;
	
	self.ClassData.Update 			= lib.Update;
	self.ClassData.RePosition		= lib.RePosition;
	
	-- Variables
	self.lastUpdate 				= 0.0;
	
	-- Update Methods
	self.UpdateAll					= lib.UpdateAll
	self.UpdateAfterCombat 			= lib.UpdateAfterCombat
	self.SafeShow 					= lib.SafeShow
	self.SafeHide 					= lib.SafeHide
	
	-- Get Methods
	self.GetButton 					= lib.GetButton
	self.GetButtons 				= lib.GetButtons
end
function lib.CreateActionButtons(self, buttonCount, addonName, addonGroup)
	self.buttons 		= self.buttons or {};
	self.buttonCount	= buttonCount or 12;
	local button 		= nil;
	local lastButton 	= nil; 
	for i = 0, #self.buttons, 1 do
		if (self.buttons[i] ~= nil) then
			self.buttons[i]:Hide();
		end
	end
	for i = 0, (self.buttonCount - 1), 1 do
		if (self.buttons[i] == nil) then
			self.buttons[i] = libActionButton.Functions.New(self:GetName(), nil, i, false, 32, 32, addonName, addonGroup);
		else
			self.buttons[i] = libActionButton.Functions.Setup.All(self.buttons[i], i, false, 32, 32, addonName, addonGroup);
		end
		
		button = self.buttons[i];
		if (button ~= nil) then
			-- Set Parent
			button:SetParent(self);
			
			-- Show
			button:Show();
		end
	end
	
	-- Update Button Position
	self:UpdateButtonPosition();
	self:UpdateSize();
end

-- Update Functions
function lib.UpdateSize(self)
	if (self.DontAutoUpdateSize) then
		self.NeedUpdateSize = false;
		return;
	end
	if (InCombatLockdown()) then
		self.NeedUpdateSize = true;
		return;
	end
	if (self.buttons[0] == nil) then
		return;
	end
	if (self.Orientation == "Horizontal") then
		local buttonWidth 	= self.buttons[0]:GetWidth();
		local buttonHeight 	= self.buttons[0]:GetHeight();
		
		local width = 100;
		if (lib.GetButtonPad() > 0) then
			width = (buttonWidth * self.buttonCount) + (lib.GetButtonPad() * (self.buttonCount + 2));
		else
			width = (buttonWidth * self.buttonCount) + (lib.GetButtonPad() * (self.buttonCount) + 4);
		end
		self:SetWidth(width)
		
		local height = buttonHeight; -- + (lib.GetButtonPad() * 2);
		self:SetHeight(height)
	else
		local buttonWidth 	= self.buttons[0]:GetWidth();
		local buttonHeight 	= self.buttons[0]:GetHeight();
		
		local width = buttonWidth;
		self:SetWidth(width)
		
		local height = (buttonHeight * (self.buttonCount)) + (lib.GetButtonPad() * (self.buttonCount + 2));
		self:SetHeight(height)
	end
	self.NeedUpdateSize = false;
end
function lib.UpdateButtonPosition(self)
	if (self.DontAutoUpdateButtonPosition) then
		self.needButtonUpdate = false;
		return;
	end
	if (InCombatLockdown()) then
		self.needButtonUpdate = true;
		return;
	end
	local button 		= nil;
	local lastButton 	= nil; 
	for i = 0, (self.buttonCount - 1), 1 do
		button = self.buttons[i];
		if (self.Orientation == "Horizontal") then
			-- Set Point
			button:ClearAllPoints();
			if (i == 0) then
				button:SetPoint("Left", lib.GetButtonPad(), 0);
			else
				button:SetPoint("Left", lastButton, "Right", lib.GetButtonPad(), 0);
			end
		else
			-- Set Point
			button:ClearAllPoints();
			if (i == 0) then
				button:SetPoint("Top", 0, -lib.GetButtonPad());
			else
				button:SetPoint("Top", lastButton, "Bottom", 0, -lib.GetButtonPad());
			end
		end
		lastButton = button;
	end
	self.needButtonUpdate = false;
end

-- Class Events
function lib:OnInitialize()
	if (InCombatLockdown()) then
		-- Turn Flag On
		self.NeedsState("Initialized");
		return;
	end
	-- Load Here
	if (self.Load) then
		self:Load();
	end
	
	-- Initailize Here
	if (self.Init) then
		self:Init();
	end
	
	-- Show
	self:SafeShow();
	
	-- Remove Flag Off
	self.NeedsState("");
end
function lib:OnEnable()
	if (InCombatLockdown()) then
		-- Turn Flag On
		self.NeedsState("Enabled");
		return;
	elseif (self.NeedsState == "Initialized") then
		return;
	end
	-- Load Here
	if (self.Init) then
		self:Init();
	end
	
	-- Show
	self:SafeShow();
	
	-- Remove Flag Off
	self.NeedsState("");
end
function lib:OnDisable()
	if (InCombatLockdown()) then
		-- Turn Flag On
		self.NeedsState("Disabled");
		return;
	elseif (self.NeedsState == "Initialized") then
		return;
	end
	-- Hide
	self:SafeHide();
	
	if (self.Save) then
		self:Save();
	end
	
	-- Remove Flag Off
	self.NeedsState("");
end

-- Updates
function lib:Update(elapsed)
	if (self.lastUpdate > lib.UpdateTime) then
		-- Turn Flag On
		self.needsUpdate = true;
	else
		self.lastUpdate = self.lastUpdate + elapsed;
	end
end
function lib:UpdateAll()
	if (InCombatLockdown() ~= true) then
		self.needsUpdate = true;
		return;
	end
	self:UpdateSize();
	self:UpdateButtonPosition();
	
	-- Turn Flag Off
	self.needsUpdate = false;
end
function lib:UpdateAfterCombat()
	-- Handle Enable/Disable
	if (self.NeedsState == "Initialized") then
		self:OnInitialize();
	elseif (self.NeedsState == "Enabled") then
		self:OnEnable();
	elseif (self.NeedsState == "Disabled") then
		self:OnDisable();
	end
	
	-- Handle Main Update
	if (self.needsUpdate) then
		self:UpdateAll();
	end
	
	-- Handle Update Size
	if (self.NeedUpdateSize) then
		lib.UpdateSize(self);
	end
	
	-- Handle Button Update
	if (self.needButtonUpdate) then
		self:UpdateButtonPosition();
	end
	
	-- Handle Position Update
	if (self.needsPosition) then
		-- Move
		if (self.PositionInfo ~= nil) then
			if (type(self.PositionInfo) == "table") then
				self:RePosition(unpack(self.PositionInfo));
			end
		end
	end
end

-- Get/Set
function lib.GetButtonPad()
	return lib.ButtonPad;
end
function lib:GetButtons()
	return self.buttons;
end
function lib:GetButton(id)
	return self.buttons[id];
end

-- Events
function lib.AddEventMethods(self)
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:SetScript("OnEvent", lib.OnEvent);
end
function lib.OnEvent(self, event)
	if (event == "PLAYER_REGEN_ENABLED") then
		if (self.Update) then
			lib:UpdateAfterCombat();
		end
		if (self.SpecialUpdate) then
			self:SpecialUpdate();
		end
	elseif (event == "SPELLS_CHANGED") or (event == "LEARNED_SPELL_IN_TAB") then
		if (self.SpecialUpdate) then
			self.needSpellUpdate = true;
			self:SpecialUpdate();
		end
	end
end

-- Safe Methods
function lib:RePosition(...)
	if (InCombatLockdown() ~= true) then
		-- Move
		self:SetPoint(...);
		-- Turn Flag Off
		self.needsPosition = false;
	else
		-- Set Data
		self.PositionInfo = {...};
		-- Turn Flag On
		self.needsPosition = true;
	end
end
function lib:SafeShow()
	if (InCombatLockdown()) then
		self.NeedsVisibility = "Shown";
		return;
	end
	self:Show();
	self.needsVisibility = "";
end
function lib:SafeHide()
	if (InCombatLockdown()) then
		self.NeedsVisibility = "Hidden";
		return;
	end
	self:Hide();
	self.needsVisibility = "";
end


-- Create Horizontal Bar
--[[
local frameName = "FoxDummyFrame01";
local frame1;
if (_G[frameName] == nil) then
	frame1 = lib.New(frameName);
else
	frame1 = lib.New(_G[frameName]);
end
frame1:SetBackdropColor(1.0, 0.0, 0.0, 0.6);
]]--

-- Create Vertical Bar
--[[
local frameName = "FoxDummyFrame01";
local frame1;
if (_G[frameName] == nil) then
	frame1 = lib.New(frameName, "Vertical");
else
	frame1 = lib.New(_G[frameName], "Vertical");
end
frame1:SetBackdropColor(1.0, 0.0, 0.0, 0.6);
]]--


