-- libFoxActionBarMain.lua
-- Contains the base of all Actionbars in Addon.

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local LibraryName 		= "libFoxActionBarMain";
local LibraryVersion	= "1.0";
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName, true) == nil) then
	lib = LibStub:NewLibrary(LibraryName, LibraryVersion);
else
	lib = LibStub:GetLibrary(LibraryName);
end

----------------------------------------------------------------------------------------------------
--  Get Console
----------------------------------------------------------------------------------------------------
Console 			= LibStub:GetLibrary("FoxConsole");

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

-- Libraries
local libFrameCreator 		= LibStub:GetLibrary("FoxFrameCreator");
local libActionButton		= LibStub:GetLibrary("FoxActionButton 2.0");
local libFoxObjectClass		= LibStub:GetLibrary("FoxObjectClass");
local libFoxActionBarBase 	= LibStub:GetLibrary("libFoxActionBarBase");

-- Inherieted Methods
lib.AddClassData			= libFoxActionBarBase.AddClassData;

lib.UpdateSize				= libFoxActionBarBase.UpdateSize;
lib.UpdateButtonPosition 	= libFoxActionBarBase.UpdateButtonPosition;

lib.OnInitialize			= libFoxActionBarBase.OnInitialize;
lib.OnEnable				= libFoxActionBarBase.OnEnable;
lib.OnDisable				= libFoxActionBarBase.OnDisable

lib.Update					= libFoxActionBarBase.Update;
lib.UpdateAll				= libFoxActionBarBase.UpdateAll;

lib.RePosition				= libFoxActionBarBase.RePosition;

lib.UpdateTime				= libFoxActionBarBase.UpdateTime;

lib.SafeShow				= libFoxActionBarBase.SafeShow;
lib.SafeHide				= libFoxActionBarBase.SafeHide;
lib.UpdateAfterCombat		= libFoxActionBarBase.UpdateAfterCombat;

function lib.New(name, orientation, count, static, addonName, addonGroup)
	local self;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name, true);
	else
		self = libFrameCreator.Setup(_G[name], true);
	end
	self.ClassData = libFoxObjectClass.Class.New(name);
	-- Setup
	lib.Setup(self, orientation, count, static, addonName, addonGroup)
	
	-- Return
	return self;
end

-- Setup
function lib.Setup(self, orientation, count, static, addonName, addonGroup)
	-- Add Class Data
	lib.AddClassData(self);
	
	-- Position
	self:ClearAllPoints();
	self:SetPoint("CENTER", UIParent, 0, 0);
	
	-- Set Appearance
	self.Orientation = orientation or "Horizontal";
	
	-- Add New Methods
	
	-- Create Buttons
	lib.CreateActionButtons(self, count, addonName, addonGroup);
	
	-- Return
	return self;
end
function lib.CreateActionButtons(self, buttonCount, addonName, addonGroup)
	self.buttons 		= self.buttons or {};
	self.buttonCount	= buttonCount or 12;
	local button 		= nil;
	local lastButton 	= nil; 
	for i = 0, #self.buttons, 1 do
		if (self.buttons[i] ~= nil) then
			self.buttons[i]:Hide();
		end
	end
	for i = 0, (self.buttonCount - 1), 1 do
		if (self.buttons[i] == nil) then
			self.buttons[i] = libActionButton.Functions.New(self:GetName(), nil, i, false, 32, 32, addonName, addonGroup);
		else
			self.buttons[i] = libActionButton.Functions.Setup.All(self.buttons[i], i, false, 32, 32, addonName, addonGroup);
		end
		button = self.buttons[i];
		if (button ~= nil) then
			-- Set Parent
			button:SetParent(self);
			
			button:MakeLikeOrigButton(i + 1);
			
			-- Show
			button:Show();
		end
	end
	
	-- Update Button Position
	self:UpdateButtonPosition();
	self:UpdateSize();
end

function lib.GetButtonPad()
	return libFoxActionBarBase.GetButtonPad();
end

-- Create Bar
--[[
local frameName = "FoxDummyFrame01";
local frame1;
if (_G[frameName] == nil) then
	frame1 = lib.New(frameName);
else
	frame1 = lib.New(_G[frameName]);
end
frame1:SetBackdropColor(1.0, 0.0, 0.0, 0.6);
]]--
