-- libFoxActionBarPet.lua
-- Contains the base of all Actionbars in Addon.

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local LibraryName 		= "libFoxActionBarPet";
local LibraryVersion	= "1.0";
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName, true) == nil) then
	lib = LibStub:NewLibrary(LibraryName, LibraryVersion);
else
	lib = LibStub:GetLibrary(LibraryName);
end

----------------------------------------------------------------------------------------------------
--  Get Console
----------------------------------------------------------------------------------------------------
Console 			= LibStub:GetLibrary("FoxConsole");

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

-- Libraries
local libFrameCreator 		= LibStub:GetLibrary("FoxFrameCreator");
local libActionButton		= LibStub:GetLibrary("FoxActionButton 2.0");
local libFoxObjectClass		= LibStub:GetLibrary("FoxObjectClass");
local libFoxActionBarBase 	= LibStub:GetLibrary("libFoxActionBarBase");

-- State Controls
local VisibilityConditional = "[pet] pet; nopet;";

local VisibilityStateSnippet = [[
	-- arguments: self, stateid, newstate
	local state = newstate or SecureCmdOptionParse("[pet] pet; nopet;");
	if (state == "pet") then
		self:Show();
	else
		self:Hide();
	end
]]

-- Inherieted Methods
lib.AddClassData			= libFoxActionBarBase.AddClassData;

lib.UpdateSize				= libFoxActionBarBase.UpdateSize;
lib.UpdateButtonPosition 	= libFoxActionBarBase.UpdateButtonPosition;

lib.OnInitialize			= libFoxActionBarBase.OnInitialize;
lib.OnEnable				= libFoxActionBarBase.OnEnable;
lib.OnDisable				= libFoxActionBarBase.OnDisable

lib.Update					= libFoxActionBarBase.Update;
lib.UpdateAll				= libFoxActionBarBase.UpdateAll;

lib.RePosition				= libFoxActionBarBase.RePosition;

lib.UpdateTime				= libFoxActionBarBase.UpdateTime;

lib.SafeShow				= libFoxActionBarBase.SafeShow;
lib.SafeHide				= libFoxActionBarBase.SafeHide;
lib.UpdateAfterCombat		= libFoxActionBarBase.UpdateAfterCombat;

function lib.New(name, orientation, count, static, addonName, addonGroup)
	local self;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name, false);
	else
		self = libFrameCreator.Setup(_G[name], false);
	end
	self.ClassData = libFoxObjectClass.Class.New(name);
	-- Setup
	lib.Setup(self, orientation, count, static, addonName, addonGroup)
	
	-- Set the Visibility State Driver
	self:SetAttribute("_onstate-currentState", VisibilityStateSnippet);
	RegisterStateDriver(self, "currentState", VisibilityConditional);
	
	self:SetAttribute("updateState", VisibilityStateSnippet);
	
	-- Return
	return self;
end

-- Setup
function lib.Setup(self, orientation, count, static, addonName, addonGroup)
	-- Add Class Data
	lib.AddClassData(self);
	
	-- Position
	self:ClearAllPoints();
	self:SetPoint("CENTER", UIParent, 0, 0);
	
	-- Set Appearance
	self.Orientation = orientation or "Horizontal";
	
	-- Set the Manual Size Update Flag
	self.DontAutoUpdateSize = true;
	
	-- Add Methods
	self.ResetPosition 		= lib.ResetPosition;
	
	-- Create Buttons
	lib.CreateActionButtons(self, count, addonName, addonGroup);
	
	-- Return
	return self;
end
function lib.CreateActionButtons(self, buttonCount, addonName, addonGroup)
	self.buttons 		= self.buttons or {};
	self.buttonCount	= buttonCount or 10;
	local button 		= nil;
	local lastButton 	= nil; 
	for i = 0, (self.buttonCount - 1), 1 do
		if (self.buttons[i] == nil) then
			self.buttons[i] = libActionButton.Functions.New(self:GetName(), nil, (i + 1), false, nil, nil, addonName, addonGroup);
		else
			self.buttons[i] = libActionButton.Functions.Setup.All(self.buttons[i], (i + 1), false, nil, nil, addonName, addonGroup);
		end
		button = self.buttons[i];
		if (button ~= nil) then
			-- Set Parent
			button:SetParent(self);
			
			-- Show
			button:Show();

			-- Set to Pet Action
			libActionButton.Functions.SetupAsPetAction(button, (i + 1));
			
			-- Update
			libActionButton.Functions.UpdateChanges(button);
		end
	end
	
	-- Update Button Position
	self:UpdateButtonPosition();
	self:UpdateSize();
	self:ResetPosition();
	
end

function lib:ResetPosition()
	local button 	= self.buttons[0];
	local width 	= (button:GetWidth() * self.buttonCount) + (lib.GetButtonPad() * (self.buttonCount + 1)) + (4 * 2);
	local height 	= button:GetHeight() + (5 * 2);
	self:SetWidth(width);
	self:SetHeight(height);
	
	-- Set first button's Position
	button:SetPoint("Left", self, "Left", lib.GetButtonPad() + 4, 0);
end

function lib.GetButtonPad()
	return libFoxActionBarBase.GetButtonPad();
end

