-- libFoxActionBarStance.lua
-- Contains the base of all Actionbars in Addon.

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local LibraryName 		= "libFoxActionBarStance";
local LibraryVersion	= "1.0";
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName, true) == nil) then
	lib = LibStub:NewLibrary(LibraryName, LibraryVersion);
else
	lib = LibStub:GetLibrary(LibraryName);
end

----------------------------------------------------------------------------------------------------
--  Get Console
----------------------------------------------------------------------------------------------------
Console 			= LibStub:GetLibrary("FoxConsole");

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

-- Libraries
local libFrameCreator 		= LibStub:GetLibrary("FoxFrameCreator");
local libActionButton		= LibStub:GetLibrary("FoxActionButton 2.0");
local libFoxObjectClass		= LibStub:GetLibrary("FoxObjectClass");
local libFoxActionBarBase 	= LibStub:GetLibrary("libFoxActionBarBase");

-- Inherieted Methods
lib.AddClassData			= libFoxActionBarBase.AddClassData;

lib.UpdateSize				= libFoxActionBarBase.UpdateSize;
lib.UpdateButtonPosition 	= libFoxActionBarBase.UpdateButtonPosition;

lib.OnInitialize			= libFoxActionBarBase.OnInitialize;
lib.OnEnable				= libFoxActionBarBase.OnEnable;
lib.OnDisable				= libFoxActionBarBase.OnDisable

lib.Update					= libFoxActionBarBase.Update;
lib.UpdateAll				= libFoxActionBarBase.UpdateAll;

lib.RePosition				= libFoxActionBarBase.RePosition;

lib.UpdateTime				= libFoxActionBarBase.UpdateTime;

lib.SafeShow				= libFoxActionBarBase.SafeShow;
lib.SafeHide				= libFoxActionBarBase.SafeHide;
lib.UpdateAfterCombat		= libFoxActionBarBase.UpdateAfterCombat;

lib.AddEventMethods 		= libFoxActionBarBase.AddEventMethods;

function lib.New(name, orientation, count, static, addonName, addonGroup)
	local self;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name, false);
	else
		self = libFrameCreator.Setup(_G[name], false);
	end
	self.ClassData = libFoxObjectClass.Class.New(name);
	-- Setup
	lib.Setup(self, orientation, count, static, addonName, addonGroup)
	
	-- Return
	return self;
end

-- Setup
function lib.Setup(self, orientation, count, static, addonName, addonGroup)
	-- Add Class Data
	lib.AddClassData(self);
	
	-- Position
	self:ClearAllPoints();
	self:SetPoint("CENTER", UIParent, 0, 0);
	
	-- Set Appearance
	self.Orientation = orientation or "Horizontal";
	
	-- Add New Methods
	self.SetActions				= lib.SetActions;
	self.CheckStatus 			= lib.CheckStatus;
	self.ResetPosition			= lib.ResetPosition;
	self.SpecialUpdate 			= lib.SpecialUpdate;
	self.OnStanceSpellsUpdate	= lib.OnStanceSpellsUpdate;
	self.GetStanceAction		= lib.GetStanceAction;
	self.UseOriginalLayout		= true;
	
	-- Add Event Setup
	lib.AddEventMethods(self);

	-- Register Spell Update
	self:RegisterEvent("SPELLS_CHANGED");
	self:RegisterEvent("LEARNED_SPELL_IN_TAB")
	
	-- Create Buttons
	lib.CreateActionButtons(self, count, addonName, addonGroup);
	
	-- Check Status
	if (self.doNotShow) then
		self:SafeHide();
	end
	
	-- Return
	return self;
end
function lib.CreateActionButtons(self, buttonCount, addonName, addonGroup)
	self.buttons 		= self.buttons or {};
	self.buttonCount	= buttonCount or 10;
	local button 		= nil;
	local lastButton 	= nil; 
	for i = 0, (self.buttonCount - 1), 1 do
		if (self.buttons[i] == nil) then
			self.buttons[i] = libActionButton.Functions.New(self:GetName(), nil, i, true, nil, nil, addonName, addonGroup);
		else
			self.buttons[i] = libActionButton.Functions.Setup.All(self.buttons[i], i, true, nil, nil, addonName, addonGroup);
		end
		button = self.buttons[i];
		if (button ~= nil) then
			-- Set Parent
			button:SetParent(self);
			
			-- Set Layer
			button:SetFrameLevel(5);
			
			-- Show
			button:Hide();

			-- Set Post Update
			button.PostUpdate = lib.UpdateStance;
		end
	end
	
	-- Update Button Position
	self:UpdateButtonPosition();
	self:UpdateSize();
	
	self:SetActions();
	self:ResetPosition();
end

function lib.UpdateStance(button)
	local self = button;
	local _, className = UnitClass("player");
	if (className == "DRUID") then
		local buttonType = self:GetAttribute("type");
		if (buttonType == "spell") then
			local spellId 	= self:GetAttribute(buttonType);
			local spellName = GetSpellInfo(spellId);
			local formId 	= GetShapeshiftForm();
			if (formId > 0) then
				local _, name, isActive, _ = GetShapeshiftFormInfo(formId);
				if (spellName == name) then
					-- Show Enabled Texture
					self.enabledTexture:Show();
					self.drawFrame.enabledGlowTexture:Hide();
					self.drawFrame.activeTexture:Hide();
				else
					-- Hide Enabled Texture
					self.enabledTexture:Hide();
					self.drawFrame.enabledGlowTexture:Hide();
					self.drawFrame.activeTexture:Hide();
				end
			else
				-- Hide Enabled Texture
				self.enabledTexture:Hide();
				self.drawFrame.enabledGlowTexture:Hide();
				self.drawFrame.activeTexture:Hide();
			end
		end
	end
end

function lib.SetActions(self)
	local kind;
	local value;
	self:CheckStatus();
	for i = 0, (self.buttonCount - 1), 1 do
		if (self:IsShown()) then
			actionType, id, originalId = self:GetStanceAction(i);
			if (actionType == "spell") then
				--self.buttons[i]:SetAction(actionType, id);
				local _, className = UnitClass("player");
				--if (className == "DRUID") then
					local spellName = GetSpellInfo(id);
					-- Extra
					self.buttons[i]:SetAttribute("type", "macro");
					self.buttons[i]:SetAttribute("macrotext", "/cast "..(spellName or "")..";");
					self.buttons[i]:SetAttribute("customSpellIcon", id);
					
					self.buttons[i]:SetAttribute("tooltip", id);
					--self:SetAttribute("type", "macro");
					--self:SetAttribute("macrotext", "/cast "..(spellName or "")..";");
					
					--self:SetAttribute("type1", "macro");
					--self:SetAttribute("*macrotext1", "/cast [nostance] "..(spellName or "").."; /cancelaura [stance] "..(spellName or "")..";");
					--self:SetAttribute("type2", "macro");
					--self:SetAttribute("*macrotext2", "/cast "..(spellName or "")..";" );
					--self:SetAttribute("*macrotext2", "/cast [nostance] "..(spellName or "").."; /cancelaura [stance] "..(spellName or "")..";" );
				--end
			else
				self.buttons[i]:SetAction(actionType, id);
			end
			libActionButton.Functions.UpdateChanges(self.buttons[i]);
		end
	end
end

function lib:GetStanceAction(id)
	local _, className = UnitClass("player");
	if (self.UseOriginalLayout) then
		return lib.GetStanceData(className, id);
	else
		return lib.GetStanceDataReOrdered(className, id);
	end
end

function lib.CheckStatus(self)
	local _, className = UnitClass("player");
	local isKnown = false;
	local button;
	for i = 0, (self.buttonCount), 1 do
		if (self.UseOriginalLayout) then
			if (lib.Actions[className] == nil) then
				isKnown = (i+1 <= GetNumShapeshiftForms());
			elseif (lib.Actions[className][i+1] == nil) then
				isKnown = (i+1 <= GetNumShapeshiftForms());
			else
				isKnown = IsSpellKnown(lib.Actions[className][i+1]);
			end
		else
			if (lib.ActionsMod[className] == nil) then
				isKnown = (i+1 <= GetNumShapeshiftForms());
			elseif (lib.ActionsMod[className][i+1] == nil) then
				isKnown = (i+1 <= GetNumShapeshiftForms());
			else
				isKnown = IsSpellKnown(lib.ActionsMod[className][i+1]);
			end
		end
		button = self.buttons[i];
		if (isKnown) then
			if (button ~= nil) then
				self.buttons[i]:Show();
			end
		else
			if (button ~= nil) then
				self.buttons[i]:Hide();
			end
		end
	end
end

function lib.ResetPosition(self)
	local buttonsShown 		= 0;
	local button			= nil;
	local lastShownButton 	= nil;
	
	-- Set the Manual Size Update Flag
	self.DontAutoUpdateSize = true;
	
	-- Set the Manual Position Update Flag
	self.DontAutoUpdateButtonPosition = true;
	
	for i = 0, (self.buttonCount - 1), 1 do
		button = self.buttons[i];
		if (button:IsShown()) then
			-- Size
			buttonsShown = buttonsShown + 1;
			
			if (lastShownButton == nil) then
				button:SetPoint("Left", self, "Left", lib.GetButtonPad() + 4, 0);
			else
				button:SetPoint("Left", lastShownButton, "Right", lib.GetButtonPad(), 0);
			end
			lastShownButton = button;
		end
	end
	if (buttonsShown > 0) then
		local width = (button:GetWidth() * buttonsShown) + (lib.GetButtonPad() * (buttonsShown + 1)) + (4 * 2);
		local height = button:GetHeight() + (5 * 2);
		self:SetWidth(width);
		self:SetHeight(height);
		self.doNotShow = false;
	else
		self.doNotShow = true;
	end
end

function lib.GetButtonPad()
	return libFoxActionBarBase.GetButtonPad();
end

function lib.OnStanceSpellsUpdate(self)
	if (InCombatLockdown()) then
		self.needSpellUpdate = true;
		return;
	end
	-- Update Spells
	self:SetActions();
	-- Update Positions
	self:ResetPosition();
	
	-- Check Status
	if (self.doNotShow) then
		self:SafeHide();
	else
		self:SafeShow();
	end
	
	-- Remove Update Flag
	self.needSpellUpdate = false;
end

function lib.SpecialUpdate(self)
	-- Handle Spell Update
	if (self.needSpellUpdate) then
		self:OnStanceSpellsUpdate();
	end
end


