-- libFoxObjectClass.lua
-- Contains the base of all Objects in Addon.

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local LibraryName 		= "FoxObjectClass";
local LibraryVersion	= "1.0";
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName, true) == nil) then
	lib = LibStub:NewLibrary(LibraryName, LibraryVersion);
else
	lib = LibStub:GetLibrary(LibraryName);
end

----------------------------------------------------------------------------------------------------
--  Get Console
----------------------------------------------------------------------------------------------------
Console 			= LibStub:GetLibrary("FoxConsole");

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Local Functions
----------------------------------------------------------------------------------------------------
-- lib.Class:GetName;				-- Get's the Name of the Object

-- lib.Class:EmptyFuctionError; 	-- Empty Function, for Errors
-- lib.Class:OnInitialize;			-- Empty OnInitialize, for Errors
-- lib.Class:OnEnable;				-- Empty OnEnable, for Errors
-- lib.Class:OnDisable;				-- Empty OnDisable, for Errors

-- lib.Class:Update					-- The Optional Update Method
-- lib.Class:RePosition				-- The Optional RePosition Method
-- lib.Class:Load					-- The Optional Load Method
-- lib.Class:Save					-- The Optional Save Method
-- lib.Class:Show					-- The Optional Show Method
-- lib.Class:Hide					-- The Optional Hide Method

----------------------------------------------------------------------------------------------------
-- Object Class Definition
----------------------------------------------------------------------------------------------------
lib.Class 			= {};
lib.Class.__index 	= lib.Class;

lib.Classes 		= {};

function lib.Class.New(name)
	Console.CondError(LibraryName, ((name == nil) or (name == "")), "Class Name can not be nil.");
	Console.CondError(LibraryName, (lib.Classes[name] ~= nil), "Class Name \""..(tostring(name) or "nil").."\" already taken, name must be unique.");
	-- Create Empty Array for Class
	local self = {}; 						-- Set local self
	setmetatable(self, lib.Class);			-- Set Metatable
	
	-- Required Variables
	self.Name = name;
	
	-- Assign the Object to the Array
	lib.Classes[name] = self;
	
	-- Return new Object
	return self;
end
lib.New = lib.Class.New;

----------------------------------------------------------------------------------------------------
-- Built-In Methods
----------------------------------------------------------------------------------------------------
function lib.Class:GetName()
	return self.Name;
end
function lib.Class:EmptyFuctionError(funcName)
	Console:Error(LibraryName, "Object \""..(self:GetName() or "nil").."\" doesn't contain function \""..name.."\".")
end

----------------------------------------------------------------------------------------------------
-- Base Methods, Required
----------------------------------------------------------------------------------------------------
function lib.Class:OnInitialize()
	self:EmptyFuctionError("OnInitialize");
end
function lib.Class:OnEnable()
	self:EmptyFuctionError("OnEnable");
end
function lib.Class:OnDisable()
	self:EmptyFuctionError("OnDisable");
end

----------------------------------------------------------------------------------------------------
-- Base Methods, Optional
----------------------------------------------------------------------------------------------------
function lib.Class:Update(elapsed)
	-- Update Method is not required, but is here in case it is needed.
	return;
end

function lib.Class:RePosition()
	-- RePosition Method is not required, but is here in case it is needed.
	return;
end

function lib.Class:Load()
	-- Load Method is not required, but is here in case it is needed.
	return;
end

function lib.Class:Save()
	-- Save Method is not required, but is here in case it is needed.
	return;
end

function lib.Class:Show()
	-- Show Method is not required, but is here in case it is needed.
	return;
end

function lib.Class:Hide()
	-- Hide Method is not required, but is here in case it is needed.
	return;
end


