-- Main.lua
-- Main functions of the Addon

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

local MainFrameName 	= "FoxSmoothUIFrame";

local Loaded 			= false;
local PlayerIsInWorld 	= false;

----------------------------------------------------------------------------------------------------
-- Events
----------------------------------------------------------------------------------------------------

function Addon:CheckRequiredAddons()
	local libActionButton	= LibStub:GetLibrary("FoxActionButton 2.0", true);
	local libConsole 		= LibStub:GetLibrary("FoxConsole", true);
	-- Check for Required Libs
	if (libActionButton == nil) then
		return false;
	end
	if (libConsole == nil) then
		-- Throw Error
		return false;
	end
	
	return true;
end

function Addon:OnDisable()
	
	-- Check for Required Libs
	if (not Addon:CheckRequiredAddons()) then
		return;
	end
	
	if (InCombatLockdown()) then
		Addon.NeedsState = "Disable";
		return;
	end
	
	for i = 1, #Addon.Objects, 1 do
		if (Addon.Objects[i].OnDisable ~= nil) then
			Addon.Objects[i]:OnDisable();
		end
	end
	--Addon.Console.Print(Addon:GetName(), "Goodbye World!", Addon.MessageColor);
	
	-- Register Addon's Events
	Addon.RegisterEvents(true);
	
	if (PlayerIsInWorld and Loaded) then
		-- Save Buttons
		Addon.SaveButtons();
		
		-- Setup Hotkey Loading
		Loaded = false;
		--Addon:RegisterEvent("UPDATE_BINDINGS");
	
	end
	
	-- Show Old UI
	MainMenuBar:Show();
	MultiBarRight:Show();
	MultiBarLeft:Show();
	
	-- Set Value
	Addon.db.profile.Enabled = false;
	
	-- Hide Main Frame
	Addon.MainFrame:Hide();
	
	-- Update UI
	--Addon.Update_BottomController(true);
	
	-- Run OnDisable Modular Functions
	Addon.Module_OnDisable();
end

function Addon:OnEnable()
	
	-- Check for Required Libs
	if (not Addon:CheckRequiredAddons()) then
		return;
	end
	
	if (InCombatLockdown()) then
		Addon.NeedsState = "Enable";
		return;
	end
	
	for i = 1, #Addon.Objects, 1 do
		if (Addon.Objects[i].OnEnable ~= nil) then
			Addon.Objects[i]:OnEnable();
		end
	end
	--Addon.Console.Print(Addon:GetName(), "Hello World!", Addon.MessageColor);	
	
	if (PlayerIsInWorld and not Loaded) then
		-- Update the Stance Bar
		--Addon.StanceBar:SetActions();
		
		-- Set the Keybindings
		Addon.LoadButtons();
		
		-- Update Bars
		--Addon.Update_BottomController(true);
		
		-- Set Flag HasLoaded to true
		Loaded = true;
		
	end
	
	-- Register Addon's Events
	Addon.RegisterEvents();
	
	-- Hide Old UI
	MainMenuBar:Hide();
	MultiBarRight:Hide();
	MultiBarLeft:Hide();
	
	-- Un-Parent the Extra Button Frame
	ExtraActionBarFrame:SetParent(UIParent);
	
	-- Show Main Frame
	Addon.MainFrame:Show();
	
	-- Set Value
	Addon.db.profile.Enabled = true;
	
	Addon.Config.Load();
	
	-- Update UI
	--Addon.Update_BottomController(true);
	
	-- Run OnEnable Modular Functions
	Addon.Module_OnEnable();
end

function Addon:OnInitialize()
	
	-- Check for Required Libs
	if (not Addon:CheckRequiredAddons()) then
		return;
	end
	
	-- Create Main Frame, if needed
	if (Addon.MainFrame == nil) then
		if (_G[MainFrameName] == nil) then
			Addon.MainFrame = CreateFrame("FRAME", MainFrameName, UIParent, "SecureHandlerStateTemplate, SecureHandlerShowHideTemplate");
		else
			Addon.MainFrame = _G[MainFrameName];
		end
		
		-- Register Addon's Leave Combat Event
		Addon:RegisterEvent("PLAYER_REGEN_ENABLED");
	end
	
	if (InCombatLockdown()) then
		Addon.NeedsState = "Initialize";
		return;
	end

	-- Setup Config
	Addon.InitializeConfig();
	
	-- Set Update Time
	Addon.TimeBetweenUpdates = 30.0;
	
	-- Create ActionBar Base Frame
	--Addon.CreateBottomControllerFrame();
	
	-- Create the Side ActionBars
	--Addon.CreateSideActionBarsController();
	
	-- Create Micro Button Controller
	--Addon.CreateMicroButtonController();
	
	-- Create Bag Bar
	--Addon.CreateBagBar();
	
	for i = 1, #Addon.Objects, 1 do
		if (Addon.Objects[i].OnInitialize ~= nil) then
			Addon.Objects[i]:OnInitialize();
		end
	end
	--Addon.Console.Print(Addon:GetName(), "World Initialized!", Addon.MessageColor);
	
	-- Run Module Initializations
	Addon.Module_Initialization()
	
	-- Show Main Frame
	Addon.MainFrame:Show();
	
	Addon:SetEnabledState(Addon.db.profile.Enabled);
	
	if (Addon.db.profile.Enabled) then
		Addon:Enable();
	else
		Addon:Disable();
	end
end

function Addon:ADDON_LOADED(name)
	if (name == Addon.FolderName) then
		Addon.VariablesLoad();
		
		-- LoadButtons Fails!
	end
end

function Addon:PLAYER_ENTERING_WORLD()
	-- Update the Stance Bar
	--Addon.StanceBar:SetActions();
	
	-- If player has not Loaded UI before this Session
	if (not Loaded) then
		-- Set the Keybindings
		Addon.LoadButtons();
		
		-- Setup if Necessary
		if (self.db.char.LoadedBefore == false) then
			
			-- Setup First Bar
			--Addon.SetupKeybindingDataBar(Addon.ActionBarMain);
			
			-- Prompt for Pulling Data
			--Addon.FirstLoad.Prompt();
		end
	end
	-- Set Flag HasLoaded to true
	Loaded = true
	
	-- Update Bars
	--Addon.Update_BottomController(true);
	
	-- Module Additions
	Addon.Module_PEW_Func()
end

function Addon:PLAYER_LEAVING_WORLD()
	if (Loaded) then
		-- Save Buttons
		Addon.SaveButtons();
	end
	
	-- Setup Hotkey Loading
	Loaded = false;
	Addon:RegisterEvent("UPDATE_BINDINGS");
	
	-- Set Player in world Flag
	PlayerIsInWorld = false;
	
end

function Addon:UPDATE_BINDINGS()
	-- Load Buttons
	if (Loaded) then
		Addon:UnregisterEvent("UPDATE_BINDINGS");
		
		-- Update the Stance Bar
		--Addon.StanceBar:SetActions();
		
		-- Set the Keybindings
		Addon.LoadButtons();
	end
end

function Addon:PLAYER_REGEN_ENABLED()
	if ((Addon.NeedsState ~= nil) and (not InCombatLockdown())) then
		if (Addon.NeedsState == "Initialize") then
			Addon:OnInitialize();
		elseif (Addon.NeedsState == "Enable") then
			Addon:OnEnable();
		elseif (Addon.NeedsState == "Disable") then
			Addon:OnDisable();
		end
	end
end

function Addon.VariablesLoad()
	Addon.ConfigVariablesLoaded();
end

function Addon.RegisterEvents(clear)
	if (not clear) then
		-- Register Addon's Events
		Addon:RegisterEvent("PLAYER_REGEN_ENABLED");
		
		-- Setup Variable Load
		Addon:RegisterEvent("ADDON_LOADED");

		Addon:RegisterEvent("PLAYER_ENTERING_WORLD");
		Addon:RegisterEvent("PLAYER_LEAVING_WORLD");
		Addon:RegisterEvent("UPDATE_BINDINGS");
	else
		
		-- Setup Variable Load
		Addon:UnregisterEvent("ADDON_LOADED");

		Addon:UnregisterEvent("PLAYER_ENTERING_WORLD");
		Addon:UnregisterEvent("PLAYER_LEAVING_WORLD");
		Addon:UnregisterEvent("UPDATE_BINDINGS");
	end
end
-- Create a Dummy Test Object
local function DummyObjectTest01()
	local name = "test01";
	local obj1 = Addon.ObjectClass.New(name);
	obj1.OnInitialize = function() Addon.Console.Print(Addon:GetName(), obj1:GetName().." Initialized!", Addon.MessageColor); end
	obj1.OnEnable = function() Addon.Console.Print(Addon:GetName(), obj1:GetName().." Enabled!", Addon.MessageColor); end
	obj1.OnDisable = function() Addon.Console.Print(Addon:GetName(), obj1:GetName().." Disabled!", Addon.MessageColor); end
	Addon:AddObject(obj1);
end

-- Create a Dummy Test Object
local function DummyFrameTest01()
	local lib = LibStub:GetLibrary("FoxFrameCreator");
	local frame1 = lib.New("FoxDummyFrame01");
	local frame2 = lib.New("FoxDummyFrame02", true);
	lib.MakeFrameDraggable(frame1)
	frame2:ClearAllPoints();
	frame2:SetPoint("LEFT", frame1, "RIGHT", 0, 0);
	frame2:SetBackdropColor(1.0, 0.0, 0.0, 0.6);
end

local function MainActionBarTest01()
	local lib 	= LibStub:GetLibrary("libFoxActionBarMain");
	local frameName1 = "FoxActionBarMainFrame";
	local frame1;
	if (_G[frameName1] == nil) then
		frame1 = lib.New(frameName1);
	else
		frame1 = lib.Setup(_G[frameName1])
	end
	frame1:SetBackdropColor(1.0, 0.0, 0.0, 0.6);
	frame1:ClearAllPoints();
	frame1:RePosition("Center", UIParent, "Center", 0, 0);
end

local function ActionBarsTest01()
	local lib 			= LibStub:GetLibrary("libFoxActionBarBase");
	local frameName1 	= "FoxActionBarFrame1";
	local frameName2 	= "FoxActionBarFrame2";
	local frameNameMain = "FoxActionBarMainFrame";
	local frame1;
	local frame2;
	if (_G[frameName1] == nil) then
		frame1 = lib.New(frameName1);
	else
		frame1 = lib.Setup(_G[frameName1])
	end
	if (_G[frameName2] == nil) then
		frame2 = lib.New(frameName2);
	else
		frame2 = lib.Setup(_G[frameName2])
	end
	frame1:SetBackdropColor(1.0, 0.0, 0.0, 0.6);
	frame1:ClearAllPoints();
	frame1:RePosition("Bottom", _G[frameNameMain], "Top", 0, lib.GetButtonPad());
	
	frame2:SetBackdropColor(1.0, 0.0, 0.0, 0.6);
	frame2:ClearAllPoints();
	frame2:RePosition("Bottom", frame1, "Top", 0, lib.GetButtonPad());
	--frame2:RePosition("Left", frame1, "Right", lib.GetButtonPad(), 0);
end

local function StanceBarTest01()
	local lib 		= LibStub:GetLibrary("libFoxActionBarStance");
	local frameName = "FoxStanceBar1";
	local frame;
	if (_G[frameName] == nil) then
		frame = lib.New(frameName);
	else
		frame = lib.Setup(_G[frameName]);
	end
	frame:SetBackdropColor(1.0, 0.0, 0.0, 0.6);
	
	local frame2 = _G["FoxActionBarFrame2"];
	frame:RePosition("BottomLeft", frame2, "TopLeft", 0, lib.GetButtonPad())
end

-- Modular Functions
-- Tables
-- Addon.DisableFunctions
-- Addon.EnableFunctions
-- Addon.InitializationFunctions
-- Addon.Module_PEW
function Addon.Module_OnDisable()
	-------------------------------------------------------------
	-- Added Code for a Mimimalistic Appoach to Adding Modules
	-------------------------------------------------------------
	-- Safety Check
	Addon.DisableFunctions = Addon.DisableFunctions or {};
	
	-- Safety Check
	if (#Addon.DisableFunctions > 0) then
		-- Loop through Functions in table
		for i = 1, #Addon.DisableFunctions, 1 do
			-- Call Function i
			local status = pcall(Addon.DisableFunctions[i]);
		end
	end
end
function Addon.Module_OnEnable()
	-------------------------------------------------------------
	-- Added Code for a Mimimalistic Appoach to Adding Modules
	-------------------------------------------------------------
	-- Safety Check
	Addon.EnableFunctions = Addon.EnableFunctions or {};
	
	-- Safety Check
	if (#Addon.EnableFunctions > 0) then
		-- Loop through Functions in table
		for i = 1, #Addon.EnableFunctions, 1 do
			-- Call Function i
			local status = pcall(Addon.EnableFunctions[i]);
		end
	end
end
function Addon.Module_Initialization()
	-------------------------------------------------------------
	-- Added Code for a Mimimalistic Appoach to Adding Modules
	-------------------------------------------------------------
	-- Safety Check
	Addon.InitializationFunctions = Addon.InitializationFunctions or {};
	
	-- Safety Check
	if (#Addon.InitializationFunctions > 0) then
		-- Loop through Functions in table
		for i = 1, #Addon.InitializationFunctions, 1 do
			-- Call Function i
			--local status;
			local status = pcall(Addon.InitializationFunctions[i]);
		end
	end
end
function Addon.Module_PEW_Func()
	-------------------------------------------------------------
	-- Added Code for a Mimimalistic Appoach to Adding Modules
	-------------------------------------------------------------
	-- Safety Check
	Addon.Module_PEW = Addon.Module_PEW or {};
	
	-- Safety Check
	if (#Addon.Module_PEW > 0) then
		-- Loop through Functions in table
		for i = 1, #Addon.Module_PEW, 1 do
			-- Call Function i
			local status = pcall(Addon.Module_PEW[i]);
		end
	end
end

function Addon.LoadModules()
	Addon.Module_Initialization();
	Addon.Module_PEW_Func();
	Addon.Module_OnEnable();
	
	Addon.AddConfigOptions();
end