-- Styler.lua
-- Handle Styling the UI

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--


--- Add to new file
Addon.StyleFrames 	= Addon.StyleFrames or {};
Addon.StyleStatBars = Addon.StyleStatBars or {};
Addon.StyleColor 	= {r = 0.1, g = 0.1, b = 0.1, a = 0.6}

Addon.Styler = Addon.Styler or {};

Addon.Styler.Colors = {}
Addon.Styler.Colors['Black'] 	= {r = 0.1, g = 0.1,  b = 0.1, a = 0.6};
Addon.Styler.Colors['Grey'] 	= {r = 0.4, g = 0.4,  b = 0.4, a = 0.6};
Addon.Styler.Colors['Red'] 		= {r = 0.4, g = 0.0,  b = 0.0, a = 0.6};
Addon.Styler.Colors['Blue'] 	= {r = 0.0, g = 0.0,  b = 0.3, a = 0.6};
Addon.Styler.Colors['Green'] 	= {r = 0.0, g = 0.3,  b = 0.0, a = 0.6};
Addon.Styler.Colors['Teal'] 	= {r = 0.0, g = 0.15, b = 0.4, a = 0.6};
Addon.Styler.Colors['Light'] 	= {r = 0.7, g = 0.7,  b = 0.7, a = 0.6};
Addon.Styler.Colors['Custom'] 	= "Custom";


function Addon.Styler.SetStyle(styleName)
	Addon.DebugPrint("UseStyle: "..styleName);
	Addon.Styler.UseStyle(styleName);
end
function Addon.Styler.UseStyle(styleName)
	Addon.DebugPrint("UseStyle: "..styleName);
	
	if ((styleName == "Round") or (styleName == "Original")) then
		-- Default
		Addon.DebugPrint("UseStyle: ".."round");
		Addon.Styler.UseStyle_Round();
		
	elseif (styleName == "Square") then
		Addon.DebugPrint("UseStyle: ".."square");
		-- New Square Style
		Addon.Styler.UseStyle_Square();
		
	end

end

function Addon.Styler.UseStyle_Round()
	
	-- Style Frames
	if (#Addon.StyleFrames > 0) then
		local i;
		for i = 1, #Addon.StyleFrames, 1 do
			-- for loop
			local self = Addon.StyleFrames[i];
			self:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
				edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
				edgeSize = 12, 
				insets = { left = 3, right = 3, top = 3, bottom = 3 }});
			
			if (Addon.db.profile.ColorPre == "Custom") then
				Addon.Styler.Style_SetColor(Addon.db.profile.Color);
			else
				Addon.Styler.Style_SetColor(Addon.Styler.Colors[Addon.db.profile.ColorPre]);
			end
			
			if (self.OnStyle) then
				self:OnStyle();
			end
		end
	end
	
	-- Style Stat Bars
	if (#Addon.StyleStatBars > 0) then
		local i;
		for i = 1, #Addon.StyleStatBars, 1 do
			-- for loop
			local self = Addon.StyleStatBars[i];
			
			-- Set Height

			-- Set Background
			self:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
				edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
				edgeSize = 12, 
				insets = { left = 3, right = 3, top = 3, bottom = 3 }});

			-- Set Background Color and Texture
			if (Addon.db.profile.ColorPre == "Custom") then
				Addon.Styler.Style_SetColor(Addon.db.profile.Color);
			else
				Addon.Styler.Style_SetColor(Addon.Styler.Colors[Addon.db.profile.ColorPre]);
			end
			--self:SetBackdropBorderColor(1, 0, 0, 0.6);
			--self.background:SetTexture( 0, 0, 0, 0.7);
			self:SetupBackground();

			-- Set Text
			-- First
			self.bar.labelFirst:SetFont("Fonts\\FRIZQT__.TTF", 10, "Outline");
			if (not self.isVertical) then
				self.bar.labelFirst:ClearAllPoints();
				self.bar.labelFirst:SetPoint("BottomLeft", self.bar, "BottomLeft", 2, 0);
			end

			-- Middle
			self.bar.labelMiddle:SetFont("Fonts\\FRIZQT__.TTF", 10, "Outline");
			if (not self.isVertical) then
				self.bar.labelMiddle:ClearAllPoints();
				self.bar.labelMiddle:SetPoint("Bottom");
			end

			-- Last
			self.bar.labelLast:SetFont("Fonts\\FRIZQT__.TTF", 10, "Outline");
			if (not self.isVertical) then
				self.bar.labelLast:ClearAllPoints();
				self.bar.labelLast:SetPoint("BottomRight", self.bar, "BottomRight", -2, 0);
			end
			
		end
	end
	
end

function Addon.Styler.UseStyle_Square()
	
	-- Style Frames
	--print(#Addon.StyleFrames);
	if (#Addon.StyleFrames > 0) then
		local i;
		for i = 1, #Addon.StyleFrames, 1 do
			-- for loop
			local self = Addon.StyleFrames[i];
			self:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
			edgeFile = "Interface\\Addons\\FoxSmoothUI-v3.1\\Graphics\\Tooltips\\UI-Tooltip-Border-White", 
			edgeSize = 10, 
			insets = { left = 2, right = 2, top = 3, bottom = 2 }});
			
			if (Addon.db.profile.ColorPre == "Custom") then
				Addon.Styler.Style_SetColor(Addon.db.profile.Color);
			else
				Addon.Styler.Style_SetColor(Addon.Styler.Colors[Addon.db.profile.ColorPre]);
			end
			
			if (self.OnStyle) then
				self:OnStyle();
			end
		end
	end
	
	-- Style Stat Bars
	if (#Addon.StyleStatBars > 0) then
		local i;
		for i = 1, #Addon.StyleStatBars, 1 do
			-- for loop
			local self = Addon.StyleStatBars[i];
			
			-- Set Height
			--self:SetHeight(20);

			-- Set Background
			self:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
				  edgeFile = "Interface\\Addons\\FoxSmoothUI-v3.1\\Graphics\\Tooltips\\UI-Tooltip-Border-White", 
				  edgeSize = 10, 
				  insets = { left = 2, right = 2, top = 3, bottom = 2 }});

			-- Set Background Color and Texture
			if (Addon.db.profile.ColorPre == "Custom") then
				Addon.Styler.Style_SetColor(Addon.db.profile.Color);
			else
				Addon.Styler.Style_SetColor(Addon.Styler.Colors[Addon.db.profile.ColorPre]);
			end
			--self:SetBackdropBorderColor(1, 0, 0, 0.6);
			self.background:SetTexture( 0, 0, 0, 0.7);

			-- Set Text
			-- First
			self.bar.labelFirst:SetFont("Fonts\\FRIZQT__.TTF", 10);
			if (not self.isVertical) then
				self.bar.labelFirst:ClearAllPoints();
				self.bar.labelFirst:SetPoint("BottomLeft", self.bar, "BottomLeft", 2, 0);
			end

			-- Middle
			self.bar.labelMiddle:SetFont("Fonts\\FRIZQT__.TTF", 10);
			if (not self.isVertical) then
				self.bar.labelMiddle:ClearAllPoints();
				self.bar.labelMiddle:SetPoint("Bottom");
			end

			-- Last
			self.bar.labelLast:SetFont("Fonts\\FRIZQT__.TTF", 10);
			if (not self.isVertical) then
				self.bar.labelLast:ClearAllPoints();
				self.bar.labelLast:SetPoint("BottomRight", self.bar, "BottomRight", -2, 0);
			end
			
		end
	end
	
end

function Addon.Styler.Style_SetColor(color, borderColor)
	borderColor = borderColor or {};
	if (#borderColor == 0) then
		-- Calc Border Color
		if (color.r > 0) then
			borderColor.r = math.min(color.r + 0.3, 1);
		end
		if (color.g > 0) then
			borderColor.g = math.min(color.g + .3, 1);
		end
		
		if (color.b > 0) then
			borderColor.b = math.min(color.b + .3, 1);
		end
		
		borderColor.a = color.a;
	end
	
	if (#Addon.StyleFrames > 0) then
		local i;
		for i = 1, #Addon.StyleFrames, 1 do
			local self = Addon.StyleFrames[i];
			
			-- Set Background Color
			self:SetBackdropColor(color.r, color.g, color.b, color.a);

			-- Set Border Color
			self:SetBackdropBorderColor(borderColor.r, borderColor.g, borderColor.b, borderColor.a);
			
			if (self.OnStyle) then
				self:OnStyle();
			end
		end
	end
	
	-- Style Stat Bars
	if (#Addon.StyleStatBars > 0) then
		local i;
		for i = 1, #Addon.StyleStatBars, 1 do
			-- for loop
			local self = Addon.StyleStatBars[i];
			
			--self:SetBackdropColor(color.r, color.g, color.b, color.a);
			self:SetBackdropColor(0, 0, 0, 0.7);
			self:SetBackdropBorderColor(borderColor.r, borderColor.g, borderColor.b, borderColor.a);
			
			if (self.OnStyle) then
				self:OnStyle();
			end
		end
	end
	
	for i=1, NUM_CHAT_WINDOWS, 1 do
	local self = _G["ChatFrame"..i.."Tab"]   
		_G["ChatFrame"..i.."Tab"]:SetBackdropColor(color.r, color.g, color.b, 1)
		_G["ChatFrame"..i.."TabLeft"]:SetVertexColor(color.r, color.g, color.b, 1)
		_G["ChatFrame"..i.."TabMiddle"]:SetVertexColor(color.r, color.g, color.b, 1)
		_G["ChatFrame"..i.."TabRight"]:SetVertexColor(color.r, color.g, color.b, 1)
	end
end

--function Addon.SetButtonPad()