-- libFoxFrameCreator.lua
-- Handles Creating Basic Frames.


--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
local libFrameCreator 			= LibStub:GetLibrary("FoxFrameCreator");
local frameName = "FSUI_Exp_Tooltip";

local function commaFormat(number)
	local formatedString = string.gsub(number, "(%d)(%d%d%d)$", "%1,%2", 1)
	while true do
		formatedString, found = string.gsub(formatedString, "(%d)(%d%d%d),", "%1,%2,", 1)
		if found == 0 then break end
	end
	return formatedString;
end

local function hideSelf(self)
	if (InCombatLockdown()) then return; end
	self.tooltip:Hide();
end

local function showSelf(self)
	if (InCombatLockdown()) then return; end
	if (UnitLevel("player") == GetMaxPlayerLevel()) then return; end
	self.tooltip:Show();
end

local function UpdateText(self)
	-- Data
	local unit 			= "player"
	local xp 			= UnitXP(unit);
	local xpMax 		= UnitXPMax(unit);
	local xpRested 		= GetXPExhaustion() or 0;
	local xpLeft 		= xpMax - xp;
	local xpPer 		= math.floor((xp / xpMax) * 100);
	local xpLeftPer 	= math.ceil(((xpMax - xp) / xpMax) * 100);
	local xpResterPer 	= math.floor((xpRested / xpMax) * 100);
	
	-- Setup Current Amount Label
	local labelCurrentAmountText = commaFormat(xp).." ("..xpPer.."%)";
	if (xpPer < 10) then
		labelCurrentAmountText = commaFormat(xp).." (  "..xpPer.."%)";
	end
	self.labelCurrentAmount:SetText(labelCurrentAmountText);
	
	-- Setup Needed Amount Label
	line 	= 2;
	local labelNeededAmountText = commaFormat(xpMax).."           ";
	self.labelNeededAmount:SetText(labelNeededAmountText);
	
	-- Setup Left Amount  Label
	line 	= 3;
	local labelLeftAmountText = commaFormat(xpLeft).." ("..xpLeftPer.."%)";
	if (xpLeftPer < 10) then
		labelLeftAmountText = commaFormat(xpLeft).." (  "..xpLeftPer.."%)";
	end
	self.labelLeftAmount:SetText(labelLeftAmountText);
	
	-- Setup Rested Amount Label
	local labelRestedAmountText = commaFormat(xpRested).." ("..xpResterPer.."%)";
	if (xpResterPer < 10) then
		labelRestedAmountText = commaFormat(xpRested).." (  "..xpResterPer.."%)";
	end
	self.labelRestedAmount:SetText(labelRestedAmountText)
end

local function onEvent(self)
	UpdateText(self);
end

local function SetupXPBarTooltipLayout(bar)
	local name 		= frameName;
	local self 		= _G[name];
	local width 	= 180;
	local height 	= 72;
	local parent 	= UIParent;

	local anchorInfo = {
		point 			= "Bottom";
		relativeFrame 	= bar;
		relativePoint 	= "Top";
		relativeX		= 0;
		relativeY		= 0;
	}

	-- Setup Frame
	if (self == nil) then
		self = libFrameCreator.New(name, false, width, height, parent, anchorInfo)
	else
		self:SetWidth(width);
		self:SetHeight(height);
		self:SetParent(parent);
	end

	-- Setup Frame Strata
	self:SetFrameStrata("TOOLTIP");
	
	-- Setup Background
	libFrameCreator.SetupFrameWithBorder(self);
	
	-- Set Color
	self:SetBackdropColor(0.1, 0.1, 0.1, 1.0);
	
	local topOffset 	= -4;
	local lineOffset 	= -12;
	local line			= 0;
	local justifyH		= "CENTER";
	local anchorPoint	= "Top";
	local leftOffset	= 0;
	
	-- Setup Label
	local labelTitleText = "Exp";
	self.labelTitle = libFrameCreator.CreateLabel(self.labelTitle, self, labelTitleText, anchorPoint, self, anchorPoint, leftOffset, topOffset + (lineOffset * line), 100, 20, justifyH);
	libFrameCreator.MakeTitle(self.labelTitle, true);
	
	-- Setup Current Label
	justifyH 	= "LEFT";
	anchorPoint	= "TopLeft";
	line 		= 1;
	topOffset 	= -8;
	leftOffset	= 4;
	local labelCurrentText = "Current: ";
	self.labelCurrent = libFrameCreator.CreateLabel(self.labelCurrent, self, labelCurrentText, anchorPoint, self, anchorPoint, leftOffset, topOffset + (lineOffset * line), 100, 20, justifyH);
	
	-- Setup Needed Label
	line 		= 2;
	local labelNeededText = "Needed: ";
	self.labelNeeded = libFrameCreator.CreateLabel(self.labelNeeded, self, labelNeededText, anchorPoint, self, anchorPoint, leftOffset, topOffset + (lineOffset * line), 100, 20, justifyH);
	
	-- Setup Rested Label
	line = 3;
	local labelLeftText = "Left: ";
	self.labelLeft = libFrameCreator.CreateLabel(self.labelLeft, self, labelLeftText, anchorPoint, self, anchorPoint, leftOffset, topOffset + (lineOffset * line), 100, 20, justifyH);
	
	-- Setup Rested Label
	line = 4;
	local labelRestedText = "Rested: ";
	self.labelRested = libFrameCreator.CreateLabel(self.labelRested, self, labelRestedText, anchorPoint, self, anchorPoint, leftOffset, topOffset + (lineOffset * line), 100, 20, justifyH);
	
	-- Data
	local unit 			= "player"
	local xp 			= UnitXP(unit);
	local xpMax 		= UnitXPMax(unit);
	local xpRested 		= GetXPExhaustion() or 0;
	local xpLeft 		= xpMax - xp;
	local xpPer 		= math.floor((xp / xpMax) * 100);
	local xpLeftPer 	= math.floor(((xpMax - xp) / xpMax) * 100);
	local xpResterPer 	= math.floor((xpRested / xpMax) * 100);
	
	-- Setup Current Amount Label
	justifyH 			= "RIGHT";
	anchorPoint			= "TopRight";
	line 				= 1;
	topOffset 			= -8;
	leftOffset			= -4;
	local textWidth		= 120;
	self.labelCurrentAmount = libFrameCreator.CreateLabel(self.labelCurrentAmount, self, "###", anchorPoint, self, anchorPoint, leftOffset, topOffset + (lineOffset * line), textWidth, 20, justifyH);
	
	-- Setup Needed Amount Label
	line 	= 2;
	self.labelNeededAmount = libFrameCreator.CreateLabel(self.labelNeededAmount, self, "###", anchorPoint, self, anchorPoint, leftOffset, topOffset + (lineOffset * line), textWidth, 20, justifyH);
	
	-- Setup Left Amount Label
	line 	= 3;
	self.labelLeftAmount = libFrameCreator.CreateLabel(self.labelLeftAmount, self, "###", anchorPoint, self, anchorPoint, leftOffset, topOffset + (lineOffset * line), textWidth, 20, justifyH);
	
	-- Setup Rested Amount Label
	line 	= 4;
	self.labelRestedAmount = libFrameCreator.CreateLabel(self.labelRestedAmount, self, "###", anchorPoint, self, anchorPoint, leftOffset, topOffset + (lineOffset * line), textWidth, 20, justifyH);
	
	UpdateText(self);
end

local function SetupXPBarTooltipScripts(bar)
	local name 		= frameName;
	local self 		= _G[name];
	
	-- Setup OnMouseOver
	bar:SetScript("OnEnter", showSelf);
	bar:SetScript("OnLeave", hideSelf);
	bar:EnableMouse(true);
	
	bar.tooltip = self;
	
	self:RegisterEvent("PLAYER_XP_UPDATE")
	self:SetScript("OnEvent", onEvent);
	
	-- Hide Frame
	self:Hide();
end

function Addon.SetupXPBarTooltip(bar)
	-- Setup Layout
	SetupXPBarTooltipLayout(bar);
	
	-- Setup Scripts
	SetupXPBarTooltipScripts(bar);
end
