-- BottomLeft.lua
-- Creates the BottomLeft ActionBars (BottomLeft.ActionBar[1-4])

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator		= LibStub:GetLibrary("FoxFrameCreator");
local libActionBarBase		= LibStub:GetLibrary("libFoxActionBarBase");
local libFoxActionBarMain 	= LibStub:GetLibrary("libFoxActionBarMain");
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
-- Array Safety
Addon.ActionBar = Addon.ActionBar or {};
local module = Addon.ActionBar;

-- Module Locals
local namePrefix 			= "FSUI_";
local moduleName 			= "undef";
local actionBarFrameName 	= "_ActionBarFrame";
local actionBarsBaseName 	= "_ActionBar";

local actionBarRowMax 		=  3;
local actionBarButtonMax 	= 12;

local barXOffset 	= 4;
local barXOffsetB 	= 8;
local barYOffset 	= 6;
local barXOffset2 	= 0;
local barYOffset2 	= 2;

-- Function Definitions
local CreateBar;
local CreateBars;
local GetFrameName;
local GetBarName;
local SetSize;
local Resize;

local CreatePageFrame;

----------------------------------------------------------------------------------------------------

function module.Create(self, parent, modName, hasMain, rowMax, barButtonMax, isVertical, addonName, addonGroup)
	-- Variables
	moduleName = modName;
	local name = GetFrameName();
	
	-- Set to defaults as Needed
	rowMax 			= rowMax 		or actionBarRowMax;
	barButtonMax 	= barButtonMax 	or actionBarButtonMax;
	
	local self;
	--
	
	-- Create Frame
	-- Check, and Create/Setup, or just Setup
	if (_G[name] == nil) then
		self = libFrameCreator.New(name);
	else
		self = libFrameCreator.Setup(_G[name]);
	end
	
	-- Set Parent
	self:SetParent(parent);
	
	-- Setup Size and Position
	self:SetWidth(418);
	self:SetHeight(114);
	self:ClearAllPoints();
	self:SetPoint("Bottom", parent, "Bottom", 0, 0);
	
	-- Set hasMain Flag
	self.hasMain 	= hasMain 		or false;
	self.isVertical = isVertical 	or false;
	
	-- Set Row & Button Max
	if (self.rowMax ~= nil) then
		if (self.rowMax < rowMax) then
			self.rowMax 		= rowMax;
		end
	else
		self.rowMax 		= rowMax;
	end
	if (self.barButtonMax ~= nil) then
		if (self.barButtonMax < barButtonMax) then
			self.barButtonMax 	= barButtonMax;
		end
	else
		self.barButtonMax 	= barButtonMax;
	end
	
	
	-- Set Row & Button Count
	self.rowCount 		= rowMax;
	self.barButtonCount	= barButtonMax;
	
	-- CreateBars
	CreateBars(self, rowMax, barButtonMax, isVertical, addonName, addonGroup);
	
	-- Add Bars to DataTable
	Addon.BarTable = Addon.BarTable or {};
	
	local startIndex = 1;
	if (self.hasMain) then
		table.insert(Addon.BarTable, {self.Bars[1], true});
		startIndex = 2;
		self.Bars[1].barId = #Addon.BarTable;
		
		self.actionBarPageFrame = CreatePageFrame(self.Bars[1], moduleName);
	end
	
	for i = startIndex, #self.Bars do
		table.insert(Addon.BarTable, {self.Bars[i], nil});
		self.Bars[i].barId = #Addon.BarTable;
	end
	
	-- Set Methods
	self.SetSize 	= SetSize;
	self.Resize 	= Resize;
	
	-- Resize Frame
	self:SetSize();
	
	return self;
end

function GetFrameName()
	return namePrefix..moduleName..actionBarFrameName;
end
function GetBarName(i, isMain)
	local iStr = ""..i;
	if (isMain) then
		iStr = "Main";
	elseif (i < 10) then
		iStr = "0"..i;
	end
	return namePrefix..moduleName..actionBarsBaseName..iStr;
end

function CreateBars(self, rowMax, barButtonMax, isVertical, addonName, addonGroup)
	local previousBar	= nil;
	local startIndex 	= 1;
	if (self.hasMain) then
		previousBar = CreateBar(self, 1, previousBar, barXOffset2, barYOffset2, barButtonMax, true, isVertical, addonName, addonGroup);
		startIndex 	= 2;
	end
	
	if (rowMax > startIndex) or (self.Bars == nil) then
		for i = startIndex, rowMax do
			previousBar = CreateBar(self, i, previousBar, barXOffset2, barYOffset2, barButtonMax, false, isVertical, addonName, addonGroup);
			previousBar:Show();
		end
	end
	if (rowMax < self.rowMax) then
		for i = rowMax + 1, self.rowMax do
			self.Bars[i]:Hide();
		end
	end
end
function CreateBar(self, i, anchorFrame, xPad, yPad, buttonCount, isMain, isVertical, addonName, addonGroup)
	-- Get Name
	local barName 		= GetBarName(i, isMain);
	
	-- Set Position Points
	local anchorPoint 	= "TopLeft";
	local point 		= "BottomLeft";
	
	-- Safety Check
	self.Bars = self.Bars or {};
	
	orientation = (isVertical and "Vertical") or "Horizontal";
	-- Check if Bar is a Main Bar
	if (isMain) then
		-- Create/Setup Main Bar
		if (_G[barName] == nil) then
			self.Bars[i] = libFoxActionBarMain.New(barName, orientation, buttonCount, nil, addonName, addonGroup);
			--self.Bars[i] = libFoxActionBarMain.New(barName, orientation, buttonCount);
		else
			self.Bars[i] = libFoxActionBarMain.Setup(_G[barName], orientation, buttonCount, nil, addonName, addonGroup);
			--self.Bars[i] = libFoxActionBarMain.Setup(_G[barName], orientation, buttonCount);
		end
		
	-- Otherwise
	else
		-- Create/Setup Normal Bar
		if (_G[barName] == nil) then
			self.Bars[i] = libActionBarBase.New(barName, orientation, buttonCount, false, addonName, addonGroup);
			--self.Bars[i] = libActionBarBase.New(barName, orientation, buttonCount, false);
		else
			self.Bars[i] = libActionBarBase.Setup(_G[barName], orientation, buttonCount, false, addonName, addonGroup);
			--self.Bars[i] = libActionBarBase.Setup(_G[barName], orientation, buttonCount, false);
		end
	end
	
	-- If this is the First Bar
	if (i == 1) then
		-- Set Custom Positioning
		xPad 		= barXOffset;
		yPad 		= 6;
		point 		= "BottomLeft";
		anchorPoint = "BottomLeft";
		anchorFrame = self;
		if (isVertical) then
			point 		= "TopRight";
			anchorPoint = "TopRight";
		end
	elseif (isVertical) then
		point 		= "TopRight";
		anchorPoint = "TopLeft";
	end
	if (i == 1) then
		-- Set Custom Positioning
		xPad 		= barXOffset;
		yPad 		= 6;
		point 		= "BottomLeft";
		anchorPoint = "BottomLeft";
		anchorFrame = self;
		if (isVertical) then
			point 		= "TopRight";
			anchorPoint = "TopRight";
			xPad 		= -barYOffset;
			yPad 		= -barXOffset;
		end
	elseif (isVertical) then
		point 		= "TopRight";
		anchorPoint = "TopLeft";
		xPad = -barYOffset2;
		yPad = -barXOffset2;
	end
	
	
	
	-- Set Location
	self.Bars[i]:ClearAllPoints();
	self.Bars[i]:SetPoint(point, anchorFrame, anchorPoint, xPad, yPad);
	
	-- Set Background
	self.Bars[i]:SetBackdrop(nil);
	
	-- Set Parent
	self.Bars[i]:SetParent(self);
	
	-- return Bar
	return self.Bars[i];
end

function SetSize(self, usePadded)
	if (self.isVertical) then
		local bar = self.Bars[1];
		self:SetWidth((bar:GetWidth() * self.rowCount) + (barYOffset2 * (self.rowCount - 1)) + (barYOffset * 2));
		
		local height = 0;
		height = height + bar:GetHeight() + barYOffset;
		--height = height + barYOffset * 2;
		--height = height + barYOffset2 * (self.rowCount - 1);
		self:SetHeight(height);
	else
		local bar = self.Bars[1];
		if (not usePadded) then
			self:SetWidth(bar:GetWidth() + barXOffset * 2);
			
			local height = 0;
			height = height + bar:GetHeight() * self.rowCount;
			height = height + barYOffset * 2;
			height = height + barYOffset2 * (self.rowCount - 1);
			self:SetHeight(height);
		else
			-- Set Custom Positioning
			local xPad 			= barXOffsetB;
			local yPad 			= 6;
			local point 		= "BottomLeft";
			local anchorPoint 	= "BottomLeft";
			local anchorFrame 	= self;

			self.Bars[1]:ClearAllPoints();
			self.Bars[1]:SetPoint(point, anchorFrame, anchorPoint, xPad, yPad);
			self:SetWidth(bar:GetWidth() + barXOffsetB + barXOffset);
			
			local height = 0;
			height = height + bar:GetHeight() * self.rowCount;
			height = height + barYOffset * 2;
			height = height + barYOffset2 * (self.rowCount - 1);
			self:SetHeight(height);
		end
		
	end
end
function Resize(self, rowCount, buttonCount)
	-- Safety Check
	if (InCombatLockdown()) then
		return false;
	end
	
	-- Set Row & Button Count
	self.rowCount 		= rowCount;
	self.barButtonCount	= buttonCount;

	CreateBars(self, rowCount, buttonCount, self.isVertical);
	
	self:SetSize();
	
	return true;
end

function CreatePageFrame(parent, moduleName)
	local frameName 	= namePrefix..moduleName.."_ActionBarPageFrame";
	
	local function OnEvent(self, event, ...)
		local pageNumber = GetActionBarPage();
		self.label = libFrameCreator.CreateLabel(self.label, self, pageNumber, "Center", self, "Center", 0, 0, 20, 30, "Center", "Middle");
	end
	
	local self;
	
	if (_G[frameName]) then
	   self = _G[frameName];
	else
	   self = libFrameCreator.New(frameName);
	end

	--local parent = Addon.Bottom.ActionBarFrame.Bars[1];

	self:SetParent(parent);
	self:ClearAllPoints();
	self:SetPoint("Left", parent, "Right", -5, 0)
	self:SetFrameStrata("LOW");
	--self:SetPoint("Center", UIParent)
	self:SetWidth(16);
	self:SetHeight(20);
	local pageNumber = GetActionBarPage();
	self.label = libFrameCreator.CreateLabel(self.label, self, pageNumber, "Center", self, "Center", 0, 0, 20, 30, "Center", "Middle");

	self:RegisterEvent("ACTIONBAR_PAGE_CHANGED");
	self:SetScript("OnEvent", OnEvent);
	
	return self;
end

-- To Use:
--module.Create(self, parent, modName, hasMain)
--module.ActionBarFrame:
--module.ActionBarFrame 	= module.ActionBar.Create(module.ActionBarFrame, module.Controller, "Bottom", true, 4, 13);
--module.ActionBarFrame:Resize(3, 12);

--foxTest = module.Create(foxTest, UIParent, "Right", false, 3, 9, true)
--foxTest:ClearAllPoints();
--foxTest:SetPoint("Right", UIParent, "Right", 0, 0);
--module.ActionBarFrame:
--module.ActionBarFrame     = module.ActionBar.Create(module.ActionBarFrame, module.Controller, "Bottom", true, 4, 13);
--module.ActionBarFrame:Resize(3, 12);