-- Controller.lua
-- Handle the Bottom UI containing: ActionBarMain, ActionBar1, ActionBar2, ActionBarStance, XPBar, RepBar, HPBar, and the MPBar.

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator	= LibStub:GetLibrary("FoxFrameCreator");
local libFoxStatusBar 	= LibStub:GetLibrary("FoxStatusbarLib-1.0");
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
Addon.Bottom 			= Addon.Bottom or {};
Addon.Bottom.ControllerArr	= Addon.Bottom.ControllerArr or {};

Addon.StyleFrames 	= Addon.StyleFrames or {};
Addon.StyleStatBars = Addon.StyleStatBars or {};

local module 		= Addon.Bottom;
local moduleName 	= "Bottom";

local moduleFrames	= Addon.Frames;

module.VisibilityHandler 	= module.VisibilityHandler or {};
module.PositionUpdateFuncs 	= module.PositionUpdateFuncs or {};

local controllerFrameName 		= "FSUI_Bottom_Controller"

-- Set Graphics Path for StatusBars
libFoxStatusBar.SetLibPath("Interface\\Addons\\FoxSmoothUI v2\\Libraries");

-- Function Definitions
local Create;
local ShowOverlay;
local AddConfigOptions;

local GetClearFunction;
local GetShowOverlayFunction;
local AddConfigClearOptionForBar;
local AddConfigShowOptionForBar;
----------------------------------------------------------------------------------------------------

function Create()
	local name = controllerFrameName;
	local self;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name);
	else
		self = libFrameCreator.Setup(_G[name]);
	end
	
	self:SetBackdrop(nil);
	self:ClearAllPoints();
	self:SetPoint("Bottom", UIParent, "Bottom", 0, 0);
	
	self:SetScript("OnUpdate", module.ControllerArr.OnUpdate);
	self:SetScript("OnEvent", module.ControllerArr.OnEvent);
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("DISPLAY_SIZE_CHANGED");
	
	return self;
end

function module.Main()
	if (module.Loaded) then
		return;
	end
	module.Loaded = true;
	-- Create ParentFrame (Secure)
	module.Controller = Create();
	
	-- Create ActionBar Frames
	--module.ActionBarFrame 	= module.ActionBar.Create(module.ActionBarFrame, module.Controller);
	--module.ActionBarFrame 	= Addon.ActionBar.Create(module.ActionBarFrame, module.Controller, "Bottom", true);
	module.ActionBarFrame 	= Addon.ActionBar.Create(module.ActionBarFrame, module.Controller, "Bottom", true, nil, nil, nil, "FoxUI", "Bottom");
	module.ActionBarFrame:SetParent(module.Controller);
	
	module.ActionBarFrame.UpdateSec = module.ControllerArr.Update;
	
	module.StanceActionBar 	= module.StanceBar.Create(module.StanceActionBar, module.Controller, module.ActionBarFrame, "FoxUI", "Stance");
	module.StanceActionBar:SetParent(module.Controller);
	--module.StanceActionBar 	= module.StanceBar.Create(module.StanceActionBar, module.Controller, module.ActionBarFrame);
	module.PetActionBar 	= module.PetBar.Create(module.PetActionBar, module.Controller, module.ActionBarFrame, "FoxUI", "Pet");
	module.PetActionBar:SetParent(module.Controller);
	--module.PetActionBar 	= module.PetBar.Create(module.PetActionBar, module.Controller, module.ActionBarFrame);
	
	-- Create Children Frames
	module.HealthBarFrame 		= moduleFrames.HealthBar.Create(	module.HealthBarFrame, 		module.Controller, moduleName);
	module.ResourceBarFrame 	= moduleFrames.ResourceBar.Create(	module.ResourceBarFrame, 	module.Controller, moduleName);
	module.XPBarFrame 			= moduleFrames.XPBar.Create(		module.XPBarFrame, 			module.Controller, moduleName);
	module.XPBarFrame:SetParent(module.Controller);
	module.RepBarFrame 			= moduleFrames.RepBar.Create(		module.RepBarFrame, 		module.Controller, moduleName);
	Addon.SetupXPBarTooltip(module.XPBarFrame);
	module.RepBarFrame:SetParent(module.Controller);
	
	-- Create InfoBar
	module.InfoBarFrame = moduleFrames.InfoBar.New(moduleFrames.InfoBarFrame, UIParent, moduleName);
	module.InfoBarFrame:SetPoint("Bottom", UIParent, "Bottom");
	module.InfoBarFrame:SetParent(module.Controller);
	module.InfoBarFrame:Hide();
	
	module.ConfigButtonFrame = Addon.InfoFrames.ConfigButton.New();
	module.InfoBarFrame:AddFrameLeft(module.ConfigButtonFrame);
	
	local Recount = LibStub("AceAddon-3.0"):GetAddon("Recount", true);
	if (Recount) then
		module.RecountButtonFrame = Addon.InfoFrames.RecountButton.New();
		module.InfoBarFrame:AddFrameLeft(module.RecountButtonFrame);
	end
	
	-- Create Unit Info
	module.UnitInfoFrame = moduleFrames.UnitInfo.New();
	module.UnitInfoFrame:Hide();
	module.UnitInfoFrame:SetParent(module.Controller);
	
	-- Create Micro Button Controller
	Addon.CreateMicroButtonController();
	
	-- Setup Frames
	moduleFrames.HealthBar.Setup(	module.HealthBarFrame, 		module.ActionBarFrame);
	moduleFrames.ResourceBar.Setup(	module.ResourceBarFrame, 	module.ActionBarFrame);
	moduleFrames.XPBar.Setup(		module.XPBarFrame, 			module.ActionBarFrame);
	moduleFrames.RepBar.Setup(		module.RepBarFrame, 		module.ActionBarFrame);
	
	-- Add Style Info
	Addon.StyleFrames 	= Addon.StyleFrames or {};
	Addon.StyleStatBars = Addon.StyleStatBars or {};
	
	-- Fix Animation Problem
	BeginActionBarTransition 	= function() end;
	ValidateActionBarTransition = function() end;
	
	module.Controller:Hide();
	
	-- Hide Health/Resource Bar
	module.HealthBarFrame:Hide()
	module.ResourceBarFrame:Hide()
	
	-- Add Children Modules
	--Addon.BottomLeft.SetParent(module.Controller);
	--Addon.BottomRight.SetParent(module.Controller);
	--Addon.Right.SetParent(module.Controller);
	
	-- Add to Style Table
	table.insert(Addon.StyleFrames, module.ActionBarFrame);
	table.insert(Addon.StyleFrames, module.ActionBarFrame.actionBarPageFrame);
	table.insert(Addon.StyleFrames, module.StanceActionBar.bar);
	table.insert(Addon.StyleFrames, module.PetActionBar.bar);
	table.insert(Addon.StyleFrames, module.PetActionBar);
	
	table.insert(Addon.StyleFrames, Addon.MicroController);
	
	table.insert(Addon.StyleStatBars, module.HealthBarFrame);
	table.insert(Addon.StyleStatBars, module.ResourceBarFrame);
	table.insert(Addon.StyleStatBars, module.XPBarFrame);
	table.insert(Addon.StyleStatBars, module.RepBarFrame);
	
	
	table.insert(Addon.StyleFrames, module.InfoBarFrame);
	
	
	table.insert(Addon.StyleFrames,   module.UnitInfoFrame);
	table.insert(Addon.StyleStatBars, module.UnitInfoFrame.hpBar);
	table.insert(Addon.StyleStatBars, module.UnitInfoFrame.resourceBar);
end

function module.ControllerArr.PEW()
	Addon.DebugPrint("ControllerArr.PEW()");
	Addon.SetupKeybindingDataBar(module.ActionBarFrame.Bars[1], 0);
	
	module.ControllerArr.Update();
	
	--libFoxStatusBar.Update(module.HealthBarFrame);
	--libFoxStatusBar.Update(module.ResourceBarFrame);
	libFoxStatusBar.Update(module.XPBarFrame);
	libFoxStatusBar.Update(module.RepBarFrame);
	
	libFoxStatusBar.Update(module.UnitInfoFrame.hpBar);
	libFoxStatusBar.Update(module.UnitInfoFrame.resourceBar);
	
	if (C_PetBattles.IsInBattle()) then
		module.Controller:Hide();
	end
	
	-- Set the Layout
	Addon.Layout.SetLayout(Addon.db.profile.Layout);
	
	-- Set the Style
	Addon.Styler.SetStyle(Addon.db.profile.Style);
	
	if (#module.PositionUpdateFuncs > 0) then
		-- for
		for i = 1, #module.PositionUpdateFuncs, 1 do
			module.PositionUpdateFuncs[i]();
		end
	end
	
	-- Check if class is DK
	local class, classFileName = UnitClass("player")
	if (classFileName ~= "DEATHKNIGHT") then
		module.UnitInfoFrame:HideRunes();
	end
	module.UnitInfoFrame:Show();
	
	Addon.Bottom.UpdateOptions();
	
	UpdateVisibility();
end

-- Update Functions
function module.ControllerArr.Update()
	Addon.DebugPrint("BarsUpdate()");
	local self = module.Controller;
	if (InCombatLockdown()) then
		Addon.DebugPrint("In Combat Exiting for now.");

		self.NeedUpdateBars = true;
		return;
	end
	
	
	-- Update the Stance Bar
	if (module.StanceActionBar ~= nil) then
		module.StanceActionBar:UseOriginalLayout(not Addon.db.profile.RearrangeFormBar);
		module.StanceActionBar:SetActions();
		module.StanceActionBar:ResetPosition();
	end
	
	-- Update Visibility Handler
	if (module.VisibilityHandler.Update ~= nil) then
		module.VisibilityHandler.Update();
	end
	
	-- Update PositionHandler
	if (module.PositionHandler.Update ~= nil) then
		module.PositionHandler.Update();
	end
	
	--libFoxStatusBar.Update(module.HealthBarFrame);
	--libFoxStatusBar.Update(module.ResourceBarFrame);
	libFoxStatusBar.Update(module.XPBarFrame);
	libFoxStatusBar.Update(module.RepBarFrame);
	
	if (#module.PositionUpdateFuncs > 0) then
		-- for
		for i = 1, #module.PositionUpdateFuncs, 1 do
			module.PositionUpdateFuncs[i]();
		end
	end
	
end

function module.ControllerArr.UpdateHandler(forceUpdate)
	Addon.DebugPrint("Update_Bars("..tostring(forceUpdate)..")");
	local self = module.Controller;
	if ((self.NeedUpdateBars) or (forceUpdate)) then
		module.ControllerArr.Update();
	end
end
function module.ControllerArr.OnUpdate(self, elapsed)
	self.lastUpdate = self.lastUpdate or 0;
	if (self.lastUpdate > Addon.TimeBetweenUpdates) then
		module.ControllerArr.UpdateHandler(true);
		self.lastUpdate = 0;
	else
		self.lastUpdate = self.lastUpdate + elapsed;
	end
end

function module.ControllerArr.OnEvent(self, event, arg1)
	Addon.DebugPrint("BarsOnEvent("..tostring(self)..",\""..event.."\")");
	if (event == "PLAYER_REGEN_ENABLED") then
		-- Update Bars, if needed
		module.ControllerArr.UpdateHandler();
	elseif (event == "DISPLAY_SIZE_CHANGED") then
		if (not InCombatLockdown()) then
			Addon.Layout.SetLayout(Addon.db.profile.Layout);
		end
		
	else
		--if (module.ControllerArr.Update ~= nil) then
			-- Update Bars
			module.ControllerArr.Update();
		--end
	end
end

-- Override the blank Function
function Addon.GetMainActionBar()
	return module.ActionBarFrame.Bars[1];
end

function ShowOverlay(self, name)
	local backdrop = {
	   -- path to the background texture
	   bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",  
	   -- path to the border texture
	   edgeFile = "",--"Interface\\Tooltips\\UI-Tooltip-Border",
	   -- true to repeat the background texture to fill the frame, false to scale it
	   tile = true,
	   -- size (width or height) of the square repeating background tiles (in pixels)
	   tileSize = 16,
	   -- thickness of edge segments and square size of edge corners (in pixels)
	   edgeSize = 12,
	   -- distance from the edges of the frame to those of the background texture (in pixels)
	   insets = {
		  left 		= 0,
		  right 	= 0,
		  top 		= 0,
		  bottom 	= 0
	   }
	}
	self.overlay:SetBackdrop(backdrop);
	self.overlay:SetFrameStrata("Dialog");
	self.overlay:SetBackdropColor(1, 0.1, 0.1, 1);
	self.overlay.label1:SetText(name);
	if (self.overlay:IsShown()) then
		self.overlay:Hide();
	else
		self.overlay:Show();
	end
end
function AddConfigOptions()
	local bars = module.ActionBarFrame.Bars;
	
	local Config = Addon.Config;
	
	AddConfigClearOptionForBar(bars, 1, "Main Bar", 0);
	AddConfigShowOptionForBar(bars,  1, "Main Bar", 0);
	Config.SetActionBarOptionsCount(1);
	
	for i = 2, #bars do
		Config.SetActionBarOptionsCount(Config.GetActionBarOptionsCount() + 1);
		local name = "Bar "..Config.GetActionBarOptionsCount();
		AddConfigClearOptionForBar(bars, i, name, -1);
		AddConfigShowOptionForBar(bars,  i, name, -1);
	end
end

Addon.Bars = Addon.Bars or {}
Addon.Bars.ShowOverlay = ShowOverlay;
------------------------------------------
-- Config For Each Controller
------------------------------------------
local Config = Addon.Config;
-- Clear Template Function
function GetClearFunction(bars, i, useKeys)
	local funcStr = [[
		local barName = "]].. bars[i]:GetName()..[["; 
		local self = _G[barName]; 
		local i 		= ]]..(i or -1)..[[;
		local AddonName 	= "]]..AddonName..[["; 
		local AddonVersion	= "]]..AddonVersion..[["; 
		local Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
		--do stuff 
		Addon.DebugPrint("ClearBarHotkeys("..(self:GetName() or "nil")..");");
		-- 2nd Bar
		if (]]..useKeys..[[) then
			Addon.SetupKeybindingDataBar(self, ]]..useKeys..[[);
		else
			Addon.SetupKeybindingDataBar(self);
		end
	]]
	local func, errorStr = loadstring(funcStr);
	if (errorStr) then
		error(errorStr);
	end
	
	return func;
end
-- Show Overlay Template Function
function GetShowOverlayFunction(bars, i)
	local funcStr = [[
		local barName 	= "]].. bars[i]:GetName()..[[";
		local self 		= _G[barName]; 
		local i 		= ]]..(i or -1)..[[;
		local AddonName 	= "]]..AddonName..[["; 
		local AddonVersion	= "]]..AddonVersion..[["; 
		local Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
		--do stuff 
		Addon.DebugPrint("ShowBar("..(self:GetName() or "nil")..");");
		Addon.Bars.ShowOverlay(self, (self.BarNameData or "Bar "..i));
	]]
	local func, errorStr = loadstring(funcStr);
	if (errorStr) then
		error(errorStr);
	end
	
	return func;
end
function AddConfigClearOptionForBar(bars, i, customName, useKeys)
	if (customName) then
		bars[i].BarNameData = customName;
	end
	
	local option = {
		type = 'execute',
		name = 'Clear '..(customName or "Bar "..i),
		desc = 'Clear '..(customName or "Bar "..i),
		
		func = GetClearFunction(bars, i, useKeys),
		order = Config.GetActionBarOptionsOffset();
	}
	
	
	local options = Addon.GetOptions();
	
	options.args.mainOptions.args.barHotkeyDefaults.args["ClearBar"..i] = option;
	
	-- Update
	LibStub("AceConfigRegistry-3.0"):NotifyChange(AddonName.."-"..AddonVersion);
	
	Config.SetActionBarOptionsOffset(Config.GetActionBarOptionsOffset() + 1);
end
function AddConfigShowOptionForBar(bars, i, customName)
	if (customName) then
		bars[i].BarNameData = customName;
	end
	option = {
		type  = 'execute',
		name  = 'Show '..(customName or "Bar "..i),
		desc  = 'Show '..(customName or "Bar "..i),
		func  = GetShowOverlayFunction(bars, i),
		order = Config.GetActionBarOptionsOffset();
	}
	
	local options = Addon.GetOptions();
	options.args.mainOptions.args.barHotkeyDefaults.args["ShowBar"..i] = option;
	
	-- Update
	LibStub("AceConfigRegistry-3.0"):NotifyChange(AddonName.."-"..AddonVersion);
	
	Config.SetActionBarOptionsOffset(Config.GetActionBarOptionsOffset() + 1);
end
------------------------------------------

-- Add Following Lines to Config Setup in each Controller
--AddConfigClearOptionForBar(bars, i, customName);
--AddConfigShowOptionForBar(bars, i, customName);



function Addon.Bottom.UpdateOptions()
	if (not module.Loaded) then
		return;
	end
	
	if (Addon.MicroController) then
		if (Addon.db.profile.ShowBlizzOptions) then
			Addon.MicroController:Show();
		else
			Addon.MicroController:Hide();
		end
	end
	
	if (module.XPBarFrame) then
		module.XPBarFrame:SetAttribute("overrideVisibility", Addon.db.profile.HideXPBar)
		if (Addon.db.profile.HideXPBar) then
			module.XPBarFrame:Hide();
		else
			module.XPBarFrame:Show();
		end
	end
	
	-- Handle the Resource Bar
	if (module.UnitInfoFrame) then
		if (Addon.db.profile.ShowResourceBar) then
			module.UnitInfoFrame:Show();
		else
			module.UnitInfoFrame:Hide();
		end
	end
	
	--Addon.BottomLeft.TimeFrame:UseTime(Addon.db.profile.useLocalTime);
	if (Addon.BottomLeft.TimeFrame) then
		Addon.BottomLeft.TimeFrame:OnUpdate(3.0);
	end
	
	-- Update Frame Position
	if (module.ControllerArr) then
		module.ControllerArr.Update();
	end
	
	if (Addon.BottomLeft.CurrencyBarFrame) then
		if (Addon.Bottom.GoldTrackerFrame) then
			Addon.BottomLeft.CurrencyBarFrame:SetGoldTooltip(Addon.Bottom.GoldTrackerFrame);
		end
	end
end


table.insert(Addon.InitializationFunctions, module.Main);
table.insert(Addon.Module_PEW, 				module.ControllerArr.PEW);
--table.insert(Addon.ConfigOptionsFuncs, 1, AddConfigOptions);
table.insert(Addon.ConfigOptionsFuncs, AddConfigOptions);

--table.insert(Addon.Module_PEW, module.ChatBox.Setup);
--table.insert(Addon.DisableFunctions, module.ChatBox.ResetChatWindow);
--table.insert(Addon.EnableFunctions, module.ChatBox.Setup);

-- For Debug use:
--module.Main()

