-- StatBars.lua
-- Handles the StatBar Creation

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
--...
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
Addon.Bottom 	= Addon.Bottom or {};

-- Safety Check
Addon.Bottom.PositionHandler 	= Addon.Bottom.PositionHandler or {};
local moduleParent 				= Addon.Bottom;
local module 					= Addon.Bottom;

-- Safety Check
-- Safety Check
Addon.InitializationFunctions 	= Addon.InitializationFunctions or {};

-- Secure Snippets
-- StatBar Position Update Snippet
module.UpdateStatBarPositionSnippet = [[
	local InfoBarRef 	= self:GetFrameRef("InfoBarRef");
	
	local XPBarRef 		= self:GetFrameRef("XPBarRef");
	local RepBarRef 	= self:GetFrameRef("RepBarRef");
	local MainFrameRef 	= self:GetFrameRef("MainFrameRef");
	
	if (InfoBarRef:IsShown()) then
		MainFrameRef:SetPoint("BOTTOM", InfoBarRef, "TOP", 0, 0);
	else
		MainFrameRef:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 0);
	end
	control:CallMethod("UpdateSec", self);
	
	if (RepBarRef:IsShown()) then
		self:SetPoint(			"BOTTOM", 	RepBarRef, 		"TOP", 		0, 0);
		if (XPBarRef:IsShown()) then
			RepBarRef:SetPoint(	"BOTTOM", 	XPBarRef, 		"TOP", 		0, 0);
			XPBarRef:SetPoint(	"BOTTOM", 	MainFrameRef, 	"BOTTOM", 	0, 0);
		else
			RepBarRef:SetPoint(	"BOTTOM", 	MainFrameRef, 	"BOTTOM", 	0, 0);
		end
	elseif (XPBarRef:IsShown()) then
		XPBarRef:SetPoint(		"BOTTOM", 	MainFrameRef, 	"BOTTOM", 	0, 0);
		self:SetPoint(			"BOTTOM", 	XPBarRef, 		"TOP", 		0, 0);
	else
		self:SetPoint(			"BOTTOM", 	MainFrameRef, 	"BOTTOM", 	0, 0);
	end
]]

-- Pet/Stance Bar Position Update Snippet
module.UpdatePetStanceBarPosition = [[
	local PetBarRef 	= self:GetFrameRef("PetBarRef");
	local StanceBarRef 	= self:GetFrameRef("StanceBarRef");
	
	if (StanceBarRef:IsShown()) then
		if (PetBarRef:IsShown()) then
			StanceBarRef:SetPoint(	"BOTTOMLEFT", PetBarRef, 	"TOPLEFT", 0, 0);
			PetBarRef:SetPoint(		"BOTTOMLEFT", self, 		"TOPLEFT", 0, 0);
		else
			StanceBarRef:SetPoint(	"BOTTOMLEFT", self, 		"TOPLEFT", 0, 0);
		end
	elseif (PetBarRef:IsShown()) then
		PetBarRef:SetPoint(			"BOTTOMLEFT", self, 		"TOPLEFT", 0, 0);
	end
]]


-- Function Definitions
--...
----------------------------------------------------------------------------------------------------

function module.Main()
	
	-- Setup Stat Bars
	-- Setup the Position State Driver for the Stat Bars
	local InfoBarRef 	= "InfoBarRef"
	local XPBarRef 		= "XPBarRef"
	local RepBarRef 	= "RepBarRef";
	local MainFrameRef 	= "MainFrameRef";
	
	moduleParent.ActionBarFrame:SetFrameRef(InfoBarRef, 	moduleParent.InfoBarFrame);
	
	moduleParent.ActionBarFrame:SetFrameRef(XPBarRef, 		moduleParent.XPBarFrame);
	moduleParent.ActionBarFrame:SetFrameRef(RepBarRef, 		moduleParent.RepBarFrame);
	moduleParent.ActionBarFrame:SetFrameRef(MainFrameRef, 	moduleParent.Controller);
	
	moduleParent.ActionBarFrame:SetAttribute("FoxUpdatePositionStatBars", module.UpdateStatBarPositionSnippet);
	
	-- Show the Exp and Rep Bars
	moduleParent.XPBarFrame:Show();
	moduleParent.RepBarFrame:Show();
	
	
	-- Setup the Position State Driver for the Stance and Pet Bars
	local PetBarRef 	= "PetBarRef";
	local StanceBarRef 	= "StanceBarRef";
	
	moduleParent.ActionBarFrame:SetFrameRef(PetBarRef, 		moduleParent.PetActionBar);
	moduleParent.ActionBarFrame:SetFrameRef(StanceBarRef, 	moduleParent.StanceActionBar);
	
	-- Set the Position State Driver
	moduleParent.ActionBarFrame:SetAttribute("FoxUpdatePetStancePosition", module.UpdatePetStanceBarPosition);
	
	-- Show PetBar
	moduleParent.PetActionBar:Hide();
	moduleParent.PetActionBar:Show();
	moduleParent.PetActionBar:Execute(moduleParent.PetActionBar:GetAttribute("updateState"));
	
	
	-- Handle Vehicle Leave Button Position
	module.MoveVehicleLeaveButton(MainMenuBarVehicleLeaveButton);
	MainMenuBarVehicleLeaveButton:SetScript("onshow", module.MoveVehicleLeaveButton);
end

function module.MoveVehicleLeaveButton(self)
	self:SetParent(UIParent)
	self:ClearAllPoints();
	self:SetPoint("BottomRight", FoxActionBarFrame, "TopRight", -6, 2);
end

function module.PositionHandler.Update()
	if (not moduleParent.Loaded) then
		print("not loaded");
		return;
	end
	
	-- Update Width
	local width = moduleParent.ActionBarFrame:GetWidth() + 4;
	if (moduleParent.HealthBarFrame) then
		if (moduleParent.HealthBarFrame:IsShown()) then
			width = width + moduleParent.HealthBarFrame:GetWidth() * 2;
		end
	end
	moduleParent.Controller:SetWidth(width);
	
	-- Position Blizz Micro Menu Bar
	if (Addon.MicroController ~= nil) then
		if (Addon.db.profile.ShowBlizzOptions) then
			local microBarState = 0;
			if (Addon.BottomRight ~= nil) then
				if (Addon.BottomRight.Controller ~= nil) then
					if (Addon.BottomRight.Controller:IsShown()) then
						microBarState = 1;
					end
				end
			end

			-- Middle Position
			if (microBarState == 1) then
				Addon.MicroController:Show();
				Addon.MicroController:ClearAllPoints();
				Addon.MicroController:SetPoint("BottomRight", FSUI_BagFrame, "TopRight", 	0, 0);
				
			-- Default Position (BottomLeft)
			else
				Addon.MicroController:Show();
				Addon.MicroController:ClearAllPoints();
				Addon.MicroController:SetPoint("BottomLeft", moduleParent.Controller, "BottomRight", 	0, 0);
			end
			
		else
			Addon.MicroController:Hide();
		end
	end
end

table.insert(Addon.InitializationFunctions, module.Main);