-- BottomLeft.lua
-- Creates the BottomLeft ActionBars (BottomLeft.ActionBar[1-4])

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator			= LibStub:GetLibrary("FoxFrameCreator");
local libActionBarBase			= LibStub:GetLibrary("libFoxActionBarBase");
local libFoxActionBarStance 	= LibStub:GetLibrary("libFoxActionBarStance");
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
-- Array Safety
Addon.Bottom = Addon.Bottom or {};
local module = Addon.Bottom;

-- Array Setup
module.StanceBar 		= {};
module.StanceBar.Bars 	= module.StanceBar.Bars or {};

-- Module Locals
local actionBarFrameName 	= "FSUI_Bottom_StanceBarFrame";
local actionBarsBaseName 	= "FSUI_Bottom_StanceBar";
local actionBarRowCount 	= 1;
local actionBarButtonCount 	= 12;


-- Function Definitions
local CreateBar;
local CreateBars;

local UseOriginalLayout;
local SetActions;
local ResetPosition;
local doNotShow;

----------------------------------------------------------------------------------------------------



function module.StanceBar.Create(self, parent, anchor, addonName, addonGroup)
	-- Variables
	local name = actionBarFrameName;
	local self;
	--
	
	-- Create Frame
	-- Check, and Create/Setup, or just Setup
	if (_G[name] == nil) then
		self = libFrameCreator.New(name, true);
	else
		self = libFrameCreator.Setup(_G[name], true);
	end
	-- Set Parent
	self:SetParent(parent);
	
	-- Setup Size and Position
	--self:SetWidth(418);
	--self:SetHeight(114);
	self:ClearAllPoints();
	self:SetPoint("BottomLeft", anchor, "TopLeft", 0, 0);
	self:SetBackdrop(nil);
	
	-- CreateBars
	self.bar = CreateBar(self, addonName, addonGroup);
	
	-- Add Methods
	self.UseOriginalLayout 	= UseOriginalLayout;
	self.SetActions 		= SetActions;
	self.ResetPosition 		= ResetPosition;
	self.doNotShow 			= doNotShow;
	
	-- Add Bars to DataTable
	Addon.BarTable = Addon.BarTable or {};
	table.insert(Addon.BarTable, {self.bar, true});
	
	--for i = 2, #module.StanceBar.Bars do
		--table.insert(Addon.BarTable, {module.StanceBar.Bars[i], nil});
	--end
	
	return self;
end

function CreateBar(frame, addonName, addonGroup)
	local barName 		= actionBarsBaseName;
	local point 		= "BottomLeft";
	local anchorPoint 	= "BottomLeft";
	local xPad 			= 0;
	local yPad 			= 0;
	local self;
	
	if (_G[barName] == nil) then
		self = libFoxActionBarStance.New(barName, nil, nil, true, addonName, addonGroup);
	else
		self = libFoxActionBarStance.Setup(_G[barName], nil, nil, true, addonName, addonGroup);
	end
	-- Set Location
	self:SetPoint(point, frame, anchorPoint, xPad, yPad);
	
	-- Set Background
	--self:SetBackdrop(nil);
	
	self:SetParent(frame);
	
	-- return Bar
	return self;
end

function SetActions(self)
	self.bar:SetActions();
end
function ResetPosition(self)
	self.bar:ResetPosition();
end
function UseOriginalLayout(self, value)
	self.bar.UseOriginalLayout = value;
end
function doNotShow(self)
	return self.bar.doNotShow;
end




--module.ActionBar.Create(self, parent)
--module.StanceActionBar 	= module.StanceBar.Create(module.StanceActionBar, module.Controller, module.ActionBarFrame);