-- Controller.lua
-- Handles the Bottom UI's Visibility

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFoxStatusBar 			= LibStub:GetLibrary("FoxStatusbarLib-1.0");
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
Addon.Bottom 	= Addon.Bottom or {};

-- Safety Check
Addon.Bottom.VisibilityHandler 	= Addon.Bottom.VisibilityHandler or {};
local moduleParent 				= Addon.Bottom;
local module 					= Addon.Bottom;

-- Secure Snippets
-- Setup Main Visibility
module.VisibilityConditional = "[vehicleui] Vehicle; [possessbar] Possessbar; [overridebar] Overridebar; [petbattle] PetBattle; [pet] Pet; Normal;";
module.VisibilitySnippet = [[
	-- arguments: self, stateid, newstate
	print(newstate);
	
	local vehicle 		= (newstate == "Possessbar") or (newstate == "Overridebar") or (newstate == "Vehicle");
	local petBattle 	= (newstate == "PetBattle");
	
	local vehicleRef 	= self:GetFrameRef("vehicleRef");
	local petBattleRef 	= self:GetFrameRef("petBattleRef");
	local ActionBarFrameRef = self:GetFrameRef("ActionBarFrameRef");
	
	if (vehicle or petBattle) then
		-- Hide My UI
		if (self:IsShown()) then
			self:Hide();
		end
	else
		-- Show My UI
		if (not self:IsShown()) then
			self:Show();
		end
	end
	
	if (vehicle) then
		-- Show Vehicle UI
		if (not vehicleRef:IsShown()) then
			vehicleRef:Show();
		end
	else
		-- Hide Vehicle UI
		if (vehicleRef:IsShown()) then
			vehicleRef:Hide();
		end
	end
	
	if (petBattle) then
		-- [!Insecure] Show Pet UI
		petBattleRef:CallMethod("Show");
	else
		-- [!Insecure] Hide Pet UI
		petBattleRef:CallMethod("Hide");
	end
	
]]

-- StatBar Show and Hide Snippet
module.StatBarShowHideSnippet = [[
	local ActionBarFrameRef = self:GetFrameRef("ActionBarFrameRef");
	local value = ActionBarFrameRef:GetAttribute("FoxUpdatePositionStatBars");
	if (value ~= nil) then
		ActionBarFrameRef:RunAttribute("FoxUpdatePositionStatBars");
	end
]]

-- Pet/Stance Show and Hide Snippet
module.PetStanceBarShowHideSnippet = [[
	local ActionBarFrame = self:GetFrameRef("ActionBarFrameRef")
	if (ActionBarFrame:GetAttribute("FoxUpdatePetStancePosition") ~= nil) then
		ActionBarFrame:RunAttribute("FoxUpdatePetStancePosition");
	end
]]


-- Function Definitions
local UpdateVisibility;
----------------------------------------------------------------------------------------------------

function module.Main()
	moduleParent.Controller:Hide();
	
	-- Add Reference for Show/Hide Function
	local ActionBarFrameRef = "ActionBarFrameRef";
	
	moduleParent.XPBarFrame:SetFrameRef(ActionBarFrameRef, 	moduleParent.ActionBarFrame);
	moduleParent.RepBarFrame:SetFrameRef(ActionBarFrameRef, moduleParent.ActionBarFrame);
	moduleParent.InfoBarFrame:SetFrameRef(ActionBarFrameRef, moduleParent.ActionBarFrame);
	
	-- Set the Show and Hide Attributes
	moduleParent.XPBarFrame:SetAttribute("_onshow", module.StatBarShowHideSnippet);
	moduleParent.XPBarFrame:SetAttribute("_onhide", module.StatBarShowHideSnippet);
	
	moduleParent.RepBarFrame:SetAttribute("_onshow", module.StatBarShowHideSnippet);
	moduleParent.RepBarFrame:SetAttribute("_onhide", module.StatBarShowHideSnippet);
	
	moduleParent.InfoBarFrame:SetAttribute("_onshow", module.StatBarShowHideSnippet);
	moduleParent.InfoBarFrame:SetAttribute("_onhide", module.StatBarShowHideSnippet);

	-----------------------------------------
	-- Setup Bottom Controller Visibility
	-----------------------------------------
	local vehicleRef 	= "vehicleRef";
	local petBattleRef 	= "petBattleRef";
	
	moduleParent.Controller:SetFrameRef(vehicleRef, 	OverrideActionBar);
	moduleParent.Controller:SetFrameRef(petBattleRef, 	PetBattleFrame);
	
	moduleParent.Controller:SetAttribute("_onstate-currentState", 	module.VisibilitySnippet);
	RegisterStateDriver(moduleParent.Controller, "currentState", 	module.VisibilityConditional);
	-----------------------------------------
	
	
	-----------------------------------------
	-- Pet Bar
	-----------------------------------------
	-- Add Reference for Show/Hide Function
	local ActionBarFrameRef = "ActionBarFrameRef";
	moduleParent.PetActionBar:SetFrameRef(ActionBarFrameRef, moduleParent.ActionBarFrame);

	-- Set Show and Hide Snippet
	moduleParent.PetActionBar:SetAttribute("_onshow", module.PetStanceBarShowHideSnippet);
	moduleParent.PetActionBar:SetAttribute("_onhide", module.PetStanceBarShowHideSnippet);
	
	-- Show PetBar
	moduleParent.PetActionBar:Hide();
	moduleParent.PetActionBar:Show();
	moduleParent.PetActionBar:Execute(moduleParent.PetActionBar:GetAttribute("updateState"));
	-----------------------------------------
	
	
	-- Show StanceBar as Needed
	if (moduleParent.StanceActionBar:doNotShow() == false) then
		moduleParent.StanceActionBar:Show();
	end
	
	
	-- Register Events
	moduleParent.Controller:RegisterEvent("PLAYER_XP_UPDATE");
	moduleParent.Controller:RegisterEvent("DISABLE_XP_GAIN");
	moduleParent.Controller:RegisterEvent("ENABLE_XP_GAIN");
	moduleParent.Controller:RegisterEvent("UPDATE_FACTION");
	
	
	UpdateVisibility();
end

function module.OnEnable()
	UpdateVisibility();
end
function module.OnDisable()
	UpdateVisibility();
end

function UpdateVisibility()
	if (Addon.db.profile.Enabled) then
		-- Hide Old UI
		MainMenuBar:Hide();
		MultiBarRight:Hide();
		MultiBarLeft:Hide();
		
		if ((C_PetBattles.IsInBattle()) and (moduleParent.Controller:IsShown())) then
			moduleParent.Controller:Hide();
			return;
		elseif (UnitHasVehicleUI("player")) then
			moduleParent.Controller:Hide();
			
			return;
			
		elseif (C_PetBattles.IsInBattle()) then
			return;
		else
			moduleParent.Controller:Show();
			
			-- Show Main Frame
			Addon.MainFrame:Show();
		end
	else
		moduleParent.Controller:Hide();
	
		-- Hide Main Frame
		Addon.MainFrame:Hide();
		
		-- Show Old UI
		MainMenuBar:Show();
		MultiBarRight:Show();
		MultiBarLeft:Show();
	end
	
	
	-- Handle ExpBar
	if (moduleParent.XPBarFrame ~= nil) then
		if ((IsXPUserDisabled()) or (UnitLevel("player") == GetMaxPlayerLevel())) then
			moduleParent.XPBarFrame:Hide();
			
		elseif (moduleParent.XPBarFrame:GetAttribute("overrideVisibility")) then
				moduleParent.XPBarFrame:Hide();
			
		else
			moduleParent.XPBarFrame:Show();
			libFoxStatusBar.Update(moduleParent.XPBarFrame);
		end
	end
	
	-- Handle RepBar
	if (moduleParent.RepBarFrame ~= nil) then
		if (GetWatchedFactionInfo() == nil) then
			moduleParent.RepBarFrame:Hide();
		else
			moduleParent.RepBarFrame:Show();
			libFoxStatusBar.Update(moduleParent.RepBarFrame);
		end
	end
	
	if ((C_PetBattles.IsInBattle()) and (moduleParent.Controller:IsShown())) then
		moduleParent.Controller:Hide();
	end
end

function module.VisibilityHandler.Update()
	UpdateVisibility();
end

table.insert(Addon.InitializationFunctions, module.Main);
table.insert(Addon.EnableFunctions, 		module.OnEnable);
table.insert(Addon.DisableFunctions, 		module.OnDisable);

--module.Main()





	
--[[

function Addon.MoveVehicleLeaveButton(self)
	self:SetParent(UIParent)
	self:ClearAllPoints();
	self:SetPoint("BottomRight", FoxActionBarFrame, "TopRight", -6, 2);
end

--]]