-- BottomLeft.lua
-- Creates the BottomLeft ActionBars (BottomLeft.ActionBar[1-4])

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator	= LibStub:GetLibrary("FoxFrameCreator");
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
-- Array Safety
Addon.BottomLeft = Addon.BottomLeft or {};
local module = Addon.BottomLeft;

module.ChatBox 	= module.ChatBox or {};
Addon.Options 				= Addon.Options or {};

-- Function Definitions
local FocusOnShow;
local FocusOnHide;
local OnShow;

----------------------------------------------------------------------------------------------------

function module.ChatBox.Create(self, anchorPoint, parent)
	local name = "FSUI_ChatBoxFrame";
	if (_G[name] == nil) then
	   self = libFrameCreator.New(name);
	else
	   self = libFrameCreator.Setup(_G[name]);
	end
	self:SetFrameStrata("background");
	self:SetFrameLevel(0);
	self:SetPoint("Bottom", anchorPoint, "Top", 0, 0);
	self:SetPoint("Left", UIParent, "Left", 0, 0);
	self:SetWidth(100);
	self:SetHeight(100);
	self:SetBackdrop(nil);
	self:SetParent(parent);
	self:SetScript("OnShow", OnShow);
	
	name = "FSUI_ChatBoxFrame_Main";
	if (_G[name] == nil) then
	   self.ChatBox = libFrameCreator.New(name);
	else
	   self.ChatBox = libFrameCreator.Setup(_G[name]);
	end
	self.ChatBox:SetParent(self);

	-- Create Secondary Currency Frame
	name = "FSUI_ChatBoxFrame_LeftSide";
	if (_G[name] == nil) then
	   self.ChatLeftSide = libFrameCreator.New(name);
	else
	   self.ChatLeftSide = libFrameCreator.Setup(_G[name]);
	end
	self.ChatLeftSide:SetParent(self);
	
	-- Create Secondary Currency Frame
	name = "FSUI_ChatBoxFrame_EditBox";
	if (_G[name] == nil) then
	   self.EditBox = libFrameCreator.New(name);
	else
	   self.EditBox = libFrameCreator.Setup(_G[name]);
	end
	self.EditBox:SetParent(self);
	
	-- Setup ChatBox LeftSide
	self.ChatLeftSide:SetFrameStrata("background");
	self.ChatLeftSide:SetPoint("BottomLeft", self, "BottomLeft", 0, 0);
	self.ChatLeftSide:SetWidth(  36);
	self.ChatLeftSide:SetHeight(164);
	
	-- Setup ChatBox EditBox
	self.EditBox:SetFrameStrata("background");
	self.EditBox:SetPoint("BottomLeft", self.ChatLeftSide, "BottomRight", 0, 0);
	self.EditBox:SetWidth( 386);
	self.EditBox:SetHeight( 30);
	
	-- Setup ChatBox Main
	self.ChatBox:SetFrameStrata("background");
	self.ChatBox:SetWidth( 382);
	self.ChatBox:SetHeight(134);
	-- Set Position
	self.ChatBox:SetPoint("Left", 	self.ChatLeftSide, 	"Right", 0, 0);
	self.ChatBox:SetPoint("Bottom", self.EditBox, 		"Top", 	 0, 0);
	
	self:Hide();
	
	return self;
end

local FCF_RestorePositionAndDimensionsOrig;
local function FCF_RestorePositionAndDimensionsNew()
	
end
function module.ChatBox.Setup(self)
	local self = module.ChatBoxFrame;
	if (self == nil) then
		return;
	end
	
	-- Setup Custom Editbox Size
	self.EditBox:SetHeight(25);
	self.EditBox:SetWidth(386);

	-- Setup Custom Main Size
	self.ChatBox:SetWidth( 386);
	self.ChatBox:SetHeight(173);

	-- Setup Custom Side Size
	self.ChatLeftSide:SetHeight(198);
	self.ChatLeftSide:SetWidth(  34);
	
	-- Loop through the ChatFrames
	for i = 1, NUM_CHAT_WINDOWS do
		-- Clear EditBox Normal Textures
		--_G["ChatFrame"..i.."EditBox"]:SetBackdrop(nil);
		--_G["ChatFrame"..i.."EditBox"]:SetAlpha(0.75);
		--_G["ChatFrame"..i.."EditBox"]:SetBackdropColor(1, 1, 1, 1);
		
		_G["ChatFrame"..i.."EditBoxLeft"]:Hide();
		_G["ChatFrame"..i.."EditBoxMid"]:Hide();
		_G["ChatFrame"..i.."EditBoxRight"]:Hide();
		
		local alpha = 0;
		_G["ChatFrame"..i.."EditBoxFocusLeft"]:SetAlpha(alpha);
		_G["ChatFrame"..i.."EditBoxFocusMid"]:SetAlpha(alpha);
		_G["ChatFrame"..i.."EditBoxFocusRight"]:SetAlpha(alpha);
		
		--- Clear ChatBox Graphics
		sections = {"Background", "TopRightTexture", "TopLeftTexture", "BottomRightTexture", "BottomLeftTexture", "TopTexture", "BottomTexture", "RightTexture", "LeftTexture" };
		for j = 1, #sections do
			--_G["ChatFrame"..i..sections[j]]:SetTexture(nil);
			_G["ChatFrame"..i..sections[j]]:Hide();
		end
		
		-- Clear ChatBox Button Frame Graphics
		for j = 1, #sections do
			--_G["ChatFrame"..i.."ButtonFrame"..sections[j]]:SetTexture(nil);
			_G["ChatFrame"..i.."ButtonFrame"..sections[j]]:Hide();
		end
		
		
		-- Set Chat EditBox Size
		if (Addon.Options.UsingPrat()) then
			_G["ChatFrame"..i.."EditBox"]:SetWidth(386);
			_G["ChatFrame"..i.."EditBox"]:SetHeight(24);
			_G["ChatFrame"..i.."EditBox"]:ClearAllPoints();
			_G["ChatFrame"..i.."EditBox"]:SetPoint("TopLeft", _G["FSUI_ChatBoxFrame_EditBox"], "TopLeft", 0, 0);
			_G["ChatFrame"..i.."EditBox"]:SetWidth(386);
			_G["ChatFrame"..i.."EditBox"]:SetHeight(24);
		else
			_G["ChatFrame"..i.."EditBox"]:SetWidth(390);
			_G["ChatFrame"..i.."EditBox"]:SetHeight(24);
			_G["ChatFrame"..i.."EditBox"]:ClearAllPoints();
			_G["ChatFrame"..i.."EditBox"]:SetPoint("TopLeft", _G["FSUI_ChatBoxFrame_EditBox"], "TopLeft", 0, 0);
		end
		
		-- Set ChatBox Button Frame Size
		_G["ChatFrame"..i.."ButtonFrame"]:SetHeight(124);
		_G["ChatFrame"..i.."ButtonFrame"]:ClearAllPoints();
		_G["ChatFrame"..i.."ButtonFrame"]:SetPoint("BottomRight", _G["ChatFrame"..i], "BottomLeft", -8, 0);
		_G["ChatFrame"..i.."ButtonFrame"]:SetHeight(124);
		
		-- Hook OnShow & OnHide Scripts
		_G["ChatFrame"..i.."EditBox"]:HookScript("OnEditFocusGained", FocusOnShow);
		_G["ChatFrame"..i.."EditBox"]:HookScript("OnEditFocusLost", FocusOnHide);
	end

	-- Save ChatFrame1 Position
	ChatFrame1_Position = ChatFrame1:GetPoint(1);
	
	-- Set ChatFrame Position
	ChatFrame1:ClearAllPoints();
	--ChatFrame1:SetPoint("BottomLeft", self.ChatBox, "BottomLeft", 3, 8);
	ChatFrame1:SetPoint("BottomLeft", self.ChatBox, "BottomLeft", 5, 8);
	ChatFrame1:SetWidth(370);
	ChatFrame1:SetHeight(160);
	--ChatFrame1:SetParent(self.ChatBox);
	
	local chatFrame = ChatFrame1;
	SetChatWindowSavedDimensions(chatFrame:GetID(), chatFrame:GetWidth(), chatFrame:GetHeight());
	
	-- Hook Annoying Chat Positioning Function
	if (FCF_RestorePositionAndDimensionsOrig == nil) then
		FCF_RestorePositionAndDimensionsOrig 	= FCF_RestorePositionAndDimensions;
		FCF_RestorePositionAndDimensions 		= FCF_RestorePositionAndDimensionsNew;
	end
	ChatFrame1:SetUserPlaced(true);
	
	--- Set Dock Position
	GeneralDockManager:ClearAllPoints();
	GeneralDockManager:SetPoint("BottomLeft", ChatFrame1, "TopLeft", 0, 6);
	GeneralDockManager:SetPoint("BottomRight", ChatFrame1, "TopRight", 0, 6);
	
	-- Edit The Combat Log Quick Button Frame
	CombatLogQuickButtonFrame_Custom:SetPoint("BottomRight", ChatFrame2, "TopRight",  6, 2)
	CombatLogQuickButtonFrame_Custom:SetPoint("BottomLeft",  ChatFrame2, "TopLeft",  -1, 2)
	
	self:Show();
end

function OnShow(self)
	self:SetFrameLevel(0);
end

-- Fix ChatFrames and Buttons
function FocusOnShow()
	if (not Addon:IsEnabled()) then
		return;
	end
	-- AddPost Hook for ChatGainedFocus
	--module.ChatBoxFrame.EditBox:SetBackdropColor(0, 0, 0, 1);
end
function FocusOnHide()
	if (not Addon:IsEnabled()) then
		return;
	end
	-- AddPost Hook for ChatLostFocus
	--module.ChatBoxFrame.EditBox:SetBackdropColor(0.1, 0.1, 0.1, 0.6);
end

function module.ChatBox.ResetChatWindow()
	-- Loop through the ChatFrames
	for i = 1, NUM_CHAT_WINDOWS do
		
		local alpha = 1;
		_G["ChatFrame"..i.."EditBoxFocusLeft"]:SetAlpha(alpha);
		_G["ChatFrame"..i.."EditBoxFocusMid"]:SetAlpha(alpha);
		_G["ChatFrame"..i.."EditBoxFocusRight"]:SetAlpha(alpha);
		
		--- Clear ChatBox Graphics
		sections = {"Background", "TopRightTexture", "TopLeftTexture", "BottomRightTexture", "BottomLeftTexture", "TopTexture", "BottomTexture", "RightTexture", "LeftTexture" };
		for j = 1, #sections do
			_G["ChatFrame"..i..sections[j]]:Show();
		end
		-- Clear ChatBox Button Frame Graphics
		for j = 1, #sections do
			_G["ChatFrame"..i.."ButtonFrame"..sections[j]]:Show();
		end
		
		if (Addon.Options.UsingPrat()) then
			_G["ChatFrame"..i.."EditBox"]:ClearAllPoints();
			_G["ChatFrame"..i.."EditBox"]:SetPoint("TopLeft", _G["ChatFrame"..i], "BottomLeft", 0, -8);
			_G["ChatFrame"..i.."EditBox"]:SetPoint("TopRight", _G["ChatFrame"..i], "BottomRight", 0, -8);
		else
			_G["ChatFrame"..i.."EditBoxLeft"]:Show();
			_G["ChatFrame"..i.."EditBoxMid"]:Show();
			_G["ChatFrame"..i.."EditBoxRight"]:Show();
		
			_G["ChatFrame"..i.."EditBox"]:SetWidth(380);
			_G["ChatFrame"..i.."EditBox"]:SetHeight(32);
		end
	end
	
	-- ChatFrame1
	ChatFrame1:SetPoint("BottomLeft", UIParent, "BottomLeft", 35, 150);

	-- GeneralDockManager
	GeneralDockManager:SetPoint("BottomLeft", ChatFrame1, "TopLeft", 0, 6);
	GeneralDockManager:SetPoint("BottomRight", ChatFrame1, "TopRight", 0, 6);

	-- CombatLogQuickButtonFrame_Custom
	CombatLogQuickButtonFrame_Custom:SetPoint("BottomLeft", ChatFrame2, "TopLeft", 0, 0);
	CombatLogQuickButtonFrame_Custom:SetPoint("BottomRight", ChatFrame2, "TopRight", 0, 0);

	local self = module.ChatBoxFrame;
	self:Hide();
end

function Addon.Options.UsingPrat()
	return (LibStub("AceAddon-3.0"):GetAddon("Prat", true) ~= nil);
end