-- modBottomLeft\Controller.lua
-- Handle the Bottom UI containing: ActionBarMain, ActionBar1, ActionBar2, ActionBarStance, XPBar, RepBar, HPBar, and the MPBar.
-- Handle the Side   UI containing: ActionBar3, ActionBar4.

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator	= LibStub:GetLibrary("FoxFrameCreator");
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
Addon.BottomLeft 	= Addon.BottomLeft or {};
local module 		= Addon.BottomLeft;
local moduleFrames	= Addon.Frames;

-- Safety Check
Addon.InitializationFunctions 	= Addon.InitializationFunctions or {};
Addon.Module_PEW 				= Addon.Module_PEW or {};
Addon.DisableFunctions 			= Addon.DisableFunctions or {};
Addon.EnableFunctions 			= Addon.EnableFunctions or {};

Addon.Bottom.PositionUpdateFuncs = Addon.Bottom.PositionUpdateFuncs or {};

local controllerFrameName 	= "FSUI_BottomLeft_Controller"

-- Function Definitions
local Create;
local AddBarOption;
local ShowOverlay;
local AddShowBarOption;
local AddConfigOptions;
----------------------------------------------------------------------------------------------------
function Create()
	local name = controllerFrameName;
	local self;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name);
	else
		self = libFrameCreator.Setup(_G[name]);
	end
	
	self:SetBackdrop(nil);
	self:ClearAllPoints();
	self:SetPoint("BottomLeft", UIParent, "BottomLeft", 0, 0);
	
	self:Hide();
	
	return self;
end

function module.Main()
	if (module.Loaded) then
		return;
	end
	-- Create 
	module.Controller = Create();
	
	-- Create Protected Frames
	--module.ActionBarFrame 		= module.ActionBar.Create(	module.ActionBarFrame, 		module.Controller);
	module.ActionBarFrame 	= Addon.ActionBar.Create(module.ActionBarFrame, module.Controller, "BottomLeft", false, 3, 12, nil, "FoxUI", "BottomLeft");
	
	-- Create Children Frames
	module.CurrencyBarFrame 	= module.CurrencyBarFrame or module.CurrencyBar.Create(module.CurrencyBarFrame, 	module.ActionBarFrame, 		module.Controller);
	module.ChatBoxFrame 		= module.ChatBoxFrame or module.ChatBox.Create(	module.ChatBoxFrame, 		module.CurrencyBarFrame, 	module.Controller)
	module.DurabilityFrame 		= module.DurabilityFrame or module.Durability.Create(	module.DurabilityFrame, 	module.CurrencyBarFrame, 	module.Controller);
	module.TimeFrame 			= module.TimeFrame or module.Time.Create(		module.TimeFrame, 			module.DurabilityFrame, 	module.Controller);
	
	module.InfoBarFrame = module.InfoBarFrame or Addon.Frames.InfoBar.New(module.InfoBarFrame, module.ActionBarFrame, "BottomLeft", 0)
	module.InfoBarFrame:SetPoint("Bottom", module.ActionBarFrame, "Top");
	module.InfoBarFrame:SetParent(module.Controller);
	
	-- Extra Setup
	module.CurrencyBar.Setup(module.CurrencyBarFrame, module.ActionBarFrame, module.Controller);
	
	-- Move ChatFrame
	module.ChatBoxFrame:SetPoint("Bottom", module.InfoBarFrame, "Top", 0, 0);
	
	module.InfoBarFrame:AddFrameLeft(module.TimeFrame);
	module.InfoBarFrame:AddFrameLeft(module.DurabilityFrame);
	module.InfoBarFrame:AddFrameRight(module.CurrencyBarFrame);
	
	-- !!!!!Handle Positioning in PositionHandler
	module.ActionBarFrame:ClearAllPoints();
	module.ActionBarFrame:SetPoint("BottomLeft", module.Controller, "BottomLeft", 0, 0);
	
	-- Add Config
	--AddConfigOptions()
	
	-- Set Parent
	module.Controller:SetParent(Addon.Bottom.Controller);
	
	-- Table Inserts
	if (not module.Loaded) then
		-- Add Update Func
		table.insert(Addon.Bottom.PositionUpdateFuncs, module.Controller_UpdatePos);
		
		
		-- Add to Style Table
		table.insert(Addon.StyleFrames, module.ActionBarFrame);
		table.insert(Addon.StyleFrames, module.CurrencyBarFrame);
		table.insert(Addon.StyleFrames, module.ChatBoxFrame.ChatBox);
		table.insert(Addon.StyleFrames, module.ChatBoxFrame.ChatLeftSide);
		table.insert(Addon.StyleFrames, module.ChatBoxFrame.EditBox);
		table.insert(Addon.StyleFrames, module.DurabilityFrame);
		table.insert(Addon.StyleFrames, module.TimeFrame);
		
		table.insert(Addon.StyleFrames, module.InfoBarFrame);
		
		-- Loop through the ChatFrames
		for i = 1, NUM_CHAT_WINDOWS do
			if (Addon.Options.UsingPrat()) then
				table.insert(Addon.StyleFrames, _G["ChatFrame"..i.."EditBox"]);
				table.insert(Addon.StyleFrames, _G["ChatFrame"..i.."EditBox"].frame);
			end
		end
		
		-- Setup Flags
		module.AnchorToStatBars = false;
		
	end
	module.Loaded = true;
end

function module.SetParent(parent)
	module.Controller:SetParent(parent);
end

function module.Controller_OnEnable()
	module.Controller:Show();
	
	-- Run ChatBox Setup
	module.ChatBox.Setup();
	
end
function module.Controller_OnDisable()
	module.Controller:Hide();
	
	-- Reset ChatBox Setup
	module.ChatBox.ResetChatWindow();
	
end

function module.Controller_UpdatePos()
	local anchor = UIParent;
	local anchorPoint = "BottomLeft";
	if (Addon.Bottom.InfoBarFrame) then
		if (module.AnchorToStatBars) then
			if (Addon.Bottom.RepBarFrame:IsShown()) then
				anchor = Addon.Bottom.RepBarFrame;
				anchorPoint = "TopLeft";
			elseif (Addon.Bottom.XPBarFrame:IsShown()) then
				anchor = Addon.Bottom.XPBarFrame;
				anchorPoint = "TopLeft";
			elseif (Addon.Bottom.InfoBarFrame:IsShown()) then
				anchor = Addon.Bottom.InfoBarFrame;
				anchorPoint = "TopLeft";
			end
		elseif (Addon.Bottom.InfoBarFrame:IsShown()) then
			anchor = Addon.Bottom.InfoBarFrame;
			anchorPoint = "TopLeft";
		end
	end
	
	module.Controller:ClearAllPoints();
	module.Controller:SetPoint("BottomLeft", anchor, anchorPoint, 0, 0);
end

function module.SetAnchorToStatBars(value)
	module.AnchorToStatBars = value;
	if (not InCombatLockdown()) then
		module.Controller_UpdatePos();
	end
end

local baseOffset = 6;
function AddBarOption(barId, customName, offset)
	local option = nil;
	-- Get Options		
	local options = Addon.GetOptions();

	if (barId == 1) then
		-- Edit Options
		option = {
					type = 'execute',
					name = 'Clear Bar '..(customName or barId + baseOffset/2),
					desc = 'Clear Bar '..barId + baseOffset/2,
					func = function(bar)
								local bar = module.ActionBarFrame.Bars[1];
								Addon.DebugPrint("ClearBarHotkeys("..(bar:GetName() or "nil")..");");

								-- 2nd Bar
								Addon.SetupKeybindingDataBar(bar);
							end,
					order = offset;
				}
	elseif (barId == 2) then
		-- Edit Options
		option = {
					type = 'execute',
					name = 'Clear Bar '..(customName or barId + baseOffset/2),
					desc = 'Clear Bar '..barId + baseOffset/2,
					func = function(bar)
								local bar = module.ActionBarFrame.Bars[2];
								Addon.DebugPrint("ClearBarHotkeys("..(bar:GetName() or "nil")..");");

								-- 2nd Bar
								Addon.ClearKeybindingDataBar(bar);
							end,
					order = offset;
				}
	elseif (barId == 3) then
		-- Edit Options
		option = {
					type = 'execute',
					name = 'Clear Bar '..(customName or barId + baseOffset/2),
					desc = 'Clear Bar '..barId + baseOffset/2,
					func = function(bar)
								local bar = module.ActionBarFrame.Bars[3];
								Addon.DebugPrint("ClearBarHotkeys("..(bar:GetName() or "nil")..");");

								-- 2nd Bar
								Addon.ClearKeybindingDataBar(bar);
							end,
					order = offset;
				}
	end
	options.args.mainOptions.args.barHotkeyDefaults.args["ClearBar"..(barId + baseOffset)] = option
	
	-- Update
	--LibStub("AceConfig-3.0"):RegisterOptionsTable(AddonName.."-"..AddonVersion, options, {"foxsmoothui", "fsui"});
	LibStub("AceConfigRegistry-3.0"):NotifyChange(AddonName.."-"..AddonVersion)
	LibStub("AceConfigDialog-3.0"):Open(AddonName.."-"..AddonVersion)
end
function ShowOverlay(self, name)
	local backdrop = {
	   -- path to the background texture
	   bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",  
	   -- path to the border texture
	   edgeFile = "",--"Interface\\Tooltips\\UI-Tooltip-Border",
	   -- true to repeat the background texture to fill the frame, false to scale it
	   tile = true,
	   -- size (width or height) of the square repeating background tiles (in pixels)
	   tileSize = 16,
	   -- thickness of edge segments and square size of edge corners (in pixels)
	   edgeSize = 12,
	   -- distance from the edges of the frame to those of the background texture (in pixels)
	   insets = {
		  left 		= 0,
		  right 	= 0,
		  top 		= 0,
		  bottom 	= 0
	   }
	}
	self.overlay:SetBackdrop(backdrop);
	self.overlay:SetFrameStrata("Dialog");
	self.overlay:SetBackdropColor(1, 0.1, 0.1, 1);
	self.overlay.label1:SetText(name);
	if (self.overlay:IsShown()) then
		self.overlay:Hide();
	else
		self.overlay:Show();
	end
end
function AddShowBarOption(barId, customName, offset)
	local option = nil;
	-- Get Options		
	local options = Addon.GetOptions();
	
	if (barId == 1) then
		-- Edit Options
		option = {
					type = 'execute',
					name = 'Show Bar '..(customName or barId + baseOffset/2),
					desc = 'Show Bar '..barId + baseOffset/2,
					func = function()
								local bar = module.ActionBarFrame.Bars[1];
								Addon.DebugPrint("ShowBar("..(bar:GetName() or "nil")..");");

								-- 2nd Bar
								ShowOverlay(bar, "Bar 4");
							end,
					order = offset + 1;
				}
	elseif (barId == 2) then
		-- Edit Options
		option = {
					type = 'execute',
					name = 'Show Bar '..(customName or barId + baseOffset/2),
					desc = 'Show Bar '..barId + baseOffset/2,
					func = function()
								local bar = module.ActionBarFrame.Bars[2];
								Addon.DebugPrint("ShowBar("..(bar:GetName() or "nil")..");");

								-- 2nd Bar
								ShowOverlay(bar, "Bar 5");
							end,
					order = offset + 1;
				}
	elseif (barId == 3) then
		-- Edit Options
		option = {
					type = 'execute',
					name = 'Show Bar '..(customName or barId + baseOffset/2),
					desc = 'Show Bar '..barId + baseOffset/2,
					func = function()
								local bar = module.ActionBarFrame.Bars[3];
								Addon.DebugPrint("ShowBar("..(bar:GetName() or "nil")..");");

								-- 2nd Bar
								ShowOverlay(bar, "Bar 6");
							end,
					order = offset + 1;
				}
	end
	options.args.mainOptions.args.barHotkeyDefaults.args["ShowBar"..(barId + baseOffset)] = option
	
	-- Update
	--LibStub("AceConfig-3.0"):RegisterOptionsTable(AddonName.."-"..AddonVersion, options, {"foxsmoothui", "fsui"});
	LibStub("AceConfigRegistry-3.0"):NotifyChange(AddonName.."-"..AddonVersion)
	LibStub("AceConfigDialog-3.0"):Open(AddonName.."-"..AddonVersion)
end
function AddConfigOptions()
	AddShowBarOption(1, nil, 6)
	AddShowBarOption(2, nil, 8)
	AddShowBarOption(3, nil, 10)

	AddBarOption(1, nil, 6)
	AddBarOption(2, nil, 8)
	AddBarOption(3, nil, 10)
end


------------------------------------------
-- Config For Each Controller
------------------------------------------
local GetClearFunction;
local GetShowOverlayFunction;
local AddConfigClearOptionForBar;
local AddConfigShowOptionForBar;
local Config = Addon.Config;
local moduleName = "BottomLeft";

function AddConfigOptions()
	local bars = module.ActionBarFrame.Bars;
	
	local Config = Addon.Config;
	
	for i = 1, #bars do
		Config.SetActionBarOptionsCount(Config.GetActionBarOptionsCount() + 1);
		local name = "Bar "..Config.GetActionBarOptionsCount();
		AddConfigClearOptionForBar(bars, i, name, -1);
		AddConfigShowOptionForBar(bars,  i, name, -1);
		
	end
end
-- Clear Template Function
function GetClearFunction(bars, i, useKeys)
	local funcStr = [[
		local barName = "]].. bars[i]:GetName()..[["; 
		local self = _G[barName]; 
		local i 		= ]]..(i or -1)..[[;
		local AddonName 	= "]]..AddonName..[["; 
		local AddonVersion	= "]]..AddonVersion..[["; 
		local Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
		--do stuff 
		Addon.DebugPrint("ClearBarHotkeys("..(self:GetName() or "nil")..");");
		-- 2nd Bar
		if (]]..useKeys..[[) then
			Addon.SetupKeybindingDataBar(self, ]]..useKeys..[[);
		else
			Addon.SetupKeybindingDataBar(self);
		end
	]]
	local func, errorStr = loadstring(funcStr);
	if (errorStr) then
		error(errorStr);
	end
	
	return func;
end
-- Show Overlay Template Function
function GetShowOverlayFunction(bars, i)
	local funcStr = [[
		local barName 	= "]].. bars[i]:GetName()..[[";
		local self 		= _G[barName]; 
		local i 		= ]]..(i or -1)..[[;
		local AddonName 	= "]]..AddonName..[["; 
		local AddonVersion	= "]]..AddonVersion..[["; 
		local Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
		--do stuff 
		Addon.DebugPrint("ShowBar("..(self:GetName() or "nil")..");");
		Addon.Bars.ShowOverlay(self, (self.BarNameData or "Bar "..i));
		print(self.BarNameData);
	]]
	local func, errorStr = loadstring(funcStr);
	if (errorStr) then
		error(errorStr);
	end
	
	return func;
end
function AddConfigClearOptionForBar(bars, i, customName, useKeys)
	if (customName) then
		bars[i].BarNameData = customName;
	end
	
	local option = {
		type = 'execute',
		name = 'Clear '..(customName or "Bar "..i),
		desc = 'Clear '..(customName or "Bar "..i),
		
		func = GetClearFunction(bars, i, useKeys),
		order = Config.GetActionBarOptionsOffset();
	}
	
	local options = Addon.GetOptions();
	
	options.args.mainOptions.args.barHotkeyDefaults.args["ClearBar_"..moduleName.."_"..i] = option;
	
	-- Update
	LibStub("AceConfigRegistry-3.0"):NotifyChange(AddonName.."-"..AddonVersion)
	LibStub("AceConfigDialog-3.0"):Open(AddonName.."-"..AddonVersion)
	
	Config.SetActionBarOptionsOffset(Config.GetActionBarOptionsOffset() + 1)
end
function AddConfigShowOptionForBar(bars, i, customName)
	if (customName) then
		bars[i].BarNameData = customName;
	end
	option = {
		type  = 'execute',
		name  = 'Show '..(customName or "Bar "..i),
		desc  = 'Show '..(customName or "Bar "..i),
		func  = GetShowOverlayFunction(bars, i),
		order = Config.GetActionBarOptionsOffset();
	}
	
	local options = Addon.GetOptions();
	options.args.mainOptions.args.barHotkeyDefaults.args["ShowBar_"..moduleName.."_"..i] = option;
	
	-- Update
	LibStub("AceConfigRegistry-3.0"):NotifyChange(AddonName.."-"..AddonVersion)
	LibStub("AceConfigDialog-3.0"):Open(AddonName.."-"..AddonVersion)
	
	Config.SetActionBarOptionsOffset(Config.GetActionBarOptionsOffset() + 1)
end

table.insert(Addon.ConfigOptionsFuncs, AddConfigOptions);
------------------------------------------


-- Main
table.insert(Addon.InitializationFunctions, module.Main);
table.insert(Addon.EnableFunctions, module.Controller_OnEnable);
table.insert(Addon.DisableFunctions, module.Controller_OnDisable);

table.insert(Addon.Module_PEW, module.ChatBox.Setup);

Addon.LoadModules();

