-- BottomLeftCurrency.lua
-- Handles the CurrencyBar

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator	= LibStub:GetLibrary("FoxFrameCreator");
local libFoxObjectClass	= LibStub:GetLibrary("FoxObjectClass");
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
-- Array Safety
Addon.BottomLeft 	= Addon.BottomLeft or {};
local module 		= Addon.BottomLeft;

-- Array Setup
module.CurrencyBar 			= {};
module.CurrencyBar.Textures 	= {};
module.CurrencyBar.Labels 	= {};

-- Module Locals
local frameName 	= "FSUI_CurrencyFrame";
local goldTextStr 	= "Interface\\MONEYFRAME\\UI-GoldIcon";

local frameName_1 	= "FSUI_CurrencyFrame1";
local frameName_2 	= "FSUI_CurrencyFrame2";
local frameName_3 	= "FSUI_CurrencyFrame3";

-- Function Definitions
local CreateCurrency;
local OnEvent;
local Update;
local Hooked_BackpackTokenFrame_Update;
local origBackpackTokenFrame_Update;
local OnEnter_Tooltip;
local OnLeave_Tooltip;

local subValue;
local OnEnter_TooltipGold;
local OnLeave_TooltipGold;

----------------------------------------------------------------------------------------------------

function module.CurrencyBar.Create(self, anchorFrame, parent)
	local name = frameName;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name);
	else
		self = libFrameCreator.Setup(_G[name]);
	end
	
	-- Set Parent
	self:SetParent(parent);
	
	-- Set Position
	self:SetPoint("BottomRight", anchorFrame, "TopRight", 0, 0);
	self:SetWidth(418);
	self:SetHeight(14);
	
	self:SetScript("OnEnter", OnEnter_TooltipGold);
	self:SetScript("OnLeave", OnLeave_TooltipGold);
	
	self.SetGoldTooltip = SetGoldTooltip;
	
	-- Setup CurrencyFrame 1
	name = frameName_1;
	if (_G[name] == nil) then
		self.currency1Frame = libFrameCreator.New(name);
	else
		self.currency1Frame = libFrameCreator.Setup(_G[name]);
	end
	-- Set Parent
	self.currency1Frame:SetParent(self);
	
	-- Set Position
	self.currency1Frame:ClearAllPoints();
	self.currency1Frame:SetPoint("Right", self, "Right", -60, 0);
	self.currency1Frame:SetWidth(74);
	self.currency1Frame:SetHeight(28);
	self.currency1Frame:SetBackdrop(nil);
	self.currency1Frame.id = 1;
	self.currency1Frame:SetScript("OnEnter", OnEnter_Tooltip);
	self.currency1Frame:SetScript("OnLeave", OnLeave_Tooltip);
	
	-- Setup CurrencyFrame 2
	name = frameName_2;
	if (_G[name] == nil) then
		self.currency2Frame = libFrameCreator.New(name);
	else
		self.currency2Frame = libFrameCreator.Setup(_G[name]);
	end
	-- Set Parent
	self.currency2Frame:SetParent(self);
	
	-- Set Position
	self.currency2Frame:ClearAllPoints();
	self.currency2Frame:SetPoint("Right", self, "Right", -124, 0);
	self.currency2Frame:SetWidth(74);
	self.currency2Frame:SetHeight(28);
	self.currency2Frame:SetBackdrop(nil);
	self.currency2Frame.id = 2;
	self.currency2Frame:SetScript("OnEnter", OnEnter_Tooltip);
	self.currency2Frame:SetScript("OnLeave", OnLeave_Tooltip);
	
	-- Setup CurrencyFrame 3
	name = frameName_3;
	if (_G[name] == nil) then
		self.currency3Frame = libFrameCreator.New(name);
	else
		self.currency3Frame = libFrameCreator.Setup(_G[name]);
	end
	-- Set Parent
	self.currency3Frame:SetParent(self);
	
	-- Set Position
	self.currency3Frame:ClearAllPoints();
	self.currency3Frame:SetPoint("Right", self, "Right", -188, 0);
	self.currency3Frame:SetWidth(74);
	self.currency3Frame:SetHeight(28);
	self.currency3Frame:SetBackdrop(nil);
	self.currency3Frame.id = 3;
	
	self.currency3Frame:SetScript("OnEnter", OnEnter_Tooltip);
	self.currency3Frame:SetScript("OnLeave", OnLeave_Tooltip);
	
	return self;
end
function module.CurrencyBar.Setup(self, anchorFrame, parent)
	if (InCombatLockdown()) then
		return
	end
	
	-- Set Parent
	self:SetParent(parent);
	
	
	-- Set Position
	self:SetPoint("BottomRight", anchorFrame, "TopRight", 0, 0);
	--self:SetWidth(418);
	self:SetWidth(252);
	self:SetHeight(28);
	
	-- Setup Events
	self:SetScript("OnEvent", OnEvent);
	self:RegisterEvent("PLAYER_MONEY");
	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
	self:RegisterEvent("BAG_UPDATE");
	
	
	-- Set Variables
	local count 	= math.floor(GetMoney() / 10000);
	
	-- Create Gold Texture
	module.CurrencyBar.Textures[0] = libFrameCreator.CreateTexture(module.CurrencyBar.Textures[0], self, goldTextStr, "Center", nil, nil, 0, 0, 16, 16);
	module.CurrencyBar.Textures[0]:ClearAllPoints();
	module.CurrencyBar.Textures[0]:SetPoint("Right", self, "Right", 0, 0);
	
	module.CurrencyBar.Labels[0] = libFrameCreator.CreateLabel(module.CurrencyBar.Labels[0], self, count, "Right", module.CurrencyBar.Textures[0], "Left", -1, 0, 100, self:GetHeight(), "Right", "MIDDLE");
	
	for i = 1, 3 do
		CreateCurrency(self, i);
	end
	
	if (origBackpackTokenFrame_Update == nil) then
		origBackpackTokenFrame_Update = BackpackTokenFrame_Update;
		BackpackTokenFrame_Update = Hooked_BackpackTokenFrame_Update
	end
	
	
end

function CreateCurrency(parent, i)
	-- Get Texture Path
	local name, count, icon, itemID = GetBackpackCurrencyInfo(i);
	if (name == nil) then
		if (module.CurrencyBar.Textures[i]) then
			module.CurrencyBar.Textures[i]:Hide();
		end
		if (module.CurrencyBar.Labels[i]) then
			module.CurrencyBar.Labels[i]:Hide();
		end
		return false;
	end
	
	-- Create Currency Texture
	module.CurrencyBar.Textures[i] = libFrameCreator.CreateTexture(module.CurrencyBar.Textures[i], parent, icon, "Center", nil, nil, 0, 0, 16, 16);
	module.CurrencyBar.Textures[i]:ClearAllPoints();
	module.CurrencyBar.Textures[i]:SetPoint("Right", parent, "Right", ((((i - 1) * 64) + 64) * -1), 0);
	
	-- Create Label
	module.CurrencyBar.Labels[i] = libFrameCreator.CreateLabel(module.CurrencyBar.Labels[i], parent, subValue(count), "Right", module.CurrencyBar.Textures[i], "Left", -1, 0, 100, 12, "Right", "middle");
	if ((count == 0) and (icon == nil)) then
		module.CurrencyBar.Labels[i]:SetText("");
	end
	
	module.CurrencyBar.Textures[i]:Show();
	module.CurrencyBar.Labels[i]:Show();
	
	return true;
end

function Hooked_BackpackTokenFrame_Update(...)
	origBackpackTokenFrame_Update(...);
	Update(_G[frameName]);
end

function OnEvent(self, event, arg1, ...)
	Update(self);
end
function Update(self)
	if (self == nil) then
		return;
	end
	
	if (InCombatLockdown()) then
		return
	end
	
	-- Update Money
	local count 	= math.floor(GetMoney() / 10000);
	module.CurrencyBar.Labels[0] = libFrameCreator.CreateLabel(module.CurrencyBar.Labels[0], self, (subValue(count) or ""), "Right", module.CurrencyBar.Textures[0], "Left", -1, 0, 100, 30, "Right", "Middle");
	
	-- Update Currency
	local currencies = 0;
	for i = 1, 3 do
		local bool = CreateCurrency(self, i);
		if (bool) then
			currencies = currencies + 1;
		end
	end
	
	self:SetWidth(64 + 64 * currencies);
end

function SetGoldTooltip(self, frame)
	frame:ClearAllPoints();
	frame:SetPoint("BottomRight", self, "TopRight", 0, 0);
	
	self.goldToolTip = frame;
	
	Addon.StyleFrames = Addon.StyleFrames or {};
	table.insert(Addon.StyleFrames, self.goldToolTip);
		
	self.goldToolTip.OnStyle = function(self)
		local red, green, blue, alpha = self:GetBackdropColor();
		self:SetBackdropColor(red, green, blue, 1);
		
	end
end
function OnEnter_TooltipGold(self)
	if (self.goldToolTip) then
		self.goldToolTip:Show();
		self.goldToolTip:SetFrameStrata("TOOLTIP");
	end
end
function OnLeave_TooltipGold(self)
	if (self.goldToolTip) then
		self.goldToolTip:Hide();
	end
end


function OnEnter_Tooltip(self)
	--print(self.id, GetCurrencyListSize())
	if (self.id <= GetNumWatchedTokens()) then
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
		GameTooltip:SetBackpackToken(self.id);
	end
end
function OnLeave_Tooltip(self)
	GameTooltip:Hide();
end


function subValue(value)
	if (value) then
		if (value > 10000) then
			value = math.floor(value/1000)
			value = value.."k";
		end
	end
	return value;
end

--module.CurrencyBarFrame 	= module.CurrencyBar.Create(module.CurrencyBarFrame, 	module.ActionBarFrame, 		module.Controller);
-- Extra Setup
--module.CurrencyBar.Setup(module.CurrencyBarFrame, module.ActionBarFrame, module.Controller);