-- BottomLeftCurrency.lua
-- Handles the CurrencyBar

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator	= LibStub:GetLibrary("FoxFrameCreator");

----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
-- Array Safety
Addon.BottomLeft = Addon.BottomLeft or {};
local module = Addon.BottomLeft;

-- Array Setup
module.Durability = {};

local frameName = "FSUI_DurabilityFrame";
local frameWidth 		= 48;
local lowThreshold 		= 20;
local meduimThreshold	= 50;

-- Function Definitions
local SetText;
local SetTextColor;

local OnEvent;
local Update;
----------------------------------------------------------------------------------------------------


function module.Durability.Create(self, anchorFrame, parent)
	local name = frameName;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name);
	else
		self = libFrameCreator.Setup(_G[name]);
	end

	-- Set Parent
	self:SetParent(parent);
	
	-- Set Position
	self:ClearAllPoints();
	self:SetPoint("TopRight", anchorFrame, "TopLeft", 0, 0);
	self:SetWidth(frameWidth);
	self:SetHeight(28);
	
	self.icon = self.icon or self:CreateTexture(self:GetName().."Icon", "Artwork");

	self.icon:SetTexture("Interface\\Durability\\UI-Durability-Icons");
	self.icon:SetWidth(20);
	self.icon:SetHeight(20);
	self.icon:SetPoint("Right", self, "Right");
	self.icon:SetVertexColor(1, 1, 1, 1)
	self.icon:SetTexCoord(0.1875, 0.375, 0.3203125, 0.5546875)
	
	self.label = libFrameCreator.CreateLabel(self.label, self, "", "Right", self.icon, "Left", 5, 0, 100, 30, "Right", "middle");
	
	-- Add Methods
	self.SetText 		= SetText;
	self.SetTextColor 	= SetTextColor;
	self.OnEvent 		= OnEvent;
	self.Update 		= Update;
	
	-- Set Events
	self:SetScript("OnEvent", OnEvent);
	self:RegisterEvent("UPDATE_INVENTORY_DURABILITY");
	
	-- Update Text
	self:Update();
	
	return self;
end

-- Methods
function SetText(self, text)
	if (self == nil) then
		return;
	end
	
	self.label:SetText(text);
end
function SetTextColor(self, r, g, b, a)
	if (self == nil) then
		return;
	end
	
	self.label:SetTextColor(r, g, b, a);
end

function OnEvent(self, event, arg1, ...)
	Update(self);
end

function Update(self)
	--local slots = {"AmmoSlot", "BackSlot", "Bag0Slot", "Bag1Slot", "Bag2Slot", "Bag3Slot", "ChestSlot", "FeetSlot", "Finger0Slot", "Finger1Slot", "HandsSlot", "HeadSlot", "LegsSlot", "MainHandSlot", "NeckSlot", "SecondaryHandSlot", "ShirtSlot", "ShoulderSlot", "TabardSlot", "Trinket0Slot", "Trinket1Slot", "WaistSlot", "WristSlot"}
	-- id, texture, checkRelic = GetInventorySlotInfo(slots[i])
	local durability = 0;
	local maxDurability = 0; 
	local slotIds = {0, 15, 20, 21, 22, 23, 5, 8, 11, 12, 10, 1, 7, 16, 2, 17, 4, 3, 19, 13, 14, 6, 9}
	for i = 1, #slotIds do
		local d, md = GetInventoryItemDurability(slotIds[i]);
		if (d ~= nil) then
			durability = durability + d;
			maxDurability = maxDurability + md;
		end
	end
	
	-- Set Text
	local durabilityPercent = math.floor((durability / maxDurability) * 100);
	self:SetText(""..durabilityPercent.."%");
	
	-- Set Text Color
	if (durabilityPercent < lowThreshold) then
		-- Color Red
		self:SetTextColor(1, 0, 0, 1);
		
	elseif (durabilityPercent < meduimThreshold) then
		-- Color Yellow
		self:SetTextColor(1, 1, 0, 1);
		
	else
		-- Color Green
		self:SetTextColor(0, 1, 0, 1);
		
	end
end