-- BottomLeftCurrency.lua
-- Handles the CurrencyBar

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator	= LibStub:GetLibrary("FoxFrameCreator");

----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
-- Array Safety
Addon.BottomLeft = Addon.BottomLeft or {};
local module = Addon.BottomLeft;

-- Array Setup
module.Time = {};

local frameName 		= "FSUI_TimeFrame";
local frameWidth 		= 96;
local updateTime 		= 1.0;
local updateTimeElapsed = 0.0;

-- Function Definitions
local SetText;
local SetTextColor;
local GetTimeString;
local OnUpdate;
----------------------------------------------------------------------------------------------------

function module.Time.Create(self, anchorFrame, parent)
	local name = frameName;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name);
	else
		self = libFrameCreator.Setup(_G[name]);
	end
	
	-- Set Parent
	self:SetParent(parent);

	-- Set Position
	self:ClearAllPoints();
	self:SetPoint("TopRight", anchorFrame, "TopLeft", 0, 0);
	self:SetWidth(frameWidth);
	self:SetHeight(28);
	self.label = libFrameCreator.CreateLabel(self.label, self, "", "Right", self, "Right", -1, 0, 100, 28, "Right", "MIDDLE");
	
	-- Add Methods
	self.SetText 		= SetText;
	self.SetTextColor 	= SetTextColor;
	
	self.OnUpdate = OnUpdate;
	self:SetScript("OnUpdate", OnUpdate);
	
	local str = GetTimeString();
	self:SetText(str);
	
	return self;
end

-- Methods
function SetText(self, text)
	if (self == nil) then
		return;
	end
	
	self.label:SetText(text);
end
function SetTextColor(self, r, g, b, a)
	if (self == nil) then
		return;
	end
	
	self.label:SetTextColor(r, g, b, a);
end
function GetTimeString()
	local hour = 0;
	local minute = 0;
	
	
	if (GetTimeSource() == "Both") then
		-- Get Player's Time
		local dateValue = date("*t");
		local hourL 	= dateValue.hour;
		local minuteL 	= dateValue.min;
		
		-- Get Realm Time
		local hourR, minuteR = GetGameTime();
		
		local pmL = false;
		if (hourL > 12) then
			hourL = hourL - 12;
			pmL = true;
		elseif (hourL == 0) then
			hourL = 12;
		end
		
		local pmR = false;
		if (hourR > 12) then
			hourR = hourR - 12;
			pmR = true;
		elseif (hourR == 0) then
			hourR = 12;
		end
		
		if (minuteL < 10) then
			minuteL = "0"..minuteL;
		end
		if (minuteR < 10) then
			minuteR = "0"..minuteR;
		end
		
		if (hourL < 10) then
			hourL = " "..hourL;
		end
		if (hourR < 10) then
			hourR = " "..hourR;
		end
		
		local str_Local = "Local: "..hourL..":"..minuteL..(pmL and "PM" or "AM");
		local str_Realm = "Realm: "..hourR..":"..minuteR..(pmR and "PM" or "AM");
		
		
		return str_Local.."\n"..str_Realm
	else
		-- Using Player's Time
		if (Addon.Options.UsingPlayerTime()) then
			local dateValue = date("*t");
			hour 	= dateValue.hour;
			minute 	= dateValue.min;
		else
			hour, minute = GetGameTime();
		end
		
		local pm = false;
		if (hour > 12) then
			hour = hour - 12;
			pm = true;
		elseif (hour == 0) then
			hour = 12;
		end
		
		if (minute < 10) then
			minute = "0"..minute;
		end
		
		if (hour < 10) then
			hour = " "..hour;
		end
		
		return ""..(Addon.Options.UsingPlayerTime() and "Local: " or "Realm: ")..hour..":"..minute..(pm and "PM" or "AM");
	end
end
function OnUpdate(self, elapsed)
	updateTimeElapsed = updateTimeElapsed + elapsed;
	
	if (updateTimeElapsed < updateTime) then
		return;
	end
	updateTimeElapsed = 0.0;
	
	local str = GetTimeString();
	self:SetText(str);
end
function Addon.Options.UsingPlayerTime()
	if (GetTimeSource() == "Realm") then
		return false;
	end
	return true;
end

function GetTimeSource()
	return Addon.db.profile.useLocalTime;
end