-- FoxSmoothUIBagFrame.lua
-- Handles the Bag Buttons.

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--


----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator = LibStub:GetLibrary("FoxFrameCreator");
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
-- Array Safety
Addon.Frames 		= Addon.Frames or {};
Addon.Frames.BagBar = Addon.Frames.BagBar or {};
local module 		= Addon.Frames.BagBar;


local frameWidth  = 171;
local frameHeight = 44;
----------------------------------------------------------------------------------------------------

-- Variables
local FrameName = "FSUI_BagFrame";
local OnShow;
local OnHide;
local SetVertical;
local SetHorizontal;

function module.Create(self, parent, isVertical)
	-- Create Frame, if needed
	if (_G[FrameName] == nil) then
		self = libFrameCreator.New(FrameName, false, 160, 80, UIParent, nil, "nil");
	else
		self = libFrameCreator.Setup(_G[FrameName], false, 160, 80, UIParent, nil, "nil");
		
	end
	
	self:SetBackdropColor(0.1, 0.1, 0.1, 0.6);
	self:SetWidth(frameWidth);
	self:SetHeight(frameHeight);
	
	-- Set Position
	self:ClearAllPoints();
	self:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", 0, 0);
	
	self:SetScript("OnShow", OnShow);
	self:SetScript("OnHide", OnHide);
	
	-- Set Parent
	self:SetParent(parent);
	
	-- Bag Frame
	self:Show();
	OnShow(self);
	
	if (isVertical) then
		SetVertical(self);
	else
		SetHorizontal(self);
	end
	
	-- Set Methods
	self.SetCustomPoint = module.SetCustomPoint;
	self.FixPosition 	= module.FixPosition;
	
	return self;
end
function module.SetCustomPoint(self, point, anchor, anchorPoint, x, y)
	self.point = point;
	self.anchor = anchor;
	self.anchorPoint = anchorPoint;
	self._x = x;
	self._y = y;
	
	self:FixPosition();
end
function module.FixPosition(self)
	self:ClearAllPoints();
	self:SetPoint(self.point, self.anchor, self.anchorPoint, self._x, self._y);
end

function OnShow(self)
	-- Set Position
	MainMenuBarBackpackButton:ClearAllPoints();
	MainMenuBarBackpackButton:SetParent(self);
	MainMenuBarBackpackButton:SetPoint("BOTTOMRIGHT", -6, 7);
	
	-- Set Parent
	CharacterBag0Slot:SetParent(self);
	CharacterBag1Slot:SetParent(self);
	CharacterBag2Slot:SetParent(self);
	CharacterBag3Slot:SetParent(self);
end
function OnHide(self)
	-- Set Position
	MainMenuBarBackpackButton:ClearAllPoints();
	MainMenuBarBackpackButton:SetParent(MainMenuBarArtFrame);
	MainMenuBarBackpackButton:SetPoint("BOTTOMRIGHT", -6, 2);
	
	-- Set Parent
	CharacterBag0Slot:SetParent(MainMenuBarArtFrame);
	CharacterBag1Slot:SetParent(MainMenuBarArtFrame);
	CharacterBag2Slot:SetParent(MainMenuBarArtFrame);
	CharacterBag3Slot:SetParent(MainMenuBarArtFrame);
end

function SetVertical(self)
	local backpack = _G["MainMenuBarBackpackButton"];
	local bag1 = _G["CharacterBag0Slot"];
	local bag2 = _G["CharacterBag1Slot"];
	local bag3 = _G["CharacterBag2Slot"];
	local bag4 = _G["CharacterBag3Slot"];
	
	-- Set Bag Positions
	bag1:ClearAllPoints();
	bag1:SetPoint("Bottom", backpack, "Top", 0, 2);
	bag2:ClearAllPoints();
	bag2:SetPoint("Bottom", bag1, "Top", 0, 2);
	bag3:ClearAllPoints();
	bag3:SetPoint("Bottom", bag2, "Top", 0, 2);
	bag4:ClearAllPoints();
	bag4:SetPoint("Bottom", bag3, "Top", 0, 2);
	
	-- Set Frame Size
	self:SetWidth(frameHeight);
	self:SetHeight(frameWidth);
end
function SetHorizontal(self)
	local backpack = _G["MainMenuBarBackpackButton"];
	local bag1 = _G["CharacterBag0Slot"];
	local bag2 = _G["CharacterBag1Slot"];
	local bag3 = _G["CharacterBag2Slot"];
	local bag4 = _G["CharacterBag3Slot"];
	
	-- Set Bag Positions
	bag1:ClearAllPoints();
	bag1:SetPoint("Right", backpack, "Left", -2, 0);
	bag2:ClearAllPoints();
	bag2:SetPoint("Right", bag1, "Left", -2, 0);
	bag3:ClearAllPoints();
	bag3:SetPoint("Right", bag2, "Left", -2, 0);
	bag4:ClearAllPoints();
	bag4:SetPoint("Right", bag3, "Left", -2, 0);
	
	-- Set Frame Size
	self:SetWidth(frameWidth);
	self:SetHeight(frameHeight);
end
