-- Controller.lua
-- Handle the Bottom UI containing: ActionBarMain, ActionBar1, ActionBar2, ActionBarStance, XPBar, RepBar, HPBar, and the MPBar.

local addonName, addonTable = ...
----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName		= addonTable.Name;
local AddonVersion	= addonTable.Version;
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator	= LibStub:GetLibrary("FoxFrameCreator");
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
Addon.BottomRight 				= Addon.BottomRight or {};
Addon.BottomRight.ControllerArr	= Addon.BottomRight.ControllerArr or {};

local module 		= Addon.BottomRight;
local moduleFrames	= Addon.Frames;


-- Safety Check
Addon.InitializationFunctions 	= Addon.InitializationFunctions or {};
Addon.EnableFunctions 			= Addon.EnableFunctions 		or {};
Addon.DisableFunctions 			= Addon.DisableFunctions 		or {};

local controllerFrameName 		= "FSUI_BottomRight_Controller"

-- Function Definitions
local Create;
----------------------------------------------------------------------------------------------------

function Create()
	local name = controllerFrameName;
	local self;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name);
	else
		self = libFrameCreator.Setup(_G[name]);
	end
	
	self:SetBackdrop(nil);
	self:ClearAllPoints();
	self:SetPoint("BottomRight", UIParent, "BottomRight", 0, 0);
	
	return self;
end

function module.Main()
	-- Create ParentFrame (Secure)
	module.Controller = Create();
	
	module.BagBarFrame = Addon.Frames.BagBar.Create(module.BagBarFrame, module.Controller, true);
	module.BagBarFrame:SetCustomPoint("BottomRight", module.Controller, "BottomRight", 0, 0)
	
	-- Create ActionBar Frames
	module.ActionBarFrame 	= Addon.ActionBar.Create(module.ActionBarFrame, module.Controller, "BottomRight", false, 4, 10, false, "FoxUI", "BottomRight");
	module.ActionBarFrame:ClearAllPoints();
	module.ActionBarFrame:SetPoint("BottomRight", module.BagBarFrame, "BottomLeft", 0, 0);
	
	-- Create Children Frames
	module.StopWatchFrame 	= module.StopWatch.Create( module.StopWatchFrame, 	module.ActionBarFrame, 	module.Controller);
	
	local width = module.ActionBarFrame:GetWidth() - module.StopWatchFrame:GetWidth();
	module.Title 			= module.TitleFrame.Create(module.Title, 			module.StopWatchFrame, 	module.Controller, width);
	
	-- Hide Frame
	module.Controller:Hide();
	
	-- Set Parent
	module.Controller:SetParent(Addon.Bottom.Controller);
	
	-- Table Inserts
	if (not module.Loaded) then
		-- Add Update Func
		table.insert(Addon.Bottom.PositionUpdateFuncs, module.Controller_UpdatePos);
		
		-- Add to Style Table
		table.insert(Addon.StyleFrames, module.ActionBarFrame);
		table.insert(Addon.StyleFrames, module.StopWatchFrame);
		table.insert(Addon.StyleFrames, module.BagBarFrame);
		table.insert(Addon.StyleFrames, module.Title);
		
		-- Setup Flags
		module.AnchorToStatBars = false;
		
		module.Loaded = true;
	end
	
end

function module.SetParent(parent)
	module.Controller:SetParent(parent);
end

function module.Controller_OnEnable()
	module.Controller:Show();
	
	-- Run StopWatch Setup
	module.StopWatchFrame:Setup();
end
function module.Controller_OnDisable()
	module.Controller:Hide();
	
	-- Reset StopWatch Frame
	module.StopWatchFrame:Reset();
end


function module.Controller_UpdatePos()
	local anchor = UIParent;
	local anchorPoint = "BottomRight";
	if (Addon.Bottom.InfoBarFrame) then
		if (module.AnchorToStatBars) then
			if (Addon.Bottom.RepBarFrame:IsShown()) then
				anchor = Addon.Bottom.RepBarFrame;
				anchorPoint = "TopRight";
			elseif (Addon.Bottom.XPBarFrame:IsShown()) then
				anchor = Addon.Bottom.XPBarFrame;
				anchorPoint = "TopRight";
			elseif (Addon.Bottom.InfoBarFrame:IsShown()) then
				anchor = Addon.Bottom.InfoBarFrame;
				anchorPoint = "TopRight";
			end
		elseif (Addon.Bottom.InfoBarFrame:IsShown()) then
			anchor = Addon.Bottom.InfoBarFrame;
			anchorPoint = "TopRight";
		end
	end
	
	module.Controller:ClearAllPoints();
	module.Controller:SetPoint("BottomRight", anchor, anchorPoint, 0, 0);
	
	-- Handle Position with BagBar
	
	if (Addon.db.profile.ShowBagBar) then
		module.BagBarFrame:Show();
		
		module.ActionBarFrame:ClearAllPoints();
		module.ActionBarFrame:SetPoint("BottomRight", module.BagBarFrame, "BottomLeft", 0, 0);
	else
		module.BagBarFrame:Hide();
		
		module.ActionBarFrame:ClearAllPoints();
		module.ActionBarFrame:SetPoint("BottomRight", module.Controller, "BottomRight", 0, 0);
	end
	
	-- Run StopWatch Setup
	module.StopWatchFrame:Setup();
end

function module.SetAnchorToStatBars(value)
	module.AnchorToStatBars = value;
	if (not InCombatLockdown()) then
		module.Controller_UpdatePos();
	end
end

------------------------------------------
-- Config For Each Controller
------------------------------------------
local GetClearFunction;
local GetShowOverlayFunction;
local AddConfigClearOptionForBar;
local AddConfigShowOptionForBar;
local Config = Addon.Config;
local moduleName = "BottomRight";

function AddConfigOptions()
	local bars = module.ActionBarFrame.Bars;
	
	local Config = Addon.Config;
	
	for i = 1, #bars do
		Config.SetActionBarOptionsCount(Config.GetActionBarOptionsCount() + 1);
		local name = "Bar "..Config.GetActionBarOptionsCount();
		AddConfigClearOptionForBar(bars, i, name, -1);
		AddConfigShowOptionForBar(bars,  i, name, -1);
		
	end
end
-- Clear Template Function
function GetClearFunction(bars, i, useKeys)
	local funcStr = [[
		local barName = "]].. bars[i]:GetName()..[["; 
		local self = _G[barName]; 
		local i 		= ]]..(i or -1)..[[;
		local AddonName 	= "]]..AddonName..[["; 
		local AddonVersion	= "]]..AddonVersion..[["; 
		local Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
		--do stuff 
		Addon.DebugPrint("ClearBarHotkeys("..(self:GetName() or "nil")..");");
		-- 2nd Bar
		if (]]..useKeys..[[) then
			Addon.SetupKeybindingDataBar(self, ]]..useKeys..[[);
		else
			Addon.SetupKeybindingDataBar(self);
		end
	]]
	local func, errorStr = loadstring(funcStr);
	if (errorStr) then
		error(errorStr);
	end
	
	return func;
end
-- Show Overlay Template Function
function GetShowOverlayFunction(bars, i)
	local funcStr = [[
		local barName 	= "]].. bars[i]:GetName()..[[";
		local self 		= _G[barName]; 
		local i 		= ]]..(i or -1)..[[;
		local AddonName 	= "]]..AddonName..[["; 
		local AddonVersion	= "]]..AddonVersion..[["; 
		local Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
		--do stuff 
		Addon.DebugPrint("ShowBar("..(self:GetName() or "nil")..");");
		Addon.Bars.ShowOverlay(self, (self.BarNameData or "Bar "..i));
		print(self.BarNameData);
	]]
	local func, errorStr = loadstring(funcStr);
	if (errorStr) then
		error(errorStr);
	end
	
	return func;
end
function AddConfigClearOptionForBar(bars, i, customName, useKeys)
	if (customName) then
		bars[i].BarNameData = customName;
	end
	
	local option = {
		type = 'execute',
		name = 'Clear '..(customName or "Bar "..i),
		desc = 'Clear '..(customName or "Bar "..i),
		
		func = GetClearFunction(bars, i, useKeys),
		order = Config.GetActionBarOptionsOffset();
	}
	
	local options = Addon.GetOptions();
	
	options.args.mainOptions.args.barHotkeyDefaults.args["ClearBar_"..moduleName.."_"..i] = option;
	
	-- Update
	LibStub("AceConfigRegistry-3.0"):NotifyChange(AddonName.."-"..AddonVersion)
	LibStub("AceConfigDialog-3.0"):Open(AddonName.."-"..AddonVersion)
	
	Config.SetActionBarOptionsOffset(Config.GetActionBarOptionsOffset() + 1)
end
function AddConfigShowOptionForBar(bars, i, customName)
	if (customName) then
		bars[i].BarNameData = customName;
	end
	option = {
		type  = 'execute',
		name  = 'Show '..(customName or "Bar "..i),
		desc  = 'Show '..(customName or "Bar "..i),
		func  = GetShowOverlayFunction(bars, i),
		order = Config.GetActionBarOptionsOffset();
	}
	
	local options = Addon.GetOptions();
	options.args.mainOptions.args.barHotkeyDefaults.args["ShowBar_"..moduleName.."_"..i] = option;
	
	-- Update
	LibStub("AceConfigRegistry-3.0"):NotifyChange(AddonName.."-"..AddonVersion)
	LibStub("AceConfigDialog-3.0"):Open(AddonName.."-"..AddonVersion)
	
	Config.SetActionBarOptionsOffset(Config.GetActionBarOptionsOffset() + 1)
end

table.insert(Addon.ConfigOptionsFuncs, AddConfigOptions);
------------------------------------------


table.insert(Addon.InitializationFunctions, module.Main);
table.insert(Addon.EnableFunctions, module.Controller_OnEnable);
table.insert(Addon.DisableFunctions, module.Controller_OnDisable);

Addon.LoadModules();

-- For Debug use:
--module.Main()