-- Controller.lua
-- Handle the Bottom UI containing: ActionBarMain, ActionBar1, ActionBar2, ActionBarStance, XPBar, RepBar, HPBar, and the MPBar.

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local AddonName 	= "FoxSmoothUI";
local AddonVersion	= "3.1";
local Addon;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion)) then
	Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
else
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator	= LibStub:GetLibrary("FoxFrameCreator");
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
Addon.Right 				= Addon.Right or {};

local module 		= Addon.Right;
local moduleFrames	= Addon.Frames;


-- Safety Check
Addon.InitializationFunctions 	= Addon.InitializationFunctions or {};
Addon.EnableFunctions 			= Addon.EnableFunctions 		or {};
Addon.DisableFunctions 			= Addon.DisableFunctions 		or {};

local controllerFrameName 		= "FSUI_Right_Controller"

-- Function Definitions
local Create;
----------------------------------------------------------------------------------------------------

function Create()
	local name = controllerFrameName;
	local self;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name);
	else
		self = libFrameCreator.Setup(_G[name]);
	end
	
	self:SetBackdrop(nil);
	self:ClearAllPoints();
	self:SetPoint("Right", UIParent, "Right", 0, 0);
	
	
	return self;
end

function module.Main()
	if (module.Loaded) then
		return;
	end
	module.Loaded = true;
	
	-- Create ParentFrame (Secure)
	module.Controller = Create();
	
	-- Create ActionBar Frames
	module.ActionBarFrame 	= Addon.ActionBar.Create(module.ActionBarFrame, module.Controller, "Right", false, 3, 12, true, "FoxUI", "Right");
	module.ActionBarFrame:ClearAllPoints();
	module.ActionBarFrame:SetPoint("Right", 0, 0);
	
	-- Hide Frame
	module.Controller:Hide();
	
	-- Set Parent
	module.Controller:SetParent(Addon.Bottom.Controller);
	
	-- Add to Style Table
	table.insert(Addon.StyleFrames, module.ActionBarFrame);
	
end

function module.SetParent(parent)
	module.Controller:SetParent(parent);
end

function module.Controller_OnEnable()
	module.Controller:Show();
end
function module.Controller_OnDisable()
	module.Controller:Hide();
end


------------------------------------------
-- Config For Each Controller
------------------------------------------
local GetClearFunction;
local GetShowOverlayFunction;
local AddConfigClearOptionForBar;
local AddConfigShowOptionForBar;
local Config = Addon.Config;
local moduleName = "Right";

function AddConfigOptions()
	local bars = module.ActionBarFrame.Bars;
	
	local Config = Addon.Config;
	
	for i = 1, #bars do
		Config.SetActionBarOptionsCount(Config.GetActionBarOptionsCount() + 1);
		local name = "Bar "..Config.GetActionBarOptionsCount();
		AddConfigClearOptionForBar(bars, i, name, -1);
		AddConfigShowOptionForBar(bars,  i, name, -1);
		
	end
end
-- Clear Template Function
function GetClearFunction(bars, i, useKeys)
	local funcStr = [[
		local barName = "]].. bars[i]:GetName()..[["; 
		local self = _G[barName]; 
		local i 		= ]]..(i or -1)..[[;
		local AddonName 	= "]]..AddonName..[["; 
		local AddonVersion	= "]]..AddonVersion..[["; 
		local Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
		--do stuff 
		Addon.DebugPrint("ClearBarHotkeys("..(self:GetName() or "nil")..");");
		-- 2nd Bar
		if (]]..useKeys..[[) then
			Addon.SetupKeybindingDataBar(self, ]]..useKeys..[[);
		else
			Addon.SetupKeybindingDataBar(self);
		end
	]]
	local func, errorStr = loadstring(funcStr);
	if (errorStr) then
		error(errorStr);
	end
	
	return func;
end
-- Show Overlay Template Function
function GetShowOverlayFunction(bars, i)
	local funcStr = [[
		local barName 	= "]].. bars[i]:GetName()..[[";
		local self 		= _G[barName]; 
		local i 		= ]]..(i or -1)..[[;
		local AddonName 	= "]]..AddonName..[["; 
		local AddonVersion	= "]]..AddonVersion..[["; 
		local Addon = LibStub("AceAddon-3.0"):GetAddon(AddonName.." "..AddonVersion);
		--do stuff 
		Addon.DebugPrint("ShowBar("..(self:GetName() or "nil")..");");
		Addon.Bars.ShowOverlay(self, (self.BarNameData or "Bar "..i));
	]]
	local func, errorStr = loadstring(funcStr);
	if (errorStr) then
		error(errorStr);
	end
	
	return func;
end
function AddConfigClearOptionForBar(bars, i, customName, useKeys)
	if (customName) then
		bars[i].BarNameData = customName;
	end
	
	local option = {
		type = 'execute',
		name = 'Clear '..(customName or "Bar "..i),
		desc = 'Clear '..(customName or "Bar "..i),
		
		func = GetClearFunction(bars, i, useKeys),
		order = Config.GetActionBarOptionsOffset();
	}
	
	local options = Addon.GetOptions();
	
	options.args.mainOptions.args.barHotkeyDefaults.args["ClearBar_"..moduleName.."_"..i] = option;
	
	-- Update
	LibStub("AceConfigRegistry-3.0"):NotifyChange(AddonName.."-"..AddonVersion)
	
	Config.SetActionBarOptionsOffset(Config.GetActionBarOptionsOffset() + 1)
end
function AddConfigShowOptionForBar(bars, i, customName)
	if (customName) then
		bars[i].BarNameData = customName;
	end
	option = {
		type  = 'execute',
		name  = 'Show '..(customName or "Bar "..i),
		desc  = 'Show '..(customName or "Bar "..i),
		func  = GetShowOverlayFunction(bars, i),
		order = Config.GetActionBarOptionsOffset();
	}
	
	local options = Addon.GetOptions();
	options.args.mainOptions.args.barHotkeyDefaults.args["ShowBar_"..moduleName.."_"..i] = option;
	
	-- Update
	LibStub("AceConfigRegistry-3.0"):NotifyChange(AddonName.."-"..AddonVersion)
	LibStub("AceConfigDialog-3.0"):Open(AddonName.."-"..AddonVersion)
	
	Config.SetActionBarOptionsOffset(Config.GetActionBarOptionsOffset() + 1)
end

--table.insert(Addon.ConfigOptionsFuncs, 1, AddConfigOptions);
table.insert(Addon.ConfigOptionsFuncs, AddConfigOptions);
------------------------------------------


table.insert(Addon.InitializationFunctions, module.Main);
table.insert(Addon.EnableFunctions, module.Controller_OnEnable);
table.insert(Addon.DisableFunctions, module.Controller_OnDisable);

Addon.LoadModules();

-- For Debug use:
--module.Main()