local addonName, addonTable = ...
-- libFoxActionButton_ClassMethods.lua
-- Handles the Classes' Methods.

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local LibraryName		= addonTable.Name;
local LibraryVersion	= addonTable.Version;
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName.." "..LibraryVersion, true) ~= nil) then
	lib = LibStub:GetLibrary(LibraryName.." "..LibraryVersion);
else
	return;
end

----------------------------------------------------------------------------------------------------
--  Get Console
----------------------------------------------------------------------------------------------------
Console = LibStub:GetLibrary("FoxConsole");



local GetSpecData;
local SetSpecData;

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Class Methods
----------------------------------------------------------------------------------------------------
-- Action Methods
-----------------------
function lib.Methods.GetAction(self)
	-- Parameter Check
	if (self == nil) then
		return;
	end
	-- Get Values
	local kind 	= self:GetAttribute("type") or "";
	local value = self:GetAttribute(kind)  	or "";
	-- Return Values
	return kind, value;
end
function lib.Methods.SetAction(self, kind, value)
	if (InCombatLockdown()) then
		return;
	end
	-- Parameter Check
	if (self == nil) then
		return;
	end
	self:Clear();
	if ((kind == nil) or (value == nil) or (kind == "") or (value == "")) then
		if ((kind == "spell") or (kind == "item") or (kind == "macro") or (kind == "MOUNT") or (kind == "CRITTER")) then
			kind = nil;
		elseif (kind == nil) then
			value = nil;
		end
	end
	
	-- Set Values
	if ((kind ~= nil) and (kind ~= "")) then
		self:SetAttribute("type", kind);
		if (kind == "flyout") then
			self:SetAttribute("spell", value);
		elseif ((kind ~= nil) and (kind ~= "")) then
			self:SetAttribute(kind,   value);
		end
	
		local name = lib.Functions.GetActionName(self, kind, value);
		self:SetAttribute("type2", "macro");
		self:SetAttribute("*macrotext2", "/cancelaura "..(name or ""));	
	end
	
	self:ResetFlyout();
	
	-- Set Icon
	lib.Functions.SetIconFromAttribute(self);
end
function lib.Methods.Clear(self)
	if (InCombatLockdown()) then
		return;
	end
	self:SetAttribute("type", 		nil);
	self:SetAttribute("type1", 		nil);
	self:SetAttribute("type2", 		nil);
	self:SetAttribute("type*", 		nil);

	self:SetAttribute("action", 	nil);
	self:SetAttribute("*action1", 	nil)
	self:SetAttribute("spell", 		nil);
	self:SetAttribute("item", 		nil);
	self:SetAttribute("macro", 		nil);
	self:SetAttribute("macrotext",  nil);
	self:SetAttribute("macrotext1", nil);
	self:SetAttribute("macrotext2", nil);
	self:SetAttribute("battlepet",  nil);
	self.icon:SetTexture("");
end

-----------------------
-- Cooldown
-----------------------
function lib.Methods.ClearCooldown(self)
	if (self ~= nil) then
		if (self.cooldown ~= nil) then
			-- Handle Cooldown Animation
			CooldownFrame_SetTimer(self.cooldown, 0, 0, 0);
			
			self.cooldown.isSet = false;
		end
	end
end
function lib.Methods.Flash(self)
	--self.cooldown:SetCooldown(0, 0);
end

-----------------------
-- Keybinding
-----------------------
function lib.Methods.SetKeyBindHandler(self, func)
	-- Set the Keybind Handler Function
	self.keyBindHandlerFunc = func;
end
function lib.Methods.GetSaveKeyBind(self)
	return self.saveKeyBinding;
end
function lib.Methods.SetSaveKeyBind(self, value)
	self.saveKeyBinding = value;
end
function lib.Methods.UpdateKeybinding(self)
	lib.Functions.UpdateKeybinding(self);
end
function lib.Methods.SetKeybinding(self, hotkey)
	if (InCombatLockdown()) then
		return;
	end
	lib.Functions.SetKeybinding(self, hotkey);
	self:SetAttribute("hotkey", hotkey);
	lib.Functions.UpdateKeybinding(self);
end

-----------------------
-- Data
-----------------------
function lib.Methods.Save(self)
	if (self == nil) then
		return "", "", "";
	end
	-- Output: Kind, Value, Hotkey
	local kind 		= self:GetAttribute("type") or "";
	local value 	= self:GetAttribute(kind) 	or "";
	if (kind == "flyout") then
		value 	= self:GetAttribute("spell") 	or "";
	end
	local hotkey 	= self:GetAttribute("hotkey");
	
	--local spec1_kind, spec1_value, spec2_kind, spec2_value = GetSpecData(self);
	--local hotkey 	= self:GetAttribute("hotkey");
	--local hotkey 	= self:GetAttribute("hotkey");
	
	if (self:GetAttribute("battlepet")) then
		kind 	= "battlepet";
		value 	= self:GetAttribute("battlepetid");
	end
	
	
	
	return kind, value, hotkey, self.customType, self.customUsingTexture, self.customTextureIndex, self.macroText;
end
function lib.Methods.Load(self, kind, value, hotkey, customType, customUsingTexture, customTextureIndex, macroText)
	if (kind == "battlepet") then
		local text = "/run C_PetJournal.SummonPetByGUID(\'"..value.."\');";

		self:SetAttribute("battlepetid", value);

		self:SetAttribute("type", "macro");
		self:SetAttribute("macrotext", text);
		self:SetAttribute("battlepet", true);
	else
		-- Set Action
		lib.Methods.SetAction(self, kind, value);
	end
	-- Set Hotkey
	hotkey = hotkey or "";
	
	if (hotkey ~= "") then
		lib.Functions.SetKeybinding(self, hotkey);
		self:SetAttribute("hotkey", hotkey);
		lib.Functions.UpdateKeybinding(self);
	end
	
	
	
	self.customType 		= customType;
	self.customUsingTexture = customUsingTexture;
	self.customTextureIndex 		= customTextureIndex;
	if (macroText) then
		self.macroText = macroText;
		self:SetAttribute("type", "macro");
		self:SetAttribute("macrotext", macroText);
	end
	
	
	--SetSpecData(self, spec1_kind, spec1_value, spec2_kind, spec2_value);
end

function GetSpecData(button)
	local spec1_kind 	= button:GetAttribute("type_s1");
	local spec1_value 	= nil;
	if (spec1_kind ~= nil) then
		if (spec1_kind ~= "flyout") then
			spec1_value = button:GetAttribute(spec1_kind.."_s1");
		else
			spec1_value = button:GetAttribute("spell".."_s1");
		end
	end


	local spec2_kind 	= button:GetAttribute("type_s1");
	local spec2_value 	= nil;
	if (spec2_kind ~= nil) then
		if (spec2_kind ~= "flyout") then
			spec2_value = button:GetAttribute(spec2_kind.."_s1");
		else
			spec2_value = button:GetAttribute("spell".."_s1");
		end
	end
	
	return spec1_kind, spec1_value, spec2_kind, spec2_value;
end

function SetSpecData(button, spec1_kind, spec1_value, spec2_kind, spec2_value)
	button:SetAttribute("type_s1", spec1_kind);
	if (spec1_kind ~= nil) then
		if (spec1_kind ~= "flyout") then
			button:SetAttribute(spec1_kind.."_s1", spec1_value);
		else
			button:SetAttribute("spell".."_s1", spec1_value);
		end
	end
	
	button:SetAttribute("type_s2", spec2_kind);
	if (spec2_kind ~= nil) then
		if (spec2_kind ~= "flyout") then
			button:SetAttribute(spec1_kind.."_s2", spec2_value);
		else
			button:SetAttribute("spell".."_s2", spec2_value);
		end
	end
end

-----------------------
-- Update
-----------------------
function lib.Methods.UpdateIcon(self)
	lib.Functions.SetIconFromAttribute(self);
end
function lib.Methods.UpdateButton(self)
	lib.Functions.UpdateChanges(self);
end
function lib.Methods.UpdateAllHotkeys(self, frameName, hotkey)
	if (self.updateAllHotkeysFunc ~= nil) then
		self.updateAllHotkeysFunc(frameName, hotkey);
	end
end

-----------------------
-- Flyout
-----------------------
function lib.Methods.ShowFlyout(self)
	self.forceShowFlyout = true;
	lib.Functions.UpdateFlyout(self);
end
function lib.Methods.ResetFlyout(self)
	self.forceShowFlyout = false;
	self.flyout:SetVertexColor(1, 1, 1)
	lib.Functions.UpdateFlyout(self);
end

-----------------------
-- Special
-----------------------
function lib.Methods.MakeLikeOrigButton(self, id)
	self:SetAttribute("type", "action");
	self:SetID(id);
	
	local _, className = UnitClass("player");
	self:SetAttribute("playerUnitClass", className);
	
	--SecureHandlerUnwrapScript(self, "OnAttributeChanged");
	
	-- Register the Drivers
	RegisterStateDriver(self, "isstealth", 	"[stealth] true; false;");
	--RegisterStateDriver(self, "iscat", 		"[form:3] cat; notcat;");
	RegisterStateDriver(self, "form", 		"[form:0] 0; [form:1] 1; [form:2] 2; [form:3] 3; [form:4] 4; [form:5] 5; 1");
	RegisterStateDriver(self, "pages", "[actionbar:1] 1; [actionbar:2] 2; [actionbar:3] 3; [actionbar:4] 4; [actionbar:5] 5; [actionbar:6] 6; 1");
	
	RegisterStateDriver(self, "bonuspages", "[bonusbar:1] 1; [bonusbar:2] 2; [bonusbar:3] 3; [bonusbar:4] 4; [bonusbar:5] 5; [bonusbar:6] 6; 1");
	
	
	--self:SetAttribute("_onstate-pages", lib.Variables.StateChanges);
	self:SetAttribute("_onstate-pages", "print(self, stateid, value)");
	self:SetAttribute("_onstate", "print(self, stateid, value)");
	
	--RegisterStateDriver(self, "pages", "[actionbar:1] page1; [actionbar:2] page2; [actionbar:3] page3; [actionbar:4] page4; [actionbar:5] page5; [actionbar:6] page6; otherpage");
	
	--SecureHandlerUnwrapScript(self, "OnAttributeChanged");
	SecureHandlerWrapScript(self, "OnAttributeChanged", self, lib.Variables.OnAttributeChangedPre);
	
	self:SetAttribute("needsUpdate", true);
	
	
	-- Show
	self:Show();
	
	-- Update
	lib.Functions.UpdateChanges(self);
end
----------------------------------------------------------------------------------------------------

