local addonName, addonTable = ...
-- libFoxActionButton_ClassMethods.lua
-- Handles the Classes' Methods.

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local LibraryName		= addonTable.Name;
local LibraryVersion	= addonTable.Version;
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName.." "..LibraryVersion, true) ~= nil) then
	lib = LibStub:GetLibrary(LibraryName.." "..LibraryVersion);
else
	return;
end

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator    = LibStub:GetLibrary("FoxFrameCreator");
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Still Needed
--[[------------------------------------------------------------------------------------------------

 -N- Open ButtonConfig via Alt+Shift+Right Click
 -N- Close ButtonConfig via Close Button
 -N- Save ("Export") Settings to Button
 -N- Add Save to DataManager
 
 -M- Make Config Auto-Hide if user enters Combat, and Show afterwards
 -L- Make Active Button Glow
 

 - Next Features
 - Custom Flyouts
  - Handle Normal Button Data (Spells, Items, Macros, ect.) and Custom Button Data (via ButtonConfig)
 
]]--------------------------------------------------------------------------------------------------

local MACRO_ICON_FILENAMES = {};

local rowCount = 8;
local numberOfRows = 4;
local buttonCount = numberOfRows * rowCount;

local ConfigButton_OnClick;
local IconButton_OnClick;
local UseIconCheckBox_OnClick;
local Icon_OnClick;

local CreateCustomButtonConfigFrame;
local CreateConfigButtonFrame;
local CreateMacroTextFrame;
local CreateIconSelectionFrame;
local CreateIconSelectionButtons;

local GetMacroText;
local UseIcon;

local SelectIcon;
local SaveConfigToButton;
local SetEditBoxMacroText;

local RefreshPlayerSpellIconInfo;
local GetSpellorMacroIconInfo;

-- Items for DropDown
local dropDownItems = {
   "Normal",
   --"Custom Flyout",
   "Macro Text",
};

function ConfigButton_OnClick(self)
	local parent 	= self:GetParent();
	local index 	= UIDropDownMenu_GetSelectedID(parent.typeDropDown);
	local value 	= self.items[index];
	if (value == "Macro Text") then
		-- Show Macro Text Box
		if (not parent.MacroTextFrame:IsShown()) then
			parent.MacroTextFrame:Show();
			local macrotext = parent:GetMacroText();
			SetEditBoxMacroText(parent, macrotext);
			
			-- Hide Icon Frame
			parent.IconSelectionFrame:Hide();
			
			parent:SetWidth(612);
		else
			parent:SetWidth(260);
			parent.MacroTextFrame:Hide();
		end
		
	elseif (value == "Custom Flyout") then
		parent.MacroTextFrame:Hide();
		
		-- Show Flyout Menu
		
	else
		parent:SetWidth(260);
		
		-- Hide Everything
		parent.MacroTextFrame:Hide();
		-- Hide Icon Frame
		parent.IconSelectionFrame:Hide();
		
	end
end
function IconButton_OnClick(self)
	local parent 	= self:GetParent();
	local index 	= UIDropDownMenu_GetSelectedID(parent.typeDropDown);
	local value 	= self.items[index];
	
	if (value == "Macro Text") then
		if (not parent.IconSelectionFrame:IsShown()) then
			parent:SetWidth(625);
			parent.IconSelectionFrame:Show();
			parent.MacroTextFrame:Hide();
			if (parent.SelectedIconIndex == nil) then
				parent:SelectIcon(1);
			end
			
		else
			parent:SetWidth(260);
			parent.IconSelectionFrame:Hide();
		end
	end
end
function UseIconCheckBox_OnClick(self)
	local actionButtonConfigFrame = self:GetParent();
	
	if (actionButtonConfigFrame:UseIcon()) then
		if (actionButtonConfigFrame.SelectedIconIndex) then
			local index = actionButtonConfigFrame.SelectedIconIndex;
			local numMacroIcons = #MACRO_ICON_FILENAMES;
			local texture = GetSpellorMacroIconInfo(index);
			if ( index <= numMacroIcons and texture ) then
				if(type(texture) == "number") then
					actionButtonConfigFrame.texture:SetToFileData(texture);
					return;
				else
					actionButtonConfigFrame.texture:SetTexture("INTERFACE\\ICONS\\"..texture);
					return;
				end
			end
		end
	end
	local icon = actionButtonConfigFrame.actionButton.icon:GetTexture();
	if (icon) then
		-- Set Button Texture
		actionButtonConfigFrame.texture:SetTexture(icon);
	end
end
function Icon_OnClick(self, button)
	local iconSelectionFrame = self:GetParent();
	local actionButtonConfigFrame = iconSelectionFrame:GetParent();
	local scrollFrame = iconSelectionFrame.scrollFrame;
	actionButtonConfigFrame:SelectIcon(self:GetID() + (FauxScrollFrame_GetOffset(scrollFrame) * rowCount), true);
end

function CreateCustomButtonConfigFrame(self)
	
	----------------------------------------------
	-- Setup Primary Frames
	----------------------------------------------
	-- Create Base Frame
	local self = CreateConfigButtonFrame();
	self:SetClampedToScreen(true);
	
	----------------------------------------------
	-- Setup Secondary Frames
	----------------------------------------------
	----------------------------------------------
	--+++  Setup MacroText Frame
	----------------------------------------------
	self.MacroTextFrame = CreateMacroTextFrame(self.MacroTextFrame, self);
	
	-- Create the Custom Flyout Frame Creator
		-- Options for Setup of each button, and each button can be setup individually
			-- Drag & Drop
			-- Manual Entry
	
	
	-- Icon Choice Popout
		-- Search Option Eventually
		
	----------------------------------------------
	--+++  Setup MacroText Frame
	----------------------------------------------
	self.IconSelectionFrame = CreateIconSelectionFrame(self.IconSelectionFrame, self);
	
	self.SelectIcon = SelectIcon;
	self:SelectIcon(1);
	
	self:Hide();
	
	return self;
end

function CreateConfigButtonFrame()
	local name = "ActionButtonConfigFrame";
	local self;
	if (_G[name] == nil) then
	   self = libFrameCreator.New(name);
	else
	   self = libFrameCreator.Setup(_G[name]);
	end
	self:SetBackdrop(nil);
	self:ClearAllPoints();
	self:SetPoint("Center", 200, 0);
	self:SetWidth(260);
	--self:SetWidth(625);
	
	name = name.."Background";
	if (_G[name] == nil) then
	   self.background = libFrameCreator.New(name);
	else
	   self.background = libFrameCreator.Setup(_G[name]);
	end
	self.background:ClearAllPoints();
	self.background:SetPoint("TopLeft", self, "TopLeft", 0, 0);
	self.background:SetWidth(260);
	self.background:SetParent(self);
	self.background:SetFrameLevel(0);
	
	-- Function Declarations
	self.GetMacroText = GetMacroText;
	
	-- Title Label
	local titleText = "Button Config";
	self.labelTitle = libFrameCreator.CreateLabel(self.labelTitle, self, titleText, "Top", self.background, "Top", 0, 0, self:GetWidth(), 20, "Center", "Middle");

	-- Icon Texture
	self.texture = self.texture or self:CreateTexture(nil, "Artwork");
	self.texture:ClearAllPoints();
	self.texture:SetPoint("TopLeft", 8, -22);
	self.texture:SetTexture("Interface/ICONS/INV_Misc_QuestionMark");
	
	-- Type Selection DropDown
	local typeText = "Type:";
	self.labelType = libFrameCreator.CreateLabel(self.labelType, self, typeText, "TopLeft", self.texture, "TopRight", 8, -8, 34, 20, "Left", "Middle");
	
	-- Add DropDown - Normal, Custom Flyout, MacroText
	if not self.typeDropDown then
	   self.typeDropDown = CreateFrame("Button", self:GetName().."TypeDropDown", UIParent, "UIDropDownMenuTemplate");
	end
	self.typeDropDown:ClearAllPoints();
	self.typeDropDown:SetPoint("Left", self.labelType, "Right", -4, 0);
	self.typeDropDown:SetParent(self);
	
	self.typeDropDown:Show();
	
	self.typeDropDown.hasInit = false;
	local items = dropDownItems;
	self.items = items;
	
	
	-- Button Type
	-- Disabled if DropDown == Normal
	local typeConfigText = "Config:";
	self.labelTypeConfig = libFrameCreator.CreateLabel(self.labelTypeConfig, self, typeConfigText, "TopLeft", self.texture, "TopRight", 8, -34, 44, 20, "Left", "Middle");

	-- Button
	--   Custom Flyout: Pops a new Flyout Frame
	--   MacroText:     Pops a Window for Entering Macro Text
	self.btnConfig = self.btnConfig or CreateFrame("Button", self:GetName().."BtnConfig", self, "UIPanelButtonTemplate");

	-- Location Info
	self.btnConfig:ClearAllPoints();
	self.btnConfig:SetPoint("Left", self.labelTypeConfig, "Right", 2, 0);
	
	-- Parent Info
	self.btnConfig:SetParent(self);
	
	-- Size
	self.btnConfig:SetWidth(120);
	self.btnConfig:SetHeight(16);
	
	-- Font & Text
	self.btnConfig:GetFontString():SetFont("Fonts\\FRIZQT__.TTF", 10, "Outline");
	self.btnConfig:SetText("Config");
	
	self.typeDropDown:SetFrameLevel(2)
	
	-- Items for Scripts
	self.btnConfig.items = items;
	self.btnConfig:SetScript("OnClick", ConfigButton_OnClick);
	self.btnConfig:Disable();
	
	
	-- Icon Selection Label
	local iconText = "Icon:";
	self.labelIcon = libFrameCreator.CreateLabel(self.labelIcon, self, iconText, "TopLeft", self.texture, "TopRight", 8, -54, 44, 20, "Left", "Middle");
	
	-- Icon Selection Menu
	-- Disabled if DropDown ~= MacroText
	self.btnIcon = self.btnIcon or CreateFrame("Button", self:GetName().."BtnIcon", self, "UIPanelButtonTemplate");
	
	-- Location Info
	self.btnIcon:ClearAllPoints();
	self.btnIcon:SetPoint("Left", self.labelIcon, "Right", 2, 0);
	
	-- Parent Info
	self.btnIcon:SetParent(self);
	
	-- Size
	self.btnIcon:SetWidth(120);
	self.btnIcon:SetHeight(16);
	
	-- Font & Text
	self.btnIcon:GetFontString():SetFont("Fonts\\FRIZQT__.TTF", 10, "Outline");
	self.btnIcon:SetText("Set Icon");
	
	self.typeDropDown:SetFrameLevel(2)
	
	-- Items for Scripts
	self.btnIcon.items = items;
	self.btnIcon:SetScript("OnClick", IconButton_OnClick);
	self.btnIcon:Disable();
	
	-- Use Icon Label
	local useIconText = "Use Custom Icon:";
	self.labelUseIcon = libFrameCreator.CreateLabel(self.labelUseIcon, self, useIconText, "TopLeft", self.texture, "TopRight", 8, -74, 100, 20, "Left", "Middle");
	
	-- Use Icon Checkbox
	self.checkbox = self.checkbox or CreateFrame("CheckButton", self:GetName().."CheckBox", self, "ChatConfigCheckButtonTemplate");
	
	-- Parent Info
	self.checkbox:SetParent(self);
	
	-- Location Info
	self.checkbox:ClearAllPoints();
	self.checkbox:SetPoint("TopLeft", self.labelUseIcon, "TopRight", 0, 0);
	
	-- Script Info
	self.checkbox:SetScript("OnClick", UseIconCheckBox_OnClick);
	self.checkbox:Disable();
	self.checkbox:SetChecked(false);
	
	-- Add Method
	self.UseIcon = UseIcon;
	
	
	
	-- Close Button
	--   Closes the Config Window
	self.btnClose = self.btnClose or CreateFrame("Button", self:GetName().."BtnClose", self, "UIPanelCloseButton");

	-- Location Info
	self.btnClose:ClearAllPoints();
	self.btnClose:SetPoint("TopLeft", self.background, "TopRight", -32, 0);
	
	-- Parent Info
	self.btnClose:SetParent(self);
	
	
	-- Save Button
		-- Button Exports Current Settings to the Active Button
	self.btnSave = self.btnSave or CreateFrame("Button", self:GetName().."BtnSave", self, "UIPanelButtonTemplate");

	-- Location Info
	self.btnSave:ClearAllPoints();
	self.btnSave:SetPoint("Left", self.texture, "Right", 8, 0);
	self.btnSave:SetPoint("Top", self.labelUseIcon, "Bottom", 4, 0);
	
	-- Parent Info
	self.btnSave:SetParent(self);
	
	-- Size
	self.btnSave:SetWidth(166);
	self.btnSave:SetHeight(24);
	
	-- Font & Text
	self.btnSave:GetFontString():SetFont("Fonts\\FRIZQT__.TTF", 10, "Outline");
	self.btnSave:SetText("Save");
	
	-- Items for Scripts
	--self.btnSave.items = items;
	self.btnSave:SetScript("OnClick", SaveConfigToButton);
	--self.btnSave:Disable();
	
	
	-- Arrow Texture
	self.arrow = self.arrow or self:CreateTexture(nil, "OVERLAY");
	
	-- Size Info
	self.arrow:SetWidth(32);
	self.arrow:SetHeight(16);
	
	-- Texture Info
	self.arrow:SetTexture("Interface\\Buttons\\ActionBarFlyoutButton");
	self.arrow:SetTexCoord(0.625, 0.984375, 0.74218750, 0.82812500);
	
	-- Ensure Arrow is over Button
	self:SetFrameStrata("DIALOG");
	self:SetFrameLevel(3);
	
	
	-- Return Frame
	return self;
end
function CreateMacroTextFrame(self, parent)
	self = self or CreateFrame("Frame", parent:GetName().."_MacroTextFrame", parent);
	
	-----------------
	-- Set Background
	libFrameCreator.SetupFrameWithBorder(self);
	
	-- Location Info
	self:ClearAllPoints();
	self:SetPoint("TopLeft", parent.background, "TopRight", 1, 0);
	
	-- Parent Info
	self:SetParent(parent);
	
	-- Size Info
	self:SetWidth(350);
	self:SetHeight(200);
	-----------------
	
	
	-----------------
	-- Title Label
	local labelmacroText_Text = "Macro Text";
	self.labelmacroText = libFrameCreator.CreateLabel(self.labelmacroText, self, labelmacroText_Text, "Top", self, "Top", 0, 0, 80, 20, "Left", "Middle");
	-----------------
	
	
	-----------------
	-- Scroll Info
	local scrollFrameHeight = 168;
	
	-- Add Scroll Frame Parent for MacroText EditBox
	self.scrollFrame = self.scrollFrame or CreateFrame("ScrollFrame", self:GetName().."ScrollFrame", self, "UIPanelScrollFrameTemplate");
	
	-- Parent Info
	self.scrollFrame:SetParent(self);
	
	-- Location Info
	self.scrollFrame:ClearAllPoints();
	self.scrollFrame:SetPoint("Top", self, "Top", -8, -24);
	
	-- Size Info
	self.scrollFrame:SetWidth(286);
	self.scrollFrame:SetHeight(scrollFrameHeight);
	-----------------
	
	
	-----------------
	-- Add EditBox for MacroText
	self.EditBox = self.EditBox or CreateFrame("EditBox", self:GetName().."EditBox", self);
	
	-- Set Parent
	self.EditBox:SetParent(self.scrollFrame);
	self.scrollFrame:SetScrollChild(self.EditBox);
	
	-- Location Info
	self.EditBox:ClearAllPoints();
	--self.EditBox:SetPoint("TOPLEFT", self, "TOPRight", 0, 0)
	self.EditBox:SetPoint("TOPLEFT", self.scrollFrame, "TopLeft", 0, 0);
	
	-- Size Info
	self.EditBox:SetWidth(286);
	self.EditBox:SetHeight(85);
	
	-- Other Info
	self.EditBox:SetFrameStrata("DIALOG");
	self.EditBox:SetMaxLetters(255);
	self.EditBox:SetMultiLine(true);
	self.EditBox:SetAutoFocus(false);
	self.EditBox:SetTextInsets(5, 5, 4, 4);
	self.EditBox:SetFontObject("GameFontHighlightSmall");
	--self.EditBox:SetFocus();
	
	-- Script Info
	self.EditBox:SetScript("OnEscapePressed", EditBox_ClearFocus);
	self.EditBox:SetScript("OnTextChanged", function(self) ScrollingEdit_OnTextChanged(self, self:GetParent()); end);
	self.EditBox:SetScript("OnCursorChanged", ScrollingEdit_OnCursorChanged);
	self.EditBox:SetScript("OnUpdate", function(self) ScrollingEdit_OnUpdate(self, elapsed, self:GetParent()); end);
	
	

	local function LinkGrab_Spell(self, button)
		local frame = _G["ActionButtonConfigFrame"].MacroTextFrame;
		local slot = SpellBook_GetSpellBookSlot(self);
		if ( slot > MAX_SPELLS ) then
			return;
		end
		if ( IsModifiedClick("CHATLINK") ) then
			if ( frame.EditBox and frame.EditBox:HasFocus() ) then
				local spellName, subSpellName = GetSpellBookItemName(slot, SpellBookFrame.bookType);
				if ( spellName and not IsPassiveSpell(slot, SpellBookFrame.bookType) ) then
					if ( subSpellName and (strlen(subSpellName) > 0) ) then
						local text = "/use "..spellName.."("..subSpellName..")";
						frame.EditBox:Insert(text);
					else
						local text = "/use "..spellName;
						frame.EditBox:Insert(text);
					end
				end
			end
		end
	end

	hooksecurefunc("SpellButton_OnModifiedClick", LinkGrab_Spell);
	
	local function LinkGrab_Toy(self, button) 
		local frame = _G["ActionButtonConfigFrame"].MacroTextFrame;
		if ( IsModifiedClick("CHATLINK") ) then
			local itemLink = C_ToyBox.GetToyLink(self.itemID);
			if ( itemLink ) then
				local itemName = GetItemInfo(itemLink)
				local text = "/use "..itemName;
				frame.EditBox:Insert(text);
			end
		end
	end
	hooksecurefunc("ToySpellButton_OnModifiedClick", LinkGrab_Toy);
	-----------------
	
	
	
	-----------------
	-- Create Background for MacroText Edit Field
	self.EditBoxBack = self.EditBoxBack or CreateFrame("Frame", self:GetName().."EditBoxBack", self);
	libFrameCreator.SetupFrameWithBorder(self.EditBoxBack);
	
	self.EditBoxBack:SetParent(self.EditBox);
	
	self.EditBoxBack:ClearAllPoints();
	self.EditBoxBack:SetPoint("TopLeft", self.EditBox, "TopLeft", 0, 0);
	
	self.EditBoxBack:SetWidth(286);
	self.EditBoxBack:SetHeight(scrollFrameHeight);
	
	self.EditBoxBack:SetBackdropColor(0, 0, 0, 1);
	self.EditBoxBack:SetFrameStrata("LOW");
	-----------------
	
	
	-----------------
	-- Add a Button to set the focus to the MacroText Edit Field
	self.EditBoxBtn = self.EditBoxBtn or CreateFrame("Button", self.EditBox:GetName().."Btn", self);
	
	self.EditBoxBtn:ClearAllPoints();
	self.EditBoxBtn:SetPoint("TopLeft", 0, 0);
	
	self.EditBoxBtn:SetWidth(322);
	self.EditBoxBtn:SetHeight(scrollFrameHeight);
	
	--self.EditBoxBtn:SetParent(self.EditBox);
	
	self.EditBoxBtn.box = self.EditBox;
	self.EditBoxBtn:SetScript("OnClick", function(self) self.box:SetFocus(); end);
	-----------------
	
	
	-- Set Methods
	parent.SetEditBoxMacroText = SetEditBoxMacroText;
	
	
	-- Hide Parent Scroll Frame
	self:Hide();
	
	
	-- Return frame
	return self;
end
function CreateIconSelectionFrame(self, parent)
	self = self or CreateFrame("Frame", parent:GetName().."IconSelectionFrame", self);
	
	-----------------
	-- Background
	libFrameCreator.SetupFrameWithBorder(self);
	
	-- Parent Info
	self:SetParent(parent);
	
	-- Location Info
	self:ClearAllPoints()
	self:SetPoint("TopLeft", parent.background, "TopRight", 1, 0);
	
	-- Size Info
	self:SetWidth(364);
	self:SetHeight(200);
	-----------------
	
	-----------------
	-- Scroll Info
	local scrollFrameHeight = 164;
	
	-- Add Scroll Frame Parent for MacroText EditBox
	self.scrollFrame = self.scrollFrame or CreateFrame("ScrollFrame", self:GetName().."ScrollFrame", self, "ListScrollFrameTemplate");
	
	-- Parent Info
	self.scrollFrame:SetParent(self);
	
	-- Location Info
	self.scrollFrame:ClearAllPoints();
	self.scrollFrame:SetPoint("Top", self, "Top", 0, -30);
	
	-- Size Info
	self.scrollFrame:SetWidth(294);
	self.scrollFrame:SetHeight(scrollFrameHeight);
	
	-- Scripts Info
	self.scrollFrame:SetScript("OnVerticalScroll", function(self, offset) FauxScrollFrame_OnVerticalScroll(self, offset, 36, IconSelectionButtons_Update); end);
	-----------------
	
	RefreshPlayerSpellIconInfo();
	
	self:Hide();
	
	-- Create Buttons
	CreateIconSelectionButtons(self);
	
	local numMacroIcons = #MACRO_ICON_FILENAMES;
	FauxScrollFrame_Update(self.scrollFrame, ceil(numMacroIcons / rowCount), numberOfRows, 36);
	--FauxScrollFrame_Update(frame, numItems, numToDisplay, buttonHeight, button, smallWidth, bigWidth, highlightFrame, smallHighlightWidth, bigHighlightWidth, alwaysShowScrollBar )
	
	return self;
end
function CreateIconSelectionButtons(self)
	self.iconbuttons = self.iconbuttons or {}
	
	
	
	local lastButton = nil;
	local firstRowButton = nil;
	-- Create For Loop to handle Creation
		-- Create ~ 20 buttons for Icon Selection
	for i = 1, buttonCount, 1 do
		self.iconbuttons[i] = self.iconbuttons[i] or CreateFrame("CheckButton", self:GetName().."IconButton"..i, self, "MacroPopupButtonTemplate");
	
		self.iconbuttons[i]:SetParent(self);
		
		self.iconbuttons[i]:ClearAllPoints();
		if (lastButton == nil) then
			self.iconbuttons[i]:SetPoint("TopLeft", self, "TopLeft", 12, -30);
		else
			self.iconbuttons[i]:SetPoint("TopLeft", lastButton, "TopRight", 4, 0);
		end
		
		local index_Y = math.floor(i / (rowCount));
		if (index_Y < 1) then
			index_Y = 1;
		end
		local index_X = i % (rowCount * index_Y + 1);
		
		if (index_X == 0) then
			if (i ~= 1) then
				self.iconbuttons[i]:ClearAllPoints();
				self.iconbuttons[i]:SetPoint("TopLeft", firstRowButton, "BottomLeft", 0, -6);
			end
			firstRowButton = self.iconbuttons[i];
		end
		if (i == 1) then
			firstRowButton = self.iconbuttons[i];
		end
		
		self.iconbuttons[i].icon = _G[self.iconbuttons[i]:GetName().."Icon"];
		
		self.iconbuttons[i]:SetID(i);
		
		local texture = GetSpellorMacroIconInfo(i)
		if (texture) then
			self.iconbuttons[i].icon:SetTexture("Interface\\Icons\\"..texture);
		end
		
		self.iconbuttons[i]:SetScript("OnClick", Icon_OnClick)
		
		
		lastButton = self.iconbuttons[i];
	end
end

function GetMacroText(self)
	if (self.actionButton) then
		return self.actionButton:GetAttribute("macrotext");
	else
		return "";
	end
end
function UseIcon(self)
	return self.checkbox:GetChecked();
end

function SelectIcon(self, index, setUseIcon)
	self.SelectedIconIndex = index;
	IconSelectionButtons_Update(self.IconSelectionFrame.scrollFrame);
	
	if (setUseIcon) then
		self.checkbox:SetChecked(true);
	end
	
	local texture = GetSpellorMacroIconInfo(index);
	local numMacroIcons = #MACRO_ICON_FILENAMES;
	if (self:UseIcon()) then
		if (index ~= nil) then
			if ( index <= numMacroIcons and texture ) then
				if(type(texture) == "number") then
					self.texture:SetToFileData(texture);
				else
					self.texture:SetTexture("INTERFACE\\ICONS\\"..texture);
				end
			end
		end
	end
end
function SaveConfigToButton(self)
	local buttonPressed = self;
	local parent = self:GetParent();
	local self = parent;
	
	local button = self.actionButton;
	local selectedTypeId = self.typeDropDown.selectedID;
	local customType = self.items[selectedTypeId];
	button.customType = customType;
	
	if (customType == "Macro Text") then
		button.customUsingTexture = self:UseIcon();
		button.customTextureIndex = self.SelectedIconIndex;
		lib.Functions.SetIconFromAttribute(button);
		
		local macroText = self.MacroTextFrame.EditBox:GetText();
		
		button.macroText = macroText;
		button:SetAttribute("type", "macro");
		button:SetAttribute("macrotext", macroText);
		
	else
		button.customUsingTexture = nil;
		button.customTextureIndex = nil;
		button.macroText = nil;
		
		if ((button:GetAttribute("macrotext") ~= nil) and (button:GetAttribute("macrotext") ~= "")) then
			button:SetAttribute("type", nil);
			button:SetAttribute("macrotext", nil);
		end
	end
	
end
function SetEditBoxMacroText(self, text)
	text = text or "";
	self.MacroTextFrame.EditBox:SetText(text)
end

function IconSelectionButtons_Update(self)
	local iconSelectionFrame = self:GetParent();
	local actionButtonConfigFrame = iconSelectionFrame:GetParent();
	
	local parent = iconSelectionFrame;
	
    local numMacroIcons = #MACRO_ICON_FILENAMES;
    local macroPopupIcon, macroPopupButton;
	
    local macroPopupOffset = FauxScrollFrame_GetOffset(self);
    local index;
	 
    -- Icon list
    local texture;
    for i=1, buttonCount do
		
		macroPopupIcon 		= parent.iconbuttons[i].icon;
        macroPopupButton 	= parent.iconbuttons[i];
		
		index = (macroPopupOffset * rowCount) + i;
        texture = GetSpellorMacroIconInfo(index);
		
        if ( index <= numMacroIcons and texture ) then
			if(type(texture) == "number") then
                macroPopupIcon:SetToFileData(texture);
            else
				macroPopupIcon:SetTexture("INTERFACE\\ICONS\\"..texture);
			end
            macroPopupButton:Show();
        else
			macroPopupIcon:SetTexture("");
            macroPopupButton:Hide();
        end
        if (actionButtonConfigFrame.SelectedIconIndex and (index == actionButtonConfigFrame.SelectedIconIndex) ) then
            macroPopupButton:SetChecked(1);
        elseif ( MacroPopupFrame.selectedIconTexture ==  texture ) then
            --macroPopupButton:SetChecked(1);
        else
            macroPopupButton:SetChecked(nil);
        end
    end
	
    -- Scrollbar stuff
    FauxScrollFrame_Update(parent.scrollFrame, ceil(numMacroIcons / rowCount), numberOfRows, 36);
end
function UpdateConfigFrame(self, button)
	self.actionButton = button;
	local icon = button.icon:GetTexture();
	if (icon) then
		-- Set Button Texture
		self.texture:SetTexture(icon);
	else
		self.texture:SetTexture("Interface/ICONS/INV_Misc_QuestionMark");
	end
	
	self:ClearAllPoints();
	self:SetPoint("Bottom", self.actionButton, "Top", 0, 0);

	-- Location Info
	self.arrow:ClearAllPoints();
	self.arrow:SetPoint("Bottom", self.actionButton, "Top", 0, -8);
	
	if (self.typeDropDown.hasInit == false) then
		-- Items for DropDown
		local items = dropDownItems
		
		local name = self.typeDropDown:GetName();
		local funcStr = [[ 
		return function(self) 
			local name = "]]..name..[[";
			local frame = _G[name];
			UIDropDownMenu_SetSelectedID(frame, self:GetID());
			
			local parent = frame:GetParent();
			parent.MacroTextFrame:Hide();
			parent.IconSelectionFrame:Hide();
			
			parent:SetWidth(260);
			parent:ClearAllPoints();
			parent:SetPoint("Bottom", parent.actionButton, "Top", 0, 0);
			
			local option = parent.items[self:GetID()];
			if (option == "Macro Text") then
				parent.btnConfig:Enable();
				parent.btnIcon:Enable();
				parent.checkbox:Enable();
				parent.checkbox:SetChecked(false);
			else
				parent.btnConfig:Disable();
				parent.btnIcon:Disable();
				parent.checkbox:Disable();
				parent.checkbox:SetChecked(false);
			end
		end
		]]
		
		local func, errorMessage = loadstring(funcStr);
		if (errorMessage) then
		   print(errorMessage);
		end
		
		local OnClick = func();
		
		local function initialize(self, level)
		   local info = UIDropDownMenu_CreateInfo();
		   for k,v in pairs(items) do
			  info = UIDropDownMenu_CreateInfo();
			  info.text = v;
			  info.value = v;
			  info.func = OnClick;
			  info.owner = self;
			  UIDropDownMenu_AddButton(info, level);
		   end
		end
		
		-- DropDown Menu Setup
		UIDropDownMenu_Initialize(self.typeDropDown, initialize);
		UIDropDownMenu_SetWidth(self.typeDropDown, 100);
		UIDropDownMenu_SetButtonWidth(self.typeDropDown, 124);
		UIDropDownMenu_SetSelectedID(self.typeDropDown, 1);
		UIDropDownMenu_JustifyText(self.typeDropDown, "LEFT");
		--self.typeDropDown:SetFrameStrata("DIALOG");
		
		self.typeDropDown.hasInit = true;
	end
	
	if ((not self.mountNotInit) and (IsPlayerInWorld())) then
		if ( not PetJournalParent ) then
			PetJournal_LoadUI();
		end
		
		local function LinkGrab_Mount(self, button)
			local frame = _G["ActionButtonConfigFrame"].MacroTextFrame;
			if ( IsModifiedClick("CHATLINK") ) then
				local id = self.spellID;
				if ( frame and frame:IsShown() ) then
					local spellName = GetSpellInfo(id);
					local text = "/use "..spellName;
					frame.EditBox:Insert(text);
				end
			end
		end
		
		for i=1, 11, 1 do
			local buttonName = "MountJournalListScrollFrameButton"..i
			button = _G[buttonName];
			button:HookScript("OnClick", LinkGrab_Mount);
		end
		
		local function LinkGrab_Mount2(self, button)
			local frame = _G["ActionButtonConfigFrame"].MacroTextFrame;
			local parent = self:GetParent();
			if ( IsModifiedClick("CHATLINK") ) then
				local id = parent.spellID;
				if ( frame and frame:IsShown() ) then
					local spellName = GetSpellInfo(id);
					local text = "/use "..spellName;
					frame.EditBox:Insert(text);
				end
			end
		end
		for i=1, 11, 1 do
			local buttonName = "MountJournalListScrollFrameButton"..i
			button = _G[buttonName].DragButton;
			button:HookScript("OnClick", LinkGrab_Mount2);
		end
		
		self.mountNotInit = true;
	end
	
	local customType 			= button.customType;
	local customUsingTexture 	= button.customUsingTexture;
	local customTexture 		= button.customTextureIndex;
	
	local index = 1;
	if (customType ~= nil) then
		for i = 1, #self.items, 1 do
			if (self.items[i] == customType) then
				index = i;
			end
		end
	end
	
	-- Get CustomType
	if (customType == "Custom Flyout") then
		UIDropDownMenu_SetSelectedID(self.typeDropDown, index);
		
	elseif (customType == "Macro Text") then
		-- Set the DropDown Menu Id 
		UIDropDownMenu_SetSelectedID(self.typeDropDown, index);
		
		-- Enable buttons
		self.checkbox:Enable();
		self.btnIcon:Enable();
		self.btnConfig:Enable();
		
		-- Get Macro Text
		local macroText = GetMacroText(self);
		
		-- Set Macro Text
		self:SetEditBoxMacroText(macroText);
		
		-- Get Custom Texture
		self.checkbox:SetChecked(customUsingTexture)
		
		-- Set Custom Texture
		if (customUsingTexture) then
			self:SelectIcon(customTexture, true);
			IconSelectionButtons_Update(self.IconSelectionFrame.scrollFrame);
		end
		
	else
		-- Set the DropDown Menu Id 
		UIDropDownMenu_SetSelectedID(self.typeDropDown, 1);
		
		-- Disable buttons
		self.checkbox:Disable();
		self.btnIcon:Disable();
		self.btnConfig:Disable();
	end
	
	self:Show();
end

function RefreshPlayerSpellIconInfo()
 
    MACRO_ICON_FILENAMES = {};
    MACRO_ICON_FILENAMES[1] = "INV_MISC_QUESTIONMARK";
    local index = 2;
    local numFlyouts = 0;
 
    for i = 1, GetNumSpellTabs() do
        local tab, tabTex, offset, numSpells, _ = GetSpellTabInfo(i);
        offset = offset + 1;
        local tabEnd = offset + numSpells;
        for j = offset, tabEnd - 1 do
            --to get spell info by slot, you have to pass in a pet argument
            local spellType, ID = GetSpellBookItemInfo(j, "player"); 
            if (spellType ~= "FUTURESPELL") then
                local spellTexture = strupper(GetSpellBookItemTexture(j, "player"));
                if ( not string.match( spellTexture, "INTERFACE\\BUTTONS\\") ) then
                    MACRO_ICON_FILENAMES[index] = gsub( spellTexture, "INTERFACE\\ICONS\\", "");
                    index = index + 1;
                end
            end
            if (spellType == "FLYOUT") then
                local _, _, numSlots, isKnown = GetFlyoutInfo(ID);
                if (isKnown and numSlots > 0) then
                    for k = 1, numSlots do
                        local spellID, overrideSpellID, isKnown = GetFlyoutSlotInfo(ID, k)
                        if (isKnown) then
                            MACRO_ICON_FILENAMES[index] = gsub( strupper(GetSpellTexture(spellID)), "INTERFACE\\ICONS\\", ""); 
                            index = index + 1;
                        end
                    end
                end
            end
        end
    end
	GetLooseMacroIcons( MACRO_ICON_FILENAMES );
    GetLooseMacroItemIcons( MACRO_ICON_FILENAMES );
	
    GetMacroIcons( MACRO_ICON_FILENAMES );
    GetMacroItemIcons( MACRO_ICON_FILENAMES );
end
function GetSpellorMacroIconInfo(index)
    if ( not index ) then
        return;
    end
    return MACRO_ICON_FILENAMES[index];
end


--self.btn:SetPushedTexture("Interface\\Buttons\\UI-Panel-Button-UP");

--foxFrame01 = foxFrame01 or CreateCustomButtonConfigFrame(foxFrame01);

--UpdateConfigFrame(foxFrame01, FSUI_BottomRight_ActionBar04_FAButton0)

--SaveConfigToButton(foxFrame01)



lib.ActionButtonConfigFrame = CreateCustomButtonConfigFrame(lib.ActionButtonConfigFrame);
RefreshPlayerSpellIconInfo();

lib.UpdateConfigFrame 		= UpdateConfigFrame;
--lib.SaveConfigToButton 	= SaveConfigToButton;
lib.MACRO_ICON_FILENAMES 	= MACRO_ICON_FILENAMES;
lib.GetSpellorMacroIconInfo = GetSpellorMacroIconInfo;