local addonName, addonTable = ...
-- libFoxActionButton_LibVariables.lua
-- Handles the Local Libraries' Variables.

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local LibraryName		= addonTable.Name;
local LibraryVersion	= addonTable.Version;
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName.." "..LibraryVersion, true) ~= nil) then
	lib = LibStub:GetLibrary(LibraryName.." "..LibraryVersion);
else
	return;
end

----------------------------------------------------------------------------------------------------
--  Get Console
----------------------------------------------------------------------------------------------------
Console = LibStub:GetLibrary("FoxConsole");

if (true) then
	return;
end

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- libs
----------------------------------------------------------------------------------------------------
local libFrameCreator	= LibStub:GetLibrary("FoxFrameCreator");
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Module Variables
----------------------------------------------------------------------------------------------------
-- Safety Check
lib.Debug = lib.Debug or {};

-- local Variables
local frameName = "FAB_DebugFrame v"..LibraryVersion;

-- Module Variables
lib.Debug.UpdateInterval = 1.0;

-- Function Definitions
local Create;

----------------------------------------------------------------------------------------------------

function Create()
	local name = frameName;
	local self;
	if (_G[name] == nil) then
		self = libFrameCreator.New(name);
	else
		self = libFrameCreator.Setup(_G[name]);
	end
	
	--self:SetBackdrop(nil);
	self:ClearAllPoints();
	self:SetPoint("Center", UIParent, "Center", 0, 0);
	
	-- Make Moveable
	self:SetMovable(true);
	self:EnableMouse(true);
	self:RegisterForDrag("LeftButton");
	self:SetScript("OnDragStart", 	self.StartMoving);
	self:SetScript("OnDragStop", 	self.StopMovingOrSizing);
	
	return self;
end

function lib.Debug.Main()
	-- Create Debug Frame
	lib.Debug.MainFrame = Create();
	
	lib.Debug.CreateButton(lib.Debug.MainFrame);
	
	lib.Debug.Setup();
end

function lib.Debug.CreateButton(parent)
	local parentName 	= parent:GetName();
	local name 			= "TestButton";
	local id 			= 0;
	local self;
	
	if (_G[name] == nil) then
		self = lib.Functions.New(parentName, name, id, false, 32, 32);
	else
		self = lib.Functions.Setup.All(_G[name], id, false);
	end
	
	parent.button = self;
	
	self:SetParent(parent);
	self:SetPoint("TopLeft", 6, -28);
	
	-- Show
	self:Show();
	
	return self;
end

function lib.Debug.Setup()
	local self = lib.Debug.MainFrame;
	
	self.title = libFrameCreator.CreateLabel(self.title, self, "FAB_Debug", "Top", self, "Top", 0, -6, 100, 100, "Middle", "Top");
	libFrameCreator.MakeTitle(self.title, true);

	self.label1 = libFrameCreator.CreateLabel(self.label1, self, "Kind:", "TopLeft", self.button, "TopRight", 8, -6, 100, 12, "Left", "Top");
	self.label1b = libFrameCreator.CreateLabel(self.label1b, self, "xxx", "TopLeft", self.label1, "TopLeft", 60, 0, 200, 12, "Right", "Top");

	self.label2 = libFrameCreator.CreateLabel(self.label2, self, "Value:", "TopLeft", self.label1, "BottomLeft", 0, 0, 100, 12, "Left", "Top");
	self.label2b = libFrameCreator.CreateLabel(self.label2b, self, "xxx", "TopLeft", self.label2, "TopLeft", 60, 0, 200, 12, "Right", "Top");
	
	self.button:ClearAllPoints();
	self.button:SetPoint("TopLeft", 6, -28);

	self:SetWidth(308);
	
	self:SetScript("OnUpdate", lib.Debug.OnUpdate);
	
	lib.Debug.Update(self);
end

function lib.Debug.OnUpdate(self, elapsed)
	self.timeElapsed = (self.timeElapsed or 0) + elapsed; 	

	if (self.timeElapsed > lib.Debug.UpdateInterval) then
		self.timeElapsed = 0;

		lib.Debug.Update(self);
	end
end
function lib.Debug.Update(self)
	local self = self or lib.Debug.MainFrame;
	local text;
	
	local kind  = self.button:GetAttribute("type") 	or "nil";
	local value = self.button:GetAttribute(kind or "") or "nil";
	
	text = kind;
	self.label1 	= libFrameCreator.CreateLabel(self.label1, self, "Kind:", "TopLeft", self.button, "TopRight", 8, -6, 100, 12, "Left", "Top");
	self.label1b 	= libFrameCreator.CreateLabel(self.label1b, self, text, "TopLeft", self.label1, "TopLeft", 60, 0, 200, 12, "Right", "Top");
	
	text = value;
	self.label2 	= libFrameCreator.CreateLabel(self.label2, self, "Value:", "TopLeft", self.label1, "BottomLeft", 0, 0, 100, 12, "Left", "Top");
	self.label2b 	= libFrameCreator.CreateLabel(self.label2b, self, text, "TopLeft", self.label2, "TopLeft", 60, 0, 200, 12, "Right", "Top");
	
	-- Spec 2
	kind  = self.button:GetAttribute("type".."_s1") 	or "nil";
	value = self.button:GetAttribute(kind.."_s1" or "") or "nil";
	
	text = kind;
	self.label3 	= libFrameCreator.CreateLabel(self.label3, self, "Spec 1 Kind:", "TopLeft", self.label2, "BottomLeft", 0, 0, 100, 12, "Left", "Top");
	self.label3b 	= libFrameCreator.CreateLabel(self.label3b, self, text, "TopLeft", self.label3, "TopLeft", 60, 0, 200, 12, "Right", "Top");
	
	text = value;
	self.label4 	= libFrameCreator.CreateLabel(self.label4, self, "Spec 1 Value:", "TopLeft", self.label3, "BottomLeft", 0, 0, 100, 12, "Left", "Top");
	self.label4b 	= libFrameCreator.CreateLabel(self.label4b, self, text, "TopLeft", self.label4, "TopLeft", 60, 0, 200, 12, "Right", "Top");
	
	-- Spec 2
	kind  = self.button:GetAttribute("type".."_s2") 	or "nil";
	value = self.button:GetAttribute(kind.."_s2" or "") or "nil";
	
	text = kind;
	self.label5 	= libFrameCreator.CreateLabel(self.label5, self, "Spec 2 Kind:", "TopLeft", self.label4, "BottomLeft", 0, 0, 100, 12, "Left", "Top");
	self.label5b 	= libFrameCreator.CreateLabel(self.label5b, self, text, "TopLeft", self.label5, "TopLeft", 60, 0, 200, 12, "Right", "Top");
	
	text = value;
	self.label6 	= libFrameCreator.CreateLabel(self.label6, self, "Spec 2 Value:", "TopLeft", self.label5, "BottomLeft", 0, 0, 100, 12, "Left", "Top");
	self.label6b 	= libFrameCreator.CreateLabel(self.label6b, self, text, "TopLeft", self.label6, "TopLeft", 60, 0, 200, 12, "Right", "Top");
end

function lib.Debug.SetButtonSize(w, h)
	if (h == nil) then
		h = w;
	end
	lib.Debug.MainFrame.button:SetWidth(w);
	lib.Debug.MainFrame.button:SetHeight(h);
end

