local addonName, addonTable = ...
-- libFoxActionButton_LibFunctions.lua
-- Handles All the Local Libraries' Functions.

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local LibraryName		= addonTable.Name;
local LibraryVersion	= addonTable.Version;
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName.." "..LibraryVersion, true) ~= nil) then
	lib = LibStub:GetLibrary(LibraryName.." "..LibraryVersion);
else
	return;
end

----------------------------------------------------------------------------------------------------
--  Get Console
----------------------------------------------------------------------------------------------------
Console = LibStub:GetLibrary("FoxConsole");

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

-- Masque
local MSQ = LibStub("Masque", 1);

local GetFlyoutSpellBookId;
local PickupFlyout;

local PickupMount;
local PickupCritter;
local GetMountId;
local GetCritterId;

local UpdateCompanionInfo;

-- Add Spell Data
local SpellCountTypeSpells = {};
SpellCountTypeSpells[49998] = true;
SpellCountTypeSpells[45529] = true;
SpellCountTypeSpells[107427] = true;

----------------------------------------------------------------------------------------------------
-- Creation
----------------------------------------------------------------------------------------------------
function lib.Functions.New(parentName, name, id, static, width, height, addonName, addonGroup)
	local fullName = lib.Functions.GetFullName(parentName, name, id);
	Console.CondError(LibraryName, (_G[name] ~= nil) , "Frame Name \""..(name or "").."\" already taken, name must be unique.");
	Console.CondError(LibraryName, (type(id) ~= "number"), "Id must be a number.");
	
	-- Create Frame
	local self = CreateFrame("Button", fullName, UIParent, "SecureActionButtonTemplate, SecureHandlerDragTemplate, SecureHandlerStateTemplate, SecureHandlerShowHideTemplate");
	
	-- Setup
	lib.Functions.Setup.All(self, id, static, width, height)
	
	if (MSQ) then
		addonName 	= addonName  or "FoxButtons";
		addonGroup  = addonGroup or nil;
		
		self.addonName 	= addonName;
		self.addonGroup = addonGroup;
		
		local myGroup = MSQ:Group(addonName, addonGroup);

		local buttonData = {
			FloatingBG 		= false,
			Icon 			= self.icon,
			Cooldown 		= self.cooldown,
			Flash 			= false,
			Pushed 			= nil,
			Normal 			= nil,
			Disabled 		= false,
			Checked 		= false,
			Border 			= false,
			AutoCastable 	= false,
			Highlight 		= nil,
			HotKey 			= self.hotkeyLabel,
			Count 			= self.itemCountLabel,
			Name 			= self.macroLabel,
			Duration 		= false,
			AutoCast 		= false,
		}
		myGroup:AddButton(self, buttonData);
	end

	-- Return self
	return self;
end
lib.New = lib.Functions.New;

----------------------------------------------------------------------------------------------------
-- Setup
----------------------------------------------------------------------------------------------------
-- Setup All
-----------------------
function lib.Functions.Setup.All(self, id, static, width, height)
	-- Add Frame Methods & Variables
	lib.Functions.Setup.AddMethods(self);
	lib.Functions.Setup.AddVariables(self);
	
	-- Setup Frame
	lib.Functions.Setup.Frame(self, width, height, static, id);
	
	-- Setup Icon
	lib.Functions.Setup.Icon(self);
	
	-- Setup Cooldown
	lib.Functions.Setup.Cooldown(self);
	
	-- Setup CooldownLabel
	lib.Functions.Setup.CooldownLabel(self);
	
	-- Setup DrawFrame
	lib.Functions.Setup.DrawFrame(self);
	
	-- Setup ModLayer
	lib.Functions.Setup.ModLayer(self);
	
	-- Setup HotkeyLabel
	lib.Functions.Setup.HotkeyLabel(self);
	
	-- Setup MacroLabel
	lib.Functions.Setup.MacroLabel(self);
	
	-- Setup ItemCountLabel
	lib.Functions.Setup.ItemCountLabel(self);
	
	-- Setup ActiveTexture
	lib.Functions.Setup.ActiveTexture(self);
	
	-- Setup EnabledTexture
	lib.Functions.Setup.EnabledTexture(self);
	
	-- Setup EnabledGlowTexture
	lib.Functions.Setup.EnabledGlowTexture(self);
	
	-- Setup SpellProcFrame
	lib.Functions.Setup.SpellProcFrame(self);
	
	-- Setup Blink
	lib.Functions.Setup.Blink(self);
	
	-- Setup Flyout
	lib.Functions.Setup.Flyout(self);
	
	-- Update Button
	lib.Functions.UpdateChanges(self);
	
	--RegisterStateDriver(self, "isstealth", 	"[stealth] stealth; nostealth;");
	--RegisterStateDriver(self, "iscat", 		"[form:3] cat; notcat;");
	--RegisterStateDriver(self, "form", 		"[form:0] noform; [form:1] form1; [form:2] form2; [form:3] form3; [form:4] form4; [form:5] form5;");
	--RegisterStateDriver(self, "pages", "[actionbar:1] page1; [actionbar:2] page2; [actionbar:3] page3; [actionbar:4] page4; [actionbar:5] page5; [actionbar:6] page6; otherpage");
	
	--SecureHandlerUnwrapScript(self, "OnAttributeChanged");
	--SecureHandlerWrapScript(self, "OnAttributeChanged", self, lib.Variables.SpecStateChange);
	--self:SetAttribute("type_s1", "nil");
	--self:SetAttribute("type_s2", "nil");
	--self:SetAttribute("nil_s1", "nil");
	--self:SetAttribute("nil_s2", "nil");
	--RegisterStateDriver(self, "currentSpec", 	"[spec:1] spec1; [spec:2] spec2");
	--self:SetAttribute("_onstate-currentSpec", 	lib.Variables.SpecStateChange);
	
	-- Add Button to Array
	lib.Buttons[#lib.Buttons] = self;
	
	-- UpdateCompanionInfo
	self.UpdateCompanions = UpdateCompanionInfo;
	
	-- Return self
	return self;
end


-- Add Methods
function lib.Functions.Setup.AddMethods(self)
	-- Action Methods
	self.GetAction 			= lib.Methods.GetAction;
	self.SetAction			= lib.Methods.SetAction;
	self.Clear 				= lib.Methods.Clear;
	
	-- Cooldown
	self.ClearCooldown 		= lib.Methods.ClearCooldown;
	self.Flash 				= lib.Methods.Flash;
	
	-- Keybinding
	self.SetKeyBindHandler	= lib.Methods.SetKeyBindHandler;
	self.GetSaveKeyBind		= lib.Methods.GetSaveKeyBind;
	self.SetSaveKeyBind		= lib.Methods.SetSaveKeyBind;
	self.UpdateKeybinding 	= lib.Methods.UpdateKeybinding;
	self.SetKeybinding 		= lib.Methods.SetKeybinding;
	
	-- Data
	self.Save 				= lib.Methods.Save
	self.Load 				= lib.Methods.Load
	
	-- Update
	self.UpdateIcon 		= lib.Methods.UpdateIcon;
	self.UpdateAllHotkeys 	= lib.Methods.UpdateAllHotkeys;
	self.UpdateButton 		= lib.Methods.UpdateButton;
	
	-- Flyout
	self.ResetFlyout 		= lib.Methods.ResetFlyout;
	self.ShowFlyout 		= lib.Methods.ShowFlyout;
	
	-- Special
	self.MakeLikeOrigButton = lib.Methods.MakeLikeOrigButton; -- Used in Main ActionBar and Shapeshift
	
	self.HotkeyUsableUpdate = lib.Functions.HotkeyUsableUpdate;
	
	self.PickupFlyout 		= PickupFlyout;
end
function lib.Functions.Setup.AddVariables(self)
	
end

-- Setup Frame
function lib.Functions.Setup.Frame(self, width, height, static, id)
	-- Setup Parameters
	width 	= width  or 32;
	height 	= height or 32;
	static  = static or false;
	id 		= id 	 or 0;
	
	-- Set Variables
	self:SetID(id);
	self.static 		= static;
	self.updateTime 	= 0;
	
	-- Set Appearance
	self:SetSize(width, height);
	
	-- Set Graphics
	lib.Functions.Setup.Graphics(self);
	
	--------------------
	-- Set Attributes
	--------------------
	if (self.static == false) then
		-- Set OnDragReceive/OnDragStart for Dragging
		self:SetAttribute("_onreceivedrag", lib.Variables.DropAction);
		self:SetAttribute("_ondragstart", 	lib.Variables.PickupAction);
		
		-- Setup Mouse Drag
		self:RegisterForDrag("LeftButton");
		
		-- Enable Mouse Interaction
		self:EnableMouse(true);
		
	else
		-- Clear OnDragReceive/OnDragStart to prevent Dragging
		self:SetAttribute("_onreceivedrag", nil);
		self:SetAttribute("_ondragstart", 	nil);
		
		-- Enable Mouse Interaction
		self:EnableMouse(true);
		
	end
	
	self:SetAttribute("alt-type2", "none");
	
	-- Handle Button Hiding and Showing
	self:SetAttribute("_onshow", lib.Variables.OnShow);
	self:SetAttribute("_onhide", lib.Variables.OnHide);
	
	-- Register Button for Any Click
	self:RegisterForClicks("AnyUp");
	--------------------
	
	--------------------
	-- Set Scripts
	--------------------
	-- Set OnEvent Function
	self:SetScript("OnEvent", lib.Functions.Events.OnEvent);
	
	--self:RegisterEvent("PLAYER_ENTERING_WORLD");
	
	-- Register Icon Update Events
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	
	-- Register Cooldown Updating Events
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	self:RegisterEvent("BAG_UPDATE_COOLDOWN");
	self:RegisterEvent("BAG_UPDATE");
	self:RegisterEvent("PET_BAR_UPDATE_COOLDOWN");
	self:RegisterEvent("UPDATE_SHAPESHIFT_COOLDOWN");
	
	
	-- Register ModLayer Updating Events
	self:RegisterEvent("SPELL_UPDATE_USABLE");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");
	self:RegisterEvent("UNIT_POWER");
	
	-- Register Macro Events
	self:RegisterEvent("MODIFIER_STATE_CHANGED");
	
	-- Register Binding Events
	self:RegisterEvent("UPDATE_BINDINGS");
	
	-- Register Spell-Proc Events
	self:RegisterEvent("SPELL_ACTIVATION_OVERLAY_GLOW_SHOW");
	self:RegisterEvent("SPELL_ACTIVATION_OVERLAY_GLOW_HIDE");
	
	-- Register Actionbar Changing Events
	self:RegisterEvent("ACTIONBAR_PAGE_CHANGED");
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED");
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
	self:RegisterEvent("UPDATE_STEALTH");
	
	
	self:RegisterEvent("COMPANION_UPDATE");
	--------------------
	
	
	--------------------
	-- Set Scripts
	--------------------
	
	-- Hook the OnLeave to Show Tooltip
	self:SetScript("OnEnter", lib.Functions.Events.OnEnter);
	
	-- Hook the OnLeave to Allow Dragging, and Hide Tooltip
	self:SetScript("OnLeave", lib.Functions.Events.OnLeave);
	
	self:SetScript("PreClick", lib.Functions.Events.PreClick);
	
	-- Hook the PostClick to Allow Dragging
	self:SetScript("PostClick", lib.Functions.Events.PostClick);
	
	-- Hook the OnLeave to Allow Dragging
	self:SetScript("OnUpdate", lib.Functions.Events.OnUpdate);
	
	self:SetScript("OnAttributeChanged", lib.Functions.Events.OnAttributeChanged);
	
	-- Set Drag
	--self:SetScript("OnDragStart", lib.Functions.Events.OnDragStart);
	--self:RegisterForDrag("LeftButton");
	
	-- Set Save Key Bindings to false
	self.saveKeyBinding = false;
	
	-- Handle Actionbar Changes
	self.iconNeedsUpdate = false;
end

function lib.Functions.Setup.Graphics(self)
	self:SetNormalTexture(lib.Functions.Graphics.GetNormalTexture());
	self:SetPushedTexture(lib.Functions.Graphics.GetPushedTexture());
	self:SetHighlightTexture(lib.Functions.Graphics.GetHighlightTexture());
end
-- Setup Icon
function lib.Functions.Setup.Icon(self)
	-- Create
	if (self.icon == nil) then
		self.icon = self:CreateTexture(self:GetName().."Icon");
	end
	-- Setup Appearance
	self.icon:ClearAllPoints();
	self.icon:SetParent(self);
	self.icon:SetPoint("TOPLEFT",     self, "TopLeft", lib.Variables.Inset, -lib.Variables.Inset);
	self.icon:SetPoint("BOTTOMRIGHT", self, "BottomRight", -lib.Variables.Inset,  lib.Variables.Inset);
	
	self.icon:SetDrawLayer("ARTWORK", -8);
	--self.icon:SetFrameLevel(0);
end

-- Setup Cooldown
function lib.Functions.Setup.Cooldown(self)
	-- Create
	if (self.cooldown == nil) then
		self.cooldown = CreateFrame("Cooldown", self:GetName().."Cooldown", nil, "CooldownFrameTemplate");
	end
	-- Setup Appearance
	self.cooldown:SetParent(self);
	self.cooldown:ClearAllPoints();
	self.cooldown:SetPoint("TOPLEFT",      lib.Variables.Inset, -lib.Variables.Inset);
	self.cooldown:SetPoint("BOTTOMRIGHT", -lib.Variables.Inset,  lib.Variables.Inset);
	self.cooldown:SetFrameLevel(1);
end

-- Setup CooldownLabel
function lib.Functions.Setup.CooldownLabel(self)
	if (self.cooldown == nil) then
		return;
	end
	-- Create
	if (self.cooldown.label == nil) then
		self.cooldown.label = self.cooldown:CreateFontString(self:GetName().."CooldownLabel");
	end
	-- Setup Appearance
	-- Initialize Cooldown Label
	lib.Functions.SetupText(self.cooldown.label, "CENTER", 0, 0, "CENTER", "MIDDLE", "OVERLAY", 5, 20, lib.Variables.Colors.CooldownColor[1]);
	
	-- Initialize Cooldown Label Flashing
	self.cooldownFlashing = false;
end

-- Setup DrawFrame
function lib.Functions.Setup.DrawFrame(self)
	-- Create
	if (self.drawFrame == nil) then
		self.drawFrame = CreateFrame("Frame", self:GetName().."DrawFrame");
	end
	-- Setup Appearance
	self.drawFrame:SetParent(self);
	self.drawFrame:ClearAllPoints();
	self.drawFrame:SetPoint("TOPLEFT",     0, 0);
	self.drawFrame:SetPoint("BOTTOMRIGHT", 0, 0);
	-- Possible Error Here Hiding the Hotkey, Macro/ItemCount Label, Ect.. Rise As needed, or find new Fix.
	self.drawFrame:SetFrameLevel(5);
end

-- Setup ModLayer
function lib.Functions.Setup.ModLayer(self)
	-- Create
	if (self.modLayer == nil) then
		self.modLayer = self:CreateTexture(self:GetName().."ModLayer");
	end
	-- Setup Appearance
	self.modLayer:ClearAllPoints();
	self.modLayer:SetPoint("TOPLEFT",      (lib.Variables.Inset+1), -(lib.Variables.Inset+1));
	self.modLayer:SetPoint("BOTTOMRIGHT", -(lib.Variables.Inset+1),  (lib.Variables.Inset+1));
	self.modLayer:SetDrawLayer("OVERLAY", 3);
	self.modLayer.modType = "none";
end

-- Setup HotkeyLabel
function lib.Functions.Setup.HotkeyLabel(self)
	-- Create
	if (self.drawFrame.hotkeyLabel == nil) then
		self.drawFrame.hotkeyLabel = self.drawFrame:CreateFontString(self:GetName().."HotkeyLabel");
	end
	-- Setup Appearance
	-- Initialize Hotkey Label
	lib.Functions.SetupText(self.drawFrame.hotkeyLabel, "TOPRIGHT", 2, 2, "RIGHT", "TOP", "OVERLAY", 5, 10, COLOR_WHITE);
	-- Set Additional Point
	self.drawFrame.hotkeyLabel:SetPoint(	"TOPLEFT", 	-2, 2);
	self.drawFrame.hotkeyLabel:SetDrawLayer("OVERLAY", 4);
	
	-- Set Hotkey
	lib.Functions.UpdateKeybinding(self);
end

-- Setup MacroLabel
function lib.Functions.Setup.MacroLabel(self)
	-- Create
	if (self.drawFrame.macroLabel == nil) then
		self.drawFrame.macroLabel = self.drawFrame:CreateFontString(self:GetName().."MacroLabel");
	end
	-- Setup Appearance
	-- Initialize Hotkey Label
	lib.Functions.SetupText(self.drawFrame.macroLabel, 	"BOTTOMLEFT", 	0, 2, "CENTER", "TOP", "OVERLAY", 4, 8, COLOR_WHITE);
	-- Set Additional Point
	self.drawFrame.macroLabel:SetPoint(	"BOTTOMRIGHT", 	0, 2);
end

-- Setup ItemCountLabel
function lib.Functions.Setup.ItemCountLabel(self)
	-- Create
	if (self.drawFrame.itemCountLabel == nil) then
		self.drawFrame.itemCountLabel = self.drawFrame:CreateFontString(self:GetName().."ItemCountLabel");
	end
	-- Initialize Hotkey Label
	lib.Functions.SetupText(self.drawFrame.itemCountLabel, 	"BOTTOMLEFT", 	0, 2, "RIGHT", "TOP", "OVERLAY", 4, 10, COLOR_WHITE);
	-- Set Additional Point
	self.drawFrame.itemCountLabel:SetPoint(		"BOTTOMRIGHT", 	0, 2);
end

-- Setup ActiveTexture
function lib.Functions.Setup.ActiveTexture(self)
	-- Create
	if (self.drawFrame.activeTexture == nil) then
		self.drawFrame.activeTexture = self.drawFrame:CreateTexture(self:GetName().."ActiveTexture");
	end
	-- Setup Appearance
	self.drawFrame.activeTexture:SetTexture(unpack(lib.Variables.ModLayer.Active));
	self.drawFrame.activeTexture:ClearAllPoints();
	self.drawFrame.activeTexture:SetPoint("TOPLEFT",      (lib.Variables.Inset+1), -(lib.Variables.Inset+1));
	self.drawFrame.activeTexture:SetPoint("BOTTOMRIGHT", -(lib.Variables.Inset+1),  (lib.Variables.Inset+1));
	self.drawFrame.activeTexture:SetDrawLayer("OVERLAY", 4);
	self.drawFrame.activeTexture:SetBlendMode("ADD")
	self.drawFrame.activeTexture:SetVertexColor(0, 1, 0);
end

-- Setup EnabledTexture
function lib.Functions.Setup.EnabledTexture(self)
	-- Create
	if (self.enabledTexture == nil) then
		self.enabledTexture = self:CreateTexture(self:GetName().."EnabledTexture");
	end
	-- Setup Appearance
	self.enabledTexture:ClearAllPoints();
	self.enabledTexture:SetPoint("TOPLEFT",      lib.Variables.Inset, -lib.Variables.Inset);
	self.enabledTexture:SetPoint("BOTTOMRIGHT", -lib.Variables.Inset,  lib.Variables.Inset);
	self.enabledTexture:SetDrawLayer("OVERLAY", 1);
	self.enabledTexture:SetTexture(unpack(lib.Variables.ModLayer.Enable));
	self.enabledTexture:Hide();
end

-- Setup EnabledGlowTexture
function lib.Functions.Setup.EnabledGlowTexture(self)
	-- Create
	if (self.drawFrame.enabledGlowTexture == nil) then
		self.drawFrame.enabledGlowTexture = self.drawFrame:CreateTexture(self:GetName().."EnabledGlowTexture");
	end
	-- Setup Appearance
	self.drawFrame.enabledGlowTexture:ClearAllPoints();
	self.drawFrame.enabledGlowTexture:SetPoint("TOPLEFT",      (lib.Variables.Inset+1), -(lib.Variables.Inset+1));
	self.drawFrame.enabledGlowTexture:SetPoint("BOTTOMRIGHT", -(lib.Variables.Inset+1),  (lib.Variables.Inset+1));
	self.drawFrame.enabledGlowTexture:SetDrawLayer("OVERLAY", 2);
	self.drawFrame.enabledGlowTexture:SetBlendMode("ADD")
	self.drawFrame.enabledGlowTexture:SetTexture(unpack(lib.Variables.ModLayer.EnableGlow));
	self.drawFrame.enabledGlowTexture:SetVertexColor(1, 1, 1);
	self.drawFrame.enabledGlowTexture:Hide();
end


local function OnFinished(self)
   self:GetParent():Hide();
end
-- Setup SpellProcFrame
function lib.Functions.Setup.SpellProcFrame(self)
	if (self.spellProc == nil) then
		self.spellProc = CreateFrame("Frame", self:GetName().."SpellProc", self, "ActionBarButtonSpellActivationAlert");
	end
	
	-- Set Parent
	self.spellProc:SetParent(self);
	
	-- Set Points
	local offset = -8;
	self.spellProc:ClearAllPoints();
	self.spellProc:SetPoint("TOPLEFT", 		 offset, -offset);
	self.spellProc:SetPoint("BOTTOMRIGHT",	-offset,  offset);
	
	-- Set Level
	self.spellProc:SetFrameLevel(2);
	
	-- Set OnFinished So Default UI doesn't screw things up...
	self.spellProc.animOut:SetScript("OnFinished", OnFinished);
end

-- Setup Blink
function lib.Functions.Setup.Blink(self)
	-- Create
	if (self.blink == nil) then
		self.blink = self:CreateTexture(self:GetName().."Blink");
	end
	-- Setup Appearance
	self.blink:ClearAllPoints();
	self.blink:SetPoint("TOPLEFT",      (lib.Variables.Inset+1), -(lib.Variables.Inset+1));
	self.blink:SetPoint("BOTTOMRIGHT", -(lib.Variables.Inset+1),  (lib.Variables.Inset+1));
	self.blink:SetDrawLayer("OVERLAY", 3);
	
	-- Hide
	self.blink:Hide();
	
end

-- Setup Flyout
function lib.Functions.Setup.Flyout(self)
	-- Create
	if (self.flyout == nil) then
		self.flyout = self:CreateTexture(self:GetName().."FlyoutTexture");
	end
	-- Setup Appearance
	self.flyout:ClearAllPoints();
	self.flyout:SetPoint("TOPLEFT", -3, 0);
	self.flyout:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT", 4,  -11);
	self.flyout:SetDrawLayer("OVERLAY", 0);
	
	-- Set Texture
	self.flyout:SetTexture("Interface\\Buttons\\ActionBarFlyoutButton");
	self.flyout:SetTexCoord(0.625, 0.984375, 0.74218750, 0.82812500);
	
	-- Hide
	self.flyout:Hide();
end
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Events
----------------------------------------------------------------------------------------------------
-- Update Appearance & Behavior
function lib.Functions.Events.OnEvent(self, event, arg1, arg2)
	-- Update Icon
	if (event == "PLAYER_ENTERING_WORLD") then
		-- Update Icon
		lib.Functions.SetIconFromAttribute(self);
		
		-- Update Changes
		lib.Functions.UpdateChanges(self);
		
		self:ClearCooldown();
		
		self:SetAttribute("needsUpdate", true);
		
		-- Update Companion Info
		UpdateCompanionInfo(self);
		
		-- Masque Update
		if (MSQ) then
			local myGroup = MSQ:Group(self.addonName, self.addonGroup);
			myGroup:ReSkin();
		end
	end
	
	-- Update Cooldown	
	if ((event == "SPELL_UPDATE_COOLDOWN") or (event == "ACTIONBAR_UPDATE_COOLDOWN") or (event == "BAG_UPDATE_COOLDOWN") or (event == "BAG_UPDATE") or (event == "PET_BAR_UPDATE_COOLDOWN") or (event == "UPDATE_SHAPESHIFT_COOLDOWN")) then
		local kind, value 	= lib.Functions.GetButtonMacroAction(self);
		-- Update Cooldown
		lib.Functions.UpdateCooldown(self, kind, value);
	end
	
	-- Update Hotkey
	if (event == "UPDATE_BINDINGS") then
		lib.Functions.UpdateKeybinding(self);
	end
	
	-- Update Item Count
	if (event == "BAG_UPDATE") then
		local kind, value 	= lib.Functions.GetButtonMacroAction(self);
		-- Update Item Count
		lib.Functions.UpdateItemCount(self, kind, value);
	end
	
	-- Update Layers
	if ((event == "SPELL_UPDATE_USABLE") or (event == "UNIT_POWER") or (event == "PLAYER_TARGET_CHANGED")) then
		local kind, value 	= lib.Functions.GetButtonMacroAction(self);
		-- Update ModLayer
		lib.Functions.UpdateModLayer(self, kind, value);
	end
	
	-- Update Macro Name
	if ((event == "UPDATE_MACROS") or (event == "MODIFIER_STATE_CHANGED")) then
		-- Update the Macro's Index if needed
		lib.Functions.UpdateMacroIndex(self);
		-- Update the Macro's Name if Needed
		lib.Functions.UpdateChanges(self);
	end
	
	-- Update Action
	if ((event == "ACTIONBAR_PAGE_CHANGED") or (event == "ACTIONBAR_SLOT_CHANGED") or (event == "UPDATE_SHAPESHIFT_FORM") or (event == "UPDATE_STEALTH")) then
		if (self:GetAttribute("type") == "action") then
			self.iconNeedsUpdate = true
		end
		
		if (self:GetAttribute("type") == "action") then
			lib.Functions.SetIconFromAttribute(self);
		end
		local kind, value 	= lib.Functions.GetButtonMacroAction(self);
		-- Update the Macro's Name if Needed
		lib.Functions.UpdateChanges(self);
	end
	
	-- Check for Spell Proc Changes
	if ((event == "SPELL_ACTIVATION_OVERLAY_GLOW_SHOW") or (event == "SPELL_ACTIVATION_OVERLAY_GLOW_HIDE")) then
		local state = "show";
		if (event == "SPELL_ACTIVATION_OVERLAY_GLOW_HIDE") then
			state = "hide";
		end
		
		-- Check/Update Spell-Proc
		lib.Functions.CheckSpellProc(self, state, arg1);
		
		-- Update Any Changes to Button
		lib.Functions.UpdateChanges(self);
	end
	
	if ((event == "COMPANION_UPDATE") or (event == "PET_BATTLE_PET_CHANGED")) then
		UpdateCompanionInfo(self);
	end
	if (not InCombatLockdown()) then
		lib.Functions.HotkeyUsableUpdate(self);
	end
end
function lib.Functions.Events.OnAttributeChanged(self, name, value)
	lib.Functions.UpdateChanges(self);
	lib.Functions.HotkeyCheck(self, name, value);
	lib.Functions.UpdateFlyout(self);
end
function lib.Functions.Events.OnUpdate(self, elapsed)
	lib.Functions.Update(self, elapsed);
end

-- Pickup & Drop
function lib.Functions.Events.PreClick(self, button)
	if ((button == "RightButton") and (IsAltKeyDown()) and (not IsShiftKeyDown()) and (not IsControlKeyDown())) then
		return;
	end
	
	if ((button == "RightButton") and (IsAltKeyDown()) and (IsShiftKeyDown()) and (not IsControlKeyDown())) then
		return;
	end
	
	lib.Variables.SwapNoAction = false;
	if (InCombatLockdown()) then
		return;
	end
	if (self:GetAttribute("type") == "action") then
		local id = lib.Functions.GetEffectiveActionID(self);
		lib.Variables.SwapKind 	= nil;
		lib.Variables.SwapValue 	= nil;
		lib.Variables.SwapNoAction = true;
		return;
	end
	local kind, value 	= GetCursorInfo();
	if ((kind ~= nil) and (self.static == false)) then
		if (kind == "action") then
			local actionType, actionID, actionSubType, actionSpellID = GetActionInfo(lib.Functions.GetEffectiveActionID(self));
			local kind 	= actionType;
			local value = actionID;
			if (actionType == "companion") then
				kind 	= actionSubType;
			elseif (actionType == "spell") then
				value 	= actionSpellID;
			end
			
			return;
		end
		lib.Variables.SwapKind 	= self:GetAttribute("type")   or "";
		lib.Variables.SwapValue = self:GetAttribute(lib.Variables.SwapKind) or "";
		
		-- BattlePet
		if (self:GetAttribute("battlepet") == true) then
			lib.Variables.SwapKind 	= "battlepet";
			lib.Variables.SwapValue = self:GetAttribute("battlepetid");
		end
		
		if (lib.Variables.SwapKind == "flyout") then
			lib.Variables.SwapValue = self:GetAttribute("spell") or "";
		end
		self:Clear();
		
	end
end
function lib.Functions.Events.PostClick(self, mouseButton)
	if (InCombatLockdown()) then
		return;
	end
	if (lib.Variables.SwapNoAction == true) then
		lib.Variables.SwapNoAction = false;
		return;
	end
	local kind, value, arg1 	= GetCursorInfo();
	-- Handle Dropping
	if ((arg1 == "MOUNT") or (arg1 == "CRITTER")) then
		kind = arg1;
	end
	if ((kind=="item" or kind=="spell" or kind=="macro" or kind=="MOUNT" or kind=="CRITTER") and (self.static == false)) then
		self:SetAttribute("type", kind)
		if kind=="item" then
			value = "item:"..value
		elseif kind=="spell" then
			value = select(4, GetCursorInfo()) -- the raw spell id (4th return GetCursorInfo)
		end
		self:SetAttribute(kind, value);
		lib.Functions.SetIconFromAttribute(self);
		
		-- Clear Cusor
		ClearCursor();
		
		-- Moved Swapping Outside of If/Else
		
	-- Handle Setting Binding
	-- RightClick + Alt
	elseif ((mouseButton == "RightButton") and (IsAltKeyDown()) and (not IsShiftKeyDown()) and (not IsControlKeyDown())) then
		lib.Functions.ShowBindingFrame(self);
		
	-- Handle Config Menu
	-- RightClick + Alt + Shift
	elseif ((mouseButton == "RightButton") and (IsAltKeyDown()) and (IsShiftKeyDown()) and (not IsControlKeyDown())) then
		-- Handle
		lib.UpdateConfigFrame(lib.ActionButtonConfigFrame, self);
	
	-- Handle Companions
	elseif ((self:GetAttribute("type") == "MOUNT") or (self:GetAttribute("type") == "CRITTER")) then
		-- Call Companion
		local kind = self:GetAttribute("type");
		local value = self:GetAttribute(kind);
		if ((self:GetAttribute("type") == "MOUNT") and (IsMounted())) then
			-- if Same MOUNT
			local creatureID, creatureName, spellId, icon, active, MountFlags = GetCompanionInfo(kind, value);
			if (lib.Functions.IsSameMount(spellId) == true) then
				Dismount();
			else
				CallCompanion(kind, value);
			end
		else
			CallCompanion(kind, value);
		end
		
		--print("Testing");
		--UpdateCompanionInfo(self);
		
	elseif (kind == "flyout") then
		self:SetAttribute("type", kind);
		self:SetAttribute("spell", value);
		
		-- Set Icon
		lib.Functions.SetIconFromAttribute(self);
		
		-- Clear Cusor
		ClearCursor();
		
	elseif (kind == "battlepet") then
		local text = "/run C_PetJournal.SummonPetByGUID(\'"..value.."\');";
		
		self:SetAttribute("battlepetid", value);
		
		self:SetAttribute("type", "macro")
		self:SetAttribute("macrotext", text);
		self:SetAttribute("battlepet", true);
		
		-- Clear Cusor
		ClearCursor();
	end
	
	-- if Button had something previously
	if ((lib.Variables.SwapKind ~= "") and (lib.Variables.SwapValue ~= "")) then
		-- Set Cursor
		if (lib.Variables.SwapKind == "spell") then
			-- Pickup Spell
			PickupSpell(lib.Variables.SwapValue);
		
		elseif (lib.Variables.SwapKind == "macro") then
			-- Pickup Macro
			PickupMacro(lib.Variables.SwapValue);
			
		elseif (lib.Variables.SwapKind == "item") then
			-- Pickup Item
			PickupItem(lib.Variables.SwapValue);
			
		elseif ((lib.Variables.SwapKind == "MOUNT") or (lib.Variables.SwapKind == "CRITTER")) then
			-- Pickup Item
			PickupCompanion(lib.Variables.SwapKind, lib.Variables.SwapValue);
		
		--elseif (lib.Variables.SwapKind == "battlepet") then
			-- Pickup Item
			--PickupCompanion(lib.Variables.SwapKind, lib.Variables.SwapValue);
			
		elseif (lib.Variables.SwapKind == "flyout") then
			-- Pickup Flyout
			lib.Functions.PickupFlyout(lib.Variables.SwapValue);
			
		elseif (lib.Variables.SwapKind == "battlepet") then
			C_PetJournal.PickupPet(lib.Variables.SwapValue);
		
		end
	end
	
	lib.Variables.SwapKind 	= nil;
	lib.Variables.SwapValue	= nil;
end

-- Pickup Flyout
function lib.Functions.PickupFlyout(id)
	if (id == -1) then
		return;
	end
	local iii=1
	while true do
		local sbtype, sbid = GetSpellBookItemInfo(iii, "spell")
		if sbtype=="FLYOUT" and sbid==id then
			PickupSpellBookItem(iii, "spell")
			return true
		elseif sbtype==nil and sbid==nil then
			return false
		end
		iii = iii+1
	end
end

function lib.Functions.Events.PreClickPet(self, button)
	if ((button == "RightButton") and (IsAltKeyDown())) then
		return;
	else
		local kind, value = GetCursorInfo();
		if (kind == "petaction") then
			if (IsModifiedClick("PICKUPACTION")) then
				PickupPetAction(self:GetID());
				return;
			end
		end
	end
end
function lib.Functions.Events.PostClickPet(self, mouseButton)
	-- Handle Setting Binding
	-- RightClick + Alt
	if ((mouseButton == "RightButton") and (IsAltKeyDown()) and (not IsShiftKeyDown()) and (not IsControlKeyDown())) then
		lib.Functions.ShowBindingFrame(self);
	else
		return;
	end
end

-- Tooltip
function lib.Functions.Events.OnEnter(self)
	lib.Functions.ShowTooltip(self);
end
function lib.Functions.Events.OnLeave(self)
	lib.Functions.HideTooltip();
end

function lib.Functions.Events.OnDragStart(self, button)
	if (InCombatLockdown()) then
		return;
	end
	if (button == "LeftButton") then
		local kind, value 	= GetCursorInfo();
		lib.Variables.SwapKind 	= self:GetAttribute("type")   or "";
		lib.Variables.SwapValue = self:GetAttribute(lib.Variables.SwapKind) or "";
		if (lib.Variables.SwapKind == "flyout") then
			lib.Variables.SwapValue = self:GetAttribute("spell") or "";
		end
		
		
		if ((kind=="item" or kind=="spell" or kind=="macro" or kind=="MOUNT" or kind=="CRITTER") and (self.static == false)) then
			self:SetAttribute("type", kind)
			if kind=="item" then
				value = "item:"..value
			elseif kind=="spell" then
				value = select(4, GetCursorInfo()) -- the raw spell id (4th return GetCursorInfo)
			end
			self:SetAttribute(kind, value);
			lib.Functions.SetIconFromAttribute(self);
			
			-- Clear Cusor
			ClearCursor();
			
			-- Moved Swapping Outside of If/Else
			
		-- Handle Setting Binding
		-- RightClick + Alt
		elseif ((mouseButton == "RightButton") and (IsAltKeyDown()) and (not IsShiftKeyDown()) and (not IsControlKeyDown())) then
			--lib.Functions.ShowBindingFrame(self)
		
		-- Handle Companions
		elseif ((self:GetAttribute("type") == "MOUNT") or (self:GetAttribute("type") == "CRITTER")) then
			-- Call Companion
			local kind = self:GetAttribute("type");
			local value = self:GetAttribute(kind);
			if ((self:GetAttribute("type") == "MOUNT") and (IsMounted())) then
				-- if Same MOUNT
				local creatureID, creatureName, spellId, icon, active, MountFlags = GetCompanionInfo(kind, value);
				if (lib.Functions.IsSameMount(spellId) == true) then
					Dismount();
				else
					CallCompanion(kind, value);
				end
			else
				CallCompanion(kind, value);
			end
		elseif (kind == "flyout") then
			self:SetAttribute("type", kind);
			self:SetAttribute("spell", value);
			
			-- Set Icon
			lib.Functions.SetIconFromAttribute(self);
			
			-- Clear Cusor
			ClearCursor();
			
		else
			self:Clear();
		end
		
		-- if Button had something previously
		if ((lib.Variables.SwapKind ~= "") and (lib.Variables.SwapValue ~= "")) then
			-- Set Cursor
			if (lib.Variables.SwapKind == "spell") then
				-- Pickup Spell
				PickupSpell(lib.Variables.SwapValue);
				
			elseif (lib.Variables.SwapKind == "macro") then
				-- Pickup Macro
				PickupMacro(lib.Variables.SwapValue);
				
			elseif (lib.Variables.SwapKind == "item") then
				-- Pickup Item
				PickupItem(lib.Variables.SwapValue);
				
			elseif ((lib.Variables.SwapKind == "MOUNT") or (lib.Variables.SwapKind == "CRITTER")) then
				-- Pickup Item
				PickupCompanion(lib.Variables.SwapKind, lib.Variables.SwapValue);
				
			elseif (lib.Variables.SwapKind == "flyout") then
				-- Pickup Flyout
				lib.Functions.PickupFlyout(lib.Variables.SwapValue);
				
			end
		end
		
		if (self:GetAttribute("type") == "flyout") then
			local id = self:GetAttribute("spell") or -1;
			-- Pickup Flyout
			lib.Functions.PickupFlyout(id);
			
			-- Clear Old Data
			self:Clear();
		end
	end
end
----------------------------------------------------------------------------------------------------
-- Get, Set
----------------------------------------------------------------------------------------------------
function lib.Functions.GetEffectiveActionID(self)
	local id = self:GetID();
	local page = GetActionBarPage();
	local effectiveId = 0;
	local _, className = UnitClass("player");
	if (className == "DRUID") then
		local form 			= GetShapeshiftFormID();
		local isStealthed 	= IsStealthed();
		if (form == CAT_FORM) then
			if (isStealthed) then
				effectiveId = id + ((7) * NUM_ACTIONBAR_BUTTONS);
				return effectiveId;
			end
		elseif (form == 4) then
			
		elseif (form == 3) then
			return effectiveId;
		end
	end
	
	if (GetBonusBarOffset() > 0) then
		page = NUM_ACTIONBAR_PAGES + GetBonusBarOffset();
	end
	effectiveId = id + ((page - 1) * NUM_ACTIONBAR_BUTTONS);
	return effectiveId;
end
function lib.Functions.GetFlyoutIcon(id)
	local iii=1
	while true do
		local sbtype, sbid = GetSpellBookItemInfo(iii, "spell")
		if sbtype=="FLYOUT" and sbid==id then
			local icon = GetSpellBookItemTexture(iii, "spell");
			return icon
		elseif sbtype==nil and sbid==nil then
			return icon
		end
		iii = iii+1
	end
end

-- Action Info
function lib.Functions.GetItemId(item)
	if ((item == nil) or (item == "")) then
		return nil;
	end
	local name, itemLink = GetItemInfo(item);
	if (item == "Xu-Fu, Cub of Xuen") then
		return "101771";
	end
	local startPos, endPos, firstWord, restOfString = string.find(itemLink, "item:(%d*)");
	
	return firstWord;
end
function lib.Functions.GetSpellNameFromId(spellId)
	if ((spellId == nil) or (spellId == "")) then
		return "Languages";
	end
	local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellId);
	name = name or "Languages";
	return name;
end
function lib.Functions.GetButtonMacroAction(self)
	local kind  		= self:GetAttribute("type") or "";
	local value 		= self:GetAttribute(kind)   or "";
	local macroKind 	= nil;
	local macroValue 	= nil;
	local macroBody		= nil;
	local command 		= nil;
	local target 		= nil;
	
	if ((kind == "macro") and (value ~= "")) then
		-- Get Macro's Target
		macroBody = GetMacroBody(value);
		if (macroBody ~= nil) then
			command, target 	= SecureCmdOptionParse(macroBody);
		end
		
		-- Get Macro's Spell/Item
		local spellName = GetMacroSpell(value);
		local itemName = GetMacroItem(value);
		if (spellName ~= nil) then
			macroKind	= "spell";
			macroValue 	= spellName;
		elseif (itemName ~= nil) then
			macroKind	= "item";
			macroValue 	= itemName;
		end
	end
	-- Return kind, value, & target
	return macroKind, macroValue, target;
end
function lib.Functions.GetSpellId(spell)
	if ((spell == nil) or (spell == "")) then
		return;
	end
	local spellLink = GetSpellLink(spell);
	local startPos, endPos, firstWord, restOfString = string.find(spellLink, "spell:(%d*)");
	return firstWord;
end
function lib.Functions.GetActionName(self, kind, value)
	local name = "";
	if (kind == "macro") then
		kind, value = lib.Functions.GetButtonMacroAction(self);
	elseif (kind == "action") then
		local ActionType, id, subType, spellID = GetActionInfo(self:GetID())
		if (kind == "MOUNT") then
			kind = ActionType;
			value = spellID;
		else
			kind = ActionType;
			value = id;
		end
	end
	
	if (kind == "spell") then
		local spellName, spellRank, spellIcon, spellPowerCost, spellIsFunnel, spellPowerType, spellCastingTime, spellMinRange, spellMaxRange = GetSpellInfo(value);
		name = spellName;
		
	elseif (kind == "item") then
		local itemId = "";
		if (self:GetAttribute("type") == "item") then
			itemId = string.gsub(value, "item:", "");
		elseif (self:GetAttribute("type") == "macro") then
			local macroId		= self:GetAttribute(self:GetAttribute("type"));
			local itemName 		= GetMacroItem(macroId);
			itemId				= lib.Functions.GetItemId(itemName);
		end
		local itemName, itemLink, itemQuality, itemILevel, itemReqLevel, itemClass, itemSubclass, itemMaxStack, itemEquipSlot, itemTexture, itemVendorPrice = GetItemInfo(itemId);
		name = itemName;
	
	elseif (kind == "MOUNT") then
		 local creatureID, creatureName, spellID, icon, active, MountFlags = GetCompanionInfo(kind, value)
		 name = creatureName;
		 
	end
	
	return name
end
function lib.Functions.IsSameMount(spellId)
	if (IsMounted()) then
		local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff, value1, value2, value3;
		local i = 1;
		while (true) do
			name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff, value1, value2, value3 = UnitBuff("player", i)
			if (name == nil) then
				break;
			end
			
			-- Check if the Same MOUNT is being Called
			if (spellId == spellID) then
				return true;
			end
			
			-- Increase i for Next Loop
			i = i + 1;
		end
	end
	return nil;
end


-- Cooldown Info
function lib.Functions.GetCooldownTime(self, kind, value)
	local kind  	= kind  or self:GetAttribute("type") or "";
	local value 	= value or self:GetAttribute(kind)   or "";
	local itemId 	= self:GetAttribute("itemID")   or "";
	local startTime = 0;
	local duration 	= 0;
	local enabled 	= 0;
	if (kind == "item") then
		-- Get Item Cooldown Time
		if (value ~= "") then
			--local itemId = "";
			if (self:GetAttribute("type") == "item") then
				--itemId = string.gsub(value, "item:", "");
				
			elseif (self:GetAttribute("type") == "macro") then
				local macroId		= self:GetAttribute(self:GetAttribute("type"));
				local itemName		= GetMacroItem(macroId);
				itemId				= lib.Functions.GetItemId(itemName);
			end
			if (itemId ~= "") then
				itemId = tonumber(itemId);		
				startTime, duration, enabled = GetItemCooldown(itemId);
			end
		end
		
	elseif (kind == "spell") then
		startTime, duration, enabled = GetSpellCooldown(value);
		
	elseif (kind == "macro") then
		--startTime, duration, enabled = GetItemCooldown(itemId);
		local customSpellID = self:GetAttribute("customSpellIcon");
		if (customSpellID ~= nil) then
			startTime, duration, enabled = GetSpellCooldown(customSpellID);
		end
		
	elseif (self:GetAttribute("type1") == "pet") then
		startTime, duration, enabled = GetPetActionCooldown(self:GetID());
		
	elseif (kind == "action") then
		-- Get Action Cooldown Time
		startTime, duration, enabled = GetActionCooldown(lib.Functions.GetEffectiveActionID(self));
	end
	
	startTime 	= startTime or 0;
	duration 	= duration  or 0;
	enabled 	= enabled   or 0;
	-- Gets the time left on the cooldown
	local timeLeft = (startTime + duration - GetTime());
	-- Insures that the Time left is 0 or greater
	timeLeft = max(timeLeft, 0);
	-- Rounds the time down to a integer
	timeLeft = floor(timeLeft);
	-- return timeLeft
	return timeLeft;
end
function lib.Functions.SetCooldownText(self, text, displayType, displayColor)
	if (self.cooldown == nil) then
		return;
	end
	displayType 	= displayType  or 1;
	--displayColor 	= displayColor or 1;
	local fontSize;
	local fontColor;
	if (displayType == 1) then
		fontSize = 20;
	elseif (displayType == 2) then
		fontSize = 12;
	else
		fontSize = 20;
	end
	fontColor 		= displayColor or lib.Variables.Colors.CooldownColor[0];
	
	-- Set Size
	self.cooldown.label:SetFont("Fonts\\FRIZQT__.TTF", fontSize, "OUTLINE");
	
	-- Set Color
	self.cooldown.label:SetTextColor(unpack(fontColor));
	
	-- Set Text
	self.cooldown.label:SetText(text);
	
	-- Show Cooldown Label
	self.cooldown.label:Show();
end
function lib.Functions.SetCooldownLabelText(self, timeLeft, modifier, color)
	timeLeft = timeLeft or 0;
	modifier = modifier or "";
	
	local str = "";
	if (timeLeft > 0) then
		str = ("%d%s"):format(timeLeft, modifier);
	end
	
	--  Change font if using Letters
	if (lib.Variables.CooldownDisplayType ~= lib.CONSTANTS.COOLDOWNUSECOLORS) then
		if ((modifier ~= "") and (modifier ~= nil)) then
			displayType = 1;
		else
			displayType = 1;
		end
	else
		displayType = 1;
	end
	
	-- Set the Cooldown Label's Text, Size, and Color
	lib.Functions.SetCooldownText(self, str, displayType, color);
end

-- Icon
function lib.Functions.SetIconFromAttribute(self)
	local kind  	= self:GetAttribute("type") 			or "";
	local value 	= self:GetAttribute(kind)   			or "";
	local custom 	= self:GetAttribute("customTexture") 	or "";
	local customSpellId = self:GetAttribute("customSpellIcon") 	or "";
	local texture	= "";
	
	if ((custom ~= nil) and (custom ~= ""))  then
		texture = custom;
		
	elseif (self.customUsingTexture) then
		-- Use Custom Texture
		local index = self.customTextureIndex;
		local MACRO_ICON_FILENAMES = lib.MACRO_ICON_FILENAMES;
		local texture = lib.GetSpellorMacroIconInfo(index);
		local numMacroIcons = #MACRO_ICON_FILENAMES;
		if (self.customUsingTexture) then
			if (index ~= nil) then
				if ( index <= numMacroIcons and texture ) then
					if(type(texture) == "number") then
						self.icon:SetToFileData(texture);
						return;
					else
						self.icon:SetTexture("INTERFACE\\ICONS\\"..texture);
						return;
					end
				end
			end
		end
	
	elseif ((customSpellId ~= nil) and (customSpellId ~= ""))  then
		local name, rank, icon, powerCost, isFunnel, powerType, castingTime, minRange, maxRange = GetSpellInfo(customSpellId);
		texture = icon;

	-- Custom Icon for BattlePet
	elseif (self:GetAttribute("battlepet")) then
		texture = "Interface\\Icons\\INV_Misc_QuestionMark";		
		local guid = self:GetAttribute("battlepetid");
		if (guid ~= nil) then
			local speciesID, customName, level, xp, maxXp, displayID, isFavorite, petName, petIcon, petType, creatureID = C_PetJournal.GetPetInfoByPetID(guid);
		
			texture = petIcon;
		end
	
	elseif ((value == "") and (kind ~= "action") and (kind ~= "flyout")) then
		if (kind == "macro") then
			if ((value == nil) or (value == "")) then
				local macrotext = self:GetAttribute("macrotext");
				if (macrotext) then
					local action, target = SecureCmdOptionParse(macrotext)
					action, numMatched = gsub(action, "/use ", "");
					action, numMatched = gsub(action, "#show", "");
					action, numMatched = gsub(action, "tooltip", "");
					action, numMatched = gsub(action, "\n", "");
					
					local name, _, icon = GetSpellInfo(action);
					if (icon == nil) then
						name, _, _, _, _, _, _, _, _, icon = GetItemInfo(action);
					end
					texture = icon;
					
				end
			end
		else
			texture = "";
		end
		
	elseif (kind == "item") then
		-- Get Texture Path
		texture = GetItemIcon(value);
		
		self:UnregisterEvent("UPDATE_MACROS");
		self.isMacro = false;
		self.macroName = "";
		
	elseif (kind == "spell") then
		-- Get Texture Path
		texture = lib.Functions.GetSpellIconFromId(value);
		
		self:UnregisterEvent("UPDATE_MACROS");
		self.isMacro = false;
		self.macroName = "";
		
	elseif (kind == "macro") then
		local arg1, arg2 = nil;
		-- Get Texture Path
		name, texture, arg1, arg2 = GetMacroInfo(value);
		self:RegisterEvent("UPDATE_MACROS");
		self.isMacro = true;
		self.macroName = name;
		
	elseif ((kind == "CRITTER") or (kind == "MOUNT")) then
		local arg1, arg2, arg3 = nil;
		-- Get Texture Path
		if (value ~= nil) then
			if (value <= GetNumCompanions(kind)) then
				arg1, arg2, arg3, texture = GetCompanionInfo(kind, value);
				
				self:UnregisterEvent("UPDATE_MACROS");
				self.isMacro = false;
				self.macroName = "";
			end
		end
		
	elseif (kind == "action") then
		-- Get Texture Path
		texture = GetActionTexture(lib.Functions.GetEffectiveActionID(self));
      
		self:UnregisterEvent("UPDATE_MACROS");
		self.isMacro = false;
		self.macroName = "";
		
	elseif (kind == "flyout") then
		-- Get Texture Path
		local id = self:GetAttribute("spell");
		texture = lib.Functions.GetFlyoutIcon(id);
      
		self:UnregisterEvent("UPDATE_MACROS");
		self.isMacro = false;
		self.macroName = "";
	
	end
	local bool1 = (string.lower((texture or "")) ~= string.lower(self.icon:GetTexture() or "")..".blp")
	local bool2 = (string.lower(texture or "") ~= string.lower(self.icon:GetTexture() or ""))
	if (bool1 and bool2) then
		if (self.icon ~= nil) then
			-- Set Texture
			self.icon:SetTexture(texture);
			
			-- Clear the Cooldown
			self:ClearCooldown();
		end
	end
	
	
end
function lib.Functions.GetSpellIconFromId(spellId)
	if (spellId == nil) then
		return "";
	end
	local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellId);
	return icon;
end

function lib.Functions.GetFullName(parentName, name, id)
	local failed = (((name == nil) or (name == "")) and ((parentName == nil) or (parentName == "")));
	Console.CondError(LibraryName, failed, "Name and Parent Name cannot both be nil.");
	
	local usingName = ((name ~= nil) and (name ~= ""));
	
	if (usingName) then
		return name;
	else
		return parentName.."_FAButton"..tostring(id);
	end
end

----------------------------------------------------------------------------------------------------
-- Update
----------------------------------------------------------------------------------------------------
function lib.Functions.Update(self, elapsed)
	self.updateTime = self.updateTime + elapsed;
	if (self.updateTime > lib.Variables.UpdateTime) then
		-- Run Main Update
		lib.Functions.UpdateMain(self);
		
		-- Reset Update Time
		self.updateTime = 0;
	end
	lib.Functions.ShowTooltip(self);
	
	-- Update Blink Effect
	lib.Functions.UpdateBlink(self);
	
	if (self.iconNeedsUpdate == true) then
		lib.Functions.SetIconFromAttribute(self);
		self.iconNeedsUpdate = false;
		
		-- Update Cooldown
		local kind, value 	= lib.Functions.GetButtonMacroAction(self);
		lib.Functions.UpdateCooldown(self, kind, value);
	end
	
	if (self.PostUpdate ~= nil) then
		if (type(self.PostUpdate) == "function") then
			self:PostUpdate();
		end
	end
	
		
	-- Update BloodTap
	lib.Functions.UpdateSpellData(self);
end
function lib.Functions.UpdateMain(self, kind, value)
	local kind, value 	= lib.Functions.GetButtonMacroAction(self);
	
	-- Update Cooldown
	lib.Functions.UpdateCooldown(self, kind, value);
	
	-- Update Cooldown Label
	lib.Functions.UpdateCooldownLabel(self, kind, value);
	
	-- Update Layers
	lib.Functions.UpdateModLayer(self, kind, value);
	
	-- Update Flyout
	lib.Functions.UpdateFlyout(self, kind, value);
	
	-- Update Item Count or Macro
	local kind = self:GetAttribute("type");
	if ((kind == "macro") or (self:GetAttribute("type1") == "pet")) then
		-- Update Macro Name
		lib.Functions.UpdateChanges(self);
		
	end
end
function lib.Functions.UpdateChanges(self)
	if (self:GetAttribute("type1") ~= "pet") then
		-- Set Icon
		lib.Functions.SetIconFromAttribute(self);
	else
		lib.Functions.UpdatePetAction(self);
	end
	
	-- Get Macro Action, if needed
	local kind, value 	= lib.Functions.GetButtonMacroAction(self);
	
	-- Run Cooldown Update
	lib.Functions.UpdateCooldown(self, kind, value);
	
	-- Update Cooldown Label
	lib.Functions.UpdateCooldownLabel(self, kind, value);
	
	-- Run Update Item Count
	lib.Functions.UpdateItemCount(self, kind, value);
	
	-- Update ModLayer
	lib.Functions.UpdateModLayer(self, kind, value);
	
	-- Update Macro Label
	lib.Functions.UpdateMacroName(self);
end
function lib.Functions.UpdatePetAction(self)
	-- Get Info
	local name, subtext, texture, isToken, isActive, autoCastAllowed, autoCastEnabled = GetPetActionInfo(self:GetID());
	local inset, overlayTexture;
	
	-- Set Icon
	if (not isToken) then
		self.icon:SetTexture(texture);
	else
		self.icon:SetTexture(_G[texture]);
	end
	-- Set IsActive (ex. aggressive/defensive/passive)
	if (self.iconActiveOverlay == nil) then
		self.iconActiveOverlay = self:CreateTexture();
	end
	if (isActive == 1) then
		overlayTexture = "Interface\\Buttons\\CheckButtonHilight";
		
	else
		overlayTexture = "";
		
	end
	inset = lib.Variables.Inset;
	
	self.iconActiveOverlay:SetDrawLayer("OVERLAY", 1);
	self.iconActiveOverlay:ClearAllPoints();
	self.iconActiveOverlay:SetPoint("TOPLEFT", 		inset+1, -inset-1);
	self.iconActiveOverlay:SetPoint("BOTTOMRIGHT", -inset,  inset);
	self.iconActiveOverlay:SetTexture(overlayTexture);
	self.iconActiveOverlay:SetBlendMode("ADD")
	self.iconActiveOverlay:SetAlpha(1);
	
	-- Set Auto-Castable Texture
	if (self.iconAutoCastOverlay == nil) then
		self.iconAutoCastOverlay = self:CreateTexture();
	end
	if (autoCastAllowed == 1) then
		overlayTexture = "interface\\buttons\\UI-AutoCastableOverlay";
	else
		overlayTexture = "";
	end
	inset = -10;
	
	self.iconAutoCastOverlay:SetDrawLayer("OVERLAY", 1);
	self.iconAutoCastOverlay:ClearAllPoints();
	self.iconAutoCastOverlay:SetPoint("TOPLEFT", 		inset+1, -inset-1);
	self.iconAutoCastOverlay:SetPoint("BOTTOMRIGHT", -inset,  inset);
	
	self.iconAutoCastOverlay:SetTexture(overlayTexture);
	
	-- Set Auto-Cast
	if (self.shine == nil) then
		self.shine = CreateFrame("Frame", self:GetName().."Shine", self, "AutoCastShineTemplate");
	end
	inset = 3;
	self.shine:ClearAllPoints();
	self.shine:SetPoint("TOPLEFT", inset+1, -inset-1);
	self.shine:SetPoint("BOTTOMRIGHT", -inset, inset);
	if (autoCastEnabled) then
		AutoCastShine_AutoCastStart(self.shine);
	else
		AutoCastShine_AutoCastStop(self.shine);
	end
end
function lib.Functions.UpdateCooldown(self, kind, value)
	local kind  	= kind  or self:GetAttribute("type") or "";
	local value 	= value or self:GetAttribute(kind)   or "";
	local startTime = 0;
	local duration 	= 0;
	local enabled 	= 1;
	-- Get Cooldown
	if (kind == "spell") then
		-- Get Spell Cooldown Time
		startTime, duration, enabled = GetSpellCooldown(value);
		
	elseif (kind == "item") then
		-- Get Item Cooldown Time
		if (value ~= "") then
			local itemId = "";
			if (self:GetAttribute("type") == "item") then
				itemId = string.gsub(value, "item:", "");
			elseif (self:GetAttribute("type") == "macro") then
				local macroItem		= self:GetAttribute(kind);
				local itemName 		= GetMacroItem(macroItem);
				itemId 				= lib.Functions.GetItemId(itemName);
			end
			itemId = tonumber(itemId);
			if (itemId ~= nil) then
				startTime, duration, enabled = GetItemCooldown(itemId);
			end
		end
		
	elseif (self:GetAttribute("type1") == "pet") then
		startTime, duration, enabled = GetPetActionCooldown(self:GetID());
	
	elseif (kind == "macro") then
		--startTime, duration, enabled = GetItemCooldown(itemId);
		local customSpellID = self:GetAttribute("customSpellIcon");
		if (customSpellID ~= nil) then
			startTime, duration, enabled = GetSpellCooldown(customSpellID);
		end
	
	elseif (kind == "action") then
		startTime, duration, enabled = GetActionCooldown(lib.Functions.GetEffectiveActionID(self));
		
	end
	
	local charges = 0;
	local maxCharges = 0;
	-- if Cooldown is Running
	if ((startTime ~= 0) and (startTime ~= nil) and (enabled ~= 0)) then
		--print(startTime, enabled)
		-- Handle Cooldown Animation
		
		if (not self.cooldown.isSet) then
			self.cooldown:SetCooldown(startTime, duration, 1, charges, maxCharges);
			--self.cooldown.isSet = true;
		end
		--CooldownFrame_SetTimer(self.cooldown, startTime, duration, 1, nil, nil, true)
	else
		self.cooldown.isSet = false;
	end
end
function lib.Functions.UpdateCooldownLabel(self, kind, value)
	local timeLeft 	= lib.Functions.GetCooldownTime(self, kind, value);
	local modifier 	= "";
	local color 	= lib.Variables.Colors.CooldownColor[1];
		
	if (lib.Variables.CooldownDisplayType ~= lib.CONSTANTS.COOLDOWNUSECOLORS) then
		if (timeLeft <= 0) then
			modifier = "";
			
		-- if less than a Minute  Time Left
		elseif (timeLeft <= 60) then
			modifier = "";
			
		-- if less than a Hour left
		elseif ((timeLeft / 60) < 60) then
			modifier = "m";
			
		-- if less than a Day left
		elseif ((timeLeft / 60 / 60) < 24) then
			modifier = "h";
			
		-- if more than a Day Left
		elseif ((timeLeft / 60 / 60 / 24) < 99) then
			modifier = "d";
			
		end
	end
	
	if (lib.Variables.CooldownDisplayType ~= lib.CONSTANTS.COOLDOWNUSELETTERS) then
		if (timeLeft <= 0) then
			color = lib.Variables.Colors.CooldownColor[1];
			
		-- if less than a Minute  Time Left
		elseif (timeLeft <= 60) then
			color = lib.Variables.Colors.CooldownColor[3];
			
		-- if less than a Hour left
		elseif ((timeLeft / 60) < 60) then
			color = lib.Variables.Colors.CooldownColor[4];
			
		-- if less than a Day left
		elseif ((timeLeft / 60 / 60) < 24) then
			color = lib.Variables.Colors.CooldownColor[5];
			
		-- if more than a Day Left
		elseif ((timeLeft / 60 / 60 / 24) < 99) then
			color = lib.Variables.Colors.CooldownColor[6];
			
		end
	else
		color = lib.Variables.Colors.CooldownColor[1];
	end
	
	-- Check Flashing
	-- if less than 3 seconds left
	if (timeLeft <= 3) then
		if (self.cooldownFlashing == false) then
			color = lib.Variables.Colors.CooldownColor[2];
			self.cooldownFlashing = true;
		else
			self.cooldownFlashing = false;
		end
	end
	
	
	-- Adjust Time Left
	if (timeLeft > 60) then
		timeLeft = ceil(timeLeft / 60);
	elseif (timeLeft > 60 * 60) then
		timeLeft = ceil(timeLeft / 60 / 60);
	elseif (timeLeft > 60 * 60 * 24) then
		timeLeft = ceil(timeLeft / 60 / 60 / 24);
	end
	
	--lib.Functions.SetCooldownLabelText(self, timeLeft, modifier, color);
	lib.Functions.SetCooldownLabelText(self, 0, "", color);
end
function lib.Functions.UpdateItemCount(self, kind, value)
	if (self.drawFrame.itemCountLabel == nil) then
		return;
	end
	local kind  	= kind		or self:GetAttribute("type") or "";
	local value 	= value		or self:GetAttribute(kind)   or "";
	
	if (kind == "action") then
		local actionType, actionId, actionSubType, actionSpellID = GetActionInfo(lib.Functions.GetEffectiveActionID(self));
		if (actionType == "item") then
			if (type(actionId) == "number") then
				if (actionId > 0) then
					kind 	= "item";
					value 	= actionId
				end
			end
		end
	end
	
	local itemCountStr = "";
	if ((kind == "item") and (value ~= nil)) then
		local itemId = "";
		if (self:GetAttribute("type") == "macro") then
			itemId = value;
		elseif (self:GetAttribute("type") == "item") then
			itemId = string.gsub(value, "item:", "");
		elseif (self:GetAttribute("type") == "action") then
			itemId = value;
		else
			itemId = nil;
		end
		if ((itemId ~= nil) and (itemId ~= "")) then
			local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemId);
			if (type(maxStack) == "number") then
				--if (maxStack > 1) then
					local itemCount = GetItemCount(itemId);
					if (type(itemCount) == "number") then
						if (itemCount <= 0) then
							itemCount = 0;
						end
					else
						itemCount = 0
					end
					if (itemCount >= 10000) then
						itemCount = itemCount / 1000;
						itemCountStr = ("%d"):format(itemCount).."k";
					elseif (itemCount == 0) then
						itemCountStr = "";
					else
						itemCountStr = ("%d"):format(itemCount);
					end
				--end
			end
		end
	end
	self.drawFrame.itemCountLabel:SetText(itemCountStr);
end
function lib.Functions.UpdateModLayer(self, kind, value, target)
	if (self.modLayer == nil) then
		return;
	end
	local kind  	= kind		or self:GetAttribute("type") or "";
	local value 	= value		or self:GetAttribute(kind)   or "";
	local target	= target	or lib.Functions.GetTarget(self);
	if (self:GetAttribute("type1") == "pet") then
		lib.Functions.UpdateModLayer_Pet(self);
		return;
	end
	-- Check Disabled
	if (kind == "") then
		-- Remove TexturePath
		self.modLayer:SetTexture("");
		self.modLayer.modType = "none";
	
	elseif (self:GetAttribute("type") == "item") then
		if (GetItemCount(value) > 0) then
			-- Remove TexturePath
			self.modLayer:SetTexture("");
			self.modLayer.modType = "none";
		else
			-- Set to Dim Effect
			self.modLayer:SetTexture(unpack(lib.Variables.ModLayer.Disabled));
			self.modLayer.modType = "disabled";
			
		end
		
	-- Check Mana
	elseif (lib.Functions.ManaTooLow(self, kind, value)) then
		-- Set to Low Mana Overlay Effect
		self.modLayer:SetTexture(unpack(lib.Variables.ModLayer.Mana));
		self.modLayer.modType = "lowmana";
	
	-- Check In Range
	elseif (lib.Functions.IsOutOfRange(self, kind, value, target)) then
		-- Set to Out Of Range Overlay Effect
		self.modLayer:SetTexture(unpack(lib.Variables.ModLayer.Range));
		self.modLayer.modType = "outofrange";
		
	-- Check Disabled
	elseif (lib.Functions.IsSpellUnusable(self, kind, value)) then
		-- Set to Dim Effect
		self.modLayer:SetTexture(unpack(lib.Variables.ModLayer.Disabled));
		self.modLayer.modType = "disabled";
		
		-- Special Edit: item throws error (Unknown Reason)
		if (value == "Xu-Fu, Cub of Xuen") then
			-- Remove TexturePath
			self.modLayer:SetTexture("");
			self.modLayer.modType = "none";
		end
		
	-- Otherwise Hide
	else
		-- Remove TexturePath
		self.modLayer:SetTexture("");
		self.modLayer.modType = "none";
		
	end
	
	-- Check Active
	if (lib.Functions.HasAura(self, kind, value)) then
		-- Show Active Texture (Green Glow)
		self.drawFrame.activeTexture:SetVertexColor(0, 1, 0);
		self.drawFrame.activeTexture:Show();
		
	elseif (lib.Functions.IsButtonAutoAttack(self, kind, value)) then
		-- Show Active Texture (Yellow Glow)
		self.drawFrame.activeTexture:SetVertexColor(1, 1, 1);
		self.drawFrame.activeTexture:Show();
		self.blink:SetTexture(unpack(lib.Variables.ModLayer.TextureBlink));
		
	else
		-- Hide Active Texture (Green Glow)
		self.drawFrame.activeTexture:Hide();
		
		-- Hide Blink
		self.blink:SetTexture("");
		
	end
end
function lib.Functions.UpdateModLayer_Pet(self)
	local id = self:GetID();
	-- Check Disabled
	if (GetPetActionSlotUsable(id) == nil) then
		-- Set to Dim Effect
		self.modLayer:SetTexture(unpack(lib.Variables.ModLayer.Disabled));
		self.modLayer.modType = "disabled";
		
	-- Otherwise Hide
	else
		-- Remove TexturePath
		self.modLayer:SetTexture("");
		self.modLayer.modType = "none";
		
	end
end
function lib.Functions.UpdateMacroName(self)
	if (self.drawFrame.macroLabel == nil) then
		return;
	end
	local kind  	= self:GetAttribute("type") or "";
	local value 	= self:GetAttribute(kind)   or "";
	local macroName = "";
	if (kind == "macro") then
		local macroKind, macroValue = lib.Functions.GetButtonMacroAction(self);
		if (macroKind == "item") then
			-- Update Macro Item Count
			lib.Functions.UpdateItemCount(self, macroKind, macroValue);
		else
			-- Get Macro Name
			macroName = GetMacroInfo(value);
		end
		
	-- Action's with Macros
	elseif (kind == "action") then
		macroName = GetActionText(lib.Functions.GetEffectiveActionID(self));
		
	end
	macroName = macroName or "";
	self.drawFrame.macroLabel:SetText(macroName);
end
function lib.Functions.UpdateKeybinding(self)
	--local command = "CLICK "..self:GetName()..":LeftButton";
	--local key = GetBindingKey(command);
	local key = self:GetAttribute("hotkey");
	if (key ~= nil) then
		key = key:gsub("ALT[-]CTRL[-]SHIFT[-]", 	"!+");
		key = key:gsub("ALT[-]CTRL[-]", 			"AC+");
		key = key:gsub("ALT[-]SHIFT[-]", 			"AS+");
		key = key:gsub("CTRL[-]SHIFT[-]", 			"CS+");
		key = key:gsub("SHIFT[-]", 					"S+");
		key = key:gsub("CTRL[-]", 					"C+");
		key = key:gsub("ALT[-]", 					"A+");
		key = key:gsub("NUMPAD", 					"N.");
		
		if (key == "Button4") then
			key = "MB4";
		elseif (key == "Button5") then
			key = "MB5";
		end
		self.drawFrame.hotkeyLabel:SetText(key);
	else
		self.drawFrame.hotkeyLabel:SetText("");
	end
end
function lib.Functions.UpdateMacroIndex(self)
	if (InCombatLockdown()) then
		return;
	end
	local kind = self:GetAttribute("type");
	if (kind == nil) then
		self:SetAttribute("type", nil);
		self:UnregisterEvent("UPDATE_MACROS");
		self.isMacro = false;
		self.macroName = "";
		return;
	end
	local value = self:GetAttribute(kind) or "";
	local name, arg1, arg2 = GetMacroInfo(value);
	-- Check if the Macro was changed
	if ((self.macroName ~= "") and (self.macroName ~= nil) and (self.isMacro == true)) then
		if (self.macroName ~= name) then
			local index = GetMacroIndexByName(self.macroName);
			-- Check if user changed another macro affecting the current macro's Index.
			if (index >= 0) then
				-- Set the button's index to the new Index
				self:SetAttribute("macro", index);
			end
		end
	end
end
function lib.Functions.UpdateFlyout(self, kind, value)
	local kind = kind or self:GetAttribute("type") or "";
	if (kind == "action") then
		kind = GetActionInfo(lib.Functions.GetEffectiveActionID(self))
	end
	
	if ((kind == "flyout") or (self.forceShowFlyout)) then
		self.flyout:Show();
	else
		self.flyout:Hide();
	end
end
function lib.Functions.UpdateBlink(self)
	if (lib.Functions.IsButtonRepeating(self)) then
		self.blink:Show();
	else
		self.blink:Hide();
	end
end
function lib.Functions.UpdateSpellData(self)
	local kind  	= kind  or self:GetAttribute("type") or "";
	local value 	= value or self:GetAttribute(kind)   or "";
	if (kind == "action") then
		local actionType, actionId, actionSubType, actionSpellID = GetActionInfo(lib.Functions.GetEffectiveActionID(self));
		if (actionType == "spell") then
			if (type(actionId) == "number") then
				if (actionId > 0) then
					kind 	= "spell";
					value 	= actionId;
				end
			end
		end
	elseif (kind == "macro") then
		kind, value 	= lib.Functions.GetButtonMacroAction(self);
	end
	
	if (kind == "spell") then
		local count = GetSpellCount(value);
		if (count) then
			if (count > 0) then
				self.drawFrame.itemCountLabel:SetText(count);
			end
		end
		--if (SpellCountTypeSpells[value] ~= nil) then
			--local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff, value1, value2, value3 = UnitBuff("player", "Blood Charge")
			--local count = GetSpellCount(value) or 0; --count or 0;
			--self.drawFrame.itemCountLabel:SetText(count);
		--end
	end
	-- Death Knight
	--if ((kind == "spell") and (value == 45529)) then
		-- Blood Charge
		--local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff, value1, value2, value3 = UnitBuff("player", "Blood Charge")
		--local count = count or 0;
		--self.drawFrame.itemCountLabel:SetText(count);
	--elseif ((kind == "spell") and (value == 49998)) then
		-- Scent of Blood
		--local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff, value1, value2, value3 = UnitBuff("player", "Scent of Blood")
		--local count = count or "";
		--self.drawFrame.itemCountLabel:SetText(count);
	--end
end
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Check Action Status
----------------------------------------------------------------------------------------------------
function lib.Functions.GetUsable(self, kind, value)
	local canUse 	= nil;
	local lowMana 	= nil;
	if (kind == "spell") then
		local spellName = lib.Functions.GetSpellNameFromId(value);
		if (spellName ~= "") then
			canUse, lowMana = IsUsableSpell(spellName);
		end
	elseif (kind == "item") then
		canUse, lowMana = IsUsableItem(value);
	elseif (kind == "action") then
		canUse, lowMana = IsUsableAction(lib.Functions.GetEffectiveActionID(self));
		if (HasAction(lib.Functions.GetEffectiveActionID(self)) ~= 1) then
			canUse, lowMana = 1;
		end
	else
		return false, false;
	end
	return canUse, lowMana;
end
function lib.Functions.IsSpellUnusable(self, kind, value)
	local canUse, lowMana = lib.Functions.GetUsable(self, kind, value);
	if (kind == "action") then
		local actionType, actionId, actionSubType, actionSpellID = GetActionInfo(lib.Functions.GetEffectiveActionID(self));
		if (actionType == "spell") then
			if (type(actionId) == "number") then
				if (actionId > 0) then
					kind 	= "spell";
					value 	= actionId;
				end
			end
		end
	end
	-- Added for Blood Tap
	if ((kind == "spell") and (value == 45529)) then
		local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff, value1, value2, value3 = UnitBuff("player", "Blood Charge")
		local count = count or 0;
		if (count < 5) then
			return true;
		end
	end
	return ((canUse == false) and (lowMana == true));
end
function lib.Functions.ManaTooLow(self, kind, value)
	local canUse, lowMana = lib.Functions.GetUsable(self, kind, value);
	return (lowMana == true);
end
function lib.Functions.GetTarget(self, kind, value)
	local kind  	= kind 	or self:GetAttribute("type") or "";
	local value 	= value or self:GetAttribute(kind)   or "";
	-- if Spell is a Buff, Healing, ect., and Target is Enemy
	if (kind == "spell") then
		if ((IsHarmfulSpell(lib.Functions.GetSpellNameFromId(value)) == 1) and (UnitIsEnemy("player", "target") == 1)) then
			-- Target is Enemy
			-- Spell is a Buff, Healing, ect.
			
			-- Check if Auto-SelfCast Enabled
			if (((GetCVar("autoSelfCast")) and (IsModifiedClick("SELFCAST") == false)) or ((GetCVar("autoSelfCast")  == false) and (IsModifiedClick("SELFCAST")))) then
				-- Spell Target is Self
				return "self";
			else
				-- Spell Target is Target
				return "target";
			end
			
		else
			-- Add here for Focus Modifier
			return "target";
		end
	elseif (kind == "item") then
		if ((IsHarmfulItem(value) == 1) and (UnitIsEnemy("player", "target") == 1)) then
			-- Target is Enemy
			-- Spell is a Buff, Healing, ect.
			
			-- Check if Auto-SelfCast Enabled
			if (((GetCVar("autoSelfCast")) and (IsModifiedClick("SELFCAST") == false)) or ((GetCVar("autoSelfCast")  == false) and (IsModifiedClick("SELFCAST")))) then
				-- Spell Target is Self
				return "self";
			else
				-- Spell Target is Target
				return "target";
			end
			
		else
			-- Add here for Focus Modifier
			return "target";
		end
	else
		return "target";
	end
end
function lib.Functions.IsActionEffective(self, target, kind, value)
	-- if Action is Spell
	if (kind == "spell") then
		-- if the Spell is an Attack Spell, and Target is a Enemy
		if ((IsHarmfulSpell(lib.Functions.GetSpellNameFromId(value)) == 1) and (UnitIsEnemy("player", target) == 1)) then
			return true;
		-- if the Spell is not an Attack Spell, and Target is not a Enemy		
		elseif ((IsHelpfulSpell(lib.Functions.GetSpellNameFromId(value)) == 1) and (UnitIsFriend("player", target) == 1)) then
			return true;
		else
			return false;
		end
	elseif (kind == "action") then
		local ActionType, id, subType, spellID = GetActionInfo(GetEffectiveActionID(self))
		if (ActionType == "spell") then
			return lib.Functions.IsActionEffective(self, target, ActionType, id);
			
		elseif (ActionType == "item") then
			return lib.Functions.IsActionEffective(self, target, ActionType, id);
		
		else
			return true;
			
		end
		-- if the Spell is an Attack Spell, and Target is a Enemy
		if ((IsHarmfulSpell(lib.Functions.GetSpellNameFromId(value)) == 1) and (UnitIsEnemy("player", target) == 1)) then
			return true;
		-- if the Spell is not an Attack Spell, and Target is not a Enemy		
		elseif ((IsHelpfulSpell(lib.Functions.GetSpellNameFromId(value)) == 1) and (UnitIsFriend("player", target) == 1)) then
			return true;
		else
			return false;
		end
	else
		return true;
	end
end
function lib.Functions.IsOutOfRange(self, kind, value, target)
	local inRange 	= nil;
	-- if Action Can be used against target
	-- Spell can't be checked if any of the following
			-- Unit doesn't Exist
			-- Unit is not Visible
			-- Spell/Item is not Effective Against Unit
	if (kind == "action") then
		local ActionType, id, subType, spellID = GetActionInfo(lib.Functions.GetEffectiveActionID(self));
		if (ActionType == "spell") then
			kind = ActionType;
			value = id;
			target = lib.Functions.GetTarget(self, ActionType, id);
			
		elseif (ActionType == "item") then
			kind = ActionType;
			value = id;
			target = lib.Functions.GetTarget(self, ActionType, id);
		
		else
			return false;
			
		end
	end
	if (lib.Functions.IsActionEffective(self, target, kind, value)) then
		-- if Action is Spell
		if (kind == "spell") then
			-- Get In Range
			inRange = IsSpellInRange(lib.Functions.GetSpellNameFromId(value), target);
			
		-- if Action is Item
		elseif (kind == "item") then
			-- Get In Range
			inRange = IsItemInRange(value, target);
			
		end
	end
	-- Return if Out of Range
	return (inRange);
end
function lib.Functions.HasAura(self, kind, value)
	-- Get Kind and Value if not given
	local kind  	= kind		or self:GetAttribute("type") or "";
	local value 	= value		or self:GetAttribute(kind)   or "";
	local customSpellID 	= self:GetAttribute("customSpellIcon") or nil;
	
	if (kind == "spell") then
		if ((value == nil) or (value == "")) then
			return false;
		end
		local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(value);
		name = name or "";
		local nameOfAura, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff, value1, value2, value3 = UnitAura("player", name, nil, "CANCELABLE|PLAYER");
		if (nameOfAura ~= nil) then
			return true;
		end
	elseif (kind == "action") then
		local ActionType, id, subType, spellID = GetActionInfo(lib.Functions.GetEffectiveActionID(self))
		if (ActionType == "spell") then
			return lib.Functions.HasAura(self, ActionType, id);
		end
	elseif (customSpellID ~= nil) then
		local name, rank, icon, powerCost, isFunnel, powerType, castingTime, minRange, maxRange = GetSpellInfo(customSpellID);
		if (name ~= nil) then
			local nameOfAura, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff, value1, value2, value3 = UnitAura("player", name, nil, "CANCELABLE|PLAYER");
			if (nameOfAura ~= nil) then
				return true;
			end
		end
	end
	return false;
end
function lib.Functions.IsButtonAutoAttack(self, kind, value)
	-- Get Kind and Value if not given
	local kind  	= kind		or self:GetAttribute("type") or "";
	local value 	= value		or self:GetAttribute(kind)   or "";
	
	if (kind == "action") then
		local id = lib.Functions.GetEffectiveActionID(self);
		return ((IsAttackAction(id)) or (IsAutoRepeatAction(id)) or (IsCurrentAction(id)));
		
	elseif (kind == "spell") then
		local name, rank, icon, powerCost, isFunnel, powerType, castingTime, minRange, maxRange = GetSpellInfo(value);
		if ((name == nil) or (name == "")) then
			return false;
		end
		return ((IsCurrentSpell(value)) or (IsAutoRepeatSpell(name)) or (IsCurrentSpell(value)));
		
	end
	return false;
end
function lib.Functions.IsButtonRepeating(self, kind, value)
	-- Get Kind and Value if not given
	local kind  	= kind		or self:GetAttribute("type") or "";
	local value 	= value		or self:GetAttribute(kind)   or "";
	
	if (kind == "action") then
		local id = lib.Functions.GetEffectiveActionID(self);
		return ((IsAutoRepeatAction(id)) and (IsCurrentAction(id)));
		
	elseif (kind == "spell") then
		local name, rank, icon, powerCost, isFunnel, powerType, castingTime, minRange, maxRange = GetSpellInfo(value);
		if ((name == nil) or (name == "")) then
			return false;
		end
		return (IsAutoRepeatSpell(name));
		
	end
	return false;
end
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Spell Check Proc
----------------------------------------------------------------------------------------------------
function lib.Functions.CheckSpellProc(self, state, spellID)
	local result = false;
	
	-- Check if Spell Proc should be Shown
	-- Check if Action
	local kind 	= self:GetAttribute("type") or "";
	local value = self:GetAttribute(kind) 	or "";
	if (kind == "action") then
		local actionType, actionId, actionSubType, actionSpellID = GetActionInfo(lib.Functions.GetEffectiveActionID(self));
		if (actionType == "spell") then
			kind = "spell";
			value = actionId;
		elseif (actionType == "macro") then
			kind = "macro";
			value = actionId;
		end
	end
	
	-- Check if Macro
	if (kind == "macro") then
		local name, _ = GetMacroSpell(value);
		if (name == nil) then
			results = false;
		else
			kind = "spell";
			value = name;
		end
	end
	
	-- Run Standard Check
	if (kind == "spell") then
		local valueType = type(value);
		if (valueType == "number") then
			if (value == spellID) then
				result = true;
			end
		else
			local spellName = GetSpellInfo(spellID);
			if (value == spellName) then
				result = true;
			end
		end
	end
	
	
	-- Play Animation if needed
	if (result == false) then
		return;
		
	elseif (result == true) then
		if (state == "show") then
			-- Play Show Animation
			self.spellProc.animIn:Play();
			return;
			
		elseif (state == "hide") then
			-- Play Hide Animation
			self.spellProc.animOut:Play();
			return;
			
		end
	end
end
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Keybinding Functions
----------------------------------------------------------------------------------------------------
function lib.Functions.IsKeyValidForBinding(key)
	-- Chat Buttons
	if (key == "ENTER") then
		return false;
	elseif (key == "BACKSPACE") then
		return false;
	elseif (key == "SPACE") then
		return false;
	elseif (key == "/") then
		return false;
	elseif (key == "TAB") then
		return false;
	elseif (key == "CAPSLOCK") then
		return false;
	elseif (key == "INSERT") then
		return false;
	elseif (key == "DELETE") then
		return false;
		
	-- Special Function Keys
	elseif (key == "ESCAPE") then
		return false;
	elseif (key == "PRINTSCREEN") then
		return false;
		
	-- Camera Buttons	
	elseif (key == "HOME") then
		return false;
	elseif (key == "PAGEUP") then
		return false;
	elseif (key == "PAGEDOWN") then
		return false;
	elseif (key == "END") then
		return false;
		
	-- Direction Keys (Arrows)
	elseif (key == "UP") then
		return false;
	elseif (key == "LEFT") then
		return false;
	elseif (key == "RIGHT") then
		return false;
	elseif (key == "DOWN") then
		return false;

	-- Direction Keys (WASD)
	elseif (key == "W") then
		return false;
	elseif (key == "A") then
		return false;
	elseif (key == "S") then
		return false;
	elseif (key == "D") then
		return false;
		
	-- Mouse Buttons
	elseif (key == "LeftButton") then
		return false;
	elseif (key == "RightButton") then
		return false;
	elseif (key == "MiddleButton") then
		return false;
	
	-- Unknown Key
	elseif (key == "UNKNOWN") then
		return false;
	end
	
	-- Key is Valid
	return true;
end
function lib.Functions.SetKeybinding(self, key)
	ClearOverrideBindings(self);
	if (key ~= nil) then
		SetOverrideBindingClick(self, false, key, self:GetName(), "LeftButton");
	end
	lib.Functions.UpdateKeybinding(self);
end

-- Setup Binding Frame
function lib.Functions.BindingFrameBackdrop()
	local backdrop 			= {};
	-- path to the background texture
	backdrop.bgFile 		= "Interface\\Tooltips\\UI-Tooltip-Background";
	-- path to the border texture
	backdrop.edgeFile 		= "Interface\\Tooltips\\UI-Tooltip-Border";
	-- true to repeat the background texture to fill the frame, false to scale it
	backdrop.tile 			= true;
	-- size (width or height) of the square repeating background tiles (in pixels)
	backdrop.tileSize 		= 16;
	-- thickness of edge segments and square size of edge corners (in pixels)
	backdrop.edgeSize 		= 16;
	-- distance from the edges of the frame to those of the background texture (in pixels)
	backdrop.insets 		= {};
	backdrop.insets.left 	= 3;
	backdrop.insets.right 	= 3;
	backdrop.insets.top 	= 3;
	backdrop.insets.bottom 	= 3;
	return backdrop;
end

function lib.Functions.BindingFrame_OnShow(self)
	if ((lib.Variables.BindingButton ~= nil) and (InCombatLockdown() ~= 1)) then
		local command = "CLICK "..lib.Variables.BindingButton:GetName()..":LeftButton";
		local currentKey = GetBindingKey(command);
		if (currentKey == nil) then
			currentKey = "None";
		end
		self.timeLeft = 5.0;
		local text = "Press the Keys you want to Bind to this Button.\n\nCurrent Binding: |cFFFFFFFF"..currentKey.."|r\n\nTo Delete Binding use |cFFFFFFFFShift-Escape|r.\nTime to Bind: |cFFFFFFFF"..self.timeLeft.."";
		self.label:SetText(text);
		self:ClearAllPoints(); 
		self:SetParent(lib.Variables.BindingButton); 
		self:SetPoint("BOTTOMRIGHT", lib.Variables.BindingButton, "TOPRIGHT", 0, 0);
	else
		self:Hide();
		lib.Variables.BindingButton = nil;
		return;
	end
end

function lib.Functions.BindingFrame_OnUpdate(self, elapsed)
	self.timeLeft  = self.timeLeft - elapsed;
	if (self.timeLeft < 0) then
		self:Hide();
		lib.Variables.BindingButton = nil;
		return;
	end
	local command = "CLICK "..lib.Variables.BindingButton:GetName()..":LeftButton";
	local currentKey = GetBindingKey(command);
	if (currentKey == nil) then
		currentKey = "None";
	end
	local timeleftStr = ("%.1f"):format(self.timeLeft);	
	local text = "Press the Keys you want to Bind to this Button.\n\nCurrent Binding: |cFFFFFFFF"..currentKey.."|r\n\nTo Delete Binding use |cFFFFFFFFShift-Escape|r.\nTime to Bind: |cFFFFFFFF"..timeleftStr.."";
	self.label:SetText(text);
end

function lib.Functions.BindingFrame_OnKeyDown(self, key)
	-- for checking if Key pressed is a Modifier Key
	local modifier = key:sub(2);
	-- Insure that the Button we're Binding to Exists
	if (lib.Variables.BindingButton ~= nil) then
		-- if the user presses a Modifier Key
		if ((modifier == "SHIFT") or (modifier == "CTRL") or (modifier == "ALT") or (key == "UNKNOWN")) then
			-- Return from Function, for this Key Press
			return;
			
		-- if the user presses Escape and the Shift Keys
		elseif ((key == "ESCAPE") and (IsShiftKeyDown())) then
			-- Clear the Key, to remove Binding
			key = nil;
		
		-- if the user presses the Escape Key
		elseif (key == "ESCAPE") then
			-- Clear the Button Variable
			lib.Variables.BindingButton = nil;
			-- Hide the Binding Frame
			self:Hide();
			-- Exit
			return;
			
		-- if the user presses the "Take Screenshot" Key
		elseif (GetBindingFromClick(key) == "SCREENSHOT") then
			-- Take Screenshot
			TakeScreenshot();
			-- Return from Function, for this Key Press
			return;
			
		-- if the user presses any other key
		elseif (lib.Functions.IsKeyValidForBinding(key)) then
			-- if the user presses the Shift Key
			if (IsShiftKeyDown()) then
				-- Add the Shift Key
				key = "SHIFT-"..key;
			end
			-- if the user presses the Control Key
			if (IsControlKeyDown()) then
				key = "CTRL-"..key;
			end
			-- if the user presses the Alt Key
			if (IsAltKeyDown()) then
				key = "ALT-"..key;
			end
		else
			print("|cFF00FF00FoxCustomButtons|r: That key isn't a valid Hotkey.");
			print("|cFF00FF00FoxCustomButtons|r: Hotkeys cannot be \"|cFFFF0000"..key.."|r\".");
			return;
		end
		lib.Functions.KeyBindFinish(lib.Variables.BindingButton, key);
	end
	-- Clear the Button Variable
	lib.Variables.BindingButton = nil;
	-- Hide the Binding Frame
	self:Hide();
end

function lib.Functions.SetupBindingFrame(self, parent)
	-- Setup Binding Frame
	if (self == nil) then
		-- Create Frame, if Needed
		self = CreateFrame("Frame");
	end
	-- Enable Keyboard Input
	self:EnableKeyboard(true);
	-- Set Frame Strata
	self:SetFrameStrata("DIALOG");
	-- Set Size
	self:SetSize(210, 95);
	-- Clear Old Points
	self:ClearAllPoints();
	-- Set Point
	self:SetPoint("CENTER", 0, 0);
	self:SetClampedToScreen(true);
	-- Set Backdrop
	self:SetBackdrop(lib.Functions.BindingFrameBackdrop());
	-- Set Backdrop Color
	self:SetBackdropColor(0, 0, 0);
	
	-- Set Time Left Variable
	self.timeLeft = 5.0
	
	-- Setup Label
	if (self.label == nil) then
		-- Create Label, if Needed
		self.label = self:CreateFontString(nil);
	end
	-- Initialize Label
	lib.Functions.SetupText(self.label, 	"TOPLEFT", 		10, -10, "LEFT", "TOP", "ARTWORK", 1, 10);
	-- Set Additional Point
	self.label:SetPoint(	"BOTTOMRIGHT", -10,  10);
	-- Set Font Object
	self.label:SetFontObject(GameFontNormalSmall);
	
	-- Setup Events
	-- Set Show
	self:SetScript("OnShow", 	lib.Functions.BindingFrame_OnShow);
	-- Set Update
	self:SetScript("OnUpdate", 	lib.Functions.BindingFrame_OnUpdate);
	-- Set Keydown
	self:SetScript("OnKeyDown", lib.Functions.BindingFrame_OnKeyDown);
	self:SetScript("OnMouseDown", lib.Functions.BindingFrame_OnKeyDown);
	
	-- Hide Frame Until Needed
	self:Hide();
	
	-- Return Frame
	return self;
end

function lib.Functions.ShowBindingFrame(self)
	if (lib.Variables.BindingFrame ~= nil) then
		if (lib.Variables.BindingFrame:IsShown()) then
			return;
		end
	end
	lib.Variables.BindingButton 	= self;
	lib.Variables.BindingFrame 		= lib.Functions.SetupBindingFrame(lib.Variables.BindingFrame, lib.Variables.BindingButton);
	lib.Variables.BindingFrame:Show();
end

function lib.Functions.KeyBindStart(self)
	ShowBindingFrame(self);
end

function lib.Functions.KeyBindFinish(self, key)
	-- if Library has been given a custom KeyBinding Handler Function
	if (self.keyBindHandlerFunc ~= nil) then
		self.keyBindHandlerFunc(self, key);
	else
		-- Set KeyBinding
		lib.Functions.SetKeybinding(self, key);
		
		-- Update KeyBinding Data
		self:SetAttribute("hotkey", key);
		
		-- Update Binding Text
		lib.Functions.UpdateKeybinding(self);
		-- Run Update Function
		if (self.hotkeyUpdated ~= nil) then
			self.hotkeyUpdated(self);
		end
		-- if the Binding Should be saved
		if (self.saveKeyBinding) then
			-- Save the Current Key Binding
			SaveBindings(GetCurrentBindingSet());
		end
	end
end
function lib.Functions.HotkeyCheck(self, name, value)
	if (name == "hotkey") then
		-- Get Hotkey
		local hotkey = self:GetAttribute("hotkey");
		ClearOverrideBindings(self);
		if ((hotkey ~= nil) and (hotkey ~= "")) then
			SetOverrideBindingClick(self, false, hotkey, self:GetName(), "LeftButton");
			self:UpdateAllHotkeys(self:GetName(), hotkey);
		end
	end
end

function lib.Functions.HotkeyUsableUpdate(self)
	local hotkey = self:GetAttribute("hotkey");
	ClearOverrideBindings(self);
	if ((hotkey ~= nil) and (hotkey ~= "")) then
		if (self:IsVisible()) then
			SetOverrideBindingClick(self, false, hotkey, self:GetName(), "LeftButton");
		end
	end
end
----------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------
-- Tooltip Functions
----------------------------------------------------------------------------------------------------
-- TooltipSetAnchor, void
	-- Set's the Tooltip's Anchor Position
function lib.Functions.TooltipSetAnchor(self)
	if (GetCVar("UberTooltips") == "1") then
		GameTooltip_SetDefaultAnchor(GameTooltip, self);
	else
		GameTooltip:SetOwner(self, "ANCHOR_LEFT");
	end
end

-- ShowTooltip, void
	-- Show's the Tooltip for the Action Button
function lib.Functions.ShowTooltip(self)
	if (GetMouseFocus() ~= self) then
		return;
	end
	local kind  	= self:GetAttribute("type")  or "";
	local value 	= self:GetAttribute(kind)    or "";
	local kindPet 	= self:GetAttribute("type1") or "";
	
	local toolTipId	= self:GetAttribute("tooltip") or "";
	
	if (toolTipId ~= "") then
		-- Clears the Tooltip
		GameTooltip:ClearLines();
		-- Setup Anchor
		lib.Functions.TooltipSetAnchor(self);
		-- Set the Tooltip to the Spell's Tooltip
		GameTooltip:SetSpellByID(toolTipId); 
		-- Show the Tooltip
		GameTooltip:Show();
	end
	
	-- Show Tooltip
	if (self.tooltipName ~= nil) then
		-- Clears the Tooltip
		GameTooltip:ClearLines();
		-- Setup Anchor
		lib.Functions.TooltipSetAnchor(self);
		GameTooltip:SetText(self.tooltipName, 1.0, 1.0, 1.0);
		-- Show the Tooltip
		GameTooltip:Show();
		return;
	end
	
	if (((kind == "") or (value == "")) and (kind ~= "action") and (kind ~= "flyout")) then
		if (kindPet == "") then
			return;
		end
	end
		
	if (kind == "spell") then
		-- Clears the Tooltip
		GameTooltip:ClearLines();
		-- Setup Anchor
		lib.Functions.TooltipSetAnchor(self);
		-- Set the Tooltip to the Spell's Tooltip
		GameTooltip:SetSpellByID(value); 
		-- Show the Tooltip
		GameTooltip:Show();
	elseif (kind == "item") then
		-- Clears the Tooltip
		GameTooltip:ClearLines();
		-- Setup Anchor
		lib.Functions.TooltipSetAnchor(self);
		-- Set the Tooltip to the Items's Tooltip
		GameTooltip:SetItemByID(string.gsub(value, "item:", "")); 
		-- Show the Tooltip
		GameTooltip:Show();
	elseif (kind == "macro") then
		local macroBody = GetMacroBody(value);
		if (macroBody ~= nil) then
			local match = string.match(macroBody, "#showtooltip");
			if (match ~= nil) then
				local kind, value, target = lib.Functions.GetButtonMacroAction(self);
				if (kind == "spell") then
					-- Clears the Tooltip
					GameTooltip:ClearLines();
					-- Setup Anchor
					lib.Functions.TooltipSetAnchor(self);
					-- Set the Tooltip to the Spell's Tooltip
					GameTooltip:SetSpellByID(lib.Functions.GetSpellId(value)); 
					-- Show the Tooltip
					GameTooltip:Show();
				elseif (kind == "item") then
					-- Clears the Tooltip
					GameTooltip:ClearLines();
					-- Setup Anchor
					lib.Functions.TooltipSetAnchor(self);
					-- Set the Tooltip to the Items's Tooltip
					GameTooltip:SetItemByID(lib.Functions.GetItemId(value)); 
					-- Show the Tooltip
					GameTooltip:Show();
				end
			else
				-- Clears the Tooltip
				GameTooltip:ClearLines();
				-- Setup Anchor
				lib.Functions.TooltipSetAnchor(self);
				-- Set the Tooltip to the Macro's Name
				local name, iconTexture, body, isLocal = GetMacroInfo(value);
				-- Set the Tooltip to the Macro's Tooltip
				GameTooltip:SetText(name, 1.0, 1.0, 1.0);
				-- Show the Tooltip
				GameTooltip:Show();
			end
		end
	elseif ((kind == "MOUNT") or (kind == "CRITTER")) then
		local arg1, arg2, spellID = nil;
		-- Get Texture Path
		arg1, arg2, spellID = GetCompanionInfo(kind, value);
		
		-- Clears the Tooltip
		GameTooltip:ClearLines();
		-- Setup Anchor
		lib.Functions.TooltipSetAnchor(self);
		-- Set the Tooltip to the Spell's Tooltip
		GameTooltip:SetSpellByID(spellID);
		-- Show the Tooltip
		GameTooltip:Show();
	elseif (kindPet == "pet") then
		-- Setup Anchor
		lib.Functions.TooltipSetAnchor(self);
		local name, subtext, texture, isToken, isActive, autoCastAllowed, autoCastEnabled = GetPetActionInfo(self:GetID());
		-- Set Tooltip
		if ( not isToken ) then
			GameTooltip:SetPetAction(self:GetID());
		else
			GameTooltip:ClearLines();
			GameTooltip:SetText(_G[name], 1.0, 1.0, 1.0);
			if (subtext) then
				GameTooltip:AddLine(subtext, GameFontNormalLeftGrey.r, GameFontNormalLeftGrey.g, GameFontNormalLeftGrey.b);
			end
		end
		GameTooltip:Show();
	elseif (kind == "action") then
		-- Setup Anchor
		lib.Functions.TooltipSetAnchor(self);
		GameTooltip:SetAction(lib.Functions.GetEffectiveActionID(self));
		GameTooltip:Show();
		
	elseif (kind == "flyout") then
		local value = self:GetAttribute("spell");
		local name, description, numSlots, isKnown = GetFlyoutInfo(value);
		-- Setup Anchor
		lib.Functions.TooltipSetAnchor(self);
		GameTooltip:ClearLines();
		GameTooltip:AddLine(name, 1.0, 1.0, 1.0);
		GameTooltip:AddLine(description, GameFontNormalLeftOrange.r, GameFontNormalLeftOrange.g, GameFontNormalLeftOrange.b);
		GameTooltip:Show();
		
	end
end

-- HideTooltip, void
	-- Hide's the Tooltip for the Action Button
function lib.Functions.HideTooltip()
	GameTooltip:Hide();
end
----------------------------------------------------------------------------------------------------


function lib.Functions.SetupText(self, anchorPoint, anchorX, anchorY, justifyH, justifyV, layer, subLayer, fontSize, fontColor)
	-- Set Points
	self:ClearAllPoints();
	self:SetPoint(anchorPoint, anchorX, anchorY);
	
	-- Set Layer
	self:SetDrawLayer(layer, subLayer);
	
	-- Set Font
	self:SetFont("Fonts\\FRIZQT__.TTF", fontSize, "OUTLINE");
	-- Set Justification
	self:SetJustifyH(justifyH);
	self:SetJustifyV(justifyV);
	-- Set Shadow
	self:SetShadowOffset(1, -1);
	-- Set Color
	if (fontColor ~= nil) then
		self:SetTextColor(unpack(fontColor));
	end
end

----------------------------------------------------------------------------------------------------
-- Graphics, Get/Set
----------------------------------------------------------------------------------------------------
function lib.Functions.Graphics.GetNormalTexture()
	return lib.Variables.Graphics.NormalTexture;
end
function lib.Functions.Graphics.GetPushedTexture()
	return lib.Variables.Graphics.PushedTexture;
end
function lib.Functions.Graphics.GetHighlightTexture()
	return lib.Variables.Graphics.HighlightTexture;
end

function lib.Functions.Graphics.SetNormalTexture(path)
	lib.Variables.Graphics.NormalTexture		= path;
	lib.Functions.Graphics.Update();
end
function lib.Functions.Graphics.SetPushedTexture(path)
	lib.Variables.Graphics.PushedTexture		= path;
	lib.Functions.Graphics.Update();
end
function lib.Functions.Graphics.SetHighlightTexture(path)
	lib.Variables.Graphics.HighlightTexture	= path;
	lib.Functions.Graphics.Update();
end
function lib.Functions.Graphics.Update()
	local self;
	for i = 0, #lib.Buttons, 1 do
		self = lib.Buttons[i];
		if (lib.Buttons[i] ~= nil) then
			lib.Functions.Setup.Graphics(self);
		end
	end
end
----------------------------------------------------------------------------------------------------

function lib.Functions.SetupAsPetAction(self, id)
	-- Set the ID
	self:SetID(id);
	
	-- Set the Types
	self:SetAttribute("type1", 			"pet");
	self:SetAttribute("type2", 			"macro");
	self:SetAttribute("*action1", 		id);
	self:SetAttribute("*macrotext2", 	"/click ".."PetActionButton"..id.." RightButton 1");
	
	-- Set the Drag/Receive Events
	self:SetAttribute("_onreceivedrag", lib.Variables.DropActionPet);
	self:SetAttribute("_ondragstart", 	lib.Variables.PickupActionPet);
	
	-- Set Events
	self:SetScript("PreClick", 			lib.Functions.Events.PreClickPet);
	
	-- Hook the PostClick to Allow Dragging
	self:SetScript("PostClick", 		lib.Functions.Events.PostClickPet);
end


function GetFlyoutSpellBookId(id)
	local iii=1
	while true do
		local sbtype, sbid = GetSpellBookItemInfo(iii, "spell")
		if sbtype=="FLYOUT" and sbid==id then
			--PickupSpellBookItem(iii, "spell")
			return iii; --true
		elseif sbtype==nil and sbid==nil then
			return nil; --false
		end
		iii = iii+1
	end
end
function PickupFlyout(self, kind, value)
	if (InCombatLockdown()) then 
		return; 
	end
	-- Get Id
	local spellBookId = GetFlyoutSpellBookId(value);
	
	-- Pickup Spell
	if (spellBookId ~= nil) then
		-- Clear Cursor
		ClearCursor();
		-- Pickup New Spell
		PickupSpellBookItem(spellBookId, "spell");
	end
	
end
function PickupMount(self, kind, value)
	if (true) then
		return;
	end
	if (InCombatLockdown()) then 
		return; 
	end
	-- Get Id from Name
	local mountId = GetMountName(value);
	
	-- Pickup Spell
	if (mountId ~= nil) then
		-- Clear Cursor
		ClearCursor();
		-- Pickup Mount
		PickupCompanion("MOUNT", mountId);
	end
end
function PickupCritter(self, kind, value)
	if (true) then
		return;
	end
	if (InCombatLockdown()) then 
		return; 
	end
	-- Get Id from Name
	local critterId = GetCritterName(value);
	
	-- Pickup Spell
	if (critterId ~= nil) then
		-- Clear Cursor
		ClearCursor();
		-- Pickup Mount
		PickupCompanion("CRITTER", critterId);
	end
end


function GetMountId(name)
	for i = 1, GetNumCompanions("MOUNT"), 1 do
		local creatureID, creatureName, creatureSpellID, icon, issummoned, mountTypeID = GetCompanionInfo("MOUNT", i);
		if (name == creatureName) then
			return i;
		end
	end
end
function GetCritterId(name)
	for i = 1, GetNumCompanions("CRITTER"), 1 do
		local creatureID, creatureName, creatureSpellID, icon, issummoned, mountTypeID = GetCompanionInfo("CRITTER", i);
		if (name == creatureName) then
			return i;
		end
	end
end

function UpdateCompanionInfo(self)
	if (InCombatLockdown()) then 
		return; 
	end
	
	local companionName = self:GetAttribute("CompanionName");
	if (companionName == nil) then
		-- Companion Name Hasnt Been Set Yet
		local id = self:GetAttribute("CRITTER") or self:GetAttribute("MOUNT") or 0;
		if (id == 0) then
			return;
		end
		local comType = self:GetAttribute("type");
		local creatureID, name, creatureSpellID, icon, issummoned, mountTypeID = GetCompanionInfo(comType, id);
		self:SetAttribute("CompanionName", name);
	else
		-- Reset Companion Info
		local comType = self:GetAttribute("type");
		local id = 0;
		if (comeType ~= nil) then
			if (comeType == "CRITTER") then
				id = GetCritterId(companionName);
			elseif (comeType == "MOUNT") then
				id = GetMountId(companionName);
			end
			self:SetAttribute(comeType, id);
		end
	end
end
