local addonName, addonTable = ...
-- libFoxActionButton_LibVariables.lua
-- Handles the Local Libraries' Variables.

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local LibraryName		= addonTable.Name;
local LibraryVersion	= addonTable.Version;
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName.." "..LibraryVersion, true) ~= nil) then
	lib = LibStub:GetLibrary(LibraryName.." "..LibraryVersion);
else
	return;
end

----------------------------------------------------------------------------------------------------
--  Get Console
----------------------------------------------------------------------------------------------------
Console = LibStub:GetLibrary("FoxConsole");

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Secure Snippets
----------------------------------------------------------------------------------------------------
-- OnAttributeChanged
lib.Variables.OnAttributeChangedPre = [[
		--print(self, name, value)
		
		if ((name ~= "state-isstealth") and (name ~= "state-form") and (name ~= "state-pages") and (name ~= "state-bonusPages") and (name ~= "needsupdate")) then
			control:CallMethod("UpdateIcon", self);
			return;
		end
		
		if (name == "needsupdate") then
			if (self:GetAttribute("needsUpdate") ~= true) then
				return;
			else
				self:SetAttribute("actionpage", GetActionBarPage());
				self:SetAttribute("form", GetShapeshiftForm());
				self:SetAttribute("isStealthed", IsStealthed());
			end
		end
		
		if (name == "state-bonuspages") then
			self:SetAttribute("actionpage", 6 + value);
		elseif (name == "state-pages") then
			self:SetAttribute("actionpage", value);
		end
		
		if (name == "state-isstealth") then
			if (value == "true") then
				self:SetAttribute("isStealthed", true);
			else
				self:SetAttribute("isStealthed", false);
			end
		elseif (name == "state-form") then
			self:SetAttribute("form", value);
			
		end
		
		local className = self:GetAttribute("playerUnitClass");
		if (className == "DRUID") then
			local isStealthed = self:GetAttribute("isStealthed");
			local form = self:GetAttribute("form");
			if (form == 1) then
				-- Bear (BonusBar:3)
				self:SetAttribute("form", form);
				self:SetAttribute("actionpage", 9);
				
			elseif ((form == 2) and (isStealthed)) then
				-- Cat (BonusBar:2)
				self:SetAttribute("form", form);
				self:SetAttribute("actionpage", 8);
				
			elseif (form == 2) then
				-- Cat (BonusBar:1)
				self:SetAttribute("form", form);
				self:SetAttribute("actionpage", 7);
				
			else
				-- Others (ActionBar:1)
				self:SetAttribute("form", 0);
				self:SetAttribute("actionpage", 1);
				
			end
		else
			if (name == "state-bonuspages") then
				self:SetAttribute("actionpage", 6 + value);
			elseif (name == "state-pages") then
				self:SetAttribute("actionpage", value);
			end
		end
		
		control:CallMethod("UpdateIcon", self);
		self:SetAttribute("needsUpdate", false);
			
	]]
	
lib.Variables.StateChanges = [[
	-- self, stateid, newstate
	if (stateId == "form0") then
		-- Page #xxx
	elseif (stateId == "form0Stealth") then
		-- Page #xxx
	elseif (stateId == "form1") then
		-- Page #xxx
	elseif (stateId == "form2") then
		-- Page #xxx
	elseif (stateId == "form3") then
		-- Page #xxx
	elseif (stateId == "form3Stealth") then
		-- Page #xxx
	elseif (stateId == "form4") then
		-- Page #xxx
	elseif (stateId == "form5") then
		-- Page #xxx
	elseif (stateId == "form6") then
		-- Page #xxx
	end
	
]]

-- Drag, Drop
lib.Variables.DropAction 		= [[
	--print(kind, value, ...);
	
	if (self:GetAttribute("actionpage") == nil) then
		self:SetAttribute("needsUpdate", true);
	end
	
	local itemName = "";
	local itemId;
	if (kind == "item") then
		local s = string.match(..., "|h.*|h")
		s = string.gsub(s, "%[", "");
		s = string.gsub(s, "%]", "");
		itemName = s;
	end
	
	if (self:GetAttribute("type") == "action") then
		local id = self:GetID();
		local page = self:GetAttribute("actionpage") or 1;
		local effectiveId = 0;
		effectiveId = id + ((page - 1) * 12);
		
		if (HasAction(effectiveId)) then
			return "action", effectiveId;
		else
			return "action", effectiveId;
		end
	end

	local oldKind = self:GetAttribute("type") or "";
	local oldValue = self:GetAttribute(oldKind) or "";
	if (oldKind == "flyout") then
		oldValue = self:GetAttribute("spell") or "";
	end
	if ((kind=="item") or (kind=="spell") or (kind=="macro")) then
		if (kind=="item") then
			value = "item:"..value
			
		elseif (kind=="spell") then
			value = select(2,...) -- the raw spell id (4th return GetCursorInfo)
		end
		-- "item","item:1234" or "spell",1234 or "macro",1234
		
		-- Clear Attributes First
		self:SetAttribute("spell", nil);
		self:SetAttribute("item",  nil);
		self:SetAttribute("macro", nil);
		self:SetAttribute("type",  nil);
		self:SetAttribute("battlepet", nil);
		
		-- Set Attributes
		self:SetAttribute(kind,   value);
		self:SetAttribute("type",  kind);
		
		if ((oldKind=="item") or (oldKind=="spell") or (oldKind=="macro")) then
			return "clear", oldKind, oldValue;
			
		elseif ((oldKind=="MOUNT") or (oldKind=="CRITTER")) then
			return "clear", "companion", oldKind, oldValue;
			
		elseif (oldKind=="flyout") then
			self:CallMethod("PickupFlyout", oldKind, oldValue);
			return;
			
		else
			return "clear";
			
		end
		
	elseif (kind == "battlepet") then
		-- Clear Attributes First
		self:SetAttribute("spell", nil);
		self:SetAttribute("item",  nil);
		self:SetAttribute("macro", nil);
		self:SetAttribute("type",  nil);
		self:SetAttribute("battlepet", nil);
		
		local text = "/run C_PetJournal.SummonPetByGUID(\'"..value.."\');";
		
		self:SetAttribute("battlepetid", value);
		
		self:SetAttribute("type", "macro")
		self:SetAttribute("macrotext", text);
		self:SetAttribute("battlepet", true);
		
		return "clear", oldKind, oldValue;
	
	elseif (kind=="companion") then
		-- Clear Attributes First
		self:SetAttribute("spell", nil);
		self:SetAttribute("item",  nil);
		self:SetAttribute("macro", nil);
		self:SetAttribute("type",  nil);
		self:SetAttribute("battlepet", nil);
		
		-- Set Attributes
		local kind = select(1, ...);
		self:SetAttribute("type",  kind);
		self:SetAttribute(kind,   value);
		
		-- Update the Companion Info
		self:CallMethod("UpdateCompanions", self);
		
		if ((oldKind=="item") or (oldKind=="spell") or (oldKind=="macro")) then
			return "clear", oldKind, oldValue;
			
		elseif ((oldKind=="MOUNT") or (oldKind=="CRITTER")) then
			return "clear", "companion", oldKind, oldValue;
		
		elseif (oldKind=="flyout") then
			self:CallMethod("PickupFlyout", oldKind, oldValue);
			return;
			
		else
			return "clear";
			
		end
	elseif (kind=="flyout") then
		
		-- Clear Attributes First
		self:SetAttribute("spell", nil);
		self:SetAttribute("item",  nil);
		self:SetAttribute("macro", nil);
		self:SetAttribute("type",  nil);
		self:SetAttribute("battlepet", nil);
		
		-- Set Attributes
		self:SetAttribute("type",   kind);
		self:SetAttribute("spell", value);
		
		if ((oldKind=="item") or (oldKind=="spell") or (oldKind=="macro")) then
			return "clear", oldKind, oldValue;
			
		elseif ((oldKind=="MOUNT") or (oldKind=="CRITTER")) then
			return "clear", "companion", oldKind, oldValue;
		
		elseif (oldKind=="flyout") then
			self:CallMethod("PickupFlyout", oldKind, oldValue);
			return;
			
		else
			return "clear";
			
		end
	end
]]

lib.Variables.PickupAction 		= [[
	if (self:GetAttribute("actionpage") == nil) then
		self:SetAttribute("needsUpdate", true);
	end
	
	if (self:GetAttribute("type") == "action") then
		local id = self:GetID();
		local page = self:GetAttribute("actionpage") or 1;
		local effectiveId = 0;
		effectiveId = id + ((page - 1) * 12);
		
		if (HasAction(effectiveId)) then
			return "action", effectiveId;
		else
			return "action", effectiveId;
		end
	end

	local oldKind = self:GetAttribute("type") or "";
	local oldValue = self:GetAttribute(oldKind) or "";
	
	local isBattlePet = self:GetAttribute("battlepet") or false;
	
	if (oldKind == "flyout") then
		oldValue = self:GetAttribute("spell") or "";
	end
	local arg1 = select(1, ...);
	if ((kind=="item") or (kind=="spell") or (kind=="macro")) then
		self:SetAttribute("type",kind);
		if (kind=="item") then
			value = "item:"..value;
		elseif (kind=="spell") then
			value = select(2,...); -- the raw spell id (4th return GetCursorInfo)
		end
		-- "item","item:1234" or "spell",1234 or "macro",1234
		self:SetAttribute(kind, value);
	elseif ((arg1 == "MOUNT") or (arg1 == "CRITTER")) then
		local kind = arg1;
		self:SetAttribute("type", kind);
		self:SetAttribute(kind, value);
		
		-- Update the Companion Info
		self:CallMethod("UpdateCompanions", self);
	elseif (kind == "flyout") then
		self:SetAttribute("type", kind);
		self:SetAttribute("spell", value);
		
	else
		self:SetAttribute("type", "");
		self:SetAttribute(oldKind, "");
		
		if (isBattlePet) then
			self:SetAttribute("macrotext", "");
			self:SetAttribute("battlepet", nil);
		end
	end
	
	if ((oldKind=="item") or (oldKind=="spell") or (oldKind=="macro") or (oldKind=="action")) then
		return "clear", oldKind, oldValue;
		
	elseif ((oldKind == "MOUNT") or (oldKind == "CRITTER")) then
		return "clear", "companion", oldKind, oldValue;
		
	elseif (oldKind == "flyout") then
		self:CallMethod("PickupFlyout", oldKind, oldValue);
		return;
		
	else
		return "clear";
		
	end
]]

-- Drag, Drop {Pet Bar}
lib.Variables.DropActionPet 		= [[
	local oldKind = self:GetAttribute("type1") or "";
	local oldValue = self:GetAttribute("*action1") or "";
	if (kind == "petaction") then
		return "petaction", self:GetID();
	else
		return "clear";
	end
]]

lib.Variables.PickupActionPet	= [[
	local oldKind = self:GetAttribute("type1") or "";
	
	if ((kind == "petaction") or (kind == nil)) then
		return "petaction", self:GetID();
		
	else
		return "clear";
		
	end
]]

lib.Variables.SpecStateChange = [[
	local newstate = self:GetAttribute(name);
	if (name == "state-currentspec") then
		if (newstate == "spec1") then
			local oldSpec_kind 	= self:GetAttribute("type");
			local oldSpec_value = self:GetAttribute(oldSpec_kind or "nil");
			if (oldSpec_kind == "flyout") then
				oldSpec_value = self:GetAttribute("spell");
			end
			
			local newSpec_kind 	= self:GetAttribute("type".."_s1");
			local newSpec_value = self:GetAttribute((newSpec_kind or "nil").."_s1" or "");
			if (newSpec_kind == "flyout") then
				newSpec_value = self:GetAttribute("spell".."_s1");
			end
			
			-- Set Current Value
			self:SetAttribute("type", newSpec_kind);
			if (newSpec_kind ~= nil) then
				if (newSpec_kind ~= "flyout") then
					self:SetAttribute(newSpec_kind, newSpec_value);
				else
					self:SetAttribute("spell", newSpec_value);
				end
			end
			
			-- Set Old Spec Value
			self:SetAttribute("type".."_s2", oldSpec_kind);
			if (oldSpec_kind ~= "flyout") then
				self:SetAttribute((oldSpec_kind or "nil").."_s2", oldSpec_value);
			else
				self:SetAttribute("spell".."_s2", oldSpec_value);
			end
			
		elseif (newstate == "spec2") then
			local oldSpec_kind 	= self:GetAttribute("type");
			local oldSpec_value = self:GetAttribute(oldSpec_kind or "");
			if (oldSpec_kind == "flyout") then
				oldSpec_value = self:GetAttribute("spell");
			end
			
			local newSpec_kind 	= self:GetAttribute("type".."_s2");
			local newSpec_value = self:GetAttribute((newSpec_kind or "nil").."_s2" or "");
			if (newSpec_kind == "flyout") then
				newSpec_value = self:GetAttribute("spell".."_s2");
			end
			
			-- Set Current Value
			self:SetAttribute("type", newSpec_kind);
			if (newSpec_kind ~= nil) then
				if (newSpec_kind ~= "flyout") then
					self:SetAttribute(newSpec_kind, newSpec_value);
				else
					self:SetAttribute("spell", newSpec_value);
				end
			end
			
			-- Set Old Spec Value
			self:SetAttribute("type".."_s1", oldSpec_kind);
			if (oldSpec_kind ~= "flyout") then
				self:SetAttribute((oldSpec_kind or "nil").."_s1", oldSpec_value);
			else
				self:SetAttribute("spell".."_s1", oldSpec_value);
			end
			
		end
	end
]]

-- Show, Hide
lib.Variables.OnShow = [[
	local hotkey = self:GetAttribute("hotkey");
	self:ClearBindings();
	if ((hotkey ~= nil) and (hotkey ~= "")) then
		self:SetBindingClick(false, hotkey, self, "LeftButton");
	end
	self:CallMethod("UpdateButton", self);
]]

lib.Variables.OnHide = [[
	local hotkey = self:GetAttribute("hotkey");
	if ((hotkey ~= nil) or (hotkey ~= "")) then
		self:ClearBindings();
	end
]]

-- Cooldown Constants
lib.CONSTANTS.COOLDOWNUSECOLORS 				= 0;
lib.CONSTANTS.COOLDOWNUSELETTERS 				= 1;
lib.CONSTANTS.COOLDOWNUSEBOTH 					= 2;

-- Color Constants
lib.CONSTANTS.COLOR_WHITE 						= { 1.00, 1.00, 1.00 };

-- Default Texture Variables
lib.Variables.Defaults["TextureDisabled"] 		= { 0, 0, 0, 0.6 };
lib.Variables.Defaults["TextureActive"] 		= { "Interface\\Buttons\\CheckButtonHilight" };
lib.Variables.Defaults["TextureManaToLow"] 		= { 0, 0, 1, 0.6 };
lib.Variables.Defaults["TextureOutOfRange"] 	= { 1, 0, 0, 0.6 };
lib.Variables.Defaults["TextureEnable"] 		= { "Interface\\ICONS\\Spell_Nature_WispSplode" };
lib.Variables.Defaults["TextureEnableGlow"] 	= { "Interface\\Buttons\\CheckButtonHilight" };
lib.Variables.Defaults["TextureBlink"] 			= { 1, 0, 0, 0.6 };

-- Default Color Variables
lib.Variables.Defaults["CooldownNormal"]		= { 1.00, 0.82, 0.00 };
lib.Variables.Defaults["CooldownFlash"]			= { 1.00, 0.00, 0.00 };
lib.Variables.Defaults["CooldownSecond"]		= { 1.00, 0.82, 0.00 };
lib.Variables.Defaults["CooldownMinute"]		= { 0.00, 1.00, 0.00 };
lib.Variables.Defaults["CooldownHour"]			= { 1.00, 0.00, 0.00 };
lib.Variables.Defaults["CooldownDay"]			= { 0.00, 0.00, 1.00 };

-- Default Cooldown Type
lib.Variables.Defaults["CooldownType"] 			= lib.CONSTANTS.COOLDOWNUSEBOTH;

-- ModLayer Texture Variables
lib.Variables.ModLayer.Disabled 				= lib.Variables.Defaults["TextureDisabled"];
lib.Variables.ModLayer.Active 					= lib.Variables.Defaults["TextureActive"];
lib.Variables.ModLayer.Mana 					= lib.Variables.Defaults["TextureManaToLow"];
lib.Variables.ModLayer.Range 					= lib.Variables.Defaults["TextureOutOfRange"];
lib.Variables.ModLayer.Enable 					= lib.Variables.Defaults["TextureEnable"];
lib.Variables.ModLayer.EnableGlow				= lib.Variables.Defaults["TextureEnableGlow"];
lib.Variables.ModLayer.TextureBlink 			= lib.Variables.Defaults["TextureBlink"];

-- ModLayer Color Variables
lib.Variables.Colors.CooldownColor[1] 			= lib.Variables.Defaults["CooldownNormal"]; 	-- Normal
lib.Variables.Colors.CooldownColor[2] 	 		= lib.Variables.Defaults["CooldownFlash"]; 	-- Flash
lib.Variables.Colors.CooldownColor[3] 			= lib.Variables.Defaults["CooldownSecond"]; 	-- Seconds
lib.Variables.Colors.CooldownColor[4] 	 		= lib.Variables.Defaults["CooldownMinute"]; 	-- Minutes
lib.Variables.Colors.CooldownColor[5] 	 		= lib.Variables.Defaults["CooldownHour"]; 	-- Hours
lib.Variables.Colors.CooldownColor[6] 	 		= lib.Variables.Defaults["CooldownDay"]; 		-- Days

-- Cooldown Type Variables
lib.Variables.CooldownDisplayType 				= lib.Variables.Defaults["CooldownType"];

-- Swap Variables
lib.Variables.SwapNoAction 						= false;
lib.Variables.SwapKind 							= nil;
lib.Variables.SwapValue 						= nil;

-- Binding Variables
lib.Variables.BindingFrame						= nil;
lib.Variables.BindingButton						= nil;

-- Misc Variables
lib.Variables.UpdateTime						= 0.5;
lib.Variables.Inset 							= 2;

-- Button Graphics
lib.Variables.Graphics.NormalTexture			= "Interface\\Addons\\libFoxActionButton\\Graphics\\CustomButton01.tga";
lib.Variables.Graphics.PushedTexture			= "Interface\\Buttons\\UI-Quickslot-Depress";
lib.Variables.Graphics.HighlightTexture			= "Interface\\Buttons\\ButtonHilight-Square";



