-- libFoxConsole.lua
-- Handles, Printing Messages & Errors

----------------------------------------------------------------------------------------------------
-- Library Variables
----------------------------------------------------------------------------------------------------
local LibraryName 	= "FoxConsole";
local LibraryVersion	= "1.0";
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName, true) == nil) then
	lib = LibStub:NewLibrary(LibraryName, LibraryVersion);
else
	lib = LibStub:GetLibrary(LibraryName);
end

----------------------------------------------------------------------------------------------------
-- Console
----------------------------------------------------------------------------------------------------
Console = LibStub:GetLibrary("AceConsole-3.0");

----------------------------------------------------------------------------------------------------
-- Color Strings
----------------------------------------------------------------------------------------------------
local colorWhite 	= "|CFFFFFFFF";
local colorRed 		= "|CFFFF0000";
local colorGreen 	= "|CFF00FF00";
local colorBlue 	= "|CFF0000FF";
local colorYellow 	= "|CFFFFF569";
local colorPink 	= "|CFFF58CBA";
local colorPurple 	= "|CFF9482C9";

local colors 	= {};
colors[0] 		= colorWhite;
colors[1] 		= colorRed;
colors[2] 		= colorGreen;
colors[3] 		= colorBlue;
colors[4] 		= colorYellow;
colors[5] 		= colorPink;
colors[6] 		= colorPurple;

local colorReturn 	= "|r";
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Console & Error Methods
----------------------------------------------------------------------------------------------------
function lib.Print(addonName, message, color)
	local colorStr = "";
	if (type(color) == "number") then
		if ((color >= 0) and (color < #colors)) then
			colorStr = colors[color];
		end
	elseif (type(color) == "string") then
		colorStr = color;
	end
	Console:Print(colorStr..addonName..": "..colorReturn..message);
end
function lib.Error(addonName, message, throwError)
	throwError = throwError or false;
	if (throwError) then
		Console:Print(colorWhite..addonName..": "..colorRed..message..colorReturn);
		error(addonName..": "..message, 4);
	else
		Console:Print(colorWhite..addonName..": "..colorRed..message..colorReturn);
	end
end
function lib.CondError(addonName, expression1, message, silent)
	silent = silent or false;
	if (expression1) then
		lib.Error(addonName, message, not silent)
	end
end
